/******************************************************************************
*******************************************************************************
**
**    Copyright 2001 Sistina Software, Inc.
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      Kenneth W. Preslan <kpreslan@sistina.com>
** 
**    Module name: misc.c
**
*******************************************************************************
******************************************************************************/

#include "gfs.h"





/**
 * gfs_glock_arch_init - Initialize the arch dependent part of a gfs_glock_t
 * @sdp: the filesystem
 * @gl: the glock
 *
 */

void gfs_glock_arch_init(gfs_sbd_t *sdp, gfs_glock_t *gl)
{
  ENTER(GFN_GLOCK_ARCH_INIT);

  osi_spinlock_init(&gl->gl_arch.gl_buflock);
  osi_list_head_init(&gl->gl_arch.gl_bufs);

  EXIT(GFN_GLOCK_ARCH_INIT);
}


/**
 * gfs_trans_prep_for_ail - called before a trans is put into the AIL
 * @sdp: the filesystem
 * @tr: the transaction
 *
 */

void gfs_trans_prep_for_ail(gfs_sbd_t *sdp, gfs_trans_t *tr)
{
  ENTER(GFN_TRANS_PREP_FOR_AIL);

  osi_atomic_set(&tr->tr_arch.tr_bufs, tr->tr_num_buf);

  EXIT(GFN_TRANS_PREP_FOR_AIL);
}


/**
 * gfs_trans_check_empty - Check whether or not a trans in the AIL has been synced
 * @sdp: the filesystem
 * @tr: the transaction
 *
 */

int gfs_trans_check_empty(gfs_sbd_t *sdp, gfs_trans_t *tr)
{
  int ret;

  ENTER(GFN_TRANS_CHECK_EMPTY);

  ret = (osi_atomic_read(&tr->tr_arch.tr_bufs)) ? FALSE : TRUE;

  EXIT(GFN_TRANS_CHECK_EMPTY);

  return ret;
}


