/******************************************************************************
*******************************************************************************
**
**    Copyright 2001 Sistina Software, Inc.
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      Kenneth W. Preslan <kpreslan@sistina.com>
** 
**    Module name: page.c
**
*******************************************************************************
******************************************************************************/

#include "gfs.h"
#include "dio.h"





/**
 * zap_inode_mapping - Ensures a page fault on address space access
 * @mapping: The address space
 *
 */

STATIC void zap_inode_mapping(struct inode *inode)
{
  struct vm_area_struct *vma;

  ENTER(GFN_ZAP_INODE_MAPPING);

  for (vma = inode->i_mmap; vma; vma = vma->vm_next_share)
  {
    struct mm_struct *mm = vma->vm_mm;
    unsigned long start = vma->vm_start;
    unsigned long end = vma->vm_end;
    unsigned long len = end - start;

    if (!(vma->vm_flags & VM_SHARED))
      continue;

    flush_cache_range(mm, start, end);
    zap_page_range(mm, start, len);
    flush_tlb_range(mm, start, end);
  }

  EXIT(GFN_ZAP_INODE_MAPPING);
}


/**
 * gfs_inval_bh - Invalidate all pages associated with a glock
 * @sdp: the filesystem
 * @gl: the glock
 *
 */

void gfs_inval_pg(gfs_sbd_t *sdp, gfs_glock_t *gl)
{
  gfs_inode_t *ip = (gfs_inode_t *)gl->gl_object;
  struct inode *inode;

  ENTER(GFN_INVAL_PG);

  if (ip)
  {
    inode = ip->i_vino;
    if (inode)
    {
      zap_inode_mapping(inode);
      truncate_inode_pages(inode, 0);
    }
  }

  EXIT(GFN_INVAL_PG);
}


