/******************************************************************************
*******************************************************************************
**
**    Copyright 2001 Sistina Software, Inc.
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      Kenneth W. Preslan <kpreslan@sistina.com>
** 
**    Module name: misc.c
**
*******************************************************************************
******************************************************************************/

#include "gfs.h"
#include "dio.h"





/**
 * gfs_glock_arch_init - Initialize the arch dependent part of a gfs_glock_t
 * @sdp: the filesystem
 * @gl: the glock
 *
 */

void gfs_glock_arch_init(gfs_sbd_t *sdp, gfs_glock_t *gl)
{
  ENTER(GFN_GLOCK_ARCH_INIT);

  osi_list_head_init(&gl->gl_arch.gl_bufs);

  EXIT(GFN_GLOCK_ARCH_INIT);
}


/**
 * gfs_trans_prep_for_ail - called before a trans is put into the AIL
 * @sdp: the filesystem
 * @tr: the transaction
 *
 */

void gfs_trans_prep_for_ail(gfs_sbd_t *sdp, gfs_trans_t *tr)
{
  ENTER(GFN_TRANS_PREP_FOR_AIL);

  osi_list_head_init(&tr->tr_arch.tr_bufs);

  EXIT(GFN_TRANS_PREP_FOR_AIL);
}


/**
 * gfs_trans_check_empty - Check whether or not a trans in the AIL has been synced
 * @sdp: the filesystem
 * @tr: the transaction
 *
 */

int gfs_trans_check_empty(gfs_sbd_t *sdp, gfs_trans_t *tr)
{
  gfs_bufdata_t *bd;
  osi_buf_t *bh;

  ENTER(GFN_TRANS_CHECK_EMPTY);


  osi_spin_lock(&lru_list_lock);

  while (!osi_list_empty(&tr->tr_arch.tr_bufs))
  {
    bd = list_entry(tr->tr_arch.tr_bufs.next, gfs_bufdata_t, bd_arch.bd_ail_list);
    bh = bd->bd_bh;

    osi_atomic_inc(&bh->b_count);
    osi_spin_unlock(&lru_list_lock);


    gfs_lock_buffer(bh);

    if (!bd->bd_arch.bd_pinned && !buffer_dirty(bh) && !buffer_locked(bh))
    {
      osi_spin_lock(&lru_list_lock);
      if (bd->bd_arch.bd_ail_tr)
      {
	bd->bd_arch.bd_ail_tr = NULL;
	osi_list_del(&bd->bd_arch.bd_ail_list);
      }

      gfs_unlock_buffer(bh);
      osi_atomic_dec(&bh->b_count);
      continue;
    }

    gfs_unlock_buffer(bh);


    osi_atomic_dec(&bh->b_count);

    EXIT(GFN_TRANS_CHECK_EMPTY);
    return FALSE;
  }

  osi_spin_unlock(&lru_list_lock);


  EXIT(GFN_TRANS_CHECK_EMPTY);

  return TRUE;
}


