/******************************************************************************
*******************************************************************************
**
**    Copyright 2001 Sistina Software, Inc.
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      Kenneth W. Preslan <kpreslan@sistina.com>
** 
**    Module name: glops.c
**
*******************************************************************************
******************************************************************************/

#include "gfs.h"
#include "blklist.h"
#include "dio.h"
#include "glock.h"
#include "inode.h"
#include "log.h"
#include "page.h"
#include "super.h"





/**
 * sync_meta - Sync the dirty data for a regular metadata glock
 * @sdp: the filesystem
 * @gl: the glock
 * @flags: GFS_SYNC_WAIT or GFS_SYNC_INVISIBLE
 *
 */

STATIC void sync_meta(gfs_sbd_t *sdp, gfs_glock_t *gl, int flags)
{
}


/**
 * release_meta - prepare a regular metadata glock to be released
 * @sdp: the filesystem
 * @gl: the glock
 *
 */

STATIC void release_meta(gfs_sbd_t *sdp, gfs_glock_t *gl)
{
}


/**
 * release_inode - prepare a inode glock to be released
 * @sdp: the filesystem
 * @gl: the glock
 *
 */

STATIC void release_inode(gfs_sbd_t *sdp, gfs_glock_t *gl)
{
}


/**
 * lock_inode - operation done after an inode lock is locked by a process
 * @sdp: the filesystem
 * @gl: the glock
 * @flags: the flags passed into gfs_glock()
 *
 * Returns: 0 on success, -EXXX on failure
 */

STATIC int lock_inode(gfs_sbd_t *sdp, gfs_glock_t *gl, int flags)
{
  return 0;
}


/**
 * unlock_inode - operation done before an inode lock is unlocked by a process
 * @sdp: the filesystem
 * @gl: the glock
 * @flags: the flags passed into gfs_gunlock()
 *
 * Returns: 0 on success, -EXXX on failure
 */

STATIC void unlock_inode(gfs_sbd_t *sdp, gfs_glock_t *gl, int flags)
{
}


/**
 * acquire_rgrp - operation done after a rgrp lock is acquired
 * @sdp: the filesystem
 * @gl: the glock
 *
 */

STATIC void acquire_rgrp(gfs_sbd_t *sdp, gfs_glock_t *gl)
{
}


/**
 * release_rgrp - prepare a rgrp glock to be released
 * @sdp: the filesystem
 * @gl: the glock
 *
 */

STATIC void release_rgrp(gfs_sbd_t *sdp, gfs_glock_t *gl)
{
}


/**
 * lock_rgrp - operation done after an rgrp lock is locked by a process
 * @sdp: the filesystem
 * @gl: the glock
 * @flags: the flags passed into gfs_glock()
 *
 * Returns: 0 on success, -EXXX on failure
 */

STATIC int lock_rgrp(gfs_sbd_t *sdp, gfs_glock_t *gl, int flags)
{
  return 0;
}


/**
 * unlock_rgrp - operation done before an rgrp lock is unlocked by a process
 * @sdp: the filesystem
 * @gl: the glock
 * @flags: the flags passed into gfs_gunlock()
 *
 * Returns: 0 on success, -EXXX on failure
 */

STATIC void unlock_rgrp(gfs_sbd_t *sdp, gfs_glock_t *gl, int flags)
{
}


/**
 * release_trans - prepare the transaction glock to be released
 * @sdp: the filesystem
 * @gl: the glock
 *
 */

STATIC void release_trans(gfs_sbd_t *sdp, gfs_glock_t *gl)
{
}





gfs_glock_operations_t gfs_meta_glops =
{
  go_sync:              sync_meta,
  go_release:           release_meta,
  go_free:              gfs_free_bh,
  go_type:              LM_TYPE_META,
  go_name:              "Misc On-Disk Metadata"
};

gfs_glock_operations_t gfs_inode_glops =
{
  go_sync:              sync_meta,
  go_release:           release_inode,
  go_lock:              lock_inode,
  go_unlock:            unlock_inode,
  go_free:              gfs_free_bh,
  go_type:              LM_TYPE_INODE,
  go_name:              "Inode"
};

gfs_glock_operations_t gfs_rgrp_glops =
{
  go_sync:              sync_meta,
  go_acquire:           acquire_rgrp,
  go_release:           release_rgrp,
  go_lock:              lock_rgrp,
  go_unlock:            unlock_rgrp,
  go_free:              gfs_free_bh,
  go_type:              LM_TYPE_RGRP,
  go_name:              "Inode"
};

gfs_glock_operations_t gfs_nopen_glops =
{
  go_type:              LM_TYPE_NOPEN,
  go_name:              "N-Open"
};

gfs_glock_operations_t gfs_flock_glops =
{
  go_type:              LM_TYPE_FLOCK,
  go_name:              "Flock"
};

gfs_glock_operations_t gfs_plock_head_glops =
{
  go_type:              LM_TYPE_PLOCK_HEAD,
  go_name:              "Plock head"
};

gfs_glock_operations_t gfs_plock_glops =
{
  go_type:              LM_TYPE_PLOCK,
  go_name:              "Plock"
};

gfs_glock_operations_t gfs_nondisk_glops =
{
  go_type:              LM_TYPE_NONDISK,
  go_name:              "Non-Disk"
};

gfs_glock_operations_t gfs_trans_glops =
{
  go_release:           release_trans,
  go_type:              LM_TYPE_NONDISK,
  go_name:              "Trans-Lock"
};
