/******************************************************************************
*******************************************************************************
**
**    Copyright 1995-1999 Regents of the University of Minnesota
**    Portions Copyright 2001 Sistina Software, Inc.
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      Kenneth W. Preslan <kpreslan@sistina.com>
** 
**    Module name: blklist.h
**
*******************************************************************************
******************************************************************************/

#ifndef __BLKLIST_DOT_H__
#define __BLKLIST_DOT_H__


gfs_rgrpd_t *gfs_blk2rgrpd(gfs_sbd_t *sdp, uint64 blk);
gfs_rgrpd_t *gfs_rgrpd_get_by_num(gfs_sbd_t *sdp, unsigned int num);
gfs_rgrpd_t *gfs_rgrpd_get_with_hint(gfs_sbd_t *sdp, uint64 hint);

void __gfs_rgrpd_put(gfs_rgrpd_t *rgd);
static __inline__ void gfs_rgrpd_put(gfs_rgrpd_t *rgd)
{
  if (osi_atomic_dec_and_test(&rgd->rd_count))
    __gfs_rgrpd_put(rgd);
}

void gfs_clear_rgrpd(gfs_sbd_t *sdp);

int gfs_rindex_hold(gfs_sbd_t *sdp);
void gfs_rindex_release(gfs_sbd_t *sdp);

int gfs_rgrp_read(gfs_rgrpd_t *rgd);
void gfs_rgrp_relse(gfs_rgrpd_t *rgd);
void gfs_rgrp_verify(gfs_rgrpd_t *rgd);

void gfs_rgrp_lvb_fill(gfs_rgrpd_t *rgd);
int gfs_rgrp_lvb_init(gfs_rgrpd_t *rgd, int force);

int gfs_inplace_reserv(gfs_inode_t *ip);
void gfs_inplace_relse(gfs_inode_t *ip);

int gfs_blkalloc(gfs_inode_t *ip, uint64 *block);
int gfs_metaalloc(gfs_inode_t *ip, uint64 *block);
void gfs_dipick(gfs_sbd_t *sdp, gfs_ipreserv_t *res, gfs_inum_t *inum);
int gfs_dialloc(gfs_sbd_t *sdp, gfs_ipreserv_t *res,
		gfs_inum_t *expected_inum, gfs_glock_t *expected_gl);

int gfs_blkfree(gfs_inode_t *ip, uint64 bstart, uint32 blen);
int gfs_metafree(gfs_inode_t *ip, uint64 bstart, uint32 blen);
int gfs_difree(gfs_inode_t *ip, gfs_rgrpd_t *rgd);



struct gfs_rgrp_list
{
  unsigned int     rl_rgrps;
  gfs_rgrpd_t      **rl_list;
  unsigned int     rl_space;
};
typedef struct gfs_rgrp_list gfs_rgrp_list_t;

int gfs_rlist_add(gfs_sbd_t *sdp, gfs_rgrp_list_t *rlist, uint64 block);
void gfs_rlist_sort(gfs_rgrp_list_t *rlist);
void gfs_rlist_free(gfs_rgrp_list_t *rlist);


#endif  /*  __BLKLIST_DOT_H__  */
