/******************************************************************************
*******************************************************************************
**
**    Copyright 2000-2001 Sistina Software, Inc. 
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      Kenneth W. Preslan <kpreslan@sistina.com>
** 
**    Module name: debug.c
**
*******************************************************************************
******************************************************************************/

#include "gfs.h"





#ifdef DEBUG_PRINT


/**
 * gfs_icdb_inc
 * @add:
 *
 */

void gfs_icdb_inc(unsigned int add)
{
  gfs_icdb_point += add;

  if (gfs_icdb_point > GFS_ICDB_BUF_LEN)
    osi_panic("GFS:  debug buffer overflow\n");

  if (gfs_icdb_point + GFS_ICDB_BUF_WIDTH > GFS_ICDB_BUF_LEN)
  {
    osi_memset(gfs_icdb_buf + gfs_icdb_point, 0, GFS_ICDB_BUF_LEN - gfs_icdb_point);
    gfs_icdb_point = 0;
  }
}


/**
 * gfs_icdb_print
 *
 */

void gfs_icdb_print()
{
  int x;

  osi_printf("\nDebug log:\n\n");

  osi_sema_lock(&gfs_icdb_lock);

  for (x = gfs_icdb_point; x < GFS_ICDB_BUF_LEN; x++)
    if (gfs_icdb_buf[x])
      osi_printf("%c", gfs_icdb_buf[x]);

  for (x = 0; x < gfs_icdb_point; x++)
    if (gfs_icdb_buf[x])
      osi_printf("%c", gfs_icdb_buf[x]);

  osi_sema_unlock(&gfs_icdb_lock);

  osi_printf("\n");
}


#endif  /*  DEBUG_PRINT  */



#ifdef DEBUG_PROFILE

/**
 * gfs_memcpy - Copy memory
 * @dest: the destination pointer
 * @src: the source pointer
 * @size: the number of bytes to copy
 *
 */

void gfs_memcpy(void *dest, const void *src, unsigned int size)
{
  ENTER(GFN_MEMCPY);

  osi_memcpy(dest, src, size);

  EXIT(GFN_MEMCPY);
}


/**
 * gfs_memset - fill memory with a pattern
 * @str: the memory pointer
 * @pattern: the pattern to fill with
 * @size: the number of bytes to fill
 *
 */

void gfs_memset(void *dest, unsigned int pattern, unsigned int size)
{
  ENTER(GFN_MEMSET);

  osi_memset(dest, pattern, size);

  EXIT(GFN_MEMSET);
}

#endif  /*  DEBUG_PROFILE  */


