/******************************************************************************
*******************************************************************************
**
**    Copyright 2000-2001 Sistina Software, Inc. 
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      Kenneth W. Preslan <kpreslan@sistina.com>
** 
**    Module name: debug.h
**
*******************************************************************************
******************************************************************************/

#ifndef __DEBUG_DOT_H__
#define __DEBUG_DOT_H__


#ifdef DEBUG_PRINT


#define GFS_ICDB_BUF_LEN (1048576)
#define GFS_ICDB_BUF_WIDTH (256)

EXTERN char gfs_icdb_buf[GFS_ICDB_BUF_LEN];
EXTERN unsigned int gfs_icdb_point INIT(0);
EXTERN osi_sema_t gfs_icdb_lock;

void gfs_icdb_inc(unsigned int add);
void gfs_icdb_print(void);


#define db_printf(fmt, args...) \
do { \
  osi_sema_lock(&gfs_icdb_lock); \
  gfs_icdb_inc(osi_sprintf(gfs_icdb_buf + gfs_icdb_point, fmt, ##args)); \
  osi_sema_unlock(&gfs_icdb_lock); \
} while(0)


#else  /*  DEBUG_PRINT  */
#define db_printf(fmt, args...)
#define gfs_icdb_print()
#endif  /*  DEBUG_PRINT  */



#if DEBUG_TRACE


#define ENTER_PRINT(x) \
{ \
  if (gfs_debug_flags[(x) / 8] & (1 << ((x) % 8))) \
    db_printf("En %s %ld\n", gfs_debug_flag_names[(x)], osi_pid()); \
}

#define EXIT_PRINT(x) \
{ \
  if (gfs_debug_flags[(x) / 8] & (1 << ((x) % 8))) \
    db_printf("Ex %s %ld\n", gfs_debug_flag_names[(x)], osi_pid()); \
}

#define IFDB(x, do) \
{ \
  if (gfs_debug_flags[(x) / 8] & (1 << ((x) % 8))) \
    {do} \
}

EXTERN unsigned char gfs_debug_flags[DEBUG_FLAG_BYTES];

#else  /*  DEBUG_TRACE  */
#define ENTER_PRINT(x)
#define EXIT_PRINT(x)
#define IFDB(x, do)
#endif  /*  DEBUG_TRACE  */



#ifdef DEBUG_PROFILE


EXTERN gfs_profile_t gfs_prof_stats[DEBUG_FLAGS];
EXTERN uint64 gfs_prof_neg_count INIT(0);


#define ENTER_PROFILE(x) uint64 gfs_profile_time = osi_gettimeofday();

#define EXIT_PROFILE(x) \
{ \
  uint64 stop_time = osi_gettimeofday(); \
  if (stop_time >= gfs_profile_time) \
    gfs_profile_time = stop_time - gfs_profile_time; \
  else \
  { \
    gfs_profile_time = 0; \
    gfs_prof_neg_count++; \
  } \
  gfs_prof_stats[(x)].total_calls++; \
  gfs_prof_stats[(x)].total_micros += gfs_profile_time; \
  if (gfs_prof_stats[(x)].min_micros > gfs_profile_time) \
    gfs_prof_stats[(x)].min_micros = gfs_profile_time; \
  if (gfs_prof_stats[(x)].max_micros < gfs_profile_time) \
    gfs_prof_stats[(x)].max_micros = gfs_profile_time; \
}

#define PROFILE_TAP(x) \
{ \
  gfs_prof_stats[(x)].total_calls++; \
  gfs_prof_stats[(x)].total_micros = 1; \
  gfs_prof_stats[(x)].min_micros = 1; \
  gfs_prof_stats[(x)].max_micros = 1; \
}

void gfs_memcpy(void *dest, const void *src, unsigned int size);
void gfs_memset(void *dest, unsigned int pattern, unsigned int size);

#else  /*  DEBUG_PROFILE  */
#define ENTER_PROFILE(x)
#define EXIT_PROFILE(x)
#define PROFILE_TAP(x)
#define gfs_memcpy osi_memcpy
#define gfs_memset osi_memset
#endif  /*  DEBUG_PROFILE  */



#define DBTAG(x)
#define ENTER(x) ENTER_PROFILE((x)); ENTER_PRINT((x));
#define ENTER2(x) ENTER(x)
#define EXIT(x) EXIT_PRINT((x)); EXIT_PROFILE((x));
#define DO_PROF(x, do) \
{ \
  ENTER_PROFILE((x)); \
  ENTER_PRINT((x)); \
  {do} \
  EXIT_PRINT((x)); \
  EXIT_PROFILE((x)); \
}
#define DO_PROF2(x, do) DO_PROF(x, do)



#define con_printf(fmt, args...) \
do { \
  osi_printf(fmt, ##args); \
  db_printf(fmt, ##args); \
} while(0)


#endif  /*  __DEBUG_DOT_H__  */
