/******************************************************************************
*******************************************************************************
**
**    Copyright 1995-1999 Regents of the University of Minnesota
**    Portions Copyright 2001 Sistina Software, Inc.
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      Kenneth W. Preslan <kpreslan@sistina.com>
** 
**    Module name: file.h
**
*******************************************************************************
******************************************************************************/

#ifndef __FILE_DOT_H__
#define __FILE_DOT_H__

typedef int (*read_copy_fn_t)(osi_buf_t *bh, char *buf, int offset, int size);
typedef int (*write_copy_fn_t)(gfs_inode_t *ip, osi_buf_t *bh, const char *buf, int offset, int size, int new);


int gfs_copy2mem(osi_buf_t *bh, char *buf, int offset, int size);
int gfs_copy2user(osi_buf_t *bh, char *buf, int offset, int size);
int64 gfs_readi(gfs_inode_t *ip, char *buf, uint64 offset, uint64 size,
		read_copy_fn_t copy_fn);

int gfs_copy_from_mem(gfs_inode_t *ip, osi_buf_t *bh, const char *buf, int offset, int size, int new);
int gfs_copy_from_user(gfs_inode_t *ip, osi_buf_t *bh, const char *buf, int offset, int size, int new);
int64 gfs_writei(gfs_inode_t *ip, const char *buf, uint64 offset, uint64 size,
		 write_copy_fn_t copy_fn);


static __inline__ int64 gfs_internal_read(gfs_inode_t *ip, char *buf, uint64 offset, uint64 size)
{
  return gfs_readi(ip, buf, offset, size, gfs_copy2mem);
}

static __inline__ int64 gfs_internal_write(gfs_inode_t *ip, const char *buf, uint64 offset, uint64 size)
{
  return gfs_writei(ip, buf, offset, size, gfs_copy_from_mem);
}

#endif  /*  __FILE_DOT_H__  */

