/******************************************************************************
*******************************************************************************
**
**    Copyright 1995-1999 Regents of the University of Minnesota
**    Portions Copyright 2000-2001 Sistina Software, Inc.
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      Kenneth W. Preslan <kpreslan@sistina.com>
** 
**    Module name: gfs.h
**
*******************************************************************************
******************************************************************************/

#ifndef __GFS_DOT_H__
#define __GFS_DOT_H__



/*  OS specific files  */

#ifdef __KERNEL__
#include "osi.h"
#else  /*  __KERNEL__  */
#include "osi_user.h"
#endif  /*  __KERNEL__  */


/*  Extern Macro  */

#ifndef EXTERN
#define EXTERN extern
#define INIT(X)
#else
#undef EXTERN
#define EXTERN
#define INIT(X) =X 
#endif


/*  Static Macro  */

#ifndef DEBUG_SYMBOLS
#define STATIC static
#else
#define STATIC
#endif  /*  DEBUG_SYMBOLS  */


/*  Includes  */

#include "release.cf"
#include "global.h"

/*  Needed for incore.h  */
#include "gfs_ondisk.h"
#include "lm_interface.h"
#include "lvb.h"
#include "osi_list.h"
#include "gfs_mount.h"

#include "incore.h"

#include "gfs_ioctl.h"

#include "gfs_debug.h"
#include "debug.h"

#include "gmalloc.h"


/*  Macros  */

#ifdef DEBUG_CRASHDUMP
#define GFS_ICDB_PRINT() gfs_icdb_print()
#define GFS_PANIC_MESSAGE ("\n""   _   _           _   _                _ \n""  | | | | ___  ___| | | | __ ___      _| |\n""  | |_| |/ _ \\/ _ \\ |_| |/ _` \\ \\ /\\ / / |\n""  |  _  |  __/  __/  _  | (_| |\\ V  V /|_|\n""  |_| |_|\\___|\\___|_| |_|\\__,_| \\_/\\_/ (_)\n")
#else
#define GFS_ICDB_PRINT()
#define GFS_PANIC_MESSAGE ("GFS:  Homer sleep now...\n")
#endif  /*  DEBUG_CRASHDUMP  */

#ifdef DEBUG_BUG
/*  Call osi_panic() after osi_bug() to keep the compiler from bitching  */
#define GFS_PANIC(str) do { con_printf((str)); osi_bug(); osi_panic((str)); } while (0)
#else
#define GFS_PANIC(str) osi_panic((str));
#endif  /*  DEBUG_BUG  */

#define GFS_ASSERT(x, todo) \
do \
{ \
  if (!(x)) \
  { \
    con_printf("\nGFS:  Assertion failed on line %d of file %s\n" \
               "GFS:  assertion:  \"%s\"\n" \
               "GFS:  time = %"FMT_64"u\n", \
	       __LINE__, __FILE__, #x, osi_gettimeofday()); \
    {todo} \
    con_printf("\n"); \
    GFS_ICDB_PRINT(); \
    GFS_PANIC(GFS_PANIC_MESSAGE); \
  } \
} \
while (0)



#define NO_CREATE    (0)
#define CREATE       (1)


/*  Divide x by y.  Round up if there is a remainder.  */
#define DIV_RU(x, y) (((x) + (y) - 1) / (y))


#endif  /*  __GFS_DOT_H__  */
