/******************************************************************************
*******************************************************************************
**
**    Copyright 1995-1999 Regents of the University of Minnesota
**    Portion Copyright 2000-2001 Sistina Software, Inc.
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      Kenneth W. Preslan <kpreslan@sistina.com>
** 
**    Module name: gmalloc.h
**
**    Description: gmalloc accounting code
**
*******************************************************************************
******************************************************************************/

#ifndef __GMALLOC_DOT_H__
#define __GMALLOC_DOT_H__


#ifdef DEBUG_MEM

#define GMALLOC_HASH_SIZE 4096
EXTERN osi_list_head_t gmalloc_head[GMALLOC_HASH_SIZE];

struct gmalloc_stats
{
  unsigned int         ct; 
  unsigned int         ct_max;
  unsigned int         ct_total;
  unsigned int         bytes;   
  unsigned int         bytes_max;
  uint64               bytes_total;
};
typedef struct gmalloc_stats gmalloc_stats_t;

EXTERN gmalloc_stats_t gm_stats;
EXTERN osi_spinlock_t gmalloc_lock;

void *gmalloc_wrapper(unsigned int size, char *file, unsigned int line);
void gfree_wrapper(void *data, unsigned int size, char *file, unsigned int line);
void gmalloc_check(void);

#define gmalloc(size) gmalloc_wrapper(size, __FILE__, __LINE__)
#define gfree(data, size) gfree_wrapper(data, size, __FILE__, __LINE__) 

#else  /*  DEBUG_MEM  */

#define gmalloc_check()

#define gmalloc(size) osi_malloc(size)
#define gfree(data, size) osi_free(data, size)

#endif  /*  DEBUG_MEM  */



/*  Slab cache allocator prototypes  */

gfs_glock_t *gfs_alloc_glock(void);
void gfs_free_glock(gfs_glock_t *gl);

gfs_inode_t *gfs_alloc_inode(void);
void gfs_free_inode(gfs_inode_t *ip);

gfs_bufdata_t *gfs_alloc_bufdata(void);
void gfs_free_bufdata(gfs_bufdata_t *);

gfs_rgrpd_t *gfs_alloc_rgrpd(void);
void gfs_free_rgrpd(gfs_rgrpd_t *rgrp);


#endif /* __GMALLOC_DOT_H__ */


