/******************************************************************************
*******************************************************************************
**
**    Copyright 1995-1999 Regents of the University of Minnesota
**    Portions Copyright 1999-2001 Sistina Software, Inc.
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      Kenneth W. Preslan <kpreslan@sistina.com>
** 
**    Module name: inode.h
**
*******************************************************************************
******************************************************************************/

#ifndef __INODE_DOT_H__
#define __INODE_DOT_H__

int gfs_copyin_dinode(gfs_inode_t *ip);

gfs_inode_t *gfs_get_istruct(gfs_sbd_t *sdp, gfs_glock_t *gl, gfs_inum_t *inum, int create);
static __inline__ void gfs_hold_istruct(gfs_inode_t *ip)
{
  osi_atomic_inc(&ip->i_count);
}
void gfs_put_istruct(gfs_inode_t *ip);
void gfs_destroy_istruct(gfs_inode_t *ip);

int gfs_dealloc_inodes(gfs_sbd_t *sdp);
int gfs_limit_unlinked(gfs_sbd_t *sdp);
int gfs_change_nlink(gfs_inode_t *ip, int diff);

int gfs_lookupi(gfs_inode_t *dip, osi_qstr_t *name,
		int unlock, osi_cred_t *cred,
		gfs_inode_t **ipp);
int gfs_createi(gfs_inode_t *dip, osi_qstr_t *name,
		unsigned int type,
		int exclusive, osi_cred_t *cred,
		int *new, gfs_inode_t **ipp);
int gfs_unlinki(gfs_inode_t *dip, osi_qstr_t *dname, gfs_inode_t *ip);
int gfs_rmdiri(gfs_inode_t *dip, osi_qstr_t *dname, gfs_inode_t *ip);

int gfs_update_atime(gfs_inode_t *ip);
#define GFS_UPDATE_ATIME(gfs_inode) (((gfs_inode)->i_sbd->sd_noatime) ? 0 : gfs_update_atime((gfs_inode)))


/*  Symlink stuff  */

struct gfs_symlink_ops
{
  const char *name;
  const unsigned int len;
  unsigned int (*subst_fn)(char *str);
};

#define GFS_FAST_SYMLINK_SIZE (256)

unsigned int gfs_link_size(struct gfs_symlink_ops *ops, char *s);
void gfs_link_expand(struct gfs_symlink_ops *ops, char *out, char *in);


/*  Inlines  */

static __inline__ int gfs_is_stuffed(gfs_inode_t *ip)
{
  return !ip->i_di.di_height;
}

static __inline__ int gfs_is_jdata(gfs_inode_t *ip)
{
  return ip->i_di.di_flags & GFS_DIF_JDATA;
}


/*  These are architecture specific functions
    Either implement them or stub them out in your arch_* directory.  */

void gfs_inode_attr_in(gfs_inode_t *ip);
int gfs_permission(gfs_inode_t *ip, int mask, osi_cred_t *cred);
void gfs_drop_excess_inodes(gfs_sbd_t *sdp);


#endif  /*  __INODE_DOT_H__  */

