/******************************************************************************
*******************************************************************************
**
**    Copyright 1999-2001 Sistina Software, Inc.
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      Kenneth W. Preslan <kpreslan@sistina.com>
** 
**    Module name: 
**      trans.h
**
**    Description:
**      Transaction functions
**
*******************************************************************************
******************************************************************************/

#ifndef __TRANS_DOT_H__
#define __TRANS_DOT_H__


#define TRANS_COMPOUND          (0)

/*  Transactions that grow directories and create new dinodes  */

#define TRANS_CREATE            (1)

/*  Transactions that grow directories  */

#define TRANS_LINK              (2)
#define TRANS_RENAME            (3)

/*  Transactions that grow files  */

#define TRANS_WRITE             (4)
#define TRANS_BMAP              (5)
#define TRANS_TRUNC_GROW        (6)
#define TRANS_WRITEPAGE         (7)
#define TRANS_PREPWRITE         (8)
#define TRANS_JWRITE            (9)

/*  Transactions that don't touch rgrps  */

#define TRANS_NOTE_CH           (10)
#define TRANS_SETATTR           (11)
#define TRANS_DIR2FILE          (12)
#define TRANS_SET_SIZE          (13)
#define TRANS_UPDATE_ATIME      (14)

#define TRANS_UNLINK            (15)
#define TRANS_RMDIR             (16)

/*  Transactions that deallocate space  */

#define TRANS_LEAF_DEALLOC      (17)
#define TRANS_FILE_DEALLOC      (18)
#define TRANS_I_DEALLOC         (19)


int gfs_trans_begin(gfs_sbd_t *sdp, int type, unsigned int meta_blocks, unsigned int extra_blocks);
void gfs_trans_end(gfs_sbd_t *sdp);

void gfs_trans_add_gl(gfs_sbd_t *sdp, gfs_glock_t *gl);
void gfs_trans_add_bh(gfs_sbd_t *sdp, osi_buf_t *bh);

void gfs_trans_mark(gfs_sbd_t *sdp, unsigned int flag);
int gfs_trans_nopen_change(gfs_sbd_t *sdp, unsigned int type, gfs_inum_t *inum);

#define TRANS_IS_NEW            (53)
#define TRANS_IS_INCORE         (54)
void gfs_trans_print(gfs_sbd_t *sdp, gfs_trans_t *tr, int where);


/*  These are architecture specific functions
    Either implement them or stub them out in your arch_* directory.  */

void gfs_trans_prep_for_ail(gfs_sbd_t *sdp, gfs_trans_t *tr);
int gfs_trans_check_empty(gfs_sbd_t *sdp, gfs_trans_t *tr);


#endif  /*  __TRANS_DOT_H__  */

