/******************************************************************************
*
*    Copyright 1999 Regents of the University of Minnesota
*    Portions Copyright 1999-2001 Sistina Software, Inc.
*
*    This is free software released under the GNU General Public License.
*    There is no warranty for this software.  See the file COPYING for
*    details.
*
*    See the file CONTRIBUTORS for a list of contributors.
*
*    This file is maintained by:
*      Benjamin E Marzinski <marzinski@sistina.com>
* 
* This was heavily infulenced by sbull from the `Linux Device Drivers' book
* by Alessandro Rubini. And the original nbd. and a bunch of work I did on
* the glm.
*/

#ifdef MODVERSIONS
#include <linux/modversions.h>
#endif  /*  MODVERSIONS  */

/* #define __NO_VERSION__  don't define kernel_verion in module.h */
#include <linux/module.h>
/* #include <linux/version.h> 

char kernel_version [] = UTS_RELEASE; */

#include <linux/sched.h>
#include <linux/kernel.h> /* printk() */
#include <linux/malloc.h> /* kmalloc() */
#include <linux/fs.h>     /* everything... */
#include <linux/errno.h>  /* error codes */
#include <linux/timer.h>
#include <linux/types.h>  /* size_t */
#include <linux/fcntl.h>        /* O_ACCMODE */
#include <linux/hdreg.h>  /* HDIO_GETGEO */
#include <linux/ioctl.h>
#include <linux/proc_fs.h>
#include <linux/genhd.h> /* so we can get on /proc/partitions for pool */


#include <asm/system.h>   /* cli(), *_flags */
#include <asm/uaccess.h>  /* put_user */

#define MAJOR_NR gnbd_major /* force definitions on in blk.h */
int gnbd_major; /* must be declared before including blk.h */

#define DEVICE_NR(device) MINOR(device)   /* gnbd has no partition bits */
#define DEVICE_NAME "gnbd"               /* name for messaging */
#define DEVICE_INTR gnbd_intrptr         /* pointer to the bottom half */
#define DEVICE_NO_RANDOM                  /* no entropy to contribute */

#define DEVICE_OFF(d) /* do-nothing */
#define DEVICE_ON(d) /* do-nothing */
#define DEVICE_REQUEST gnbd_request

#define LOCAL_END_REQUEST

#define GNBD_MAX_DEV 256    /* the absolute max number of devices gnbd will
                               allow. Can't be changed */

#ifndef MAX_DEVICES         /* the max number that gnbd is configured to use*/
#define MAX_DEVICES 10      /* effects the size of config.. It makes the */
#endif                      /* interface much prettier, and no one should */
                            /* need more than 10 clients on any machine */
                            /* it can be changed in the makefile */ 

#include <linux/blk.h>

#define EXTERN
#define WANT_DEBUG_NAMES
#include "macros.h"
#include "gnbd.h"
#include "netmisc.h"
#include "debug.h"
#include "release.cf"


		

/******************************************************************************
 *
 * This structure is needed for /proc/partitions.  Making the device
 * partitionable with a max of zero partitions seems to be the easiest
 * solution to the passemble problem
 *
 */

/* The following items are obtained through kmalloc() in init_module() */

int gnbd_devs = 0;
int max_check = 0;
struct semaphore c_init;
gnbd_t *gnbd_devices[256];    /*This is so we can dynamically create devices */
int gnbd_blksizes[256];
int gnbd_sizes[256];
char *gnbd_device_names[256];  
struct hd_struct gnbd_partitions[256];

struct gendisk gnbd_gendisk = {
  0,                     /* Major Number, set by init_module */
  DEVICE_NAME,           /* defined in main.c */
  0,                     /* minor_shift. device number is the minor number */
  0,                     /* there are NO partitions for this device */
  255,                   /* there can be 255 different devices */
  NULL,                  /* no init function */
  gnbd_partitions,       /* Partition arry */
  gnbd_sizes,            /* Block sizes */
  256,                   /* actual number of devices */
  NULL,                  /* real devices pointer. unused */
  NULL,                  /* Next */
  gnbd_device_names      /* device_names */ 
};


#define GNBD_HARDSECT 512  /* .5k sectors */

#ifndef DEBUG_SYMBOLS
EXPORT_NO_SYMBOLS; /* we like to keep to ourselves. */
#else
#define EXPORT_SYMTAB
#endif

/*
 * "Non-prefixed symbols are static. They are meant to be assigned at
 * load time. Prefixed symbols are not static, so they can be used in
 * debugging."
 *
 * Yeah. what he said......
 */
static int major = 0;
MODULE_PARM(major, "i");
MODULE_PARM_DESC(major, "Used to change the major number that is used.");

static int rahead = 1; /* two sectors */
MODULE_PARM(rahead, "i");
MODULE_PARM_DESC(rahead, "Used to set the number of blocks that are read "
		 "ahead durring each block request.");


/* Now the public versions of some of those above. */


/* 
 * Few random bits of information.
 */
MODULE_AUTHOR("Ben Marzinski <marzinski@sistina.com>");
MODULE_DESCRIPTION("A network block device that is friendlier to gfs, "
		   "plus can have multiple clients to one server. "
		   "Also getting some clustery stuff. "
		   "Mostly its got bunches of newy goo. Or something like that. "
		   "More info at http://www.globalfilesystem.org");

/******************************************************************************
 * gndb_connect()
 * what needs to be done on a connect action.  Got pulled out here since
 * its called in two places. code reuse, modularity and all that funky
 * jazz.
 */
int gnbd_connect(int num, gnbd_t *dev)
{
  int err=0;

  

  ENTER(GNBD_CONNECT);

  if( ISgnbdDisConnected(dev) ) {
    printd(" loggin into %x:%x\n", dev->servip, dev->servport);
    err = trans_start_up(dev);
    if (err <0 ) {
      printd("failed trans_start_up\n");
      goto exit_gnbd_connect;
    } else {
      IFDB( GNBD_CONNECT,
	    printi("open: dev->size = %"FMT_64"x\n", dev->size););
      gnbd_sizes[num] = dev->size / 1024;
      gnbd_partitions[num].nr_sects = 2 * gnbd_sizes[num];
      gnbd_partitions[num].start_sect = -1; /* disable partition checking */
      IFDB( GNBD_CONNECT,
	    printi("partition check: "););
      resetup_one_dev(&gnbd_gendisk, num);
      IFDB( GNBD_CONNECT,
	    printk(KERN_ERR "dev %d: beg %lx, size %lx\n", num,
		   gnbd_partitions[num].start_sect,
		   gnbd_partitions[num].nr_sects););
      IFDB( GNBD_CONNECT,
	    printi("Set gnbd_sizes[%d] to %x\n", num, gnbd_sizes[num]););
    }
  }

 exit_gnbd_connect:
  EXIT(GNBD_CONNECT);
  return err;
}



/******************************************************************************
 * Open and close
 *
 * Ok, the new order of things. There is no such thing as a bad ip, thus 
 * there is no such thing as a setup state.
 * There is an Opened state that just tells us if the device is opened by
 * any process. Basically, if usage != 0 then Opened=True;
 * There is also a Connected state. This replaces the loggedin marker.
 * Since it means pretty much the exact same thing. Difference being that
 * now just because a device is disconnected, doesn't mean you can't open
 * and read or write to it...
 *
 * minor 0 is /dev/gnbdcomm and CANNOT hook to a server ever! 
 * This gives us a device that we can always open and do ioctls.
 */
int gnbd_open (struct inode *inode, struct file *filp)
{
  gnbd_t *dev; /* device information */
  int num;
  int err=0;

  ENTER(GNBD_OPEN);

  printd("Entered gnbd_open by %d\n",current->pid);

  if(!inode) {err = -EINVAL; goto exit_gnbd_open;}
  num = MINOR(inode->i_rdev);
  printd("                  by %d on %d\n",current->pid,num);
  if (num >= max_check || gnbd_devices[num] == NULL) {
    err = -ENODEV;
    goto exit_gnbd_open;
  }
  dev = gnbd_devices[num];

  printd("device %d: usage=%d Connected=%u Pending=%u\n", 
	 num, dev->usage, ISgnbdConnected(dev), ISgnbdPending(dev));

  if( num != 0) /* don't connect minor 0 */
    err = gnbd_connect(num, dev);

  dev->usage++;
  MOD_INC_USE_COUNT;
 exit_gnbd_open:
 EXIT(GNBD_OPEN);
  return 0;          /* success */
}

int gnbd_release (struct inode *inode, struct file *filp)
{
  int num;
  int err=0;
  gnbd_t *dev;

  ENTER(GNBD_RELEASE);

  if(!inode) {err = -EINVAL; goto exit_gnbd_release;}
  num = MINOR(inode->i_rdev);
  if (num >= max_check || gnbd_devices[num] == NULL) {
    err = -ENODEV;
    goto exit_gnbd_release;
  }
  dev = gnbd_devices[num];

  printd("Entered gnbd_release by %d on %d\n",current->pid, num);

  if(dev->usage <= 0)
    printe(" umm, usage is at %d, and we still"
	   " are trying to close???\n", dev->usage);

  dev->usage--;

  if (num != 0 && !dev->usage) { /* ie. ! ISgnbdOpened(dev) */
    fsync_dev(inode->i_rdev);
    invalidate_buffers(inode->i_rdev);
    /* smart enough not to do anything if its already been done */
    trans_shutdown(dev, TRUE);
  }
  MOD_DEC_USE_COUNT;
 exit_gnbd_release:
  EXIT(GNBD_RELEASE);
  return 0;
}

/******************************************************************************
 * gnbd_action
 * bust out for all the complexities of the new userland ioctl interface.
 */
int gnbd_action(int num, gnbd_action_t *arg)
{
  int err = 0;
  gnbd_t *dev;
  gnbd_action_t act;

  ENTER(GNBD_DO_ACTION);

  memset( &act, 0, sizeof(gnbd_action_t));
  err = get_user( act.action, &arg->action);
  err = get_user( act.device, &arg->device);
  err = get_user( act.ip, &arg->ip);
  err = get_user( act.port, &arg->port);

  printd( "gnbd action %x, device %x\n", act.action, act.device);

  if( act.device >= max_check || gnbd_devices[act.device] == NULL){
    err = -EINVAL;
    goto exit_gnbd_action;
  }
  /* If opened a device other than /dev/gnbdcomm, setting the device field
   * in the gnbd_action_t to zero means we want the device we opened.
   * If opened the gnbdcomm, and act.device is zero, its an error.
   */
  if( num == 0 && act.device == 0)
    {err = -EINVAL; goto exit_gnbd_action;}
  if( act.device != 0)
    num = act.device;
   
  dev = gnbd_devices[num]; 
  switch( act.action ) {
  case GNBD_ACT_SetIPPort:
    printd(" ::SetIPPort::dev %u:: ip %x port %u\n",
	   num, act.ip, act.port);
    dev->servip   = act.ip;
    dev->servport = act.port;
    break;
  case GNBD_ACT_Connect:
    printd(" ::Connect::dev %u::\n",num);
    err = gnbd_connect(num, dev);
    if (err < 0){
      printd("exitting Connect due to error : %d\n", err);
      dev->good_connect = FALSE;
      break;
    }
    dev->good_connect = TRUE;
    if (ISgnbdOpened(dev) || ISgnbdPending(dev))
      break;
    printd("rolling through to disconnect\n");
  case GNBD_ACT_Disconnect:
    printd(" ::DisConnect::dev %u::\n",num);
    if( ISgnbdConnected(dev) ) {
      printd("Shutting down now\n");
      trans_shutdown(dev, FALSE);
    }
    break;
  case GNBD_ACT_ForceClose:
    printd(" ::ForceClose::dev %u::\n",num);
    if( ISgnbdConnected(dev) ) {
      /* This is correct, but we might need to do more */
      trans_shutdown(dev, TRUE);
      /* wake any reads|writes (if any) so they can die*/
      wake_up_interruptible( &dev->gassed );
    }
    break;
  default:
    printi("Null action\n");
    break;
  }

 exit_gnbd_action:
  EXIT(GNBD_DO_ACTION);
  return err;
}


int init_device(char *name, int length)
{
  int found = -1;
  int minor_num;
  gnbd_t *temp;
  ENTER(INIT_DEVICE);
  down (&c_init);
  for(minor_num = 0; minor_num <= max_check; minor_num++){
    if (gnbd_devices[minor_num] == NULL){
      if(found == -1)
	found = minor_num;
    }
    else if (gnbd_device_names[minor_num] &&
	     strcmp(gnbd_device_names[minor_num],name) == 0){
      printe("gnbd_device name '%s' already in use\n", name);
      up(&c_init);
      EXIT(INIT_DEVICE);
      return -EEXIST;
    }
  }
  if (found < 0){
    printe("could not find a free minor number\n");
    up(&c_init);
    EXIT(INIT_DEVICE);
    return -ENODEV;
  }
  minor_num = found;
  temp = kmalloc(sizeof(gnbd_t), GFP_KERNEL);
  if (!temp){
    printe("could not allocate memory for device %s\n", name);
    up(&c_init);
    EXIT(INIT_DEVICE);
    return -ENOMEM;
  }
  gnbd_device_names[minor_num] = kmalloc(sizeof(char) * (length + 5),
					 GFP_KERNEL);
  if (gnbd_device_names[minor_num] == NULL){
    kfree(temp);
    printe("couldn't allocate memory for device name: %s\n", name);
    up(&c_init);
    EXIT(INIT_DEVICE);
    return -ENOMEM;
  }
  strcpy(gnbd_device_names[minor_num], "gnbd/");
  strncat(gnbd_device_names[minor_num], name, length);
  memset(temp, 0, sizeof(gnbd_t));
  temp->size = 0;
  temp->recvd_run = FALSE;
  temp->sender = MUTEX;
  temp->tiodmux = MUTEX_LOCKED;
  temp->tqmux = MUTEX;
  temp->Rhead = NULL;
  gnbd_devices[minor_num] = temp;
  if(minor_num == max_check && max_check < 255)
    max_check++;
  gnbd_devs++;
  up (&c_init);
  EXIT(INIT_DEVICE);
  return minor_num;
}

/* Major problem... Semaphore is pretty useless.  It needs to make sure that
 * the device is not currently in use.  Otherwise bad things could happer*/

void cleanup_device(int minor_num)
{
  ENTER(CLEANUP_DEVICE);
  down(&c_init);
  if (minor_num == max_check-1)
    max_check--;
  gnbd_devs--;
  gnbd_sizes[minor_num] = 0;
  gnbd_partitions[minor_num].nr_sects = 0;
  if (gnbd_device_names[minor_num] != NULL){
    kfree(gnbd_device_names[minor_num]);
    gnbd_device_names[minor_num] = NULL;
  }
  if (gnbd_devices[minor_num] != NULL){
    kfree(gnbd_devices[minor_num]);
    gnbd_devices[minor_num] = NULL;
  }
  up(&c_init);
  EXIT(CLEANUP_DEVICE);
  return;
}

int gnbd_device_work(gnbd_device_t *arg)
{
  int err = 0;
  int i;
  gnbd_device_t *dev;
  char *dev_name = NULL;
  ENTER(GNBD_DEVICE_WORK);
  if (!arg){
    printe("no argument was sent to gnbd_device_work!?!?\n");
    EXIT(GNBD_DEVICE_WORK);
    return -EINVAL;
  }
  dev = (gnbd_device_t *)kmalloc(sizeof(gnbd_device_t), GFP_KERNEL);
  if (!dev){
    printe("insufficient memory to create space for gnbd request\n");
    EXIT(GNBD_DEVICE_WORK);
    return -ENOMEM;
  }
  err = (int)copy_from_user((void *)dev, (void *)arg, sizeof(gnbd_device_t));
  if (err){
    printe("Bad address in argument?!?!?\n");
    goto dev_fail;
  }
  if(dev->length != 0){
    dev_name = (char *)kmalloc(sizeof(char) * dev->length, GFP_KERNEL);
    if (!dev_name){
      printe("insufficient memory to create space for device name\n");
      goto exit;
    }
    printd("dev->length = %d\n", dev->length);
    err = (int)copy_from_user((void *)dev_name, (void *)dev->name, dev->length);
    if (err){
      printe("Bad Address for device name?!?!?!\n");
      goto exit;
    }
    printd("dev_name = \"%s\"\n", dev_name);
  }
  else{
    dev->name = NULL;
  }
  if (dev->action == GNBD_DEV_CREATE){
    if (dev->name == NULL){
      printe("No device name?!?!\n");
      err = -EINVAL;
    }
    else
      err = init_device(dev_name, dev->length);
    if (err > 0){
      dev->minor_nr = err;   
      err = (int)copy_to_user((void *)arg, (void *)dev, sizeof(gnbd_device_t));
      if (err){
	printe("Bad address in argument?!?!?\n");
	goto exit;
      }
    }
    goto exit;
  }
  else if (dev->action == GNBD_DEV_REMOVE){ 
    if (dev->name != NULL){
      for(i=0; i<max_check; i++)
	if (gnbd_devices[i] != NULL &&
	    strncmp(gnbd_device_names[i], dev_name,
		    strlen(gnbd_device_names[i])) == 0){
	  cleanup_device(i);
	  goto exit;
	}
      printe("couldn't find a device named \"%s\"\n", dev_name);
      err = -EINVAL;
    }
    else{
      if (gnbd_devices[dev->minor_nr] != NULL){
	cleanup_device(dev->minor_nr);
	goto exit;
      }
      printe("couldn't find a device with minor number %d\n", dev->minor_nr);
      err = -EINVAL;
    }
  }
 exit:
  kfree(dev_name);
 dev_fail:
  kfree(dev);
  EXIT(GNBD_DEVICE_WORK);
  return err;
}


/******************************************************************************
 * The ioctl() implementation
 * I am the io control master!
 */

int gnbd_ioctl (struct inode *inode, struct file *filp,
		unsigned int cmd, unsigned long arg)
{
  gnbd_t *dev;
  int err = 0;
  int num = MINOR(inode->i_rdev);

  ENTER(GNBD_IOCTL);
  if(!inode) {err = -EINVAL; goto exit_gnbd_ioctl;}
  if (num >= max_check || gnbd_devices[num] == NULL){
    err = -ENODEV;
    goto exit_gnbd_ioctl;
  }
  dev = gnbd_devices[num];

  IFDB(GNBD_IOCTL, printi("ioctl 0x%x 0x%lx\n", cmd, arg); );

  switch(cmd) {
  case GNBD_ACTION:
    if(!suser()) {err = -EACCES; break;}
    err = gnbd_action( num, (gnbd_action_t*)arg );
    break;

  case GNBD_DEVICE:
    if(!suser()) {err = -EACCES; break;}
    err = gnbd_device_work((gnbd_device_t *)arg);
    break;

#ifdef DEBUG
  case GNBD_DEBUG_ON:
  case GNBD_DEBUG_OFF:
    {
      unsigned int flag;
      unsigned char *val;
      err = copy_from_user(&flag,(unsigned int*)arg,sizeof(unsigned int));
      if(err) break;
      if( flag >= DEBUG_FLAGS) {err=-EINVAL; break;}
      val = &debug_flags[flag / 8];

      if(cmd==GNBD_DEBUG_ON)
	*val |= 1 << (flag %8);
      else
	*val &= ~(1 << (flag %8));
    }
    break;
#else
  case GNBD_DEBUG_ON:
  case GNBD_DEBUG_OFF:
    printe("Compiled without DEBUG, so this is void.\n");
    err = -ENOSYS;
    break;
#endif /*DEBUG*/

  case GNBD_BLKSSZESET:
    /* Not sure if this is completely thought through. It really
     * should be looked at again when my mind isn't so loopie.
     */
    if(!suser()) {err = -EACCES; break;}
    if ((arg & 511) || (arg > PAGE_SIZE)) {err = -EINVAL; break;}
    gnbd_blksizes[num] = arg;
    break;

  case BLKSSZGET:
    /* Block size of media */
    if(!arg) {err = -EINVAL; break;}
    put_user_ret(gnbd_blksizes[num], (long*)arg, -EFAULT);
    break;

  case BLKGETSIZE:
    {
      u64 blocksize = gnbd_sizes[num];
      blocksize = blocksize * 1024 / GNBD_HARDSECT;
#ifdef DEBUG
      printk(KERN_ERR "gnbd_blksizes[%d] = %d\ngnbd_sizes[%d] = %d\n",num,
	     gnbd_blksizes[num], num, gnbd_sizes[num]);
#endif /* DEBUG */
      /* Return the device size, expressed in sectors */
      if (!arg) {err = -EINVAL; break;} /* NULL pointer: not valid */
      put_user_ret( blocksize, (long*)arg, -EFAULT);
      break;
    }
  case BLKFLSBUF: /* flush */
    if (!suser()) {err = -EACCES; break;}
    fsync_dev(inode->i_rdev);
    invalidate_buffers(inode->i_rdev);
    break;

  case BLKRAGET: /* return the readahead value */
    if (!arg) {err = -EINVAL; break;}
    put_user_ret(read_ahead[num], (long*)arg, -EFAULT);
    break;

  case BLKRASET: /* set the readahead value */
    if (!suser()) {err = -EACCES; break;}
    if (arg > 0xff) {err = -EINVAL; break;}/* limit it */
    read_ahead[num] = arg;
    break;

  case BLKRRPART: /* re-read partition table: can't do it */
    err = -EINVAL;
    break;

    RO_IOCTLS(inode->i_rdev, arg); /* the default RO operations */

  case HDIO_GETGEO:
    /* get geometry: we are faking it......
     * trim size to a multiple of 64 (32KB), have 16 sectors, and 4
     * heads, and whatever we need for cylinders.
     * ?data start at sector 4?
     */
    {
      struct hd_geometry *geo = (struct hd_geometry *)arg;
      int size;
      size = gnbd_sizes[num] * 1024 / GNBD_HARDSECT;
      size &= 0x3f;
      if(geo==NULL) {err = -EINVAL; break;}
      put_user(size>>6, &geo->cylinders);
      put_user(      4, &geo->heads);
      put_user(     16, &geo->sectors);
      put_user(      4, &geo->start);
      break;
    }
  default:
    err = -EINVAL;
    break;
  }

 exit_gnbd_ioctl:
  EXIT(GNBD_IOCTL);
  return err;
}
 

/******************************************************************************
 * wrappers for the read & write cmd. 
 * We're still using the default ones for the block read & write, but we
 * have to do a quick check on a bit to make sure that things are alright
 * first. Also stop activity if we get Disconnected, this way we don't
 * produce so many pending requests. (i hope)
 */
static ssize_t gnbd_read(struct file *flip, char *buf,
			 size_t cnt, loff_t *ppos)
{
  struct inode *inode = flip->f_dentry->d_inode;
  int num = MINOR(inode->i_rdev);
  gnbd_t *dev;
  int err = 0;
	
  ENTER(GNBD_READ);
  if(!inode) {err = -EINVAL; goto exit_gnbd_read;}
  if (num >= max_check || gnbd_devices[num] == NULL){
    err = -ENODEV;
    goto exit_gnbd_read;
  }
  /* no reading or writing the comm */
  if(num == 0) {err = -EPERM; goto exit_gnbd_read;}
  dev = gnbd_devices[num];

  if( ISgnbdDisConnected(dev) ) {
    if( flip->f_flags & O_NONBLOCK)
      {err = -EAGAIN; goto exit_gnbd_read;}
    else {
      printe("Read sleeping until ungassed\n");
      interruptible_sleep_on( &dev->gassed );
      /* check again just in case */
      if( ISgnbdDisConnected(dev)) {
	printe("woke up and we're still disconnected\n");
	err = -ESHUTDOWN; 
	goto exit_gnbd_read;
      }
    }
  }

  printdd("Switching to regular block_read\n");
  err = block_read(flip, buf, cnt, ppos);
 exit_gnbd_read:
  EXIT(GNBD_READ);
  return err;
}

static ssize_t gnbd_write(struct file *flip, const char *buf,
			  size_t cnt, loff_t *ppos)
{
  struct inode *inode = flip->f_dentry->d_inode;
  int num = MINOR(inode->i_rdev);
  gnbd_t *dev;
  int err = 0;
	
  ENTER(GNBD_WRITE);
  
  if(!inode) {err = -EINVAL; goto exit_gnbd_write;}
  if (num >= max_check || gnbd_devices[num] == NULL){
    printe("there is no gnbd_device with minor number = %d\n", num);
    err = -ENODEV; 
    goto exit_gnbd_write;
  }
  /* no reading or writing the comm */
  if(num == 0) {err = -EPERM; goto exit_gnbd_write;}
  dev = gnbd_devices[num];

  if( ISgnbdDisConnected(dev) ) {
    if( flip->f_flags & O_NONBLOCK)
      {err = -EAGAIN; goto exit_gnbd_write;}
    else {
      printe("Write sleeping until ungassed\n");
      interruptible_sleep_on( &dev->gassed );
      /* check again just in case */
      if( ISgnbdDisConnected(dev)) {
	printe("woke up and still disconnected\n");
	err = -ESHUTDOWN;
	goto exit_gnbd_write;
      }
    }
  }

  printdd("Switching to regular block_write\n");
  err = block_write(flip, buf, cnt, ppos);
 exit_gnbd_write:
  EXIT(GNBD_WRITE);
  return err;
}

/******************************************************************************
 * The file operations
 */

struct file_operations gnbd_fops = {
  NULL,          /* lseek: default */
  gnbd_read,
  gnbd_write,
  NULL,          /* gnbd_readdir */
  NULL,          /* gnbd_poll */
  gnbd_ioctl,
  NULL,          /* gnbd_mmap */
  gnbd_open,
  NULL,          /* flush */
  gnbd_release,
#if 0
  block_fsync,
  NULL,          /* gnbd_fasync */
  NULL,          /* gnbd_check_change, */
  NULL,          /* gnbd_revalidate */
  NULL           /* lock */
#endif
};

/******************************************************************************
 * my_end_request
 * This is how I kill a pending block request.
 */
void my_end_request(struct request *req)
{
  if (end_that_request_first( req, !req->errors, "gnbd" ))
    return;
  end_that_request_last( req );
}
/******************************************************************************
 * gnbd_request
 * This is where the kernel finally asks us to go get those blocks of data.
 */
void gnbd_request(void)
{
  gnbd_t *device;
  struct request *req;
  int num;
  u32 len;
  u64 offset;

  ENTER(GNBD_REQUEST);
  while(1) {
    if (CURRENT == &(blk_dev[MAJOR_NR].plug))
      return;
    INIT_REQUEST;

#if 0
    printi("request %p: cmd %i sec %li (nr. %li), next %p\n", CURRENT,
	   CURRENT->cmd,
	   CURRENT->sector,
	   CURRENT->current_nr_sectors,
	   CURRENT->next);
#endif
    IFDB(GNBD_REQUEST, printi("block request from pid %d\n", current->pid););

    num = DEVICE_NR(CURRENT_DEV);
    device = gnbd_devices[num];
    if( ISgnbdDisConnected(device) ) {
      printe("server disconnected, quitting request_fn\n");
      return;
    }

    req = CURRENT;                                       
    CURRENT = req->next;
    req->next = NULL;

    /* Check if the minor number is in range */
    if (num >= max_check || gnbd_devices[num] == NULL) {
      /* print the message at most five times */
      static int UNIQUE_NAME = 0; if (UNIQUE_NAME++ < 5)
	printe("request for unknown device\n");
      my_end_request(req);
      continue;
    }

    /* pointer to device structure, from the global array */
    /* mmmm, hardcoded city... */
    offset = (u64)(req->sector) * (u64)GNBD_HARDSECT;
    len    = req->current_nr_sectors << 9;

    /* should this sleep? or somesimular type action?
     * Well, it needs to do SOMEthing else.
     */
    /*
    if( ISgnbdDisConnected(device) ) {
      static int UNIQUE_NAME = 0; if (UNIQUE_NAME++ < 5)
	printe("Not logged into a server!\n");
      my_end_request(req);
      continue;
    }
    */

    IFDB(GNBD_REQUEST,
         printi("Calculated lens: offset = 0x%"FMT_64"x len = 0x%x\n",
		offset, len););

    if (offset + len > device->size) {
      static int UNIQUE_NAME = 0; if (UNIQUE_NAME++ < 5)
	printe("request past end of device\n");
      my_end_request(req);
      continue;
    }

    cmd_RW(device, req, len, offset);
  }

  EXIT(GNBD_REQUEST);
}


/******************************************************************************
 * Proc file stuff
 * So users can easily check up on these devices
 */

int gnbd_proc_read(char *buf, char **start, off_t off, int count, int *eof,
		   void *data)
{
  int i;
  gnbd_t *dev;
  unsigned int ipaddr = 0;
  char down_msg[17];
  count = 0; /* ignore how much it wants */
  ENTER(GNBD_PROC_READ);
  
  count = sprintf(buf, "gnbd major number : %d\n\n", gnbd_major);
  for (i=1; i < max_check; i++) {
    if (gnbd_devices[i] == NULL)
      continue;
    dev = gnbd_devices[i];
    ipaddr = be32_to_cpu(dev->servip);
    if (dev->good_connect == FALSE)
      strcpy(down_msg,"CONNECTION BROKE");
    else
      strcpy(down_msg,"");
    count += sprintf(buf+count, "Device: /%s/%s\n"
                                "-------------------\n"
		                " Minor # : %d\n"
		                "      IP : %u.%u.%u.%u\n"
		                "    Port : %u\n"
		                "   State : %s %s %s\n\n",
		     &gnbd_device_names[i][5], down_msg, 
	             i,
		     (unsigned char)(0xff & (ipaddr >> 24)),
                     (unsigned char)(0xff & (ipaddr >> 16)),
                     (unsigned char)(0xff & (ipaddr >> 8)),
                     (unsigned char)(0xff & ipaddr),
		     be16_to_cpu(dev->servport),
		     ISgnbdOpened(dev)?"Open":"Close",
		     ISgnbdConnected(dev)?"Connected":"Disconnected",
		     ISgnbdPending(dev)?"Pending":"Clear"
		     );
    if (count > (PAGE_SIZE-380)) break;
  }
  *eof = TRUE;
  EXIT(GNBD_PROC_READ);
  return count;
}

#if 0

int gnbd_proc_write(struct file *flip, const char *buf, unsigned long length,
		    void *data)
{
  char *cmd;
  char *ptr;
  char *delimiters = " ,\t";
  int err = 0;
  int dev;

  ENTER(GNBD_PROC_WRITE);
  if (!buf){
    EXIT(GNBD_PROC_WRITE);
    return -EINVAL;
  }
  cmd = kmalloc(length * sizeof(char), GFP_KERNEL);
  if (!cmd){
    EXIT(GNBD_PROC_WRITE);
    return -ENOMEM;
  }
  err = (int)copy_from_user((void *)cmd, (void *)buf, (int)length);
  if (err){
    printe("Bad address?!");
    err = -EFAULT;
    goto fail;
  }
  cmd[length-1] = 0;
  if (strncmp("gnbd", cmd, 4)){
    printe("invaid gnbd device\n");
    err = -EINVAL;
    goto fail;
  }
  dev = gstrtou(cmd + 4, &ptr);
  if (ptr == cmd + 4 && dev == 0){
    printe("invalid gnbd device\n");
    err = -EINVAL;
    goto fail;
  }
  if (dev > 255){
    printe("gnbd device number %u out of range\n", dev);
    err = -EINVAL;
    goto fail;
  }
  ptr = strtok(ptr, delimiters);
  if (ptr == NULL){
    printe("unknown command for gnbd%d\n", dev);
    err = -EINVAL;
    goto fail;
  }
  printd("working on gnbd%d\n", dev);
  if (strncmp("setipport", ptr, 9) == 0){ /* set ipport */
    printd("attempting to set the ip address and the port\n");
    
  }
  else if (strncmp("create", ptr, 6) == 0){ /* create a new device */
    int length;
    printd("attempting to create a new gnbd device\n");
    ptr = strtok(NULL, delimiters);
    if (ptr == NULL){
      printe("no name for gnbd device\n");
      err = -EINVAL;
      goto fail;
    }
    length = strlen(ptr);
    if (length > 255){
      printe("device name too long\n");
      err = -EINVAL;
      goto fail;
    }
    err = init_device(ptr, length);
    if (err < 0)
      goto fail;
    goto pass;
  }
  
 pass:
  kfree(cmd);
  EXIT(GNBD_PROC_WRITE);
  return length;

 fail:
  kfree(cmd);
  EXIT(GNBD_PROC_WRITE);
  return err;
}

#endif /* 0 */

struct proc_dir_entry gnbd_proc_entry = {
  0,
  4, "gnbd",
  S_IFREG | S_IRUGO,
  1, 0, 0,
  0,
  NULL,    /* ops */
  NULL,    /* get_info */
  NULL,    /* fill_inode */
  NULL, NULL, NULL, /*next, parent, subdir */
  NULL, /* misc data ptr */
  gnbd_proc_read,
  NULL,   /* gnbd_proc_write */
  /* defaults */
};
  
#if 0

/******************************************************************************
 * config string parser.
 * takes the config string and loads its parts into the exsistaning
 * devices.
 * `255.255.255.255.65536' <- longest possible ip.port
 * "10.0.1.31.5678 192.168.1.42.1565"
 */
int parse_config_str(void)
{
  int i,err=0;
  u32 tip;
  u16 tpt;

  ENTER(PARSE_CONFIG_STR);

  if(config == NULL || config[0] == NULL) goto exit_parse_config_str;
  for (i=1; i < gnbd_devs; i++){
    if (config[i-1] == NULL)
      goto exit_parse_config_str;
    if(strlen(config[i-1]) > 22){
      printe("config \"%s\" is too long to be correct\n", config[i-1]);
      err = -1;
      goto exit_parse_config_str;
    }
    printd("config = \"%s\"\n", config[i-1]);
    err = get_ip_port( config[i-1], &tip, &tpt );
    if(err) {
      printe("Bad ip.port pair \"%s\"\n",config[i-1]);
      goto exit_parse_config_str;
    }
    printd("device:%d is getting ip:0x%x port:0x%x\n", i, tip, tpt);
    gnbd_devices[i].servip=tip;
    gnbd_devices[i].servport=tpt;
  }
 exit_parse_config_str:
  EXIT(PARSE_CONFIG_STR);
  return err;
}

#endif /* 0 */

/******************************************************************************
 * Finally, the module stuff
 */
int init_module(void)
{
  int result, i;

  ENTER(INIT_MODULE);

  gnbd_major = major;
  c_init = MUTEX;
  /*
   * Register your major, and accept a dynamic number
   */
  result = register_blkdev(gnbd_major, "gnbd", &gnbd_fops);
  if (result < 0) {
    printw("can't get major %d\n",gnbd_major);
    EXIT(INIT_MODULE);
    return result;
  }
  if (gnbd_major == 0) gnbd_major = result; /* dynamic */
  gnbd_gendisk.major = gnbd_major; /* was unknown at load time */
  blk_dev[gnbd_major].request_fn = gnbd_request;

  read_ahead[gnbd_major] = rahead;
  result = -ENOMEM; /* for the possible errors */

  for(i=0; i<256; i++)
    gnbd_blksizes[i] = BLOCK_SIZE;
  blksize_size[gnbd_major]=gnbd_blksizes;

  for(i=0; i<256; i++)
    gnbd_sizes[i] = 0; /* This is zero so that unused gnbd devices don't
			  show up on /proc/partition */
  blk_size[gnbd_major] = gnbd_sizes; /* in blocks..*/

  /* 
   * allocate the devices -- we can't have them static, as the number
   * can be specified at load time
   */

  for (i=0; i < 256; i++){
    gnbd_partitions[i].nr_sects = 2 * gnbd_sizes[i]; /* 512 size sectors */
    gnbd_partitions[i].start_sect = -1; /* disable partition checking */ 
  }
  
  for (i=0; i < 256; i++){
    /* printd(KERN_ERR "GNBD partition check: "); */
    resetup_one_dev(&gnbd_gendisk, i);
    /* printd(KERN_ERR "dev %d: beg %lx, size %lx\n", i,
	   gnbd_partitions[i].start_sect,
	   gnbd_partitions[i].nr_sects); */
  }

  gnbd_gendisk.next = gendisk_head;
  gendisk_head = &gnbd_gendisk;

  result = init_device("gnbd_comm", 10);
  if (result != 0){
    if (result > 0)
      cleanup_device(result);
    printe("couldn't initialize gnbdcomm\n");
    goto fail;
  }

  proc_register( &proc_root, &gnbd_proc_entry);

#if 0
  printi("Module gnbd loaded\n");
  printi(" sizeof theader_t %u\n", sizeof(theader_t));
  printi(" sizeof tio_info_t %u\n", sizeof(tio_info_t));
#endif

  printk("GNBD %s (built %s %s) installed\n",
	 GFS_RELEASE_NAME,
	 __DATE__, __TIME__);  

  EXIT(INIT_MODULE);
  return 0; /* succeed */

 fail:
  gendisk_head = gnbd_gendisk.next;
  gnbd_gendisk.next = NULL;
  blk_size[gnbd_major] = NULL;
  blksize_size[gnbd_major] = NULL;
  read_ahead[gnbd_major] = 0;
  blk_dev[gnbd_major].request_fn = NULL;
  unregister_blkdev(gnbd_major, "gnbd");
  EXIT(INIT_MODULE);
  return result;
}

void cleanup_module(void)
{
  int i;
  struct gendisk *gendisk_ptr;

  ENTER(CLEANUP_MODULE);
  /* first of all, flush it all and reset all the data structures */

  for (i=0; i<=max_check; i++)
    if (gnbd_devices[i]){
      fsync_dev(MKDEV(gnbd_major, i)); /* flush the devices */
      cleanup_device(i);
    }

  gendisk_ptr = gendisk_head;
  if (gendisk_ptr == &gnbd_gendisk)
    gendisk_head = gnbd_gendisk.next;
  else{
    while (gendisk_ptr != NULL && gendisk_ptr->next != &gnbd_gendisk)
      gendisk_ptr = gendisk_ptr->next;
    if (gendisk_ptr == NULL)
      printe("The gnbd_gendisk structure disappeared!!\n");
    else
      gendisk_ptr->next = gnbd_gendisk.next;
  }
  gnbd_gendisk.next = NULL;

  blk_dev[gnbd_major].request_fn = NULL;
  read_ahead[gnbd_major] = 0;
  blk_size[gnbd_major] = NULL;
  blksize_size[gnbd_major] = NULL;

  /* finally, the usual cleanup */

  unregister_blkdev(gnbd_major, "gnbd");

  proc_unregister( &proc_root, gnbd_proc_entry.low_ino );

  printd("Module gnbd unloaded\n");
  EXIT(CLEANUP_MODULE);
}















