/******************************************************************************
*
*    Copyright 1999 Regents of the University of Minnesota
*    Portions Copyright 1999-2001 Sistina Software, Inc.
*
*    This is free software released under the GNU General Public License.
*    There is no warranty for this software.  See the file COPYING for
*    details.
*
*    See the file CONTRIBUTORS for a list of contributors.
*
*    This file is maintained by:
*      Benjamin E Marzinski <marzinski@sistina.com>
*
* This is getting rebuilt to only hold information that is needed by both
* clients and servers.* 
*/
#ifndef __gnbd_h__
#define __gnbd_h__

#include "global.h"
#ifdef __KERNEL__
#include "osi.h"
#else
typedef   uint64  u64;
typedef   uint32  u32;
typedef   uint16  u16;
typedef   uint8   u8;
#endif /* __KERNEL__ */
#include "trans.h"

/* Various ioctl command that we have */
/* Should probably use something other than 0xeb. But what? */
#if 0
#define GNBD_SET_SERV_IP    _IO( 0xeb, 1)
#define GNBD_SET_SERV_PORT  _IO( 0xeb, 2)
#define GNBD_LOG_IN         _IO( 0xeb, 3)
#endif
#define GNBD_ACTION         _IO( 0xeb, 5) /* this will replace 1-3 */
#define GNBD_DEBUG_ON       _IO( 0xeb, 6)
#define GNBD_DEBUG_OFF      _IO( 0xeb, 7)
#define GNBD_BLKSSZESET     _IO( 0xeb, 9)
#define GNBD_DEVICE         _IO( 0xeb, 10)

#define GNBD_DEV_REMOVE 0
#define GNBD_DEV_CREATE 1


/* actions: SetIP&port, Connect, Disconnect, ForceClose
 * SetIP&Port:
 *   Real basic, just change the stored ip and port for the device.
 * Connect:
 *   make a call to trans_start_up if not already up.
 * Disconnect:
 *   ??
 *   Must close socket connection, but not return errors when access. Just
 *   block. (or something, need to think about this.)
 * ForceClose:
 *   Shutdown socket, return errors to any pending requests. If we can,
 *   force usrland clients to close the device.
 * others?
 */
#define GNBD_ACT_Connect    0
#define GNBD_ACT_Disconnect 1
#define GNBD_ACT_ForceClose 2
#define GNBD_ACT_SetIPPort  3
typedef struct gnbd_action_s {
   u16 action;
   u16 device;
   u32 ip;
   u16 port;
} gnbd_action_t;

typedef struct gnbd_device_s {
  int action;
  char *name;
  int length;
  int minor_nr;
} gnbd_device_t;


#ifdef __KERNEL__
#include <linux/blk.h>

/******************************************************************************
 * The piece of the pending message queue that is used to match received
 * replies with the request that generated them.
 */
typedef struct sdQ {
   theader_t      hdr; /* we need all of this for any possible resends */
   struct request *req; /* request this cam from */
   struct sdQ     *next; 
} transferQ_t;



/******************************************************************************
 * This structure defines the, umm, shape of this device.  We'll have one
 * of these for each device that can exist.
 *
 * ok, here I'll try to describe the possible states that each device in
 * the gnbd can be in.
 * OPEN or CLOSED 
 *    Open is usage != 0, otherwise Closed.
 * PENDING or CLEAR
 *    If Rhead == NULL then Clear, otherwise Pending.
 * CONNECTED or DISCONNECTED
 *    Connected if the socket is connected. Otherwise disconnected.
 *    Also, in the disconnected state, the transiod thread should be
 *    distroyed. That way it can be recreated on connect again.
 *    If recvd_run is false, this is Disconnected.
 * 
 * If ISgnbdDisConnected, all new reads and writes will immeaditily
 * block. (unless opened nonblocking then returns EAGAIN). This stops the
 * generation of new io requests. Since when that thread is dead, we cannot
 * process IO. And we'd flood the request queues if we didn't block things
 * off at the read&write commands. Hence the wait queue, gassed.
 */
#define ISgnbdOpened(x) ((x)->usage != 0)
#define ISgnbdConnected(x) ((x)->recvd_run != 0)
#define ISgnbdDisConnected(x) (!ISgnbdConnected(x))
#define ISgnbdPending(x) ((x)->Rhead != NULL)
typedef struct gnbd_s {
  u64 size;
  int usage;
  
  int readonly:1;
  int recvd_run:1;
  int good_connect:1;
  
  u32 servip;
  u16 servport;
  
  osi_wait_chan_t  gassed;
  struct socket    *sock;
  osi_sema_t sender;
  osi_sema_t tiodmux;
  osi_sema_t tqmux;
  transferQ_t      *Rhead;
} gnbd_t;


extern int gnbd_major;
/* prototypes so function in the other file can get back to each other. */
void my_end_request(struct request *req);
void gnbd_request(request_queue_t *q);
int trans_start_up(gnbd_t *tio);
void trans_shutdown(gnbd_t *tio, int complete);
void cmd_RW(gnbd_t *tio, struct request *req, u32 len, u64 offset);

void gdump_buffer(void *buf, int len);

#endif /*__KERNEL__*/
#endif /*__gnbd_h__*/

