/******************************************************************************
*
*    Copyright 1999 Regents of the University of Minnesota
*    Portions Copyright 1999-2001 Sistina Software, Inc.
*
*    This is free software released under the GNU General Public License.
*    There is no warranty for this software.  See the file COPYING for
*    details.
*
*    See the file CONTRIBUTORS for a list of contributors.
*
*    This file is maintained by:
*      Benjamin E Marzinski <marzinski@sistina.com>
* 
* Misc functions that we use that are not in the kernel.
* Atleast I couldn't find them.
* */

#ifdef MODVERSIONS
#include <linux/modversions.h>
#endif  /*  MODVERSIONS  */
#include <asm/byteorder.h>

#include "netmisc.h"

int gatoi(char *c)
{
  int x = 0;

  while ('0' <= *c && *c <= '9')
  {
    x = x * 10 + (*c - '0');
    c++;
  }

  return(x);
}

unsigned int gstrtou(char *c, char **tailptr) /*string to unsigned integer */
{
  unsigned long x = 0;
  char *temp = c;
  while ('0' <= *c && *c <= '9')
    {
      x = x * 10 + (*c - '0');
      c++;
    }
  if (x > UINT_MAX){
    if (tailptr)
      *tailptr = temp;
    return 0;
  }
  if (tailptr)
    *tailptr = c;
  return (unsigned int)x;
}


/*****************************************************************************
 *
 *   be32_to_ascii() takes a be32 value and prints 
 *
 */
char *be32_to_ascii(char *ascii, u32 num){
  char *ptr = (char *)&num;
  sprintf(ascii, "%u.%u.%u.%u", (unsigned int)*ptr, (unsigned int)*(ptr+1),
	  (unsigned int)*(ptr+2), (unsigned int)*(ptr+3));
  return ascii;
}


int ginet_aton(char *ascii, u32 *number)
{
	u32 ip = 0, value;
	int x;

	for (x = 0; x < 4; x++) {
		value = gatoi(ascii);
		if (value > 255)
			return(-1);

		ip = (ip << 8) | value;

		if (x != 3) {
			ascii = strstr(ascii, ".");
			if (!ascii)
				return(-1);
			ascii++;
		}
	}

	*number = cpu_to_be32p(&ip);

	return(0);
}


/* takes a quad dot, and get the ip and port out of it */
int get_ip_port(char *ascii, u32 *ipnum, u16 *ptnum)
{
   u32 ip = 0, value;
   u16 port = 0;
   int x;

   for(x=0; x < 4; x++ ){
      value = gatoi(ascii);
      if( value > 255) return -1;

      ip = (ip<<8) | value;

      ascii = strstr(ascii, ".");
      if(!ascii) return -1;
      ascii++;
   } /* got ip now get port */
   value = gatoi(ascii);
   if( value > 65536 ) return -1;
   port = value;

   *ipnum = cpu_to_be32(ip);
   *ptnum = cpu_to_be16(port);

   return 0;
}
