/******************************************************************************
 *
 *    Copyright 2000-2001 Sistina Software, Inc.
 *
 *    This is free software released under the GNU General Public License.
 *    There is no warranty for this software.  See the file COPYING for
 *    details.
 *
 *    See the file CONTRIBUTORS for a list of contributors.
 *
 *    This file is maintained by:
 *      Benjamin E Marzinski <marzinski@sistina.com>
 * 
 */
#include <unistd.h>
#include <stdlib.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <stdio.h>
#include <string.h>
#include "utils.h"

unsigned int match(char *name, serv_list_t *ptr){
  while (ptr != NULL){
    if (strcmp(name, ptr->name) == 0)
      return ptr->port;
    ptr = ptr->next;
  }
  return 0;
}

int get_pid(char *name, serv_list_t *ptr){
  while(ptr != NULL){
    if (strcmp(name, ptr->name) == 0)
      return ptr->pid;
    ptr = ptr->next;
  }
  return -1;
}

int max(int x, int y){
    return (x > y)? x : y;
}

serv_list_t *remove_serv(char *name, serv_list_t *head){
  serv_list_t *last;
  serv_list_t *current;
  last = head;
  if (head == NULL)
    return head;
  current = head->next;
  if (strcmp(head->name, name) == 0){
    free(head);
    return current;
  }
  while(current != NULL){
    if (strcmp(current->name, name) == 0){
      last->next = current->next;
      free(current);
      return head;
    }
    last = current;
    current = current->next;
  }
  return head;
}

serv_list_t *add(serv_list_t item, serv_list_t *head){
  serv_list_t *ptr;
  serv_list_t *tmp = (serv_list_t *)malloc(sizeof(serv_list_t));
  strcpy(tmp->name, item.name);
  strcpy(tmp->path, item.path);
  tmp->port = item.port;
  tmp->pid = item.pid;
  tmp->next = NULL;
  if (head == NULL)
    return tmp;
  ptr = head;
  while(ptr->next != NULL)
    ptr = ptr->next;
  ptr->next = tmp;
  return head;
}

serv_list_t *clear(serv_list_t *ptr){
  serv_list_t *tmp;
  while(ptr != NULL){
    tmp = ptr;
    ptr = ptr->next;
    free(tmp);
  }
  return NULL;
}

int close_socket(int val){
  if (val > 0)
    close(val);
  return 0;
}

int open_socket(int tcp, int port){
  int val;
  struct sockaddr_in sin;
  val = socket(AF_INET, (tcp)? SOCK_STREAM : SOCK_DGRAM , 0);
  if (val < 0){
    perror("gportd: error creating socket");
    return val;
  }
  sin.sin_family = AF_INET;
  sin.sin_addr.s_addr = INADDR_ANY;
  sin.sin_port = htons(port);
  if (bind(val, (struct sockaddr *)&sin, sizeof(struct sockaddr_in)) < 0){
    perror("gportd: error binding the socket");
    close(val);
    return -1;
  }
  if (tcp){
    if (listen(val, 5) < 0){
      perror("gportd: error listening on the socket");
      close(val);
      return -1;
    }
  }
  return val;
}












