/*
 * Global Network Block Server kernel module
 *
 *
 * Copyright (c) 1999 Marcelo Tosatti <marcelo@conectiva.com.br>
 * Portions Copyright 2000-2001 Sistina Software, Inc.
 * 
 * Based on Jeff Becker's knbd code.
 *
 * Copyright (c) 1999 Jeff Becker <becker@nas.nasa.gov>
 * Copyright (c) 1999 MRJ Technology Solutions
 * Copyright (c) 1999 National Aeronautics & Space Administration
 * All rights reserved.
 *
 * This software is based on code by Jeff Becker, MRJ Technology Solutions, 
 * of the Numerical Aerospace Simulation Facility, NASA Ames Research Center. 
 * It was ported from Pavel Machek's <pavel@atrey.karlin.mff.cuni.cz>
 * userland nbd server.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the National Aeronautics & Space Administration
 *    nor the names of its contributors may be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE NATIONAL AERONAUTICS & SPACE ADMINISTRATION
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE ADMINISTRATION OR CONTRIB-
 * UTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAG.E
 * 
 * Changes
 * Marcelo Tosatti <marcelo@conectiva.com.br> 9/11/99 
 * - Little change in the socket creation to work with 2.2
 * - Fixed a race (added lock_kernel before exit_mm/exit_fs)
 *
 * Changes
 * Mike Tilstra <tilstra@borg.umn.edu> 22-09-1999
 * - Cleared out the need for the cliserv.h file.
 * - Various merging type things to link it with the current gnbd and get
 *   it read to be part of the GFS cvs.
 *   Switched to a GFS style make file.
 * - Note: don't mke2fs on a 9gig served device with debugging turned on...
 *
 * Changes
 * Ben Marzinski <marz0016@sistina.com> 7/11/00
 * - changed init_module and serv_client to allow serving regular files
 * - changed cleanup_module so that the serv thread is killed when the
 *   module is unloaded
 * - Added the ability to start up multiple servers
 */

#ifdef MODVERSIONS 
#include <linux/modversions.h>
#endif  /*  MODVERSIONS  */

#include <linux/module.h>
 
#define __KERNEL_SYSCALLS__ /* to get kernel_threads to work on alphas */
#define __LIBRARY__
#include <asm/atomic.h>
#include <linux/kernel.h>
#include <linux/malloc.h>
#include <linux/net.h>
#include <linux/in.h>
#include <linux/inet.h>
#include <linux/signal.h>
#include <net/sock.h>
#include <net/tcp.h>
#include <asm/segment.h>
#include <asm/uaccess.h>
#include <linux/unistd.h>
#include <linux/uio.h>
#define WANT_DEBUG_NAMES
#include "kgnbdsrv.h"
#include "debug.h"
#include <linux/kdev_t.h>
#include <linux/blkdev.h>

#include <linux/blk.h>

#include <linux/fs.h>
#include <linux/file.h>
#include <linux/errno.h>
#include <linux/wait.h>
#include <linux/smp_lock.h>
#include <linux/proc_fs.h>

#include "global.h"
#include "release.cf"

#ifndef DEBUG_SYMBOLS
EXPORT_NO_SYMBOLS;
#else
#define EXPORT_SYMTAB
#endif

#define MAX_SERV 256

#ifndef __alpha__
static int errno;
#endif

struct task_struct *portdtsk;
atomic_t live_serv;    /*number of living servers. Used for module cleanup */
atomic_t stomith_serv;
int max_check;
long int port;
unsigned int serv_port;
kgnbdsrv_t *server[MAX_SERV]; /*max servers running at once */
struct wait_queue *death_watch = NULL;
struct wait_queue *stomith_watch = NULL;
struct semaphore s_init;
struct semaphore portd;
int alive;
char stomithip[9];

MODULE_PARM(port, "l");

/*
 *  Send or receive packet.
 */

unsigned int strtou(char *c, char **tailptr) /*string to unsigned integer */
{
  unsigned long x = 0;
  char *temp = c;
  while ('0' <= *c && *c <= '9')
    {
      x = x * 10 + (*c - '0');
      c++;
    }
  if (x > UINT_MAX){
    if (tailptr)
      *tailptr = temp;
    return 0;
  }
  if (tailptr)
    *tailptr = c;
  return (unsigned int)x;
}

int xmit(int send, struct socket *sock, char * buf, int size)
{
  mm_segment_t oldfs;
  int result;
  struct msghdr msg;
  struct iovec iov;
  /*  unsigned long flags;
      sigset_t oldset;*/

  ENTER(XMIT);
	
  oldfs = get_fs();
  set_fs(get_ds());
	
#if 0
  spin_lock_irqsave(&current->sigmask_lock, flags);
  oldset = current->blocked;
  /* sigfillset(&current->blocked); */
  siginitsetinv(&current->blocked, sigmask(SIGKILL));
  recalc_sigpending(current);
  spin_unlock_irqrestore(&current->sigmask_lock, flags);
#endif /* 0 */
  do {
    iov.iov_base = buf;
    iov.iov_len = size;
    msg.msg_name = NULL; 
    msg.msg_namelen = 0;
    msg.msg_iov = &iov;
    msg.msg_iovlen = 1;
    msg.msg_control = NULL; 
    msg.msg_controllen = 0;
    msg.msg_namelen = 0;
    msg.msg_flags = 0;
    /*printce("about to %s message\n", send ? "send" : "receive");*/
    if (send)
      result = sock_sendmsg(sock, &msg, size);
    else 
      result = sock_recvmsg(sock, &msg, size, 0);
    /*printce("%s message\n", send ? "sent" : "received");*/
    if (result <= 0) {
      printc("%s - sock=%ld at buf=%ld, size=%d returned %d.\n",
	      send ? "send" : "receive", (long) sock, (long) buf, size, result);
      break; 
    }
    size -= result;
    buf += result;
  } while (size > 0);
#if 0
  spin_lock_irqsave(&current->sigmask_lock, flags);
  current->blocked = oldset;
  recalc_sigpending(current);
  spin_unlock_irqrestore(&current->sigmask_lock, flags);
#endif /* 0 */
  set_fs(oldfs);
  EXIT(XMIT);
  return result;
}

/* accept connection */

struct socket * knbd_accept(kgnbdsrv_t *info, struct socket *dupsock,
			    int *retval)
{
  ENTER(KNBD_ACCEPT);
  if (!(dupsock = sock_alloc())) {
    printse("error allocating dup socket\n");
    EXIT(KNBD_ACCEPT);
    return NULL;
  }

  dupsock->type = info->knbdsock->type;
#if LINUX_VERSION_CODE < 0x20300
  *retval = info->knbdsock->ops->dup(dupsock, info->knbdsock);
  if (*retval < 0)
    goto fail;
#else
  dupsock->ops = info->knbdsock->ops;
#endif
  if ((*retval = info->knbdsock->ops->accept(info->knbdsock,
					     dupsock, O_RDONLY)) < 0) {
    if (*retval != -ERESTARTSYS){
      printse("knbdsock is %d, dupsock is %d\n", info->knbdsock->sk->err, 
	      dupsock->sk->err);
      printse("error %d accepting on socket\n", *retval);
    }
    else
      prints("accept failed with ERESTARTSYS. retrying\n");
    goto fail;
  }
  prints("Accepted connection\n");
  EXIT(KNBD_ACCEPT);
  return dupsock;
 fail:
  if (dupsock)
    sock_release(dupsock);
  EXIT(KNBD_ACCEPT);
  return NULL;
}

/* one thread per client */

int serv_client (void * args)
{ 
  int block, boffset, size, len, err = 0; 
  struct buffer_head *bh;
  char *buf;
  char *bufptr;
  mm_segment_t oldfs;
  kgnbdsrv_t *info = ((info_t *) args)->ksrv;
  struct socket *dupsock = ((info_t *) args)->sock;
  struct sock *sk = dupsock->sk;
  u32 ipaddr = be32_to_cpu(sk->daddr);
  unsigned long long offset;
  tio_info_t *tinfo;
  theader_t *iohdr;
  sigset_t oldset;
  unsigned long flags;
 
  ENTER(SERV_CLIENT);
  
  bufptr = (char *)&ipaddr;

  if(!dupsock) { 
    printce("dupsock NULL\n");
    EXIT(SERV_CLIENT);
    return -1;
  }

  lock_kernel();
  exit_mm(current);
  exit_fs(current);
  exit_files(current);

  current->session = 1;
  current->pgrp = 1;
  current->tty = NULL;
  current->tty_old_pgrp = 0;

  siginitsetinv(&current->blocked,
                (sigmask(SIGKILL)|sigmask(SIGINT)|sigmask(SIGTERM)));
 
  sprintf(current->comm, "gserv%08lx",
	  (unsigned long)ipaddr);
  
  err = -1;

  if (!(buf = (char *) kmalloc(20480, GFP_KERNEL))) {
    printce("cannot allocate mem for buf\n");
    goto exit_sock;
  }

  tinfo = kmalloc(sizeof(tio_info_t), GFP_KERNEL);

  if(!tinfo) {
    printce("cannot allocate mem for tinfo struct\n");
    kfree(buf);
    goto exit_sock;
  }

  iohdr = kmalloc(sizeof(theader_t), GFP_KERNEL);

  if(!iohdr) {
    printce("cannot allocate mem for theader struct\n");
    kfree(tinfo);
    kfree(buf);
    goto exit_sock;
  }

  /* Send login info to client */

  tinfo->devsize = cpu_to_be64(info->exportsize);
  tinfo->readonly = 0;

  xmit(1, dupsock,(char *) tinfo, sizeof(tio_info_t));
  kfree(tinfo);

  while(xmit(0, dupsock, (char *) iohdr, sizeof(theader_t)) > 0) {
    spin_lock_irqsave(&current->sigmask_lock, flags);
    oldset = current->blocked;
    siginitsetinv(&current->blocked, sigmask(SIGKILL));
    recalc_sigpending(current);
    spin_unlock_irqrestore(&current->sigmask_lock, flags);
    memset(buf, 0, 20480);

    bufptr = buf;

    printc("ioHdr: type=0x%x, key=0x%"FMT_64"x, len=0x%lx, offset=0x%llx\n",
	   ((unsigned int) iohdr->type),
	   (iohdr->key), 
	   ((long unsigned int) iohdr->len), 
	   ((long long unsigned int) iohdr->offset));

	
    if(iohdr->type == tioShutdown) { 
      xmit(1, dupsock, (char *) iohdr, sizeof(theader_t));
      goto exit;
    }
	
    len = ntohl(iohdr->len); 
    offset = be64_to_cpu(iohdr->offset);
	
    if (len > 10240) {
      printce("Request too big!\n");
      goto exit;
    }
	
    if ((offset + len) > info->exportsize) {
      printce("Request out of range!\n");
      goto exit;
    }
				
    if ((iohdr->type != tioReadReq) && (iohdr->type != tioWriteReq)) {
      printc("unknown gnbd command (%d)?!?\n", 
	      iohdr->type);
      goto exit;
    }
		
    if (iohdr->type == tioWriteReq){         /* get data to write */
      printc(KERN_ERR "attempting to get write data\n");
      xmit(0, dupsock, buf, len);	
    }
    if (S_ISBLK(info->knbdsrv_i->i_mode)){
      if (info->blksize < 512) {
	block = (iohdr->offset / GNBD_HARDSECT) * (512/info->blksize);
	boffset = 0;
      }
      
      else {
	block = (offset / GNBD_HARDSECT) / (info->blksize >> 9);
	boffset = (offset / GNBD_HARDSECT) % (info->blksize >> 9) << 9;
      }
      if (boffset > info->blksize) { 
	block++;
	boffset -= info->blksize;
      }
      
      /*	 transmit nbd buffer */
      while (len > 0) {
	size = info->blksize - boffset;
	
	if (size > len)
	  size = len;
	
	bh = getblk(info->knbdsrv, block, info->blksize);
	if (!bh) {
	  printce("can't get buffer head\n");
	  goto exit;
	}
	if (!buffer_uptodate(bh) && 
	    ((iohdr->type == tioReadReq) ||
	     (boffset || (len < info->blksize)))) {
	  ll_rw_block(READ, 1, &bh);
	  wait_on_buffer(bh);
	  if (!buffer_uptodate(bh)) {
	    brelse(bh);
	    break;
	  }
	}
	if (iohdr->type == tioReadReq) 
	  memcpy(bufptr, bh->b_data + boffset, size);
	else if(iohdr->type == tioWriteReq)
	  memcpy(bh->b_data + boffset, bufptr, size);
	
	if (iohdr->type == tioWriteReq) {
	  mark_buffer_uptodate(bh, 1);
          mark_buffer_dirty(bh, 1);
	  ll_rw_block(WRITE, 1, &bh);   /*I should try to group these */
	  wait_on_buffer(bh);             
	}
	brelse(bh);
	
	bufptr += size;
	len -= size;
	boffset = 0;
	block++;
      }
    }
    else{
      info->knbdsrv_f->f_pos = offset;
      oldfs = get_fs();
      set_fs(KERNEL_DS);
      while (len > 0){
	size = len;
	if (iohdr->type == tioReadReq)
	  size = info->knbdsrv_f->f_op->read(info->knbdsrv_f, bufptr, size,
					     &info->knbdsrv_f->f_pos);
	else if(iohdr->type == tioWriteReq)
	  size = info->knbdsrv_f->f_op->write(info->knbdsrv_f, bufptr, size,
					      &info->knbdsrv_f->f_pos);
	if (size < 0){
	  printce("error in file read/write\n");
	  set_fs(oldfs);
	  goto exit;
	}
	printc("transfered %d bytes\n", size);
	bufptr += size;
	len -= size;
      }
      set_fs(oldfs);
    }
    switch(iohdr->type) {
    case tioReadReq: 
      iohdr->type = tioReadRpl;
      break;
	
    case tioWriteReq:
      iohdr->type = tioWriteRpl;
      iohdr->len = iohdr->err = iohdr->offset = 0;
      break;
    }
	
    xmit(1, dupsock, (char *) iohdr, sizeof(theader_t));

    if (iohdr->type == tioReadRpl)  {              /* send data read */
      xmit(1, dupsock, buf, ntohl(iohdr->len));
    }
	
    memset(iohdr, 0, sizeof(theader_t));
    spin_lock_irqsave(&current->sigmask_lock, flags);
    current->blocked = oldset;
    recalc_sigpending(current);
    spin_unlock_irqrestore(&current->sigmask_lock, flags);
  }

 exit:
  kfree(iohdr);
  kfree(buf);
 exit_sock:
  if(dupsock)
    sock_release(dupsock);
  send_sig(SIGINT, info->knbdtsk, 0);
  unlock_kernel();
  EXIT(SERV_CLIENT);
  return err;
}

int serv(void * serv_addr)
{
  info_t *args;
  kgnbdsrv_t *info = (kgnbdsrv_t *)serv_addr;
  struct socket *dupsock = NULL; 
  struct task_struct *child;
  int waitpid_result;
  int err = 0;
  ENTER(SERV);
  lock_kernel();
  exit_mm(current);

  current->session = 1;
  current->pgrp = 1;
  current->tty = NULL;
  current->tty_old_pgrp = 0;

  sprintf(current->comm, "kgnbd");
  siginitsetinv(&current->blocked,
		(sigmask(SIGKILL)|sigmask(SIGINT)|sigmask(SIGCHLD)|
		 sigmask(SIGTERM)));


  info->knbdtsk = current;             /* so we can kill it on module exit */
  atomic_inc(&live_serv);
  up(&s_init);
  while (info->alive) {
    if (!(dupsock=knbd_accept(info, dupsock, &err))){
      if (err != -ERESTARTSYS){
	goto dieservdie;
      }
      else{
	if (info->stomith == 1){
	  pid_t childpid;
	  prints("entering stomith section, stomithip = %s\n", stomithip);
	  for(child = current->p_cptr; child != NULL; child = child->p_osptr){
	    childpid = child->pid;
	    if(strncmp(child->comm + 5, stomithip, 8) == 0){
	      prints("killing child with pid %d\n", (int)childpid);
	      send_sig(SIGINT, child, 0);
	    }
	  }
	  atomic_dec(&stomith_serv);
	  wake_up_interruptible(&stomith_watch);
	  prints("leaving stomith section, stomith_serv = %d\n",
		  (int)atomic_read(&stomith_serv));
	  info->stomith = 0;
	}
	err = 0;
	flush_signals(current); /* Is this what I'm supposed to do?? */
	dupsock = NULL;
	do{
	  waitpid_result = waitpid(-1,NULL,__WCLONE|WNOHANG);
	}while(waitpid_result>0);
	continue;
      } 
    }
    /*  Start thread to serve gnbd client */
    do{
      waitpid_result = waitpid(-1,NULL,__WCLONE|WNOHANG);
    }while(waitpid_result>0);
    args = kmalloc(sizeof(info_t), GFP_KERNEL);
    if (!args){
      printse("couldn't allocate memory of serv_client\n");
      err=-ENOMEM;
      goto dieservdie;
    }
    args->ksrv = info;
    args->sock = dupsock;
    err = kernel_thread(serv_client, args, CLONE_SIGHAND);
    if(err < 0) {
      printse("kernel thread failed\n");
      goto dieservdie;
    }
  }
 dieservdie:
  prints("alive went false. Killing thread\n");
  prints("killing my children first\n");
  for (child = current->p_cptr; child != NULL; child = child->p_osptr)
    send_sig(SIGINT, child, 0);
  while (current->p_cptr != NULL)
    waitpid(-1, NULL, __WCLONE);
  prints("all children are dead\n");
  kfree(info->name);
  kfree(info->file);
  fput(info->knbdsrv_f);
  blkdev_release(info->knbdsrv_i);
  invalidate_buffers(info->knbdsrv);
  if (info->knbdsock)
    sock_release(info->knbdsock);
  kfree(info);
  atomic_dec(&live_serv);
  wake_up_interruptible(&death_watch);
  unlock_kernel();
  EXIT(SERV);
  return err;
}


/****************************************************************************
 *
 * Init server is allocates space for a new server, and sets it up with a
 * socket. Init server cannot be run in parallel.  Servers have to be
 * created one at a time.. The next server can start after the kernel
 * thread has finished its initialization.
 */

int init_server(char *name, char *pathname)
{
  int num;
  int err;
  int len;
  struct sockaddr_in addr;
  int found = -1;
  struct sockaddr_in sin;
  kgnbdsrv_t *tmp = NULL;
  ENTER(INIT_SERVER);
  if (down_trylock(&s_init)){
    printd("process already in init_server\n");
    EXIT(INIT_SERVER);
    return -ERESTARTSYS;
  }
  for (num = 0; num <= max_check; num++){
    if (server[num] == NULL){
      if (found == -1)
	found = num;
    }
    else if (strcmp(server[num]->name,name) == 0){
      printe("server name \"%s\" already in use\n", name);
      EXIT(INIT_SERVER);
      up(&s_init);
      return -EINVAL;
    }
  }
  if (found >= MAX_SERV){
    printe("too many servers?!?!?\n");
    up(&s_init);
    EXIT(INIT_SERVER);
    return -ENODEV;
  }
  printd("server[%d] is NULL, using it to store server information\n", found);
  if (found == max_check && max_check < MAX_SERV){
    printd("raising max_check to %d\n", max_check + 1);
    max_check++;
  }
  tmp = kmalloc(sizeof(kgnbdsrv_t), GFP_KERNEL);
  if (!tmp){
    printe("could not allocate memory for server %s\n", name);
    err = -ENOMEM;
    goto fail_malloc;
  }
  memset(tmp, 0, sizeof(kgnbdsrv_t));
  if (strlen(name) > 255){
    printe("server name too large\n");
    err = EINVAL;
    goto fail_malloc;
  }
  tmp->name = kmalloc(strlen(name) * sizeof(char), GFP_KERNEL);
  if (!tmp->name){
    printe("could not allocate memory for server name: %s\n", name);
    err = -ENOMEM;
    goto fail_malloc;
  }
  strcpy(tmp->name, name);
  tmp->file = kmalloc(strlen(pathname) * sizeof(char), GFP_KERNEL);
  if (!tmp->file){
    printe("could not allocate memory for server pathname\n");
    err = -ENOMEM;
    goto fail_malloc;
  }
  strcpy(tmp->file, pathname);
  tmp->knbdsrv_f = filp_open(pathname, O_RDWR|O_SYNC, 0600);
  if ((err = IS_ERR(tmp->knbdsrv_f)) > 0) {
    printe("bad filp\n");
    err = -EBADF;
    goto fail_malloc;
  }
  tmp->knbdsrv_i = tmp->knbdsrv_f->f_dentry->d_inode;
  if (!tmp->knbdsrv_i){
    printe("NULL inode\n");
    err = -EINVAL;
    goto fail_file;
  }
  if (tmp->knbdsrv_i->i_writecount > 1){
    printe("served file is already in use\n");
    err = -ENODEV;
    goto fail_file;
  }
  printd("got inode #%ld\n", tmp->knbdsrv_i->i_ino);
  if (S_ISBLK(tmp->knbdsrv_i->i_mode)){
    err = blkdev_open(tmp->knbdsrv_i, tmp->knbdsrv_f);
    if (err)
      goto fail_file;
    tmp->knbdsrv = tmp->knbdsrv_i->i_rdev;
    if (blk_size[MAJOR(tmp->knbdsrv)])
      tmp->exportsize = blk_size[MAJOR(tmp->knbdsrv)][MINOR(tmp->knbdsrv)];
    else
      tmp->exportsize = MAX_DISK_SIZE;
    if (blksize_size[MAJOR(tmp->knbdsrv)]) {
      tmp->blksize = blksize_size[MAJOR(tmp->knbdsrv)][MINOR(tmp->knbdsrv)];
      if (!tmp->blksize)
	tmp->blksize = BLOCK_SIZE;
    }
    tmp->exportsize *= 1024; /* blk_size is the size in KILOBYTES */
    printd("exportsize is %llu\n", tmp->exportsize);
  }
  else if (S_ISREG(tmp->knbdsrv_i->i_mode)){
    if (tmp->knbdsrv_f->f_op->read == NULL ||
	tmp->knbdsrv_f->f_op->write == NULL){
      printe("Can't find the read/write f_ops\n");
      err = -EINVAL;
      goto fail_file;
    }
    tmp->blksize = tmp->knbdsrv_i->i_blksize;
    if (!tmp->blksize)
      tmp->blksize = BLOCK_SIZE;
    tmp->exportsize = tmp->knbdsrv_i->i_size;
    if (tmp->exportsize <= 0){
      printe("zero file size or error reading inode\n");
      err = -EINVAL;
      goto fail_file;
    }
    printd("serving a file, not a block device\n");
    printd("exportsize is %llu\n", tmp->exportsize);
  }
  else{
    printe("expected either a block device or a regular file\n");
    err = -EINVAL;
    goto fail_file;
  }
  
  /* now, set up a socket to listen on */
 
  err = sock_create(PF_INET, SOCK_STREAM, IPPROTO_TCP, &(tmp->knbdsock));
  if (err < 0){
    printe("error creating socket\n");
    goto fail_file;
  }
  sin.sin_family      = AF_INET;
  sin.sin_addr.s_addr = INADDR_ANY;
  sin.sin_port        = 0;
  err = tmp->knbdsock->ops->bind(tmp->knbdsock, (struct sockaddr *) &sin,
				 sizeof(sin));
  if (err < 0){
    printe("error binding to socket\n");
    goto fail_socket;
  }
  err = tmp->knbdsock->ops->getname(tmp->knbdsock,
				    (struct sockaddr *)&addr, &len, 0);
  if (err){
    printe("error getting socket information\n");
    goto fail_socket;
  }
  tmp->port = ntohs(addr.sin_port);
  err = tmp->knbdsock->ops->listen(tmp->knbdsock, 5);
  if (err < 0){
    printe("error listening on socket\n");
    goto fail_socket;
  }
  tmp->knbdsock->flags |= SO_ACCEPTCON;
  tmp->knbdsock->sk->nonagle = 1;
  tmp->alive = 1;
  tmp->stomith = 0;
  printd("starting up server[%d]\n", found);
  server[found] = tmp;
  err = kernel_thread(serv, tmp, 0);
  if (err < 0){
    server[found] = NULL;
    printe("error starting up kernel thread\n");
    goto fail_socket;
  }
  EXIT(INIT_SERVER);
  return 0;
  
 fail_socket:
  sock_release(tmp->knbdsock);
 fail_file:
  fput(tmp->knbdsrv_f);
 fail_malloc:
  if(tmp->file)
    kfree(tmp->file);
  if(tmp->name)
    kfree(tmp->name);
  if(tmp)
    kfree(tmp);
  up(&s_init);
  EXIT(INIT_SERVER);
  return err;
}


/*****************************************************************************
 *
 * Cleanup_server is not responsible for much.  It tells the server to die.
 * It doesn't wait to make sure that the server is dead, however.
 * Cleanup_server should not be called while a server is initializing.
 * This means that whatever calls cleanup_server, should get the init
 * semephore first
 *
 */


int cleanup_server(int num)
{
  kgnbdsrv_t *tmp;
  ENTER(CLEANUP_SERVER);
  if (num < 0 || num >= max_check || server[num] == NULL){
    printe("number out of range, or invalid server\n");
    EXIT(CLEANUP_SERVER);
    return -EINVAL;
  }
  tmp = server[num];
  tmp->alive = 0;
  send_sig(SIGINT, tmp->knbdtsk, 0);
  server[num] = NULL; /* the actual server thread handles deallocation */
  if (num == max_check-1)
    max_check--;
  EXIT(CLEANUP_SERVER);
  return 0;
} 


int kgnbd_portd(void * nothing){
  struct sockaddr_in sin;
  struct socket *portd_sock;
  struct socket *dupsock = NULL;
  struct wait_queue wait = { current, NULL };
  char buf[300];
  u32 ipaddr;
  int count;
  int err;
  int i;
  ENTER(KGNBD_PORTD);
  lock_kernel();
  exit_mm(current);
  exit_fs(current);
  exit_files(current);

  current->session = 1;
  current->pgrp = 1;
  current->tty = NULL;
  current->tty_old_pgrp = 0;

  siginitsetinv(&current->blocked,
		(sigmask(SIGKILL)|sigmask(SIGINT)|sigmask(SIGTERM)));
  
  sprintf(current->comm, "kgnbd_portd");
  err = sock_create(PF_INET, SOCK_STREAM, IPPROTO_TCP, &portd_sock);
  if (err < 0){
    printpe("error creating socket\n");
    up(&s_init);
    goto exit;
  }
  sin.sin_family = AF_INET;
  sin.sin_addr.s_addr = INADDR_ANY;
  sin.sin_port = htons(serv_port);
  err = portd_sock->ops->bind(portd_sock, (struct sockaddr *) &sin,
			      sizeof(sin));
  if (err < 0){
    printpe("error binding to socket\n");
    up(&s_init);
    goto exit_socket;
  }
  err = portd_sock->ops->listen(portd_sock, 5);
  if (err < 0){
    printpe("error listening on socket\n");
    up(&s_init);
    goto exit_socket;
  }
  printp("listening on port %d\n", serv_port);
  portd_sock->flags |= SO_ACCEPTCON;
  portd_sock->sk->nonagle = 1;
  portdtsk = current;
  up(&s_init);
  while(1){
    dupsock=sock_alloc();
    if (!dupsock){
      printpe("error allocating dup sock\n");
      goto exit_socket;
    }
    dupsock->type = portd_sock->type;
#if LINUX_VERSION_CODE < 0x20300
    err = portd_sock->ops->dup(dupsock, portd_sock);
    if (err < 0)
      goto exit_socket;
#else
    dupsock->ops = portd_sock->ops;
#endif
    printp("accepting connections\n");
    err = portd_sock->ops->accept(portd_sock, dupsock, O_RDONLY);
    if (err < 0){
      if (err == -ERESTARTSYS){
	printp("interrupted while waiting for connections\n");
	sock_release(dupsock);
	dupsock = NULL;
	if (sigismember(&(current->signal), SIGKILL) ||
	    sigismember(&(current->signal), SIGINT) ||
	    sigismember(&(current->signal), SIGTERM)){
	  printp("recieved signal to quit\n");
	  goto exit_socket;
	}
	continue;
      }
      else
	printpe("error %d on accepting socket\n", err);
      goto exit_socket;
    }
    printp("accepted connection\n");
    xmit(0, dupsock, buf, 5);
    if (strncmp(buf, "stom",4) == 0){
      xmit(0, dupsock, (char *)&ipaddr, 4);
      if(alive == 0)
	xmit(1, dupsock, "fail", 5);
      else{
	down(&s_init);
	sprintf(stomithip, "%08lx", (unsigned long)be32_to_cpu(ipaddr));
	stomithip[8] = '\0';
	printp("stomith ip is %s\n", stomithip);
	atomic_set(&stomith_serv, atomic_read(&live_serv));
	printp("stomith_serv = %d\n", atomic_read(&stomith_serv));
	for (i = 0; i < max_check; i++)
	  if (server[i] != NULL){
	    printp("SIGing server[%d]: %d\n", i,
		    (int)server[i]->knbdtsk->pid);
	    server[i]->stomith = 1;
	    send_sig(SIGINT, server[i]->knbdtsk, 0);
	  }
	add_wait_queue(&stomith_watch, &wait);
	current->state = TASK_INTERRUPTIBLE;
	while (atomic_read(&stomith_serv) > 0){
	  schedule();
	  current->state = TASK_INTERRUPTIBLE;
	}
	current->state = TASK_RUNNING;
	remove_wait_queue(&stomith_watch, &wait);
	stomithip[0] = '\0';
	printp("stomith completed\n");
	up(&s_init);
	xmit(1, dupsock, "pass", 5);
      }
      xmit(0, dupsock, buf, 4);
      sock_release(dupsock);
      dupsock = NULL;
      err = 0;
    }
    else{
      for(i = 0; i < max_check; i++){
	if(server[i] == NULL)
	  continue;
	count = sprintf(buf, "%s/%d", server[i]->name,
			server[i]->port);
	buf[count++] = 0;
	xmit(1, dupsock, buf, count);
	xmit(0, dupsock, buf, 5);
	if (strncmp(buf,"next",4) != 0)
	  break;
      }
      count = sprintf(buf, "bye");
      buf[count] = 0;
      xmit(1, dupsock, buf, 4);
      xmit(0, dupsock, buf, 4);
      sock_release(dupsock);
      dupsock = NULL;
      err = 0;
    }
  }
 exit_socket:
  if(dupsock)
    sock_release(dupsock);
  sock_release(portd_sock);
 exit:
  portdtsk = NULL;
  up(&portd);
  EXIT(KGNBD_PORTD);
  return err;
}

/***************************************************************************
 *
 * /proc functions and structures
 *
 * PROBLEM: proc read should handle file sizes over a page, otherwise,
 * bad things could start to happen if too many servers are registered
 * (say, somewhere over 20 per machine)
 */

int kgnbd_proc_read(char *buf, char **start, off_t off, int count, int *eof,
		    void *data)
{
  int i;
  char *bufptr;
  char tmp[5400];
  char check[12];
  int found = 0;
  ENTER(KGNBD_PROC_READ);
  if(down_trylock(&s_init)){
    printd("cannot read from /proc/kgnbd. resources locked\n");
    EXIT(KGNBD_PROC_READ);
    return -ERESTARTSYS;
  }
  count = 0;
  if (portdtsk != NULL)
    count += sprintf(tmp, "kgnbd_portd running on port: %d\n\n", serv_port);
  else
    count += sprintf(tmp, "kgnbd_portd not running\n\n");
  for (i = 0; i < max_check; i++){
    if (server[i] == NULL)
      continue;
    found = 1;
    printd("location of server[%d] file is %lu\n", i,
	   (long unsigned int) server[i]->knbdsrv_f);
    count += sprintf(tmp+count, "Server[%d] :  /%s/\n"
		                "--------------------------\n"
		                "      file : %-20s\n"
		                "      port : %d\n"    
                                "       pid : %d\n"
                                "     state : %ld\n"
                                "exportsize : %"FMT_64"u\n"
                                " blocksize : %d\n\n",
		     i,
                     server[i]->name,
		     server[i]->file,
		     server[i]->port,
                     server[i]->knbdtsk->pid,
                     server[i]->knbdtsk->state,
                     server[i]->exportsize,
                     server[i]->blksize);
    if (count >= PAGE_SIZE) break;
  }
  if (found == 0)
    count += sprintf(tmp+count, "no kgnbd servers running\n");
  if (count < PAGE_SIZE)
    memcpy(buf, tmp, count);
  else{
    tmp[count] = 0;
    sprintf(check, "Server[%d]", i);
    bufptr = strstr(tmp, check);
    memcpy(buf, tmp, bufptr - tmp);
  }
  *eof = 1;
  up(&s_init);
  EXIT(KGNBD_PROC_READ);
  return count;
}

/*****************************************************************************
 * the format of the input is command/<options>
 * such as create/filename/path
 * or remove/filename
 */

int kgnbd_proc_write(struct file *flip, const char *buf, unsigned long length,
		     void *data)
{
  int err = 0;
  char *cmd;
  char *fname_ptr;
  char *path_ptr;
  ENTER(KGNBD_PROC_WRITE);
  if (!buf){
    printe("null buffer!?!?!\n");
    EXIT(KGNBD_PROC_WRITE);
    return -EINVAL;
  }
  cmd = kmalloc(length * sizeof(char), GFP_KERNEL);
  if (!cmd){
    printe("no memory\n");
    EXIT(KGNBD_PROC_WRITE);
    return -ENOMEM;
  }
  err = (int)copy_from_user((void *)cmd, (void *)buf, (int)length);
  if (err){
    printe("Bad address?!\n");
    err = -EINVAL;
    goto fail;
  }
  cmd[length-1] = 0;
  fname_ptr = strchr(cmd, '/');
  if (fname_ptr != NULL){
    if ( &cmd[length-1] <= fname_ptr){
      printe("hit end of string while searching for filename\n");
      err = -EINVAL;
      goto fail;
    }
    *fname_ptr = 0;
    fname_ptr++;
  }
  if (strcmp("create", cmd) == 0 && fname_ptr != NULL){
    
    path_ptr = strchr(fname_ptr, '/');
    if (path_ptr == NULL || &cmd[length-1] <= path_ptr){
      printe("hit end of string while searching for pathname\n");
      err = -EINVAL;
      goto fail;
    }
    *path_ptr = 0;
    path_ptr++;
    printd("creating \"%s\", which serves %s\n", fname_ptr, path_ptr);
    while( (err = init_server(fname_ptr, path_ptr)) < 0){
      if (err != -ERESTARTSYS || alive == 0)
	goto fail;
      schedule();
    }
  }
  else if(strcmp("remove", cmd) == 0 && fname_ptr != NULL){
    int i;
    if (down_trylock(&s_init)){
      printd("cannot remove %s, resources locked\n", fname_ptr);
      err = -ERESTARTSYS;
      goto fail;
    }
    printd("removing \"%s\"\n", fname_ptr);
    for (i = 0; i < max_check; i++){
      if (server[i] && strcmp(server[i]->name, fname_ptr) == 0)
	break;
    }
    if (i >= max_check){
      printe("no server registered by that name\n");
      err = -EINVAL;
      goto remove_fail;
    }
    err = cleanup_server(i);
    if (err < 0)
      goto remove_fail;
    up(&s_init);
  }
  else if(strcmp("portstart", cmd) == 0){ /* start kgnbd_port */
    char *tail_ptr;
    if(down_trylock(&s_init)){
      printe("cannot start kgnbd_portd. resources locked\n");
      err = -EBUSY;
      goto fail;
    }     
    if(down_trylock(&portd)){
      printe("already started server\n");
      err = -EPERM;
      goto remove_fail;
    }
    if(portdtsk != NULL){
      printe("server running without semaphore locked!?!?\n");
      up(&portd);
      err = -EPERM;
      goto remove_fail;
    }
    serv_port = 0;
    if (fname_ptr != NULL){
      serv_port = strtou(fname_ptr, &tail_ptr);
      if (serv_port == 0 && tail_ptr == fname_ptr){
	printe("port number invalid\n");
	up(&portd);
	err = -EINVAL;
	goto remove_fail;
      }
    }
    if (serv_port == 0)
      serv_port = 14243;
    printd("attempting to start server on %d\n", serv_port);
    err = kernel_thread(kgnbd_portd, NULL, 0);
    if (err < 0){
      printe("cannot start kernel thread\n");
      up(&portd);
      goto remove_fail;
    }
    down(&s_init);
    if (portdtsk != NULL)
      printd("started up the port server\n");
    else{
      err = -EIO;
      goto remove_fail;
    }
    up(&s_init);
  }
  else if (strcmp("portstop", cmd) == 0 && fname_ptr == 0){
    int tmp;
    if (down_trylock(&s_init)){
      printd("cannot remove kgnbd_portd, resources locked\n");
      err = -ERESTARTSYS;
      goto remove_fail;
    }
    if (portdtsk == NULL){
      printe("kgnbd_portd is not running\n");
      err = -ENODEV;
      goto remove_fail;
    }
    send_sig(SIGINT, portdtsk, 0);
    if((tmp = down_interruptible(&portd)) != -EINTR){
      printd("kgnbd_portd stopped\n");
      up(&portd);
    }
    else{
      printd("interrupted while waiting for kgnbd_portd to die\n");
      up(&s_init);
      err = -EIO;
      goto fail;
    }
    up(&s_init);
  }
  else{
    printe("command not recognized\n");
    err = -EINVAL;
    goto fail;
  }
  kfree(cmd);
  EXIT(KGNBD_PROC_WRITE);
  return length;
  
 remove_fail:
  up(&s_init);
 fail:
  kfree(cmd);
  EXIT(KGNBD_PROC_WRITE);
  return err;
}
    

struct proc_dir_entry kgnbd_proc_entry = {
  0,
  5, "kgnbd",
  S_IFREG | S_IRUGO,
  1, 0, 0,
  0,
  NULL,    /* ops */
  NULL,    /* get_info */
  NULL,    /* fill_inode */
  NULL, NULL, NULL, /*next, parent, subdir */
  NULL, /* misc data ptr */
  kgnbd_proc_read,
  kgnbd_proc_write,
  /* defaults */
};
  
int init_module()
{
  int err;
#ifdef DEBUG
  int i;
  for (i = 0; i < DEBUG_FLAG_BYTES; i++)
    debug_flags[i] = 255;
#endif /* DEBUG */
  ENTER(INIT_MODULE);
  stomithip[0] = '\0';
  s_init = MUTEX;
  portd = MUTEX;
  if (port >= 0 && port < UINT_MAX){
    serv_port = port;
    down(&s_init);
    down(&portd);
    /*
    if (serv_port == 0)
      serv_port = 14243;
    */
    serv_port = 14243;
    alive = 1;
    err = kernel_thread(kgnbd_portd, NULL, 0);
    if (err < 0){
      printe("cannot start kgnbd_portd\n");
      return err;
    }
    down(&s_init);
    if (portdtsk == NULL){
      printe("failed to start kgnbd_portd\n");
      return -ENODEV;
    }
    printd("kgnbd_portd started\n");
    up(&s_init);
  }
  else
    serv_port = 0;
  atomic_set(&live_serv, 0);
  proc_register( &proc_root, &kgnbd_proc_entry);
  printk("KGNBD %s (built %s %s) installed\n",
	 GFS_RELEASE_NAME,
	 __DATE__, __TIME__);
  EXIT(INIT_MODULE);
  return 0;
}


void cleanup_module()
{
  int i;
  struct wait_queue wait = { current, NULL };
  ENTER(CLEANUP_MODULE);
  alive = 0;
  down(&s_init);
  for (i = 0; i < max_check; i++)
    if (server[i] != NULL)
      cleanup_server(i);
  add_wait_queue(&death_watch, &wait);   /* This should handle race */ 
  current->state = TASK_INTERRUPTIBLE;   /* conditions, so all threads are */
  while (atomic_read(&live_serv) > 0){   /* waited for... */
    schedule();
    current->state = TASK_INTERRUPTIBLE;
  }
  current->state = TASK_RUNNING;
  remove_wait_queue(&death_watch, &wait);
  if (portdtsk != NULL){
    send_sig(SIGINT, portdtsk, 0);
    down(&portd);
  }
  proc_unregister( &proc_root, kgnbd_proc_entry.low_ino );

  EXIT(CLEANUP_MODULE);
}











