/******************************************************************************
*
*    Copyright 1999 Regents of the University of Minnesota
*    Portions Copyright 1999-2001 Sistina Software, Inc.
*
*    This is free software released under the GNU General Public License.
*    There is no warranty for this software.  See the file COPYING for
*    details.
*
*    See the file CONTRIBUTORS for a list of contributors.
*
*    This file is maintained by:
*      Benjamin E Marzinski <marzinski@sistina.com>
*
 * I think for starters we'll do like the glmd.
 * One thread per client.  And I think we'll start with doing
 * lseek/read/write and move things to mmap later.
*/

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <pthread.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <netinet/in.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <signal.h>
#include <sys/mount.h>

#include "header.h"

/* This global keeps the server running.
 * I'll set up some signals to shut things down later.
 */
static int Server_Runs = TRUE;

/* This file descriptor is the data we're serving. */
static int data_fd=-1;
static u64 datasize=0;
static int readonly = FALSE;
/******************************************************************************
 * dump_buffer()
 * prints out the contents of a buffer for inspection.  ignores formats.
 *
 * Need to get all zero lines absorbed.
 */
void dump_buffer(void *buf, int len)
{
  int i;
  u8 *c = buf;

  for(i=0; i<len; i++) {
    if( i % 16 == 0 )
      printf("\n%4u: ", i);
    printf("%2x ", c[i]);
  }
  printf("\n");
}

/******************************************************************************
 * heh, simple recv/send wrappers to get all of the beffer sent over the
 * net.  Had forgotten I needed this....
 */
int cpt_recv(int s, void *buf, size_t len, int flags)
{
  int res;
  do {
    res = recv(s, buf, len, flags);
    if(res<0) return res;
    len -= res;
    buf += res;
  }while(len>0);
  return res;
}

int cpt_send(int s, void *buf, size_t len, int flags)
{
  int res;
  do {
    res = send(s, buf, len, flags);
    if(res<0) return res;
    len -= res;
    buf += res;
  }while(len>0);
  return res;
}

/******************************************************************************
 * clientTalk()
 * Error handling needs to be improved.....
 */
void *clientTalk(void *args)
{
  int *t = (int*)args;
  int sk = *t;
  int n;
  unsigned long len;
  unsigned long long offset;
  int client_runs = TRUE;
  int err = 0;
  theader_t ioHdr;
  char buf[20480]; /*some ba number. */
  pthread_mutex_t sender;
#if 0
  int deathbits = 50;
#endif

  /* init things */
  pthread_mutex_init(&sender, NULL);
  /* do login */
  printf("client has logged in\n");
  /* send some need-to-knows to the client. */
  {
    tio_info_t serv_stat;
    serv_stat.devsize = htonll(datasize);
    serv_stat.readonly = readonly;
    n = send(sk, &serv_stat, sizeof(tio_info_t), 0);
  }
  printf("login info sent\n");

  /* loop for requests. */
  while(client_runs && Server_Runs) {
    printff("top\n");
#if 0
    if( deathbits-- <=0 ) goto shutdown;
#endif
    n = cpt_recv(sk, &ioHdr, theader_len, 0);
    if( n<0 ) {
      err = n;
      goto shutdown;
    }

    len    = ntohl(ioHdr.len);
    offset = ntohll(ioHdr.offset);

    printff("ioHdr: type=0x%x, key=0x%"FMT_64"x, len=0x%lx, offset=0x%llx\n",
	    ioHdr.type, ioHdr.key, len, offset);

    if( len > 10240 ) {
      perror("Request too large");
      goto shutdown;
    }
    if( len + offset > datasize) {
      perror("Out of bounds.");
      goto shutdown;
    }
#if 0
    if( deathbits-- <=0 ) goto shutdown;
#endif
    switch(ioHdr.type) {
    case tioReadReq:
      printff("Doing a Read\n");
      /* Need to get len bytes from offset out of data_fd */
      if(lseek(data_fd, offset, SEEK_SET) <0)
	perror("Read: llseek");
      printff("Done seek\n");
      if(read(data_fd, buf, len) <0)
	perror("Read: read");
      printff("Done read\n");

      ioHdr.type = tioReadRpl;
			  
      pthread_mutex_lock(&sender);
      n = cpt_send(sk, &ioHdr, theader_len, 0);
      n = cpt_send(sk, buf, len, 0);
      pthread_mutex_unlock(&sender);
      printff("Done send\n");
      break;
    case tioWriteReq:
      printff("Doing a Write\n");
      /* need a readonly check....*/
      /* we have len bytes that we want to write at offset in data_fd */
      n = cpt_recv(sk, buf, len, 0);
      if(n==0) goto shutdown;
      if(lseek(data_fd, offset, SEEK_SET) <0)
	perror("Write: lseek");
      if(write(data_fd, buf, len) <0)
	perror("Write: write");

      ioHdr.type = tioWriteRpl;
      ioHdr.len  = 0;
      ioHdr.err  = 0;
      ioHdr.offset = 0;
      /*expects an ack..*/
      pthread_mutex_lock(&sender);
      n = cpt_send(sk, &ioHdr, theader_len, 0);
      pthread_mutex_unlock(&sender);
      printff("Done send\n");
      break;
    case tioShutdown:
      /* Gotta close the connection, after replying.*/
      /* ioHdr should be exactly what we waht to send back, so we'll
       * just dump it right back.
       */
      pthread_mutex_lock(&sender);
      n = cpt_send(sk, &ioHdr, theader_len, 0);
      pthread_mutex_unlock(&sender);

      /* now close shop. */
      client_runs = FALSE;
      break;
    default:
      printf("Unrecognised header.\n");
      printf("ioHdr: type=0x%x, key=0x%"FMT_64"x, len=0x%lx, offset=0x%llx\n",
	     ioHdr.type, ioHdr.key, len, offset);
      break;
    }


  }

  /* shut things down */
 shutdown:
  /* need to send an IOshutdown?*/
  printf("Client logged out\n");
  close(sk);
  pthread_exit(&err);
}

/******************************************************************************
 * signal handler.
 */
void mysigact(int sig)
{
  switch(sig) {
  case SIGUSR1:
    break;
  case SIGUSR2:
    Server_Runs = FALSE;
    break;
  }
}

/******************************************************************************
 * main()
 * Parse some options, checkout the world, listen to sockets, spawn
 * threads. You know, same old, same old.
 */
int main(int argc, char **argv) {
  int skin, sknew, len;
  struct sockaddr_in  myaddr, theiraddr;
  pthread_t tmpTid;

  int c;
  extern char *optarg;
  extern int optind;
  int port = 14243;

  /* get options. */
  while((c = getopt(argc, argv, "s:p:rh?"))!= -1){
    switch(c){
    case 's':
      datasize=atoi(optarg);
    case 'p':
      port=atoi(optarg);
      break;
    case 'r':
      readonly=TRUE;
      break;
    case '?':
    case 'h':
      printf("Usage: %s [-p port] [-s size] <device or file>\n",
	     argv[0]);
      return(1);
      break;
    }
  }

  /* setup signal handlers*/
  signal(SIGUSR1, mysigact);
  signal(SIGUSR2, mysigact);
  signal(SIGPIPE, SIG_IGN);

  /*open the device.*/
  data_fd = open( argv[optind], (readonly?O_RDONLY:O_RDWR)|O_SYNC );
  if(data_fd < 0) {
    perror("main: Failed to open data file");
    return 1;
  }
  if(datasize==0) {
    struct stat st;
    if(fstat(data_fd,&st)) {
      perror( "Could not fstat exported file");
      return 1;
    }
    if( S_ISBLK(st.st_mode) ) {
      if(ioctl(data_fd, BLKGETSIZE, (long*)&datasize) <0) {
	perror("Couldn't get sector count on block device");
	return 1;
      }
      datasize *= 512; /*sector size is hardcoded.*/
    }else{
      datasize=(u64)st.st_size;
    }
    if(datasize == 0) {
      fprintf(stderr,"Could not determin size of exported data\n");
      return 1;
    }
  }
  printf("Device size = %llx\n", datasize);

  if( (skin = socket(AF_INET, SOCK_STREAM, 0)) <0 ) {
    perror("main: Socket biffed");
    exit(1);
  }

  /* Set myself up to listen to the world. */
  myaddr.sin_family=AF_INET;
  myaddr.sin_addr.s_addr=INADDR_ANY;
  myaddr.sin_port=htons(port);
  if( bind(skin, (struct sockaddr *)&myaddr, sizeof(struct sockaddr_in)) <0) {
    perror("main: Bind Biffed");
    exit(1);
  }

  if( listen(skin, 5) <0 )
    perror("main: Listen Biffed");

  while(Server_Runs) {
    len = sizeof(struct sockaddr_in);
    if( (sknew = accept(skin, (struct sockaddr *)&theiraddr, &len)) <0)
      perror("main: Accept Biffed");

    if( pthread_create( &tmpTid, NULL, clientTalk, (void *)&sknew) <0)
      perror("main: pthread_create");
    if( pthread_detach( tmpTid ) <0)
      perror("main: pthread_detach");
  }


  close(data_fd);
  close(skin);
  return 0;
}

