/******************************************************************************
*
*    Copyright 1999 Regents of the University of Minnesota
*    Portions Copyright 1999-2001 Sistina Software, Inc.
*
*    This is free software released under the GNU General Public License.
*    There is no warranty for this software.  See the file COPYING for
*    details.
*
*    See the file CONTRIBUTORS for a list of contributors.
*
*    This file is maintained by:
*      Benjamin E Marzinski <marzinski@sistina.com>
*
* Trying a version with a single thread of execution.
*/

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <pthread.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <netinet/in.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <signal.h>
#include <sys/mount.h>
#include <poll.h>

#include "header.h"

/* some private data structures. */

/* This global keeps the server running.
 * I'll set up some signals to shut things down later.
 */
static int Server_Runs = TRUE;

/* This file descriptor is the data we're serving. */
static int data_fd=-1;
static u64 datasize=0;
static int readonly = FALSE;

/******************************************************************************
 * dump_buffer()
 * prints out the contents of a buffer for inspection.  ignores formats.
 *
 * Need to get all zero lines absorbed.
 */
void dump_buffer(void *buf, int len)
{
	int i;
	u8 *c = buf;

	for(i=0; i<len; i++) {
		if( i % 16 == 0 )
			printf("\n%4u: ", i);
		printf("%2x ", c[i]);
	}
	printf("\n");
}
/******************************************************************************
 * heh, simple recv/send wrappers to get all of the beffer sent over the
 * net.  Had forgotten I needed this....
 */
int cpt_recv(int s, void *buf, size_t len, int flags)
{
	int res;
	do {
		res = recv(s, buf, len, flags);
		if(res<0) return res;
		len -= res;
		buf += res;
	}while(len>0);
	return res;
}

int cpt_send(int s, void *buf, size_t len, int flags)
{
	int res;
	do {
		res = send(s, buf, len, flags);
		if(res<0) return res;
		len -= res;
		buf += res;
	}while(len>0);
	return res;
}

/******************************************************************************
 * handleRequest()
 */
int handleRequest(int clifd, theader_t *ioHdr)
{
   unsigned long len;
   unsigned long long offset;
   int err = 0;
   int n;
   char buf[20480];

   len = ntohl(ioHdr->len);
   offset = ntohll(ioHdr->offset);

   printf("ioHdr: type=0x%x, key=0x%lx, len=0x%lx, offset=0x%llx\n",
              ioHdr->type, ioHdr->key, len, offset);

   if( len > 10240 ) {
      perror("Request too large");
      return -1;
   }
   if( len + offset > datasize) {
      perror("Out of bounds.");
      return -1;
   }

   switch(ioHdr->type) {
      case tioReadReq:
         printff("Doing a Read\n");
         /* Need to get len bytes from offset out of data_fd */
         if(lseek(data_fd, offset, SEEK_SET) <0)
           perror("Read: llseek");
         printff("Done seek\n");
         if(read(data_fd, buf, len) <0)
           perror("Read: read");
         printff("Done read\n");

         ioHdr->type = tioReadRpl;

         n = cpt_send(clifd, ioHdr, theader_len, 0);
         n = cpt_send(clifd, buf, len, 0);
         printff("Done send\n");
         break;
      case tioWriteReq:
         printff("Doing a Write\n");
         /* need a readonly check....*/
         /* we have len bytes that we want to write at offset in data_fd */
         n = cpt_recv(clifd, buf, len, 0);
         if(n==0) return -1;
         if(lseek(data_fd, offset, SEEK_SET) <0)
           perror("Write: lseek");
         if(write(data_fd, buf, len) <0)
           perror("Write: write");

         ioHdr->type = tioWriteRpl;
         ioHdr->len  = 0;
         ioHdr->err  = 0;
         ioHdr->offset = 0;
         /*expects an ack..*/
         n = cpt_send(clifd, ioHdr, theader_len, 0);
         printff("Done send\n");
         break;
      case tioShutdown:
         /* Gotta close the connection, after replying.*/
         /* ioHdr should be exactly what we waht to send back, so we'll
         * just dump it right back.
         */
         n = cpt_send(clifd, &ioHdr, theader_len, 0);
         /* now close shop. */
         return -1;
         break;
      default:
         printf("Unrecognised header.\n");
         printf("ioHdr: type=0x%x, key=0x%lx, len=0x%lx, offset=0x%llx\n",
              ioHdr->type, ioHdr->key, len, offset);
         break;
   }
   return 0;
}

/******************************************************************************
 * loop()
 * the actual main loop of this thing
 */
void loop(void)
{
   int i, maxi, listenfd, clifd, nread;
   theader_t ioHdr;
	struct sockaddr_in  myaddr, theiraddr;
   struct pollfd *pollfd;

   /* fix 5 client Maximum!!!! */
   if( (pollfd = malloc(5 * sizeof(struct pollfd))) == NULL ) {
      perror("loop: cannot malloc poolfd");
      exit(1);
   }
   for(i=0;i<5;i++)
      pollfd[i].fd = -1;

	if( (listenfd = socket(AF_INET, SOCK_STREAM, 0)) <0 ) {
		perror("loop: Socket biffed");
		exit(1);
	}

	/* Set myself up to listen to the world. */
	myaddr.sin_family=AF_INET;
	myaddr.sin_addr.s_addr=INADDR_ANY;
	myaddr.sin_port=htons(14243);
	if( bind(listenfd, (struct sockaddr *)&myaddr,
            sizeof(struct sockaddr_in)) <0) {
		perror("loop: Bind Biffed");
		exit(1);
	}

	if( listen(listenfd, 5) <0 )
		perror("loop: Listen Biffed");

   pollfd[0].fd = listenfd;
   pollfd[0].events = POLLIN;
   maxi = 0;

   while(Server_Runs) {
      /* change the INFTIM to some value? */
      if( poll(pollfd, maxi +1, -1) <0) {
         perror("loop: poll error");
         exit(1);
      }

      if( pollfd[0].revents & POLLIN) {
         int len;
         len = sizeof(struct sockaddr_in);
         if((clifd = accept(listenfd, (struct sockaddr *)&theiraddr, &len))<0){
            perror("loop: error in accept");
            exit(1);
         }
         /* what is i?*/
         for(i=1; (i<5) && (pollfd[i].fd != -1); i++);

         pollfd[i].fd = clifd;
         pollfd[i].events = POLLIN;
         if(i > maxi) maxi=i;

         printf("Added new client fd=%d\n",clifd);
         {/*send login info*/
            tio_info_t serv_stat;
            serv_stat.devsize = htonll(datasize);
            serv_stat.readonly = readonly;
            nread = cpt_send(clifd, &serv_stat, sizeof(tio_info_t), 0);
         }
      }

      for( i=1; i<= maxi; i++) {
         if( (clifd = pollfd[i].fd) <0 )
            continue;
         if( pollfd[i].revents & POLLHUP)
            goto hungup;
         else if(pollfd[i].revents & POLLIN) {
            if( (nread = cpt_recv(clifd, &ioHdr, sizeof(theader_t), 0)) <0) {
               perror("loop: Read error");
            } else if(nread == 0) {
hungup:
               printf("loop: closed fd %d\n",clifd);
               pollfd[i].fd = -1;
               close(clifd);
            }else{
               if(handleRequest(clifd, &ioHdr) <0) {
                  printf("loop: closed fd %d\n",clifd);
                  pollfd[i].fd = -1;
                  close(clifd);
               }
            }
         }
      }
   }
}


/******************************************************************************
 * main()
 * Parse some options, checkout the world, listen to sockets, spawn
 * threads. You know, same old, same old.
 */
int main(int argc, char **argv) {
	int skin, sknew, len;
	pthread_t tmpTid;

	int c;
	extern char *optarg;
	extern int optind;
	int port = 14243;

/* get options. */
	while((c = getopt(argc, argv, "s:p:rh?"))!= -1){
		switch(c){
         case 's':
            datasize=atoi(optarg);
			case 'p':
				port=atoi(optarg);
				break;
			case 'r':
				readonly=TRUE;
				break;
			case '?':
			case 'h':
				printf("Usage: %s [-p port] [-s size] <device or file>\n",
                  argv[0]);
				return(1);
			break;
		}
	}

	/* setup signal handlers*/
	signal(SIGPIPE, SIG_IGN);

	/*open the device.*/
	data_fd = open( argv[optind], (readonly?O_RDONLY:O_RDWR)|O_SYNC );
	if(data_fd < 0) {
		perror("main: Failed to open data file");
		return 1;
	}
   if(datasize==0) {
		struct stat st;
		if(fstat(data_fd,&st)) {
			perror( "Could not fstat exported file");
			return 1;
		}
      if( S_ISBLK(st.st_mode) ) {
         if(ioctl(data_fd, BLKGETSIZE, (long*)&datasize) <0) {
            perror("Couldn't get sector count on block device");
            return 1;
         }
         datasize *= 512; /*sector size is hardcoded.*/
      }else{
         datasize=(u64)st.st_size;
      }
      if(datasize == 0) {
         fprintf(stderr,"Could not determin size of exported data\n");
         return 1;
      }
	}
   printf("Device size = %llx\n", datasize);

   /* Start Accept loop */
   loop();

	close(data_fd);
	return 0;
}

