/******************************************************************************
*******************************************************************************
**
**    Copyright 1995-1999 Regents of the University of Minnesota
**    Portions Copyright 2001 Sistina Software, Inc.
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      Kenneth W. Preslan <kpreslan@sistina.com>
** 
**    Module name: global.h
**
**    Description:
**      Globally used header for GFS, pool, etc.
**
*******************************************************************************
******************************************************************************/

#ifndef __GLOBAL_DOT_H__
#define __GLOBAL_DOT_H__

#ifdef __cplusplus
extern "C" {
#endif


#ifndef BPKB
#define	BPKB (10)	/* Bits per kilobyte */
#define KBSHIFT	BPKB
#endif

#if !defined(KILOBYTE)
#define KILOBYTE	((__uint32_t)(       1 << KBSHIFT))
#endif

#if !defined(MEGABYTE)
#define MEGABYTE	((__uint32_t)(KILOBYTE << KBSHIFT))
#endif

#if !defined(GIGABYTE)
#define GIGABYTE	((__uint32_t)(MEGABYTE << KBSHIFT))
#endif

#if !defined(TERABYTE)
#define TERABYTE	(1099511627776LLU)
#endif

#if !defined(PETABYTE)
#define PETABYTE	(1125899906842624LLU)
#endif

#if !defined(TRUE) || ((TRUE) != 1)
#define TRUE (1)
#endif

#if !defined(FALSE) || ((FALSE) != 0)
#define FALSE (0)
#endif



#ifdef linux

#include <linux/version.h>

#ifdef __alpha__
#define FMT_64 "l"
#define FMT_32 ""
#endif  /*  __alpha__  */

#ifdef __s390__ /* XXX */
#define FMT_64 "L"
#define FMT_32 ""
#endif

#ifdef __i386__
#define FMT_64 "L"
#define FMT_32 ""
#endif  /*  __i386__  */

#ifdef __powerpc__
#define FMT_64 "L"
#define FMT_32 ""
#endif  /*  __i386__  */

/* There are two definitions for the sparc. This is because
 * there is a seperate compiler for the kernel binaries.
 * So __sparc_v9__ is for the in kernel code, and 
 * __sparc__ is for the in userland code.
 */
#if defined(__sparc_v9__)
#define FMT_64 "l"
#define FMT_32 ""
#elif defined(__sparc__)
#define FMT_64 "L"
#define FMT_32 ""
#endif /* __sparc__*/


#ifdef __KERNEL__
#include <linux/types.h>
#else
#include <stdint.h> /* mostly same as below, much friendlier to net prgms */
#endif  /*  __KERNEL__  */

#endif  /*  linux  */



#ifdef IRIX

#define FMT_64 "l"
#define FMT_32 ""

#include <sys/types.h>
typedef dev_t		kdev_t;

#endif  /*  IRIX  */



#if defined(__FreeBSD__)

#include <sys/types.h>

#define FMT_64 "ll"
#define FMT_32 ""

#endif  /*  __FreeBSD  */



typedef uint64_t          uint64;
typedef uint32_t          uint32;
typedef uint16_t          uint16;
typedef uint8_t           uint8;
typedef int64_t           int64;
typedef int32_t           int32;
typedef int16_t           int16;
typedef int8_t            int8;



struct debug_flag
{
  const char             *name;
  const unsigned int     len;
  const unsigned int     *flag;
};
typedef struct debug_flag debug_flag_t;



#ifdef __cplusplus
}
#endif

#endif /* __GLOBAL_H__ */
