/******************************************************************************
*******************************************************************************
**
**    Copyright 2000-2001 Sistina Software, Inc.
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      Kenneth W. Preslan <kpreslan@sistina.com>
** 
**    Module name: main.c
**
*******************************************************************************
******************************************************************************/

#define EXTERN
#include "lock_harness.h"

#define __NO_VERSION__ 
#define EXPORT_SYMTAB
#include <linux/module.h>
#include <linux/version.h>
#ifdef MODULE
char kernel_version [] = UTS_RELEASE;
#endif
#include <linux/init.h>
#include <linux/kmod.h>



/**
 * lmh_module_load - try to load a module of a given name
 * @name: the name of the module
 *
 */

void lmh_module_load(char *name)
{
  if (capable(CAP_SYS_MODULE))
    request_module(name);
}


/**
 * init_lmh - Initialize the lock module harness
 *
 * Returns: 0 on success, -EXXX on failure
 */

int __init init_lmh(void)
{
  rlock_init(&lmh_lock);
  osi_list_head_init(&lmh_list);

  osi_printf("LockHarness %s (built %s %s) installed\n",
	     GFS_RELEASE_NAME,
	     __DATE__, __TIME__);

  return 0;
}


#ifdef MODULE

/**
 * init_module - Initialize the module
 *
 * Just call init_lmh
 *
 * Returns: 0 on success, -EXXX on failure
 */

int __init init_module(void)
{
  return init_lmh();
}


/**
 * cleanup_module - uninstall the module
 *
 */

void cleanup_module(void)
{
}

MODULE_DESCRIPTION("Lock Module Harness");
MODULE_AUTHOR("The GFS Group");

#endif  /*  MODULE  */



/*  Symbols  */

EXPORT_SYMBOL(lm_register_proto);
EXPORT_SYMBOL(lm_unregister_proto);
EXPORT_SYMBOL(lm_mount);

