/******************************************************************************
*******************************************************************************
**
**    Copyright 2000-2001 Sistina Software, Inc.
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      Kenneth W. Preslan <kpreslan@sistina.com>
** 
**    Module name: bottom.c
**
*******************************************************************************
******************************************************************************/

#include "lock_harness.h"





/**
 * lm_register_proto - Register a low-level locking protocol
 * @proto: the protocol definition
 *
 * Returns: 0 on success, -EXXX on failure
 */

int lm_register_proto(lm_lockops_t *proto)
{
  osi_list_head_t *tmp;
  lmh_wrapper_t *lw = NULL;

  rlock_lock(&lmh_lock);


  for (tmp = lmh_list.next; tmp != &lmh_list; tmp = tmp->next)
  {
    lw = osi_list_entry(tmp, lmh_wrapper_t, lw_list);

    if (osi_strcmp(lw->lw_ops->proto_name, proto->proto_name) == 0)
      break;
    else
      lw = NULL;
  }


  if (lw)
  {
    osi_printf("lock_harness:  protocol %s already exists\n", proto->proto_name);
    rlock_unlock(&lmh_lock);
    return -EEXIST;
  }


  lw = osi_malloc(sizeof(lmh_wrapper_t));
  if (!lw)
  {
    rlock_unlock(&lmh_lock);
    return -ENOMEM;
  }
  osi_memset(lw, 0, sizeof(lmh_wrapper_t));


  lw->lw_ops = proto;
  osi_list_add(&lw->lw_list, &lmh_list);


  rlock_unlock(&lmh_lock);

  return 0;
}


/**
 * lm_unregister_proto - Unregister a low-level locking protocol
 * @proto: the protocol definition
 *
 */

void lm_unregister_proto(lm_lockops_t *proto)
{
  osi_list_head_t *tmp;
  lmh_wrapper_t *lw = NULL;

  rlock_lock(&lmh_lock);


  for (tmp = lmh_list.next; tmp != &lmh_list; tmp = tmp->next)
  {
    lw = osi_list_entry(tmp, lmh_wrapper_t, lw_list);

    if (osi_strcmp(lw->lw_ops->proto_name, proto->proto_name) == 0)
      break;
    else
      lw = NULL;
  }
 

  if (lw)
  {
    osi_list_del(&lw->lw_list);
    osi_free(lw, sizeof(lmh_wrapper_t));
  }
  else
    osi_printf("lock_harness:  can't unregister lock protocol %s\n", proto->proto_name);


  rlock_unlock(&lmh_lock);
}





