/******************************************************************************
*******************************************************************************
**
**    Copyright 2001 Sistina Software, Inc.
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      Kenneth W. Preslan <kpreslan@sistina.com>
** 
**    Module name: rlock.c
**
*******************************************************************************
******************************************************************************/

#include "lock_harness.h"
#include "rlock.h"





/**
 * rlock_init - initialize a recursive lock
 * @rl: the lock
 *
 */

void rlock_init(rlock_t *rl)
{
  osi_memset(rl, 0, sizeof(rlock_t));

  osi_spinlock_init(&rl->rl_lock);
  osi_sema_init_lkd(&rl->rl_wait);
}


/**
 * rlock_lock - acquire a recursive lock
 * @rl: the lock
 *
 */

void rlock_lock(rlock_t *rl)
{
  for (;;)
  {
    osi_spin_lock(&rl->rl_lock);

    if (!rl->rl_locked)
    {
      rl->rl_pid = osi_pid();
      rl->rl_locked++;
      break;
    }
    if (rl->rl_pid == osi_pid())
    {
      rl->rl_locked++;
      break;
    }

    rl->rl_waiting++;

    osi_spin_unlock(&rl->rl_lock);
    osi_sema_lock(&rl->rl_wait);
  }

  osi_spin_unlock(&rl->rl_lock);
}


/**
 * rlock_unlock - release a recursive lock
 * @rl: the lock
 *
 */

void rlock_unlock(rlock_t *rl)
{
  osi_spin_lock(&rl->rl_lock);

  LMH_ASSERT(rl->rl_locked, );
  rl->rl_locked--;

  if (!rl->rl_locked && rl->rl_waiting)
  {
    rl->rl_waiting--;
    osi_spin_unlock(&rl->rl_lock);
    osi_sema_unlock(&rl->rl_wait);
  }
  else
    osi_spin_unlock(&rl->rl_lock);
}



