/******************************************************************************
*******************************************************************************
**
**    Copyright 2000-2001 Sistina Software, Inc.
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      Kenneth W. Preslan <kpreslan@sistina.com>
** 
**    Module name: top.c
**
*******************************************************************************
******************************************************************************/

#include "lock_harness.h"





/**
 * lm_mount - Mount a lock protocol
 * @proto_name - the name of the protocol
 * @table_name - the name of the lock space
 * @host_data - data specific to this host
 * @cb - the callback to the code using the lock module
 * @fsdata - data to pass back with the callback
 * @lockstruct - a structure returned describing the mount
 *
 * Returns: 0 on success, -EXXX on failure
 */

int lm_mount(char *proto_name, char *table_name, char *host_data,
	     lm_callback_t cb, lm_fsdata_t *fsdata,
	     lm_lockstruct_t *lockstruct)
{
  osi_list_head_t *tmp;
  lmh_wrapper_t *lw = NULL;
  int try = 0;


 retry:
  rlock_lock(&lmh_lock);


  for (tmp = lmh_list.next; tmp != &lmh_list; tmp = tmp->next)
  {
    lw = osi_list_entry(tmp, lmh_wrapper_t, lw_list);

    if (osi_strcmp(lw->lw_ops->proto_name, proto_name) == 0)
      break;
    else
      lw = NULL;
  }


  if (!lw)
  {
    if (!try)
    {
      try = 1;
      rlock_unlock(&lmh_lock);
      lmh_module_load(proto_name);
      goto retry;
    }

    osi_printf("lock_harness:  can't find protocol %s\n", proto_name);
    rlock_unlock(&lmh_lock);
    return -ENOENT;
  }


  lockstruct->ls_ops = lw->lw_ops;
  lockstruct->ls_lockspace = lw->lw_ops->mount(table_name, host_data,
					       cb, fsdata,
					       &lockstruct->ls_jid, &lockstruct->ls_first);
  if (!lockstruct->ls_lockspace)
  {
    rlock_unlock(&lmh_lock);
    return -EINVAL;
  }


  rlock_unlock(&lmh_lock);

  return 0;
}



