/******************************************************************************
*******************************************************************************
**
**    Copyright 1999-2001 Sistina Software, Inc.
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      David C. Teigland <teigland@sistina.com>
** 
**    Module name: cluster_config.h
**
*******************************************************************************
******************************************************************************/

#ifndef CLUSTER_CONFIG_DOT_H
#define CLUSTER_CONFIG_DOT_H


#include "global.h"
#include "gfs_ondisk.h"
#include "stomith.h"


#define GFS_CLUSTER_GLOBAL  (0x47465341)
#define GFS_CLUSTER_NODE    (0x47465342)

#define GFS_CLUSTER_VERSION (0x47010000)

#define CIDEV_BLOCKSIZE     (4096)


/*  First sector of cidev  */

struct cluster_global
{
  uint32              cg_magic;                           /* magic number */
  uint32              cg_version;                         /* version number */
  char                cg_lockdev[GFS_LOCKNAME_LEN];       /* lock device */
  char                cg_datadev[GFS_LOCKNAME_LEN];       /* data device */
  uint32              cg_cbport;                          /* call-back port */
  uint32              cg_node_timeout;                    /* in seconds */
};
typedef struct cluster_global cluster_global_t;


/*  One struct per sector.  Implicit: JID = block - 1  */

struct cluster_node
{
  uint32              cn_magic;         /* Magic number */
  uint32              cn_ipaddr;        /* IP address of node */
  uint32              cn_cid;           /* Client ID of node */
  uint32              cn_num_stomith;   /* Number of stomith structs */
};
typedef struct cluster_node cluster_node_t;



/*  On-disk format translation functions  */

void cluster_global_in(cluster_global_t *cg, char *buf);
void cluster_global_out(cluster_global_t *cg, char *buf);
void cluster_node_in(cluster_node_t *cn, char *buf);
void cluster_node_out(cluster_node_t *cn, char *buf);
void cluster_stomith_in(cluster_stomith_t *cs, char *buf);
void cluster_stomith_out(cluster_stomith_t *cs, char *buf);
int ondisk_stomith_size(uint32 magic);


#endif  /*  CLUSTER_CONFIG_DOT_H  */








