/******************************************************************************
*******************************************************************************
**
**    Copyright 1999-2001 Sistina Software, Inc.
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      David C. Teigland <teigland@sistina.com>
**
**    Module name: daemon.c
**
*******************************************************************************
******************************************************************************/

#ifdef MODVERSIONS
#include <linux/modversions.h>
#endif  /*  MODVERSIONS  */
#include <linux/kernel.h>
#include <asm/uaccess.h>
#include <linux/smp_lock.h>

#include "memexp.h"
#include "client_state.h"



/**
 * lock_refresh
 *
 * @data:
 *
 */


int lock_refresh(void *data)
{
  memexp_t *mep = (memexp_t *) data;
  unsigned long flags;


  MEP_ASSERT(mep, );


  /*  Set up the thread  */

  lock_kernel();

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0))
  daemonize();
#else
  {
    struct fs_struct *fs;
    exit_mm(current);
    exit_files(current);
    current->files = init_task.files;
    atomic_inc(&current->files->count);
    exit_fs(current);
    fs = init_task.fs;
    current->fs = fs;
    atomic_inc(&fs->count);
  }
#endif

  spin_lock_irqsave(&current->sigmask_lock, flags);
  sigfillset(&current->blocked);
  recalc_sigpending(current);
  spin_unlock_irqrestore(&current->sigmask_lock, flags);

  current->session = 1;
  current->pgrp = 1;
  current->tty = NULL;
  current->tty_old_pgrp = 0;

  sprintf(current->comm, "memexp_heartbeat");

  mep->lock_refresh_ts = current;  /* save this for the wakeup */

  while (!mep->lock_refresh_stop)
  {
    client_refresh(mep);
    osi_sleep(mep->node_timeout / 10);
  }

  up(&mep->lock_refresh_sem);

  unlock_kernel();
  return(0);
}



/**
 * node_monitor
 *
 * @data:
 *
 */

int node_monitor(void *data)
{
  memexp_t *mep = (memexp_t *) data;
  unsigned long flags;


  MEP_ASSERT(mep, );


  /*  Set up the thread  */

  lock_kernel();

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0))
  daemonize();
#else
  {
    struct fs_struct *fs;
    exit_mm(current);
    exit_files(current);
    current->files = init_task.files;
    atomic_inc(&current->files->count);
    exit_fs(current);
    fs = init_task.fs;
    current->fs = fs;
    atomic_inc(&fs->count);
  }
#endif

  spin_lock_irqsave(&current->sigmask_lock, flags);
  sigfillset(&current->blocked);
  recalc_sigpending(current);
  spin_unlock_irqrestore(&current->sigmask_lock, flags);

  current->session = 1;
  current->pgrp = 1;
  current->tty = NULL;
  current->tty_old_pgrp = 0;

  sprintf(current->comm, "memexp_monitor");

  mep->node_monitor_ts = current;  /* save this for the wakeup */


  while (!mep->node_monitor_stop)
  {
    reread_client_state(mep);
    check_expired_clients(mep);
    osi_sleep(mep->node_timeout);
  }

  up(&mep->node_monitor_sem);

  unlock_kernel();
  return(0);
}






