/******************************************************************************
*******************************************************************************
**
**    Copyright 2000-2001 Sistina Software, Inc.
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      Kenneth W. Preslan <kpreslan@sistina.com>
** 
**    Module name: main.c
**
*******************************************************************************
******************************************************************************/

#define EXTERN
#include "nolock.h"

#define __NO_VERSION__ 
#include <linux/module.h>
#include <linux/version.h>
char kernel_version [] = UTS_RELEASE;
#include <linux/init.h>
#include <linux/kmod.h>



/**
 * nolock_mod_inc - decrement the module's usage count
 *
 */

void nolock_mod_inc(void)
{
  MOD_INC_USE_COUNT;
}


/**
 * nolock_mod_dec - decrement the module's usage count
 *
 */

void nolock_mod_dec(void)
{
  MOD_DEC_USE_COUNT;
}


/**
 * init_nolock - Initialize the nolock module
 *
 * Returns: 0 on success, -EXXX on failure
 */

int __init init_nolock(void)
{
  int error;

  error = lm_register_proto(&nolock_ops);
  if (error)
    return error;

  osi_printf("Nolock %s (built %s %s) installed\n",
	     GFS_RELEASE_NAME,
	     __DATE__, __TIME__);

  return 0;
}


/**
 * exit_nolock - cleanup the nolock module
 *
 */

void __exit exit_nolock(void)
{
  lm_unregister_proto(&nolock_ops);
}


module_init(init_nolock);
module_exit(exit_nolock);


MODULE_DESCRIPTION("Nolock Module");
MODULE_AUTHOR("The GFS Group");

#ifndef DEBUG_SYMBOLS
EXPORT_NO_SYMBOLS;
#endif  /*  DEBUG_SYMBOLS  */




