/******************************************************************************
 *
 *    Copyright 2000-2001 Sistina Software, Inc.
 *
 *    This is free software released under the GNU General Public License.
 *    There is no warranty for this software.  See the file COPYING for
 *    details.
 *
 *    See the file CONTRIBUTORS for a list of contributors.
 *
 *    This file is maintained by:
 *      Mike C Tilstra <conrad@sistina.com>
 *
 * ALL VALUES ARE IN BIGEDIAN.
 *
 *  I will need a tool like uc to scan the disk store.  Mostly this needs to
 *  be a bit like a hex editor, BUT completely specific to the contents of
 *  this.
 *  Bah, a hexeditor works fine.....
 *
 *
 * heh, most of these defines just became fluff.
 *
 */
/*
 * Disk layout:
 *  Headers
 *  Ctable
 *  CHold
 *  EStore
 *
 * Headers is a signle block that holds information about stuff.
 * 
 * CTable is a list of IPs that are allowed to connect to this server, and
 * a flag byte. Currently only used for Bans.  This also creates an Index
 * to IP mapping that is used to locate a Client's CHold
 *
 * CHold Is just a bunch of Elements. One for each possible client. This is
 * used to lop over the possbile errors that Server failure durring Store
 * creates.  A store op first writes the Element here, then into the store
 * proper, then clears this (with zeros).
 *
 * EStore is where the Elements are all stored.
 */
#ifndef __disktypes_h__
#define __disktypes_h__

/* This size of one disk block */
#define BLK_size 512

/* cute divide-rounding-up macro */
#define DIV_RU(x, y) (((x) + (y) - 1) / (y))

/* this gets a block. */
#define DISKHEAD_MAGIC 0x6469736B68656164  /* diskhead */
#define DISKHEAD_VERSION 0x566572732D303031 /* Vers-001 */
struct diskheader_s {
   u64 magic;  /* magic to make sure this is good. */
   u64 version; /* Allow me to die peacefuly if mismatched disk versions. */
   u64 total_data_field; /* Total amount of memory allocated for data field */
   u32 elem_data_size; /* size of a single element */
   u32 num_hash_key; /* Number of hashkeys. */
   u32 num_elem; /* number of elements */
   u32 num_max_elem; /* maximum number of elements allowed */
   u32 max_cli; /* max number of clients. (determins ban[] size)*/
} __attribute__ ((packed));
typedef struct diskheader_s diskheader_t;

/* header gets one 512 block. */
#define HEADER_SIZE BLK_size

/* Client Table Element.
 * This will end up replacing the ban tables.
 * Also, the index location of these creates a id <=> IP mapping that will
 * be used elsewhere.
 */
#define BANNED (1)
#define IS_BANNED(x) ((x) & BANNED)
struct ct_elem_s {
   u32 IP;
	u8  C;  /* one char of magic is hardly that. but this should always be 'C'*/
   u8  flags;
} __attribute__ ((packed));
typedef struct ct_elem_s ct_elem_t;
/* How many client table elements per disk block */
#define CTPB (BLK_size/ (int)sizeof(ct_elem_t))
/* How many disk blocks needed for mx clients */
#define CT_BLOCKS(mx) DIV_RU((mx), CTPB)
/* Where the byte start of ct element idx is. */
#define CT_OFFSET(idx) (((idx)/CTPB)*BLK_size+(((idx)%CTPB)*(int)sizeof(ct_elem_t)))

/* */
#define CHOLDELEM_MAGIC 0x63686C64 /* chld */
struct CHelem_s {
	u32 pbn;
	u64 seq;
	u8  bid[BIDSIZE];
	u8  inUse;
	u32 crc;
	u32 magic;
}  __attribute__ ((packed));
typedef struct CHelem_s CHelem_t;

#define CH_SIZE(sze) ((int)sizeof(CHelem_t)+(sze))
#define CHPB(sze) (BLK_size/CH_SIZE(sze))
#define CH_BLOCKS(mx, sze) DIV_RU((mx), CHPB(sze))
#define CH_OFFSET(idx, sze) (((idx)/CHPB(sze))*BLK_size+(((idx)%CHPB(sze))*CH_SIZE(sze)))

/* these are the toughy.
 * They have a variable field following them, so I'll stick as many as I
 * can into a block, but its gonna be a variable amount.
 */
#define DISKELEM_MAGIC 0x656C656D /*elem*/
struct diskelem_s {
   u64 seq;  /* the sequence */
   u8  bid[BIDSIZE]; /* the buffer id */
   u8  inUse; /* usings this? */
   u32 crc;   /* checksum for the data that follows. */
   u32 magic; /* magic to make sure this is valid. */
} __attribute__ ((packed));
typedef struct diskelem_s diskelem_t;

/* The number of blocks to group together in reads and writes.  This
 * grouping make the entire process of reading and writing much faster.
 *
 * NOTE: the disk init funcs will fit things within even groupings.  This
 * means that if the device has 30 avail 512 blocks. And blkGrousp is 16,
 * only ONE block group get written.  So instead of using all 30 blocks,
 * only 16 will be used.  The speed advantage out weighs the loss.  And
 * besides, usually the device has tens of thousands of blocks.
 */
#define blkGroups 32

/* The byte size of one element with all headers, data and client sequences. */
#define EL_SIZE(sze) ((int)sizeof(diskelem_t)+(sze))

/* EPB is the number of element we can fit into one 512 block */
#define EPB(sze) (BLK_size/EL_SIZE(sze))

/* EOFFSET is the byte offset from the start of the elements to the idx one*/
#define EL_OFFSET(idx, elsze) (((idx)/EPB(elsze))*BLK_size+(((idx)%EPB(elsze))*EL_SIZE(elsze)))

/* number of blocks elements need. */
#define EL_BLOCKS(el, sze) DIV_RU((el),EPB((sze)))

#endif /*__disktypes_h__*/
