/******************************************************************************
 *
 *    Copyright 2001 Sistina Software, Inc.
 *
 *    This is free software released under the GNU General Public License.
 *    There is no warranty for this software.  See the file COPYING for
 *    details.
 *
 *    See the file CONTRIBUTORS for a list of contributors.
 *
 *    This file is maintained by:
 *      Mike C Tilstra <conrad@sistina.com>
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h> /* memset */
#include <errno.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <syslog.h>
#include <assert.h>
#include "memexp.h"
#include "info_stream.h"

/**
 * connect_info_stream - start the info stream
 * @met: <>
 * @s: < string containing "ip:port"
 * 
 * 
 * Returns: <0: Error, ==0: ok
 */
int connect_info_stream(MEtable_t *met, char *s)
{
   char *ip, *port;
   struct sockaddr_in adr;

   ip = s;
   port = strstr(s, ":");
   if( !port) return -EINVAL;
   *port++ = '\0';

   if( (met->TAsock = socket(AF_INET, SOCK_DGRAM, 0)) <0)
      return errno;

   adr.sin_family = AF_INET;
   adr.sin_addr.s_addr = inet_addr(ip);
   adr.sin_port = htons( atoi(port) );
   if( connect(met->TAsock, (struct sockaddr*)&adr,
            sizeof(struct sockaddr_in))<0) {
      close(met->TAsock);
      met->TAsock = -1;
      return errno;
   }
   return 0;
}

/**
 * dissconnect_info_stream - Stop the information stream
 * @met: <>
 * 
 */
void dissconnect_info_stream(MEtable_t *met)
{
   if(met->TAsock > 0) {
      close(met->TAsock);
      met->TAsock = -1;
   }
}

/**
 * send_info_packet - send a pack of information
 * @met: <
 * @pbn: <
 * @bid: < 
 * 
 * sends a short& sweet message via udp to another process.
 * This alternate process watches which pbns get hit and how often.  And
 * will hopefully give a visual representation of which locks get heavy
 * traffic.
 */
void send_info_packet(MEtable_t *met, u32 pbn, u8 bid[BIDSIZE])
{
   struct infost_s isp;
   int err=0;

   if(met->TAsock <0) return;

   isp.pbn = osi_cpu_to_be32(pbn);
   memcpy(isp.bid, bid, BIDSIZE);

   err = send(met->TAsock, &isp, sizeof(struct infost_s), 0);
#ifdef DEBUG
   if(err<0) log_err("Sending info stream packet returned %d:%d\n",err,errno);
#endif
}
