/******************************************************************************
 *
 *    Copyright 2000-2001 Sistina Software, Inc.
 *
 *    This is free software released under the GNU General Public License.
 *    There is no warranty for this software.  See the file COPYING for
 *    details.
 *
 *    See the file CONTRIBUTORS for a list of contributors.
 *
 *    This file is maintained by:
 *      Mike C Tilstra <conrad@sistina.com>
 *
 * There is now a single hash and a MRU/LRU.  There is only one condition
 * that a buffer element is not in the hash, and this is when the server is
 * first started.  Otherwise it will always be in the hash by its last bid,
 * even if it is unused.  The MRU is updated on every load or store, so
 * when we need a new buffer (no current bid mapping exists) we look for
 * something unused at the tail of the MRU.  If we scan the entire list
 * without finding one, the server is full, and the clients should have
 * been doing something about this a while ago.
 *
 * This method removes the need for a JustCreated list.
 *
 * Need to have a option to `swap' the data to disk.  This is going to be
 * interesting.  It will need to do a sync on everyop.  (open with O_SYNC)  I
 * think the best way to do this will be to do ala gnbdd.  Use nmap if we can,
 * if not then syncopen with lseek.  I won't need big files since all of this
 * will be smallish. (largest is 32.)  I also may only open the write only.
 * Sice you can bet that I'll keep ALL of the in memory structs. (fast loads,
 * slow stores.)  (this allows ppl to use a file in a filesystem.  Not really
 * good for what we want, but easier to debug.)
 *
 *  Using nmap won't work. We cannot get the block alignments we need.
 *
 * I need to figure out the ondisk format.  I think make a header on the first
 * part of the device/file and read that.  It tells the shape of the contents.
 *
 * Do I want the element and datafield arrays seperated? Might as well if I'm
 * not gonna change the in memory format.  duno. should think of something
 * that will have decent write preformance.  I think it will work better for
 * writes if each elmheader is paired with its datafield.
 *
 * I NEED to keep only the data fields and the element headers. The hash can
 * be rebuilt from the BIDs, and the MRU|LRU can be set to any arrangment to
 * start with. (Every load and store moves the element in the MRU anyways.)
 *
 * Also need to figure out when to do the syncs in the code.  Will need to do
 * some syncs in load when it maps a new element.
 *
 * -----Mon Nov 27 11:53:58 CST 2000
 * Ok, issue:
 *  Client can call store, data gets to server, gets written to disk,
 *  server crashes before send Ok reply.  Client now thinks that the state
 *  for buffer is one thing, and it is actually another.  Bad.  So I need a
 *  sequencing to track what state the buffer is in.  Cannot use the
 *  existing seq, since other clients might change that.  Need a Seq per
 *  connection.  This Seq need to be put on disk.  The writting of the Seq
 *  and the new elemheaders and data must be united. (there cannot be a
 *  crash between the writting of the Seq and the data.)
 *
 *  So, I am thinking of adding a Seq area where the ClientSeq and the Elem
 *  that store modified is put. In addition to the normal datastore area.
 *  So a store now has two writes. (eeww) One to record the CSeq with the
 *  data, one to record the data as it is.
 *
 *  This makes me think I should move the whole thing over to some kind of
 *  log.  Need more thought.
 *
 * -----Thu Dec 07 11:27:26 CST 2000
 *  It appears that the best way to deal with this is to pack as much as I
 *  can into a single element write.  This means that each element chunk
 *  needs an array of area for client sequencing.  Seems to be a bunch of
 *  wasted space, but given the semantics of crashes and disk writes, I
 *  cannot think of anything better.  (Ideas welcome.)
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h> /* memset */
#include <errno.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <sys/mount.h> /* BLKGETSIZE */
#include <sys/uio.h> 
#include <fcntl.h>
#include <syslog.h>
#include <assert.h>
#include "memexp.h"
#ifdef SOFT_RT
#include <sys/mman.h>
#endif /*SOFT_RT*/
#include "wiredtypes.h"
#include "disktypes.h"
#include "info_stream.h"
#include "hashes.h"
#include "diskstore.h"

#if 0
static void printbid(u8 bid[BIDSIZE])
{
   int i;
   printf("The bid is: ");
   for(i=0;i<BIDSIZE;i++) printf("%2x",bid[i]);
   printf("\n");
}
#endif

/****************************************************************************/
/**
 * bidcpy - Copy a bid.
 * @b: > space for the new bid
 * @c: < the bid you are copying.
 *
 * Just a fancy way to call memecpy. (both buffers had better be of size
 * BIDSIZE too.)
 * 
 */
void __inline__ *bidcpy( u8 *b, u8 *c)
{
   return memcpy( b, c, BIDSIZE);
}

/**
 * bidcmp - Compair to bids
 * @b: < a bid.
 * @c: < a bid.
 *
 * Both of those pointers assume to be to a u8 of length BIDSIZE.  If not
 * you should be hit with a big hammer for not doing that.
 *
 * Resturns: -1 if not equal, 0 if equal.
 */
#if 0
int __inline__ bidcmp( u8 *b, u8 *c)
{
   return memcmp( b, c, BIDSIZE);
}
#else
int bidcmp( u8 *b, u8 *c)
{
   int i;
   for (i = 0; i < BIDSIZE; i++, b++, c++)
      if (*b != *c)
         return(-1);
   return(0);
}
#endif

/**
 * create_Meltable - Initializes a MemoryExport Table
 * @met: <> pointer to a MEtable structure.
 * @tsze: < The amount of space to use for the datafield.
 * @dsz: < The inital size in bytes of each data element
 * @elcnt: < The max number of elements to build.
 *
 * Allocs all of the needed memory. Sets up startup defaults.
 *
 * Will blindly blast away whatever was there. Only intended to be called
 * once.
 *
 * Since there is a dependance between the totalsize, elementcount, and
 * elementsize, I should write up some function to figure out one from the
 * other two.
 *
 * Returns: 0 if ok, less than zero if failed.
 */
int create_Meltable( MEtable_t *met, unsigned long tsze,
      unsigned int hsze,
      unsigned int dsze, unsigned int elcnt,
      unsigned int maxcli)
{
   unsigned int i;
   unsigned long tmp=0;

   log_msg( " Hash needs %u bytes for %u keys\n",
         (unsigned int)(sizeof(u32)*hsze), hsze);
   tmp = sizeof( u32 ) * hsze;
   log_msg( " Element table needs %u bytes for %u elements\n",
         (unsigned int)(sizeof(MEelement_t)*elcnt), elcnt);
   tmp += sizeof(MEelement_t) * elcnt;
   log_msg( " Data Field needs %lu bytes\n", tsze);
   tmp += tsze;
   log_msg( " For a total of %lu bytes\n", tmp);

   /* ok, now get that memory! */
   met->hashsize = hsze;
   met->hashed = malloc( sizeof( u32 ) * hsze );
   if( ! met->hashed ) goto allfail;
   memset( met->hashed, GROUNDED, sizeof( u32 ) * hsze );
#ifdef SOFT_RT
   if( mlock( met->hashed, sizeof( u32 ) * hsze )  <0)
      die("Failed to lock hashes in RAM: %m\n");
#endif /*SOFT_RT*/

   met->Velemcnt = met->Relemcnt = elcnt;
   met->elements = malloc( sizeof(MEelement_t) * elcnt );
   if( ! met->elements ) goto allfail;
   memset( met->elements, 0, sizeof(MEelement_t) * elcnt );
#ifdef SOFT_RT
   if( mlock( met->elements, sizeof(MEelement_t) * elcnt )  <0)
      die("Failed to lock element headers in RAM: %m\n");
#endif /*SOFT_RT*/

   for(i=0; i < elcnt; i++) {
      met->elements[i].Hnext = GROUNDED;
      met->elements[i].Hprev = GROUNDED;
      met->elements[i].MRUn = i+1;
      met->elements[i].MRUp = i-1;
   }
   met->elements[elcnt-1].MRUn = GROUNDED; /* ground next for last element*/
   met->elements[0].MRUp = GROUNDED; /* ground prev for first element.*/

   met->MRUhead = 0;
   met->MRUtail = elcnt -1;

   met->datasize = dsze;
   met->datatotalsize = tsze;
   met->datafield = malloc( tsze );
   if( ! met->datafield ) goto allfail;
#ifdef SOFT_RT
   if( mlock( met->datafield, tsze ) <0)
      die("Failed to lock element data in RAM: %m\n");
#endif /*SOFT_RT*/

   met->used = 0; /* nothing used. */
   met->devnode = -1;
   met->maxclients = maxcli;

   met->IPs = malloc(sizeof(u32) * maxcli);
   if( ! met->IPs ) goto allfail;
   memset(met->IPs, 0, (sizeof(u32) * maxcli));
#ifdef SOFT_RT
   if( mlock( met->IPs, sizeof(u32) * maxcli) <0)
      die("Failed to lock IP tables in RAM: %m\n");
#endif /*SOFT_RT*/

   met->bans = malloc(maxcli);
   if( ! met->bans ) goto allfail;
   memset(met->bans, 0, maxcli);
#ifdef SOFT_RT
   if( mlock( met->bans, maxcli) <0)
      die("Failed to lock ban tables in RAM: %m\n");
#endif /*SOFT_RT*/

   return 0;
allfail:
   if( met->elements ) free( met->elements );
   if( met->datafield ) free( met->datafield );
   if( met->hashed ) free( met->hashed );
   if( met->IPs ) free( met->IPs );
   if( met->bans ) free( met->bans );
   return -ENOMEM;
}


/**
 * create_MTab - creates the internals based on max size.
 * @met: <> 
 * @totalsize: < max memory to use. in bytes
 * @eldsze: < the size of the data field of one element
 * @maxcli: < max number of clients
 * 
 * takes the total size, and breaks it into three chunks, one is for the
 * hash table, one for element headers, one for element data.  This is the
 * nice easy nobrainer interface.  Element data size is idealised for GFS
 * usage.
 *
 * Current settings: hash size is 2% of total.
 *  Then the remaining space is fitted with the max number of elements for
 *  the given element data size.
 *
 * Returns: errors if any
 */
int create_MTab( MEtable_t *met, unsigned long totalsize,
      unsigned int eldsze, unsigned int maxcli)
{
   unsigned long hashsze, datasze, bansze;
   unsigned int hashcnt, elhdcnt;

#if 1
   hashsze = totalsize /52;
   hashcnt = hashsze / sizeof(u32);
#else
   hashcnt = MIN((totalsize/(MEGABYTE *2 )), ~(u32)0) ;
   if( hashcnt > 100) hashcnt = 100;
   hashsze = hashcnt * sizeof(u32);
#endif

   bansze = sizeof(u32) * maxcli;

   elhdcnt = (totalsize - hashsze - bansze) / (sizeof(MEelement_t) + eldsze);

   datasze = eldsze * elhdcnt;

   return create_Meltable(met, datasze, hashcnt, eldsze, elhdcnt, maxcli);
}

/****************************************************************************/
/****************************************************************************/
/**
 * map_IP - Finds the index mapping for the IP.
 * @ip: <
 * 
 * Now question remains if these are Preloaded, or dynamically added.
 * 
 * Returns: <0: Not Found, >=0: Id for IP.
 */
int map_IP(MEtable_t *met, u32 ip)
{
   int i;
   for(i=0; i < met->maxclients; i++){
      if( ip == met->IPs[i] ) return i;
   }
   return -1;
}

/**
 * place_IP - makes a map for an IP.
 * @met: <>
 * @IP: <
 * 
 * 
 * Returns: -1: No Space Left, >=0: New mapping
 */
int place_IP(MEtable_t *met, u32 IP)
{
   int i;
   for(i=0; i < met->maxclients; i++) {
      if( met->IPs[i] == 0) {
         met->IPs[i] = IP;
         if(USEDISKSTORE(met)) write_ct(met, IP, 0, i);
         return i;
      }
   }
   return -1;
}

/****************************************************************************/
/**
 * set_ban - set a ban on an ip
 * @met: <>
 * @ip: <
 */
void set_ban(MEtable_t *met, u32 ip)
{
   int i;
   i = map_IP(met, ip);
   if(i <0) i = place_IP(met, ip);
   met->bans[i] = BANNED;
   if( USEDISKSTORE(met) ) write_ct(met, ip, BANNED, i);
}

/**
 * drop_ban - Removes a ban on an ip
 * @met: <>
 * @ip: <
 */
void drop_ban(MEtable_t *met, u32 ip)
{
   int i;
   i = map_IP(met, ip);
   if( i <0) return;
   met->bans[i] = 0;
   if( USEDISKSTORE(met) ) write_ct(met, ip, 0, i);
}


/**
 * test_ban - checks to see if an ip is banned
 * @met: <
 * @ip: <
 * 
 * 
 * Returns: TRUE or FALSE
 */
int test_ban(MEtable_t *met, u32 ip)
{
   int i;
   i = map_IP(met, ip);
   if( i <0) return FALSE;
   return IS_BANNED( met->bans[i] );
}

/****************************************************************************/
/**
 * dump_hash_density - how many entries in each hash?
 * @met: <
 * @srvs: >
 *
 * This does a quick scan of the hash table. It get the number of entries
 * in each bucket.  Useful to see how well the hash_key is balancing the
 * known entries.
 *
 */
int dump_hash_density(MEtable_t *met, hshcnt_t *srvs)
{
   int i,j,dens;

   for(i=0; i < met->hashsize; i++) {
      dens=0;
      for(j=met->hashed[i]; j != GROUNDED; j = met->elements[j].Hnext ) {
         dens++;
      }

      srvs->bcnt[ MIN(dens,(MAXdens-1)) ] ++;
   }

   for(i=0; i < MAXdens; i++)
      srvs->bcnt[i] = cpu_to_be16(srvs->bcnt[i]);

   srvs->inuse = cpu_to_be32(met->used);
   srvs->numkeys = cpu_to_be32(met->hashsize);
   srvs->maxelems = cpu_to_be32(met->Relemcnt);

   return 0;
}

#ifdef DEEP_DEBUG
/**
 * check_balance - Checks the balnce of the tables.
 * @met: <
 * @sfx: < string to print.
 * 
 * check the used count from various points of view.  Prints out which pov
 * and the string you supplied when the value is wrong.
 *
 * This function will probably get #ifed out soon.
 * 
 * Returns: void
 */
int check_balance(MEtable_t *met, char *sfx)

{
   u32 shouldbe = met->used;
   u32 thereis=0;
   u32 hashused=0;
   u32 MRUtailused=0;
   u32 MRUheadused=0;
   int i,j,ret=0;

   for(i=0;i < met->Relemcnt; i++)
      if( met->elements[i].InUse ) thereis++;

   for(i=0; i < met->hashsize; i++) {
      for(j=met->hashed[i]; j != GROUNDED; j = met->elements[j].Hnext ) {
         if(met->elements[j].InUse ) hashused++;
      }
   }

   for(i = met->MRUtail; i != GROUNDED; i = met->elements[i].MRUp) {
      if( met->elements[i].InUse ) MRUtailused++;
   }
   for(i = met->MRUhead; i != GROUNDED; i = met->elements[i].MRUn) {
      if( met->elements[i].InUse ) MRUheadused++;
   }

   if( shouldbe != hashused  ){printf("hash %s\n",sfx);ret=1;}
   if( shouldbe != MRUtailused ){printf("tail %s\n",sfx);ret=1;}
   if( shouldbe != MRUheadused ){printf("head %s\n",sfx);ret=1;}
   if( shouldbe != thereis ){printf(" is  %s\n",sfx);ret=1;}

   return ret;
}
#endif /*DEEP_DEBUG*/


/**
 * fullness - How full is the memory space
 * @met: < 
 *
 * TODO rewrite with out floating point.
 *
 * Returns: a value showing how many of the free bufs are used.
 */
u8 fullness(MEtable_t *met)
{
   return (u8) (255.0 * met->used / (float)met->Velemcnt);
}

/**
 * touch_element - Update the life of a element.
 * @met: <>
 * @elm: < index of element to move.
 *
 * Simply move the given element ID to the head of the MRU. 
 * This should be called once for each Load or Store.
 */
void touch_element(MEtable_t *met, u32 elm)
{
   u32 tmpP, tmpN;

   tmpP = met->elements[elm].MRUp;
   tmpN = met->elements[elm].MRUn;

   if( met->MRUhead == elm ) return; /*Already at the head. */
   assert( tmpP != GROUNDED );
   if( met->MRUtail == elm) {
      met->MRUtail = tmpP;
      met->elements[tmpP].MRUn = GROUNDED;
   } else {
      assert( tmpP != GROUNDED && tmpN != GROUNDED );
      met->elements[tmpP].MRUn = tmpN;
      met->elements[tmpN].MRUp = tmpP;
   }

   /* ok, now modify the MRUhead. */
   met->elements[met->MRUhead].MRUp = elm;
   met->elements[elm].MRUp = GROUNDED;
   met->elements[elm].MRUn = met->MRUhead;
   met->MRUhead = elm;
}

/**
 * lookup_bid - Finds a bid in the hashtables
 * @met: <> Memory Export Table pointer
 * @bid: < BufferId
 * @pbn: > Physical Buffer number (stuffed on return)
 *
 * Simple little thing to find a bid in the hash tables. Returns a negitive
 * if there isn't one in the hash.
 *
 * Return: zero if pbn found, -1 if not found
 */
int lookup_bid(MEtable_t *met, u8 bid[BIDSIZE], u64 *pbn)
{
   u32 key, i, sane=0;

   key = get_hash_key( bid );
   key %= met->hashsize;

   /* look for existing bid<=>pbn mapping */
   for(i = met->hashed[key]; i != GROUNDED; i = met->elements[i].Hnext, sane++){
      if( bidcmp( bid, met->elements[i].bid) == 0) {
         *pbn = i;
         return 0;
      }
      assert( sane <= met->Relemcnt ); /* worstcase, all elem in one bucket.*/
   }
   /* couldn't find one. pbn is invalid, whatever it holds.*/
   return -1;
}

/**
 * hash_remove - removed an element from its hash entry.
 * @met: <> memory element table thingy.
 * @elm: < element index to remove from whatever hash chain it is in.
 *
 * Evil shall strip the lands if you change the bid before calling this.
 */
void hash_remove(MEtable_t *met, u32 elm)
{
   u32 tmpP, tmpN, key;

   tmpP = met->elements[elm].Hprev;
   tmpN = met->elements[elm].Hnext;
   if( tmpP == GROUNDED && tmpN == GROUNDED ) {
      key = get_hash_key( met->elements[elm].bid );
      key %= met->hashsize;
      if( met->hashed[key] == elm )
         met->hashed[key] = GROUNDED;
      return;
   }

   if( tmpP != GROUNDED )
      met->elements[tmpP].Hnext = met->elements[elm].Hnext;
   else {
      key = get_hash_key( met->elements[elm].bid );
      key %= met->hashsize;
      assert( key < met->hashsize );
      assert( met->hashed[key] == elm );
      met->hashed[key] = met->elements[elm].Hnext;
   }
   if(tmpN != GROUNDED )
      met->elements[tmpN].Hprev = met->elements[elm].Hprev;
   /* no tail pointer for hash, so no else here */

   /* clear out the Hash pointers so this element is completely freed */
   met->elements[elm].Hnext = GROUNDED;
   met->elements[elm].Hprev = GROUNDED;

}

/**
 * hash_add - Add element to the hash table
 * @met: <>
 * @elm: < Element index we want to add.
 *
 * This usues the bid that is currently in the index element.
 * standard use is to hash_remove, set new bid, hash_add.
 *
 * We assume that both Hash links in the element are GROUNDED.  (This
 * element is not in any hash.)
 */
void hash_add(MEtable_t *met, u32 elm)
{
   u32 key,cur;

   key = get_hash_key( met->elements[elm].bid );
   key %= met->hashsize;

   cur = met->hashed[key];

   /* for debug only */
   assert( met->elements[elm].Hprev == GROUNDED );
   assert( met->elements[elm].Hnext == GROUNDED );

   if( cur != GROUNDED )
      met->elements[cur].Hprev = elm;
   met->elements[elm].Hnext = cur;
   met->elements[elm].Hprev = GROUNDED;
   met->hashed[key] = elm;
}

/**
 * map_new_bid - creates a bid to pbn mapping
 * @met: <>
 * @bid: < Buffer ID
 * @pbn: > The sparkly new Physical Buffer Number
 *
 * This maps a bid to a pbn.
 * This action requires us to find an unused element, update its bid, and
 * get it into the correct hash location. 
 *  
 * Returns: Errorvalue
 */
int map_new_bid(MEtable_t *met, u8 bid[BIDSIZE], u64 *pbn)
{
   u32 new,sane=0;
   int found=FALSE;

   /* start scan from MRU tail. Need an unused.
    * This search should not normally be long. As unsed elements will
    * filter them selves to the bottom of the MRU. The only reason we have
    * a search is because someone could use a element, then not touch it
    * for days. Thus it will fall to the top of the LRU.
    * NOTE the implied assumption that elements that are marked inuse get
    * touched regularly.*/
   for(new = met->MRUtail; new != GROUNDED;new=met->elements[new].MRUp,sane++) {
      if( ! met->elements[new].InUse ) {found=TRUE;break;}
      assert( sane <= met->Relemcnt ); /* don't look at more than possible.*/
   }
   if(found != TRUE) return -1; /*Crap, table is full. */

   assert( met->elements[new].InUse == 0 ); /* this is dumb */

   /* remove it from any possible old hash table entery. */
   hash_remove(met, new);

   /* init it */
   bidcpy( met->elements[new].bid, bid );

   /* ok, get it onto the hash. */
   hash_add(met, new);

   *pbn = new;
   return 0;
}

/****************************************************************************/
/**
 * Melreset - Call this when the size of things change
 * @met: <> 
 *
 * whe!!!! I like this new way of doing things.  All a reset does now is
 * mark everything unused. Since we only modify the MRU and hash on
 * load/store, we can ignore the pointers!
 */
void Melreset( MEtable_t *met )
{
   u32 i;

   /* Set everyone to unsed. */
   for(i=0; i < met->Relemcnt; i++) {
      met->elements[i].InUse = 0;
   }
   met->used = 0; /* nothing used. */

}

/**
 * MelVerify - Verify journal state for pbn.
 * @met: <
 * @b: <>
 * @blen: <
 * 
 * scans the journal. Finds a entry with matching pbn and IP.  If that
 * entry is JS_Clear, return Mel_Ok.
 * If entry is Dirty, return Mel_Verify_Failed    (Mark it clear here?)
 * If pbn or IP not matched, return Mel_Err_BadPbn
 *
 *
 * Returns: <=0: Error, >0: Bytes to send
 */
int MelVerify(MEtable_t *met, u8 *b, size_t blen, u32 IP, int sk)
{
   me_hdr_t *hdr = (me_hdr_t*)b;
   errorcode_t *ec = (errorcode_t*)(b + sizeof(me_hdr_t));
#if 0
   u64 *P = (u64*)(b + sizeof(me_hdr_t));
   u32 pbn;
#endif
   u32 dipbn;
   off_t offset;
   int err=0, clid;

#if 0
   printf("Beginning Verify search on %u:\n", pbn);
#endif

#if 0
   pbn = osi_be64_to_cpu( *P );
#endif
   clid = map_IP(met, IP);
   offset = HEADER_SIZE + /* skip headers */
      (CT_BLOCKS(met->maxclients) * BLK_size) + /* skip Client Table */
      CH_OFFSET(clid, met->datasize); /* jump to the element. */

   if(lseek(met->devnode, offset, SEEK_SET) <0)
      die("Failed to lseek: %m");
   if(read(met->devnode, &dipbn, sizeof(u32)) <0)
      die("Failed to read at %"FMT_64"u: %m\n", offset);

   if( dipbn == 0 ) {
      /* Client Hold was cleared. */
      hdr->Opcode = Mel_Error;
      hdr->length = cpu_to_be16(sizeof(errorcode_t));
      *ec = Mel_Verify_Failed;
      met->cnts.errors++;
      return send(sk, b, (sizeof(me_hdr_t) + sizeof(errorcode_t)), 0);
   } else {
      /* CHold has data. so it either got written by the Store or by the
       * reload. Either way it is in the Element store.
       */
      hdr->Opcode = Mel_Ok;
      hdr->length = 0;
      err = send(sk, b, sizeof(me_hdr_t), 0);
      clear_CHold(met, clid);
      return err;
   }

}

/**
 * Melload - The Memory Export Load command handler
 * @met: <>
 * @b: <> IO buffer, holds our input and we replace it with our output.
 * @blen: < How space there is in b. (expect core if less than sizeof(error)
 *
 * This handles a load request.  (we assume some wrapper function made sure
 * thats whats going on here.)
 *
 * Returns: Errorvalue or zero.
 */
int Melload(MEtable_t *met, u8 *b, size_t blen)
{
   u64 pbn=0;
   me_hdr_t *hdr = (me_hdr_t*)b;
   errorcode_t *ec = (errorcode_t*)(b + sizeof(me_hdr_t));
   me_cdb_t *cdb = (me_cdb_t*)(b + sizeof(me_hdr_t));
   me_rdf_t *rdf = (me_rdf_t*)(b + sizeof(me_hdr_t));
   char *data = b + sizeof(me_hdr_t) + sizeof(me_rdf_t);
   int copysize;

   /* We need atleast copysize waiting for us....*/
   copysize = sizeof(me_rdf_t) + met->datasize;
   if( be16_to_cpu(cdb->allocLen) != copysize || blen < copysize ) {
      hdr->Opcode = Mel_Error;
      hdr->length = cpu_to_be16(sizeof(errorcode_t));
      *ec = Mel_Err_BadSize;

      met->cnts.errors++;
      return (sizeof(me_hdr_t) + sizeof(errorcode_t));
   }


   /* Find the pbn for the bid.  Alloc one if needed. */
   if( lookup_bid( met, cdb->bid, &pbn) <0) {
      /* Not in Hash. */
      if( map_new_bid( met, cdb->bid, &pbn) <0) {
         /* Crapola. */
         log_alrt("\n****Crapola****\n"
                " %u of %u. if those aren't equal.....\n\n", 
               met->used, met->Velemcnt);
         hdr->Opcode = Mel_Error;
         hdr->length = cpu_to_be16(sizeof(errorcode_t));
         *ec = Mel_Pbn_Space_Full;
         met->cnts.errors++;
         return (sizeof(me_hdr_t) + sizeof(errorcode_t));
      } else {
#ifdef DEBUG
         /* grab some ``random'' data to make this nonzero. */
         memcpy( &met->datafield[ pbn * met->datasize ], b, met->datasize );
#endif

         /* Do I REALLY need this here? */
         if( USEDISKSTORE(met) ) /* on map new, write out bid change */
            write_elem(met, pbn);
      }
   }
   touch_element(met, pbn); /* update element */
   /* rdf never used above here, cdb never used below. */

   rdf->InUse = met->elements[pbn].InUse;
   rdf->seq = cpu_to_be64(met->elements[pbn].seq);
   rdf->pbn = cpu_to_be64(pbn);
   rdf->fullness = fullness(met);

   memcpy( data,
         &met->datafield[ pbn * met->datasize ],
         met->datasize );

   send_info_packet(met, pbn, cdb->bid);
#ifdef DEEP_DEBUG
   check_balance(met, "endload");
#endif

   hdr->Opcode = Mel_Ok;
   hdr->length = cpu_to_be16(copysize);
   return (sizeof(me_hdr_t) + copysize);
}

/**
 * Melstore - The Memory Export Store command handler
 * @met: <>
 * @b: <> IO buffer, holds our input and we replace it with our output.
 * @blen: < How space there is in b. (expect core if less than sizeof(error)
 *
 * This is a bit of a trick question.  The rdf field is used for both
 * incomming data and outgoing data.
 *
 * Returns: Errorvalue or amount to transfer.
 */
int Melstore(MEtable_t *met, u8 *b, size_t blen, u32 IP, int sk)
{
   u64 pbn=0;
   me_hdr_t *hdr = (me_hdr_t*)b;
   errorcode_t *ec = (errorcode_t*)(b + sizeof(me_hdr_t));
   me_cdb_t *cdb = (me_cdb_t*)(b + sizeof(me_hdr_t));
   me_rdf_t *rdf = (me_rdf_t*)(b + sizeof(me_hdr_t) + sizeof(me_cdb_t));
   u8 *data = b + sizeof(me_hdr_t) + sizeof(me_cdb_t) + sizeof(me_rdf_t);
   long copysize;
   int err=0;

   DDBprintf(" Entered melstore\n");
   /* We need atleast copysize waiting for us....*/
   copysize = sizeof(me_rdf_t) + met->datasize;
   if( be16_to_cpu(cdb->allocLen) < copysize || blen < copysize ) {
      /* not quite right, but good enough for now.
       * sepecial case: If allocLen can be == to sizeof(me_rdf_t) IF inuse
       * bit == 0.
       */
      hdr->Opcode = Mel_Error;
      hdr->length = cpu_to_be16(sizeof(errorcode_t));
      *ec = Mel_Err_BadSize;

      met->cnts.errors++;
      err = send(sk, b, sizeof(me_hdr_t) + sizeof(errorcode_t), 0);
      return err;
   }

   if( lookup_bid( met, cdb->bid, &pbn) <0) {
      /* not in hash */
      /*  Nowhere to be found.... */
      hdr->Opcode = Mel_Error;
      hdr->length = cpu_to_be16(sizeof(errorcode_t));
      *ec = Mel_Err_NoBid;

      met->cnts.errors++;
      err = send(sk, b, sizeof(me_hdr_t) + sizeof(errorcode_t), 0);
      return err;
   }
   touch_element(met, pbn); /* update element */

   if( be64_to_cpu(rdf->pbn) != pbn ) {
      hdr->Opcode = Mel_Error;
      hdr->length = cpu_to_be16(sizeof(errorcode_t));
      *ec = Mel_Err_BadPbn;

      met->cnts.errors++;
      err = send(sk, b, sizeof(me_hdr_t) + sizeof(errorcode_t), 0);
      return err;
   }
   if( be64_to_cpu(rdf->seq) != met->elements[pbn].seq ) {
      hdr->Opcode = Mel_Error;
      hdr->length = cpu_to_be16(sizeof(errorcode_t));
      *ec = Mel_Err_BadSeq;

      met->cnts.errors++;
      err = send(sk, b, sizeof(me_hdr_t) + sizeof(errorcode_t), 0);
      return err;
   }

   if( ! rdf->InUse ) { /* dealloc buffer. */
      if(met->elements[pbn].InUse) {
         met->elements[pbn].InUse = 0;
         met->used--;
#if 0
         printf("Freed buffer pbn%"FMT_64"u ", pbn);
         printbid(cdb->bid);
         printf("\n");
#endif
      }
      /* don't bother copying in the buffer. Good chance it isn't even
       * there.*/
   } else {
      /* its in use, so copy in the data. */

      memcpy( &met->datafield[ pbn * met->datasize ],
              data,
              met->datasize );

      DDBprintf("  data stored\n");
      if( ! met->elements[pbn].InUse) {
         met->elements[pbn].InUse = 1;
         met->used++;
      }
   }

   /* ok, inc sequence so others will need to reload. */
   met->elements[pbn].seq++;

   if( USEDISKSTORE(met) ) { /* changed data, write changes */
      write_CHold(met, pbn, map_IP(met, IP));
      write_elem(met, pbn);
   }

   /* Hmmm, I guess All I really need as a reply to a store is a single
    * byte saying that it worked.  Kinda makes sense if I stop to think.
    */

   send_info_packet(met, pbn, cdb->bid);
#ifdef DEEP_DEBUG
   check_balance(met, "endstore");
#endif
   hdr->Opcode = Mel_Ok;
   hdr->length = 0;
   met->cnts.sucstr++;
   err = send(sk, b, sizeof(me_hdr_t), 0);
   if( USEDISKSTORE(met) ) clear_CHold(met, map_IP(met, IP));
   return err;
}

#if 0
/**
 * MelDumpNextBuffer - returns the next valid melbuf from pbn
 * @met: <
 * @pbn: <> The Physical buffer number of the buffer we want.
 * @bf: > The returned buffer
 * @len: < The size field we can return in.
 *
 * This function returns the next valid buffer starting at pbn.  If all is
 * set it will return the pbn requested.  Otherwise it will advance until
 * it finds a buffer with the InUse bit set. (which could be the pbn you
 * passed in.)
 *
 * Returns: An error code.
 */
int MelDumpNextBuffer(MEtable_t *met, u64 *pbn, 
      dump_element_t *bf, size_t len)
{
   long copysize;
   char *jnk;
   u64 ppp = *pbn;

   DDBprintf("Entering MelDumpNextBuffer pbn=%"FMT_64"u\n", *pbn);

   copysize = sizeof(dump_element_t) + met->datasize;
   if( len < copysize ) return -1;

   /* valid pbn? */
   if( ppp >= met->Velemcnt ) return -1;


   /* might have bugs here. */
   for(; ppp < met->Velemcnt && ( ! met->elements[ppp].InUse); ppp++);

   /* ok, pbn is now the buffer we want */
   bf->InUse = met->elements[ppp].InUse;
   bf->pbn = cpu_to_be64(ppp);
   bf->seq = cpu_to_be64(met->elements[ppp].seq);
   bidcpy(bf->bid, met->elements[ppp].bid);

   jnk = (char*)bf;
   memcpy( (jnk + sizeof(dump_element_t)),
           &met->datafield[ ppp * met->datasize ],
           met->datasize );

   *pbn = ppp;
   return 0;
}

/**
 * MelCntToDump - How many buffers can we dump?
 * @met: <
 * @pbn: < The Physical Buffer Number to start at.
 * @max: < Stop at this value.
 *
 * Max exists so that we don't itterate over a couple thousand possiblities
 * when the client was only interested in getting the next two...
 *
 * Returns: Number of buffers starting at pbn that can be dumped up to max.
 */
u32 MelCntToDump(MEtable_t *met, u64 pbn, u32 max)
{
   u32 count;

   if( pbn >= met->Velemcnt) return 0; /* none to dump */

   for(count =0; count < max && pbn < met->Velemcnt; pbn++ ) {
      if( met->elements[pbn].InUse ) count++;
   }

   return count;
}

/**
 * Meldump - does the dump stuff.
 * @met: <
 * @hdr: <
 * @cdb: <
 * @socket: < The socket descriptor to write the membufs that are being dumped.
 *
 * This function does the memexp dump function, since a client could be
 * asking for a large amount to be dumped, it seems rather dumb to malloc
 * up a secot just for this.  So we write out each dump_element as we get
 * it.  This won't be as fast as it could be, but since this should only
 * ever be called when we are in recovery which is ideally never, it should
 * hurt normal operation.
 * 
 * Oh just in case...
 * WARNING! While this is dumping other requests are ignored!  So a really
 * big dump request could stall someones heartbeat!
 * Returns: errors.
 */
int OLD_Meldump(MEtable_t *met, me_hdr_t *hdr, dumpb_cdb_t *cdb, int socket)
{
   u64 pbn;
   long dsize;
   u32 cnttodump, max;
   int n=1;
   char *bmess;
   dump_element_t *det;

   DDBprintf("Entering Meldump socket = %d\n",socket);

   dsize = sizeof(dump_element_t) + met->datasize;
   bmess = malloc(dsize);
   det = (dump_element_t*)bmess;

   pbn = be64_to_cpu( cdb->pbn);
   max = be32_to_cpu( cdb->want );

   if( (u32)pbn > met->Velemcnt ) {
      errorcode_t ec;

      hdr->Opcode = Mel_Error;
      ec = Mel_Err_BadPbn;

      met->cnts.errors++;
      n = send(socket, hdr, sizeof(me_hdr_t), 0);
      if( n <= 0) goto exit_meldump;
      n = send(socket, &ec, sizeof(errorcode_t), 0);
      if( n <= 0) goto exit_meldump;
      n = sizeof(me_hdr_t) + sizeof(errorcode_t);
      goto exit_meldump;
   }

   cnttodump = MelCntToDump(met, pbn, max);/*number we're gonna send*/
   DDBprintf("Going to dump %u buffers\n", cnttodump);
   
   /* ok, we're go here, so send back that header...*/
   hdr->Opcode = Mel_Ok;
   hdr->length = cpu_to_be16(8+sizeof(u32)+ (cnttodump * dsize));
   n = send(socket, hdr, sizeof(me_hdr_t), 0);
   if( n <= 0) goto exit_meldump;
   n = send(socket, "DUMPSTRT", 8, 0);
   if( n <= 0) goto exit_meldump;

   /* sothen */
   max = cpu_to_be32( cnttodump );
   n = send(socket, &max, sizeof(u32), 0);
   if( n <= 0) goto exit_meldump;

   while( cnttodump > 0) {
      n = MelDumpNextBuffer(met, &pbn, det, dsize);
      n = send(socket, bmess, dsize, 0);
      if( n <= 0) goto exit_meldump;

      pbn++;/* get the next one. */
      cnttodump--;
   }

   n = 1;
exit_meldump:
   free(bmess);
   return n;
}
#endif
/* Gonna try to build this on with just one loop instead of two. */
#define MAXDUMPSIZE 64 /* Max number of element in one dump.*/
/**
 * Meldump - does the dump stuff.
 * @met: <
 * @hdr: <
 * @cdb: <
 * @socket: < The socket descriptor to write the membufs that are being dumped.
 * 
 * Whee! a new method for dump.  This time there's only one loop.  This
 * forces a wee bit of memeory, but we grab that all in the function head.
 * This also means we now have a BA writev call.  But it is much cleaner
 * than the old methods.
 * 
 * Returns: int
 */
int Meldump(MEtable_t *met, me_hdr_t *hdr, dumpb_cdb_t *cdb, int socket)
{
   u64 pbn;
   u32 cnt, max, send_count;
   size_t totalsize=0;
   dump_element_t dh[MAXDUMPSIZE];
   struct iovec iov[(MAXDUMPSIZE * 2) +3]; /*one for dump cdb, one for data.*/
   unsigned int iovstep=0;
   int err=0;

   pbn = be64_to_cpu( cdb->pbn);
   max = be32_to_cpu( cdb->want );

   if( (u32)pbn > met->Velemcnt ) {
      errorcode_t ec;

      hdr->Opcode = Mel_Error;
      ec = Mel_Err_BadPbn;

      met->cnts.errors++;

      iov[0].iov_base = hdr;
      iov[0].iov_len = sizeof(me_hdr_t);
      iov[1].iov_base = &ec;
      iov[1].iov_len = sizeof(errorcode_t);

      err = writev(socket, iov, 2 );
      goto exit_meldump;
   }

   /* setup first couple of static bufs. */
   iov[iovstep].iov_base = hdr;
   iov[iovstep].iov_len = sizeof(me_hdr_t);
   iov[++iovstep].iov_base = "DUMPSTRT";
   iov[iovstep].iov_len = 8;
   iov[++iovstep].iov_base = &send_count;
   iov[iovstep++].iov_len = sizeof(u32);

   totalsize = 8 + sizeof(u32); /* header size is not included. */

   /* build up data structures. */
   for(cnt=0; cnt < max && cnt < MAXDUMPSIZE ;cnt++, pbn++) {
      /* find next item to dump. */
      for(;pbn <= met->Velemcnt && ( ! met->elements[pbn].InUse); pbn++);
      if( pbn >= met->Velemcnt ) break; /* end of list. */

      /* pack dump cdb. */
      dh[cnt].InUse = met->elements[pbn].InUse;
      dh[cnt].pbn = cpu_to_be64(pbn);
      dh[cnt].seq = cpu_to_be64(met->elements[pbn].seq);
      bidcpy(dh[cnt].bid, met->elements[pbn].bid);

      /* pack iov. */
      iov[iovstep].iov_base = &dh[cnt];
      iov[iovstep].iov_len = sizeof(dump_element_t);
      iov[++iovstep].iov_base = &met->datafield[ pbn * met->datasize ];
      iov[iovstep++].iov_len = met->datasize;

      totalsize += sizeof(dump_element_t) + met->datasize;
   }

   hdr->Opcode = Mel_Ok;
   hdr->length = cpu_to_be16(totalsize);
   send_count = cpu_to_be32(cnt);

   /* send it. */
   err = writev(socket, iov, iovstep );

exit_meldump:
   return err;
}

/* vim:ai cin et sw=3 ts=3
 */
