/******************************************************************************
 *
 *    Copyright 2000-2001 Sistina Software, Inc.
 *
 *    This is free software released under the GNU General Public License.
 *    There is no warranty for this software.  See the file COPYING for
 *    details.
 *
 *    See the file CONTRIBUTORS for a list of contributors.
 *
 *    This file is maintained by:
 *      Mike C Tilstra <conrad@sistina.com>
 */
#ifndef __memexp_h__
#define __memexp_h__

#include "mxp_defines.h"

#include "wiredtypes.h" /* Needs that above firt. */

/* Since everything is in static memory. no mallocs or frees durring use,
 * only at start.  The `pointers' to the raw elements in the element store
 * is just the index, or the pbn.  This system will be limited to 32 bits
 * worth for the pbn, even though the transport allows 64 bits worth.
 *
 * What you may think of as a NULL pointer is now GROUNED.  Which is all
 * bits on. which you'll never be able to access. (since zero based, you
 * cannot goto elements[~0L] ).
 * 
 * well, ok. I ended up doing some mallocs for little transfer buffer
 * stuff. But mostly there are no large memory allocation once it is up and
 * running.
 */

#define BIDSIZE 10
typedef struct MEelement_s {
   u32  Hnext; /* this `pointer' is the index of the next hash element */
   u32  Hprev; /* this `pointer' is the index of the prev hash element */
   u32  MRUn; /* this `pointer' is the index of the next MRU element */
   u32  MRUp; /* this `pointer' is the index of the prev MRU element */
   u64  seq;
   u8   bid[BIDSIZE];
   u8   InUse;
} MEelement_t;

/* because I don't want to include disktypes in here. But need this struct
 * anyways.
 */
typedef struct MEtable_s {
   /* These are the raw storage of data. */
   MEelement_t *elements;
   char *datafield;
   u32 Relemcnt; /* how big is the elem array */
   u32 Velemcnt; /* how many the modepage reports. */
   unsigned long datatotalsize;
   u32 datasize; /* the size of each data segment. */

   /* Here are the logical access methods. */
   u32 *hashed; /* hash table access, keys by BID */
   unsigned int hashsize; /* maximum hash key value and number of buckets */

   u32 MRUhead; /* aka LRU Tail */
   u32 MRUtail; /* aka LRU Head */

   u32 used; /* number of elements in use. */

   int devnode; /* The static store we're using.  -1 means we're not using it */

   u32 maxclients; /* maximum number of clients that can be connected at once*/
   u32 numclients; /* current number of connected clients. */

	u32 *IPs; /* This is the IP to index mapping. In BigE!!*/
	u8  *bans; /* Using that mapping, which of these are banned. */

   counters_t cnts;  /* various bit of info */
   int TAsock;
} MEtable_t;

#define USEDISKSTORE(met) (((met)->devnode <= 0)?FALSE:TRUE)

#if 1
#define GROUNDED ((u32)~0UL)
#else
#define GROUNDED ((u32)4294967295UL)
#endif

/* prototypes */

int create_MTab( MEtable_t *met, unsigned long totalsize,
      unsigned int eldsze, unsigned int maxcli);
void create_MelDisk( char *name,
      unsigned long sze, unsigned int eldsze, unsigned int maxcli);
int load_MelDisk(char *name, MEtable_t *met);
void Melreset(MEtable_t *met);
int Melload(MEtable_t *met, u8 *b, size_t blen);
int Melstore(MEtable_t *met, u8 *b, size_t blen, u32 IP, int sk);
int Meldump(MEtable_t *met, me_hdr_t *hdr, dumpb_cdb_t *cdb, int socket);
int MelVerify(MEtable_t *met, u8 *b, size_t blen, u32 IP, int sk);

int dump_hash_density(MEtable_t *met, hshcnt_t *srvs);

void set_ban(MEtable_t *met, u32 ip);
void drop_ban(MEtable_t *met, u32 ip);
int test_ban(MEtable_t *met, u32 IP);

int map_IP(MEtable_t *met, u32 ip);
int place_IP(MEtable_t *met, u32 ip);

#endif /*__memexp_h__*/
