/******************************************************************************
 *
 *    Copyright 2000-2001 Sistina Software, Inc.
 *
 *    This is free software released under the GNU General Public License.
 *    There is no warranty for this software.  See the file COPYING for
 *    details.
 *
 *    See the file CONTRIBUTORS for a list of contributors.
 *
 *    This file is maintained by:
 *      Mike C Tilstra <conrad@sistina.com>
 *
 * All sorts of configuration like options.  Most of these you should never
 * have to even think of touching.
 */
#ifndef __MEMEXP_configure_h__
#define __MEMEXP_configure_h__

/* Default port to listen on.
 */
#define DEF_PORT (15697)

/* Default size for each element buffer.  This value can be overridden with
 * cmdline options.
 */
#define DEF_ELSIZE (32)

/* Default Max number of clients. Again overridable on the cmdline
 */
#define DEF_CLIENTS (32)

/*Default amount of RAM to pre-allocate
 */
#define DEF_MEMSIZE (32)

/* Default amount of diskspace to use.
 * This works out to be roughly 32Mb of RAM when the server runs.
 */
#define DEF_DISKSIZE (27)  /* makes about 32Mb memory use */

/* If defined, the syslog is used for logging messages.
 * Otherwise info is dumped to stdout.
 */
#define USE_SYSLOG

/* Defining this does a few simple things to attempt to improve the
 * performace of memexpd by taking advantage of soft real time features.
 * Specifically, we set the scheduling algorithm to FIFO, and lock most of
 * the memory.  If you turn this on, memexpd *HAS* to be run as root.  This
 * could make you valerable to security attacks.
 *
 * Also keep in mind that you will be locking what ever size memexpd is set
 * to in RAM.  So on defaults, you've locked 32Mb of memory.  That is a
 * lot....
 */
#undef SOFT_RT

/* Set only one of these to pick which hashing function is used to sort the
 * bids into the hash table.  Each has various streigths and weaknesses.
 * And those change depending on the size of the table as well as the data
 * presented.
 *
 * Keep in mind that the results from any of these will by modulo-ed by the
 * size of the hashtable.  (which can degrade the performacne of some of
 * the hash functions.)
 *
 */
#undef bob_HASH    /* http://burtleburtle.net/bob/hash/evahash.html */
#undef fnv_HASH    /* http://www.isthe.com/chongo/tech/comp/fnv */
#undef crc16_HASH
#define crc32_HASH

/* Some hash functions let you provide a starting value.  Set this to a
 * 32bit value for those that use it.  Hash function can preform better
 * with good start values.
 *
 * Hash function that accept a start value: bob_HASH fnv_HASH
 * Some values I've used: 0x0 0x66063bca 0x811c9dc5
 */
#define HASH_START (0x66063bca)

/* This is for the cute progress bars that initds can show off its
 * progress.  Defining this lets it adjust the size of the progress bars
 * based on the width of your terminals.
 */
#define GUESS_WINSIZE

/* Defining this turns on checks to make sure that the MRU and Hash chains
 * are not broken.  This means lots and lots of loops for each lock op.
 * This code probably isn't needed any more.
 */
#undef DEEP_DEBUG

/* LDUR <=> Lock Op Durration
 * Bunch of stat gathering code to determin how long a lock op takes.
 */
#undef LDUR_CODE

/* LPS <=> Lock Per Second.
 * Just a bunch of timing code that tries to figure the speed at which the
 * server can handle traffic.  Use the userclient to get the stats out.
 *
 * You should never need this.
 */
#undef LPS_CODE

#endif /*__MEMEXP_configure_h__*/
