/******************************************************************************
*******************************************************************************
**
**    Copyright 2000-2001 Sistina Software Inc
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      Jonathan Brassow <brassow@sistina.com>
** 
**    Module name: debug.h
**
**    Description:
**      Header for pool driver debugging code.
**
*******************************************************************************
******************************************************************************/

#ifndef __DEBUG_H__
#define __DEBUG_H__

typedef struct pool_debug_pairs_s{
  uint flag;
  char *name;
} pool_debug_pairs_t;

/* So what's with the number/string pairs ? 
 * 'all' will obviously turn on all debugging flags,
 * as for the others, if you specify a group,
 * locking/mapping/pool, it will turn on all debugging
 * for that group.  However, if you should choose {group}_1
 * it will only print out the enter/exit debugging statements
 * for the functions in that group.  If you specify {group}_2
 * you will get all misc debugging output contained in the
 * functions in that group sans the enter/exit statements
 * "your are understand !!!"
 */

#define LOCKING_1 1
#define LOCKING_2 2
#define MAPPING_1 4
#define MAPPING_2 8
#define POOL_1 16
#define POOL_2 32

typedef struct pool_debug_flag_s{
  uint flags;
  uint indent;
} pool_debug_flag_t;

#ifdef DEBUG

#define ENTER_PRINT(flag, fmt, args...) {\
  int I;\
  if(flag & df_flags.flags) { \
    for(I = 0; I < df_flags.indent; I++) osi_printf("\t"); \
    osi_printf("Entering :: "); \
    osi_printf(fmt, ##args); \
    osi_printf("\n"); \
    df_flags.indent++; \
  } \
}

#define EXIT_PRINT(flag, fmt, args...) {\
  int I;\
  if(flag & df_flags.flags) { \
    df_flags.indent--; \
    for(I = 0; I < df_flags.indent; I++) osi_printf("\t"); \
    osi_printf("Exiting :: "); \
    osi_printf(fmt, ##args); \
    osi_printf("\n"); \
  } \
}
#define DEBUG_PRINT(flag, fmt, args...) {\
  int I;\
  if(flag & df_flags.flags) { \
    for(I = 0; I < df_flags.indent; I++) osi_printf("\t"); \
    osi_printf("RAW DEBUG :: *********>\n"); \
    osi_printf(fmt, ##args); \
    osi_printf("\n"); \
    for(I = 0; I < df_flags.indent; I++) osi_printf("\t"); \
    osi_printf("<********* :: RAW DEBUG\n"); \
  } \
}

#else /* DEBUG not defined */

#define ENTER_PRINT(flag, fmt, args...) {}
#define EXIT_PRINT(flag, fmt, args...) {}
#define DEBUG_PRINT(flag, fmt, args...) {}

#endif /* DEBUG */

void turn_on_pool_debug_flag(char *);
void turn_off_pool_debug_flag(char *);

#endif /* __DEBUG_H__ */





