/******************************************************************************
*******************************************************************************
**
**    Copyright 2000-2001 Sistina Software Inc
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      Jonathan Brassow <brassow@sistina.com>
** 
**    Module name: pool.h
**
**    Description:
**	Pool driver primary header file.  THIS FILE IS NOT TO BE USED
**	BY ANY CODE OTHER THAN POOL CODE.  (It won't kill you, but it
**	just wouldn't be good.)
**
*******************************************************************************
******************************************************************************/

#ifndef __POOL_H__
#define __POOL_H__


#ifdef __KERNEL__
#include "osi.h"
#else
#include "pool_user.h"
#endif  /*  __KERNEL__  */

#include "release.cf"
#include "NSP.cf"
#include "global.h"

#include "pool_ondisk.h"
#include "pool_incore.h"
#include "abstraction.h"
#include "pool_ioctl.h"
#include "common.h"
#include "debug.h"

#define POOL_MAJOR              (121)

#define POOL_MAX_DEVICES 	32

#define POOL_MAGIC 		0x011670

#define POOL_VERSION		(NSPMajorVersion << 16 | \
				 NSPMinorVersion <<  8 | \
				 NSPUpdateLevel)
#define BBSHIFT 		9

#define POOL_BUSY(pp) 		((pp)->p_oblk > 0)


#ifndef ASSERT
#define ASSERT(x)  \
  if (!(x)) {   \
    osi_printf("ASSERTION FAILED: %s\n", #x); \
  }
#endif /* ASSERT */



/* Pool flags.   These are used in the status field of pool_t structures.
 *
 * PF_INIT:	 Temp state while pool is being added/generated.
 * PF_LOWEROPEN: In-use pool's lower devices are open. Normal state.
 * PF_VALID: 	 State when pool is not being initialized.  It's cleared to
 *		 prevent two adds from interfearing. Complement of PF_INIT.
 * PF_FREE: 	 Pool is not in use and free to be assigned.
 *
 * Unused pools have the state (PF_VALID | PF_FREE). 	  	(flag=6)
 * In-use pools have the state (PF_VALID | PF_LOWEROPEN).	(flag=3)
 * Pools being initialized are (PF_INIT).			(flag=0)
 */

#define	PF_INIT			0x00000000
#define PF_LOWEROPEN		0x00000001
#define PF_VALID		0x00000002
#define	PF_FREE			0x00000004

#endif /* __POOL_H__ */







