/******************************************************************************
*******************************************************************************
**
**    Copyright 2000-2001 Sistina Software Inc
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      Jonathan Brassow <brassow@sistina.com>
** 
**    Module name: pool_ondisk.h
**
**    Description:
**      This header file contains everything corresponding to ondisk
**	structures.
**
*******************************************************************************
******************************************************************************/

#ifndef __POOL_ONDISK_H__
#define __POOL_ONDISK_H__

#define POOL_NAME_SIZE          (256)

/* Pool label is at the head of every pool disk partition */

#define SIZEOF_POOL_LABEL       (512)

typedef struct pool_label_s
{
  uint64        pl_magic;       /* Pool magic number */
  uint64        pl_pool_id;     /* Unique pool identifier */
  char          pl_pool_name[POOL_NAME_SIZE];   /* Name of pool */
  uint32        pl_version;     /* Pool version */
  uint32        pl_subpools;    /* Number of subpools in this pool */
  uint32        pl_sp_id;       /* Subpool number within pool */
  uint32        pl_sp_devs;     /* Number of data partitions in this subpool */
  uint32        pl_sp_devid;    /* Partition number within subpool */
  uint32        pl_sp_type;     /* Partition type */
  uint64        pl_blocks;      /* Number of blocks in this partition */
  uint32        pl_striping;    /* Striping size within subpool */
  /* If the number of DMEP devices is zero, then the next field **
  ** (pl_sp_dmepid) becomes the subpool ID for redirection.  In **
  ** other words, if this subpool does not have the capability  **
  ** to do DMEP, then it must specify which subpool will do it  **
  ** in it's place                                              */
  uint32        pl_sp_dmepdevs; /* Number of dmep devices in this subpool */
  uint32        pl_sp_dmepid;   /* Dmep device number within subpool */
  uint32        pl_sp_weight;	/* if dmep dev, pref to using it */
} pool_label_t;

void pool_label_out(pool_label_t *pl, char *buf);
void pool_label_in(pool_label_t *pl, char *buf);

#endif /* __POOL_ONDISK_H__ */
