/******************************************************************************
*******************************************************************************
**
**    Copyright 2000-2001 Sistina Software Inc
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      Jonathan Brassow <brassow@sistina.com>
** 
**    Module name: rw_sema.c
**
**    Description:
**	General, non-OS specific pool driver routines.  
**
*******************************************************************************
******************************************************************************/
#include "rw_sema.h"

void pool_rw_sema_init(pool_rw_sema_t *lck){
  osi_memset(lck, 0, sizeof(pool_rw_sema_t));
  osi_spinlock_init(&(lck->prws_spin_lock));
  osi_sema_init_lkd(&(lck->prws_sema_lock));
}

void pool_rw_sema_read_lock(pool_rw_sema_t *lck){
  osi_spin_lock(&(lck->prws_spin_lock));

  if(lck->prws_readers){
    lck->prws_readers++;
    if(lck->prws_waiting){
      if(lck->prws_writers){
	lck->prws_waiting++;
	osi_spin_unlock(&(lck->prws_spin_lock));
	osi_sema_lock(&(lck->prws_sema_lock));
      } else {
	do {
	  lck->prws_waiting--;
	  osi_sema_unlock(&(lck->prws_sema_lock));
	} while(lck->prws_waiting);
	osi_spin_unlock(&(lck->prws_spin_lock));
      }
    } else {
      osi_spin_unlock(&(lck->prws_spin_lock));
    }
  } else {
    lck->prws_readers++;
    if(lck->prws_writers){
      lck->prws_waiting++;
      osi_spin_unlock(&(lck->prws_spin_lock));
      osi_sema_lock(&(lck->prws_sema_lock));
    } else {
      osi_spin_unlock(&(lck->prws_spin_lock));
    }
  }      
}


void pool_rw_sema_write_lock(pool_rw_sema_t *lck){
  osi_spin_lock(&(lck->prws_spin_lock));

  if(lck->prws_readers || lck->prws_writers){
    lck->prws_writers++;
    lck->prws_waiting++;
    osi_spin_unlock(&(lck->prws_spin_lock));
    osi_sema_lock(&(lck->prws_sema_lock));
  } else {
    lck->prws_writers++;
    osi_spin_unlock(&(lck->prws_spin_lock));
  }    
}


void pool_rw_sema_read_unlock(pool_rw_sema_t *lck){
  osi_spin_lock(&(lck->prws_spin_lock));

  lck->prws_readers--;
  if(lck->prws_waiting){
    if((lck->prws_readers+lck->prws_writers) == lck->prws_waiting){
      lck->prws_waiting--;
      osi_sema_unlock(&(lck->prws_sema_lock));
      osi_spin_unlock(&(lck->prws_spin_lock));
    }
  }
  osi_spin_unlock(&(lck->prws_spin_lock));
}


void pool_rw_sema_write_unlock(pool_rw_sema_t *lck){
  osi_spin_lock(&(lck->prws_spin_lock));

  lck->prws_writers--;
  
  if(lck->prws_waiting){
    lck->prws_waiting--;
    osi_sema_unlock(&(lck->prws_sema_lock));
  }
  osi_spin_unlock(&(lck->prws_spin_lock));    
}
