/******************************************************************************
*******************************************************************************
**
**    Copyright 2000-2001 Sistina Software, Inc.
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      David C. Teigland <teigland@sistina.com>
**
**    Module name:
**      main.c
**
**    Description:  This program uses "gclient".
**
*******************************************************************************
******************************************************************************/

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <errno.h>
#include <ctype.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/wait.h>


char *pname = "do_gclient_stm";
char args[256];
char *ipaddr = NULL;


void print_usage(void)
{
  printf("Usage:\n");

  printf("do_gclient_stm -s <ipaddr>\n\n"
	 "    -s   IP address of machine to stomith\n\n");
}


void get_options(int argc, char **argv)
{
  int c;
  char *last, *tok;
  
  if (argc > 1)
  {        
    while ((c = getopt(argc, argv, "hs:")) != -1)
    {
      switch(c)
      {
      case 'h':
	print_usage();
	exit(0);
	
      case 's':
	ipaddr = optarg;
	break;
      }
    }
  }

  else
  {
    memset(args, 0, 256);

    if (read(0, args, 256) < 0)
    {
      printf("STOMITH:failed:%s no input\n", pname);
      exit(0);
    }
    *(args + strlen(args) - 1) = '\0';

    for (last = "", tok = strtok(args, " "); tok; tok = strtok(NULL, " "))
    {
      if (!strncmp(last, "-s", 2))
        ipaddr = tok;

      else
      {
        if (!strncmp(tok, "-h", 2))
        {
          print_usage();
          exit(0);
        }
        
        last = tok;
        continue;
      }

      last = "";
    }
  }
}



int main(int argc, char **argv)
{
  int error, status, val;
  char line[256];
  int fds[2];


  get_options(argc, argv);
  

  if (pipe(fds))
  {
    printf("STOMITH:failed:%s pipe error\n", pname);
    exit(0);
  }
  
      
  if (!fork())
  {
    close(1);
    dup(fds[1]);
    close(fds[0]);
    close(fds[1]);
    close(2);
    dup(1);

    error = execlp("gclient", "gclient", "-s", ipaddr, NULL);
    
    exit(0);
  }
    
  else
  {
    close(0);
    dup(fds[0]);
    close(fds[0]);
    close(fds[1]);

    memset(line, 0, 256);

    /* set fd to nonblocking in case gclient dies with no output */
    if ((val = fcntl(0, F_GETFL, 0)) < 0)
      goto fail;

    val |= O_NONBLOCK;
    
    if (fcntl(0, F_SETFL, val) < 0)
      goto fail;

    wait(&status);
    
    if (read(0, line, 256) < 0)
    {
      printf("STOMITH:failed:%s read error\n", pname);
      exit(0);
    }

  fail:
    
    if (strstr(line, "stomithed"))
      printf("STOMITH:success:%s - %s\n", pname, ipaddr);
    else
      printf("STOMITH:failed:%s - %s\n", pname, ipaddr);
  }	

  return(0);
}


