/******************************************************************************
*******************************************************************************
**
**    Copyright 2000-2001 Sistina Software, Inc.
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      David C. Teigland <teigland@sistina.com>
**
**    Module name:
**      do_meat.c
**
**    Description:  The meatware stomith agent is two pieces.  This half
**    (do_meatware) is run by a user after manually resetting the victim
**    machine.  The other half (wait_meatware) will send a success message
**    to stomithd after hearing from do_meatware.
**
*******************************************************************************
******************************************************************************/

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <errno.h>
#include <ctype.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/wait.h>


char *pname;


void print_usage(void)
{
  printf("Usage:\n");

  printf("%s -s <ipaddr>\n\n"
	 "    -s   IP address of machine which was manually stomithed\n\n", pname);
}



int main(int argc, char **argv)
{
  int error, fd;
  char line[256];
  char *ipaddr = NULL;
  char response[3];
  int c;

  pname = argv[0];
    
  while ((c = getopt(argc, argv, "hs:")) != -1)
  {
    switch(c)
    {
    case 'h':
      print_usage();
      exit(0);
	
    case 's':
      ipaddr = optarg;
      break;
    }
  }

  if (!ipaddr)
  {
    print_usage();
    exit(0);
  }


  printf("\nWarning:  If the machine with IP address \"%s\" has not been\n"
	 "manually stomithed (i.e. power cycled or disconnected from all\n"
	 "storage devices) the GFS file system may become corrupted and all\n"
	 "its data unrecoverable!  Please verify that the above IP address\n"
	 "correctly matches the machine you just rebooted (or disconnected).\n", 
	 ipaddr);

  printf("\nAre you certain you want to continue? [yN] ");
  scanf("%s", response);

  if (tolower(response[0] != 'y'))
  {
    printf("%s operation canceled.\n", pname);
    exit(0);
  }


  fd = open("/tmp/meatware.fifo", O_WRONLY | O_NONBLOCK);
  if (fd < 0)
  {
    sprintf(line, "meatware failure %s", ipaddr);
    write(fd, line, 256);
    perror("can't open /tmp/meatware.fifo");
    exit(-1);
  }

  sprintf(line, "meatware reply %s", ipaddr);

  error = write(fd, line, 256);
  if (error < 0)
  {
    perror("can't write to /tmp/meatware.fifo");
    exit(-1);
  }
    
  printf("done\n");

  return(0);
}


