/******************************************************************************
*******************************************************************************
**
**    Copyright 2000-2001 Sistina Software, Inc.
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      David C. Teigland <teigland@sistina.com>
**
**    Module name:
**      main.c
**
**    Description:  This program uses the "vash" program to power cycle a node.
**
*******************************************************************************
******************************************************************************/

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <errno.h>
#include <ctype.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/wait.h>


char *pname = "do_vacm_emp";

char args[256];
char *nexxus = NULL, *login = NULL, *passwd = NULL, *nodeid = NULL;
int test = 0;


void print_usage(void)
{
  printf("Usage:\n");

  printf("do_vacm_emp -x <nexxus> -l <login> -p <passwd> -n <nodeid> [-t]\n\n"
	 "    -x   Hostname or IP of machine on which VACM Nexxus is running\n"
	 "    -l   Login name to nexxus\n"
	 "    -p   Password for login\n"
	 "    -n   Name of node within nexxus node_list\n"
	 "    -t   Test only (no power cycle)\n\n");
}



void get_options(int argc, char **argv)
{
  int c;
  char *last, *tok;
  
  if (argc > 1)
  {    
    while ((c = getopt(argc, argv, "hx:l:p:n:t")) != -1)
    {
      switch(c)
      {
      case 'h':
	print_usage();
	exit(0);
	
      case 'x':
	nexxus = optarg;
	break;
	
      case 'l':
	login = optarg;
	break;

      case 'p':
	passwd = optarg;
	break;

      case 'n':
	nodeid = optarg;
	break;

      case 't':
	test = 1;
	break;
      }
    
    }
  }

  else
  {
    memset(args, 0, 256);

    if (read(0, args, 256) < 0)
    {
      printf("STOMITH:failed:%s no input\n", pname);
      exit(0);
    }
    *(args + strlen(args) - 1) = '\0';

    for (last = "", tok = strtok(args, " "); tok; tok = strtok(NULL, " "))
    {
      if (!strncmp(last, "-x", 2))
	nexxus = tok;

      else if (!strncmp(last, "-l", 2))
	login = tok;

      else if (!strncmp(last, "-p", 2))
	passwd = tok;

      else if (!strncmp(last, "-n", 2))
	nodeid = tok;

      else
      {
	if (!strncmp(tok, "-h", 2))
	{
	  print_usage();
	  exit(0);
	}
	
	if (!strncmp(tok, "-t", 2))
	  test = 1;

	last = tok;
	continue;
      }

      last = "";
    }
  }
}



int main(int argc, char **argv)
{
  int error, number, success, status = 0, i;
  char cmd[256], line[256], desc[256], desc2[256];
  char c;
  int fds[2];


  get_options(argc, argv);


  if (pipe(fds))
  {
    printf("STOMITH:failed:%s pipe error\n", pname);
    exit(0);
  }
  
      
  if (!fork())
  {
    close(1);
    dup(fds[1]);
    close(fds[0]);
    close(fds[1]);
    close(2);
    dup(1);

    if (test)
      sprintf(cmd, "ipc %s emp:inquiry:%s", nexxus, nodeid);
    else
      sprintf(cmd, "ipc %s emp:power_cycle:%s", nexxus, nodeid);

    /* FIXME: password appears in ps list */

    error = execlp("vash", "vash", "-c", nexxus, "-u", login, "-p", passwd, "-x", cmd, NULL);

    printf("EMP:0:VASHNOTFOUND\n");

    exit(0);
  }
    
  else
  {
    close(0);
    dup(fds[0]);
    close(fds[0]);
    close(fds[1]);

    success = 0;

    while (1)
    {
      memset(line, 0, 256);
      i = 0;

      while ((c = getchar()) != EOF)
      {
	line[i++] = c;

	if (c == '\n' || i == 256)
	  break;
      }

      if (c == EOF)
	break;

      memset(desc, 0, 256);
      sscanf(line, "EMP:%u:%[^\n]", &number, desc);

      if (!strncmp(desc, "INQUIRY", 7))
      {
	/* check values */
	memset(desc2, 0, 256);
	memcpy(desc2, desc, strlen(desc));
      }

      if (!strncmp(desc, "JOB_COMPLETED", 13))
      {
	success = 1;
	break;
      }
    }
        
    wait(&status);
    
    if (success)
      printf("STOMITH:success:%s %s %s\n", pname, desc, desc2);

    else
      printf("STOMITH:failed:%s %s %s (%s, %s, %s)\n", 
	     pname, desc, desc2, nexxus, login, nodeid);
  }

  return(0);
}


