/******************************************************************************
*******************************************************************************
**
**    Copyright 1999-2001 Sistina Software, Inc.
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      David C. Teigland <teigland@sistina.com>
** 
**    Module name: stomith_ops.c
**
*******************************************************************************
******************************************************************************/

#ifdef MODVERSIONS
#include <linux/modversions.h>
#endif  /*  MODVERSIONS  */
#include <linux/kernel.h>
#include <asm/uaccess.h>
#include <linux/smp_lock.h>
#include <linux/tty.h>
#include <linux/file.h>
#include <linux/in.h>

#include "osi.h"

#define WANT_STOMITH_NAMES
#include "stomith.h"


/* Line buffer size for messages from switch */

#define MAXLINE 64


/* Synchronize stomithd/kernel messages */

extern osi_sema_t stmd_sema;
extern osi_sema_t stmd_write_sema;
extern osi_sema_t stmd_read_sema;
extern cluster_stomith_t stmd_victim;
extern cluster_stomith_t stmd_local;
extern int stmd_result;

osi_timer_t stmd_timer;



/**
 * stmd_timer_fn - called if stmd_timer expires
 *
 * @data: unused
 *
 */

void stmd_timer_fn(unsigned long data)
{
  printk("stomith:  stomithd timed out\n");

  stmd_result = -1;
  osi_sema_unlock(&stmd_write_sema);
}



/**
 * do_stomithd - have the user mode stomith daemon do the work
 *
 * @victim: stomith descriptor
 * @local: stomith descriptor
 *
 */

int do_stomithd(cluster_stomith_t *victim, cluster_stomith_t *local)
{
  int ret;
  int use_timer = (victim->cs_magic == STM_MEATWARE) ? 0 : 1;


  osi_sema_lock(&stmd_sema);

  memcpy(&stmd_victim, victim, sizeof(cluster_stomith_t));

  if (local)
    memcpy(&stmd_local, local, sizeof(cluster_stomith_t));
  
  osi_sema_unlock(&stmd_read_sema);

  stmd_result = -2;

  if (use_timer)
  {
/*
    osi_init_timer(&stmd_timer, (osi_timer_func *)stmd_timer_fn, NULL);
    osi_set_timer(&stmd_timer, STMD_TIMEOUT);
*/
  }

  osi_sema_init_lkd(&stmd_write_sema);
  osi_sema_lock(&stmd_write_sema);

/*
  if (use_timer)
    osi_del_timer(&stmd_timer);
*/

  ret = stmd_result;

  osi_sema_unlock(&stmd_sema);

  /* printk("stomith:  returning %d\n", ret); */

  return(ret);
}



/**
 * do_stomith
 *
 * @victim: stomith descriptor
 * @local: stomith descriptor
 *
 */

int do_stomith(cluster_stomith_t *victim, cluster_stomith_t *local)
{
  int error = 0;

  /* this extra function is left over from when we did both in-kernel and 
     user mode stomithing - it's possible we may support some in-kernel stomith
     actions again some day */

  switch (victim->cs_magic)
  {
  case STM_BRCD_ZONE:
  case STM_BRCD_PORT:
  case STM_WTI_NPS:
  case STM_VACM_EMP:
  case STM_GNBD:
  case STM_MEATWARE:
  case STM_MANUAL:
  case STM_VXHUB_BYPASS:
  case STM_APC_MS:
    error = do_stomithd(victim, local);
    break;

  default:
    break;
  }  

  return(error);
}












