/******************************************************************************
*******************************************************************************
**
**    Copyright 2000-2001 Sistina Software Inc.
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**	Jonathan Brassow <brassow@sistina.com>
** 
**    Module name: do_dmep.c
**
*******************************************************************************
******************************************************************************/
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>


#include "global.h"
#include "common.h"
#include "send_pool.h"
#include "send_sg.h"


uint32 buffer_size = DEFAULT_BUFFER_SIZE;

void print_usage(void);
int human_readable_output(u_char op_code, u_char action,
			  u_char *pdf, uint pdf_len,
			  error_code_t *ec, int error);
int is_dmep_capable(char *filename, u_char seg_num);


int main(int argc, char *argv[]){
  char *filename;
  char *tmp_str;
  u_char op_code;
  u_char action;
  u_char seg_num;
  uint64 bid = 0;
  uint64 seq_num;
  uint64 pbn;
  uint64 number_of_buffers;
  int in_use;
  char *data = NULL;
  uint32 data_len = 0;
  int error = 0;
  error_code_t ec;

  u_char *cdb;
  u_char *pdf;
  uint   pdf_len;

  /* This can be enabled by specifying the opcode **
  ** with a leading 0x.  Cheezy, but I don't want **
  ** options being used here.                     */
  int raw_output = 0;

  if(argc < 5){
    print_usage();
    exit(EXIT_FAILURE);
  }

  filename = argv[1];

  tmp_str = argv[2];
  if(strlen(tmp_str) == 2){ /* hex number w/o the leading 0x */
    op_code = tmp_str[0] - 48;
    op_code *= 16;
    op_code += tmp_str[1] - 48;
  } else if (strlen(tmp_str) == 4){ /* should be hex w/ leading 0x */
    if(tmp_str[1] != 'x'){
      fprintf(stderr, "Op code must be given in hex.\n\n");
      print_usage();
      exit(EXIT_FAILURE);
    }
    raw_output=1;
    op_code = tmp_str[2] - 48;
    op_code *= 16;
    op_code += tmp_str[3] - 48;
  } else {
    fprintf(stderr, "Invalid op code.  Must be 0x85 or 0x89.\n");
    print_usage();
    exit(EXIT_FAILURE);
  }
  action = atoi(argv[3]);
  seg_num = atoi(argv[4]);

  /* test if DMEP device is available, AND set the buffer_size */
  if(!is_dmep_capable(filename, seg_num)){
    printf("Command failure.\n");
    printf("The device you specified may not be set up for DMEP.\n");
    exit(EXIT_FAILURE);
  }

  if(op_code == 0x85){
    switch(action){
    case 0:        /* Load buffer */
      if(argc == 6){
	tmp_str = argv[5];
	bid = str_to_uint64(tmp_str);
      } else {
	fprintf(stderr, "Ill formatted command.\n\n");
	print_usage();
	exit(EXIT_FAILURE);
      }
      build_dmep_command(op_code, action, seg_num, bid,
			 0, 0, 0, NULL, buffer_size,
			 &cdb, &pdf, &pdf_len);
      break;
    case 1: /* Dump buffer */
      if(argc == 6){
	tmp_str = argv[5];
	pbn = str_to_uint64(tmp_str);
      } else {
	fprintf(stderr, "Ill formatted command.\n\n");
	print_usage();
	exit(EXIT_FAILURE);
      }
      build_dmep_command(op_code, action, seg_num, 0,
			 0, pbn, 0, NULL, 0,
			 &cdb, &pdf, &pdf_len);
      break;
    case 2: /* Sense config */
      build_dmep_command(op_code, action, seg_num, 0,
			 0, 0, 0, NULL, 0,
			 &cdb, &pdf, &pdf_len);
      break;
    default:
      fprintf(stderr, "Invalid action.\n\n");
      print_usage();
      exit(EXIT_FAILURE);
      break;
    }
  } else if(op_code == 0x89){
    switch(action){
    case 0: /* Store buffer */
      if(argc == 9){
	tmp_str = argv[5];
	bid = str_to_uint64(tmp_str);
	tmp_str = argv[6];
	seq_num = str_to_uint64(tmp_str);
	tmp_str = argv[7];
	pbn = str_to_uint64(tmp_str);
	tmp_str = argv[8];
	in_use = atoi(tmp_str);
	if(in_use){
	  tmp_str = zalloc(buffer_size);
	  data_len = read(STDIN_FILENO, tmp_str, buffer_size);
	  data = zalloc(data_len);
	  memcpy(data, tmp_str, data_len);
	} else {
	  data_len = 0;
	  data = NULL;
	}
      } else if (argc == 10){
	tmp_str = argv[5];
	bid = str_to_uint64(tmp_str);
	tmp_str = argv[6];
	seq_num = str_to_uint64(tmp_str);
	tmp_str = argv[7];
	pbn = str_to_uint64(tmp_str);
	tmp_str = argv[8];
	in_use = atoi(tmp_str);
	if(in_use){
	  data = zalloc(buffer_size);
	  memcpy(data, argv[9], strlen(argv[9]));
	  memset(data+strlen(argv[9]), '\0', buffer_size-strlen(argv[9]));
	  /* attention -- set this to 0 if you want to test errors  **
	  ** related to incorrect parameter length                  */
	  if(1){
	    data_len = buffer_size;
	  } else {
	    data_len = strlen(data);
	  }
	} else {
	  data = NULL;
	  data_len = 0;
	}
      } else {
	fprintf(stderr, "Ill formatted command.\n\n");
	print_usage();
	exit(EXIT_FAILURE);
      }
      build_dmep_command(op_code, action, seg_num, bid,
			 seq_num, pbn, in_use, data, data_len,
			 &cdb, &pdf, &pdf_len);
      break;
    case 2: /* Select config */
      if(argc == 7){
	tmp_str = argv[5];
	number_of_buffers = str_to_uint64(tmp_str);
	buffer_size = atoi(argv[6]);
      } else {
	fprintf(stderr, "Ill formatted command.\n\n");
	print_usage();
	exit(EXIT_FAILURE);
      }
      build_dmep_command(op_code, action, seg_num, number_of_buffers,
			 buffer_size, 0, 0, NULL, 0,
			 &cdb, &pdf, &pdf_len);
      break;
    case 3: /* Enable segment */
      if(argc != 5){
	fprintf(stderr, "Ill formatted command.\n\n");
	print_usage();
	exit(EXIT_FAILURE);
      }
      build_dmep_command(op_code, action, seg_num, 0,
			 0, 0, 0, NULL, 0,
			 &cdb, &pdf, &pdf_len);
      break;
    default:
      fprintf(stderr, "Invalid action.\n\n");
      print_usage();
      exit(EXIT_FAILURE);
      break;
    }
  } else {
    fprintf(stderr, "Invalid op code.  Must be 0x85 or 0x89.\n");
    print_usage();
    exit(EXIT_FAILURE);
  }

  error = send_dmep_command(filename, bid, cdb, &pdf,
			    pdf_len, &ec, 0);

  if(!raw_output){
    human_readable_output(op_code, action, pdf, pdf_len, &ec, error);
    exit(EXIT_SUCCESS);
  }

  if(error < 0) {
    printf("send_dmep_command() failure.\n");
    exit(EXIT_FAILURE);
  }

  if(write(STDOUT_FILENO, pdf, pdf_len) != pdf_len){
    fprintf(stderr, "Error during write.\n\n");
    exit(EXIT_FAILURE);
  }
  exit(EXIT_SUCCESS);
}

uint64 bit64_to_uint64(u_char *str){
  uint64 num;
  num = str[0];
  num <<=8;
  num += str[1];
  num <<=8;
  num += str[2];
  num <<=8;
  num += str[3];
  num <<=8;
  num += str[4];
  num <<=8;
  num += str[5];
  num <<=8;
  num += str[6];
  num <<=8;
  num += str[7];
  return num;
}

uint32 bit24_to_uint32(u_char *str){
  uint32 num;
  num = str[0];
  num <<=8;
  num += str[1];
  num <<=8;
  num += str[2];
  return num;
}

void print_usage(void){
  /* later I should include the option to print to stdout and stderr */
  fprintf(stdout, "Usage :: do_dmep [ Load | Dump | Sense | Store |"
	  " Select | Enable ]\n"
	  "\tLoad   :: <filename> <op code> <action> <seg num> <BID>\n"
	  "\tDump   :: <filename> <op code> <action> <seg num> <PBN>\n"
	  "\tSense  :: <filename> <op code> <action> <seg num>\n"
	  "\tStore  :: <filename> <op code> <action> <seg num> <BID>"
	  " <seq num> <PBN> <in use> [data]\n"
	  "\tSelect :: <filename> <op code> <action> <seg num> <# of Buffers>"
	  " <size of buffers>\n"
	  "\tEnable :: <filename> <op code> <action> <seg num>\n");
}

int human_readable_output(u_char op_code, u_char action,
			  u_char *pdf, uint pdf_len,
			  error_code_t *ec, int error){
  uint32 tmp;
  uint32 dump_block_size = 28+buffer_size;
  char data_str[buffer_size+1];
  int i;

  /* attention -- I need the DMEP spec for this */
  if(op_code == 0x85){
    switch(action){
    case 0: /* Load buffer */
      if(error < 0) {
	printf("Load failure.\n");
	if(ec->sense_key){
	  printf("\tSense key :: 0x%.2x\n", ec->sense_key);
	  printf("\tASC       :: 0x%.2x\n", ec->asc);
	  printf("\tASCQ      :: 0x%.2x\n", ec->ascq);
	  printf("\tSpecific  :: 0x%.2x 0x%.2x 0x%.2x\n",
		ec->specific[0],
		ec->specific[1],
		ec->specific[2]);
	}
      } else {
	printf("Load complete.\n");

	printf("\tIn use          :: %d\n", (pdf[4])?1:0);
	printf("\tFullness        :: 0x%2.2x\n", pdf[5]);
	printf("\tSequence number :: %"FMT_64"u\n", bit64_to_uint64(pdf+8));
	printf("\tPBN             :: %"FMT_64"u\n", bit64_to_uint64(pdf+16));
	memcpy(data_str, pdf+24, buffer_size);
	data_str[buffer_size] = '\0';
	printf("\tData            :: %-24s\n", data_str);
      }
      break;
    case 1: /* Dump buffer */
      if(error < 0) {
	printf("Dump failure.\n");
	if(ec->sense_key){
	  printf("\tSense key :: 0x%.2x\n", ec->sense_key);
	  printf("\tASC       :: 0x%.2x\n", ec->asc);
	  printf("\tASCQ      :: 0x%.2x\n", ec->ascq);
	  printf("\tSpecific  :: 0x%.2x 0x%.2x 0x%.2x\n",
		ec->specific[0],
		ec->specific[1],
		ec->specific[2]);
	}
      } else {
	printf("Dump complete.\n");

	tmp = bit24_to_uint32(pdf);
	
	for(i = 8; i < tmp; i+=dump_block_size){
	  printf("\tBID             :: %"FMT_64"u\n", bit64_to_uint64(pdf+i+4));
	  printf("\tSequence number :: %"FMT_64"u\n", bit64_to_uint64(pdf+i+12));
	  printf("\tPBN             :: %"FMT_64"u\n", bit64_to_uint64(pdf+i+20));
	  memcpy(data_str, pdf+28+i, buffer_size);
	  data_str[buffer_size] = '\0';
	  printf("\tData            :: %-24s\n", data_str);
	}
      }
      break;
    case 2: /* Sense config */
      if(error < 0) {
	printf("Sense failure.\n");
	if(ec->sense_key){
	  printf("\tSense key :: 0x%.2x\n", ec->sense_key);
	  printf("\tASC       :: 0x%.2x\n", ec->asc);
	  printf("\tASCQ      :: 0x%.2x\n", ec->ascq);
	  printf("\tSpecific  :: 0x%.2x 0x%.2x 0x%.2x\n",
		ec->specific[0],
		ec->specific[1],
		ec->specific[2]);
	}
      } else {
	printf("Sense complete.\n");

	printf("\tNumber of segments    :: %d\n", (int)pdf[4]);
	printf("\tMaximum # of segments :: %d\n", (int)pdf[5]);
	printf("\tNumber of buffers     :: %"FMT_64"u\n", bit64_to_uint64(pdf+8));
	printf("\tData size per buffer  :: %u\n", bit24_to_uint32(pdf+16));
      }
      break;
    default:
      break;
    }
  } else if(op_code == 0x89){
    switch(action){
    case 0: /* Store buffer */
      if(error < 0) {
	printf("Store failure.\n");
	if(ec->sense_key){
	  printf("\tSense key :: 0x%.2x\n", ec->sense_key);
	  printf("\tASC       :: 0x%.2x\n", ec->asc);
	  printf("\tASCQ      :: 0x%.2x\n", ec->ascq);
	  printf("\tSpecific  :: 0x%.2x 0x%.2x 0x%.2x\n",
		ec->specific[0],
		ec->specific[1],
		ec->specific[2]);
	}
      } else {
	printf("Store complete.\n");
      }
      break;
    case 2: /* Select config */
      if(error < 0) {
	printf("Select failure.\n");
	if(ec->sense_key){
	  printf("\tSense key :: 0x%.2x\n", ec->sense_key);
	  printf("\tASC       :: 0x%.2x\n", ec->asc);
	  printf("\tASCQ      :: 0x%.2x\n", ec->ascq);
	  printf("\tSpecific  :: 0x%.2x 0x%.2x 0x%.2x\n",
		ec->specific[0],
		ec->specific[1],
		ec->specific[2]);
	}
      } else {
	printf("Select complete.\n");
      }
      break;
    case 3: /* Enable segment */
      if(error < 0) {
	printf("Enable failure.\n");
	if(ec->sense_key){
	  printf("\tSense key :: 0x%.2x\n", ec->sense_key);
	  printf("\tASC       :: 0x%.2x\n", ec->asc);
	  printf("\tASCQ      :: 0x%.2x\n", ec->ascq);
	  printf("\tSpecific  :: 0x%.2x 0x%.2x 0x%.2x\n",
		ec->specific[0],
		ec->specific[1],
		ec->specific[2]);
	}
      } else {
	printf("Enable complete.\n");
      }
      break;
    default:
      break;
    }
  }

  return 0;
}


int is_dmep_capable(char *filename, u_char seg_num){
  int fd;
  u_char	*cdb;
  u_char       	*pdf;
  uint32	pdf_len;
  struct stat stat_info;
  dev_t *devices;
  int device_count;
  int dmep_count;
  uint32 tmp_buffer_size = 0;

  if((fd = open(filename, O_RDONLY)) < 0){
    fprintf(stderr, "Error openning file, %s\n", filename);
    exit(EXIT_FAILURE);
  }


  if(fstat(fd, &stat_info) < 0){
    fprintf(stderr, "Stat on file, %s, failed.\n", filename);
    exit(EXIT_FAILURE);
  }

  /********************************************************************
   ** It is important to note what is going on here.  If it is a pool
   ** device, remember that we only allow LOAD/STORE commands.
   ** Therefore, the DMEP devices in the pool must be configured and
   ** have the same buffer sizes.  However, if we are talking to an
   ** SG device, then nothing has to be preconfigured, but if it is,
   ** then we should use the buffer size specified by the device.
   *******************************************************************/
  if(major(stat_info.st_rdev) == 121){ /* POOL_MAJOR */
    int i;
    get_pool_device_list(fd, &devices, &device_count, &dmep_count);
    close(fd);
    if(!dmep_count){
      fprintf(stderr, "This pool has no dmep devices specified.\n");
      exit(EXIT_FAILURE);
    }
    for(i = 0; i < dmep_count; i++){
      /* build the command for Sense */
      if(0 > build_dmep_command(0x85, 0x2, seg_num, 0,
				0, 0, 0, NULL, 0, &cdb,
				&pdf, &pdf_len)){
	fprintf(stderr, "Could not build DMEP command.\n");
	exit(EXIT_FAILURE);
      }
      /* Send the command for Sense */
      if(0 > send_dmep_command(filename, 0, cdb, &pdf,
			       pdf_len, NULL, devices[i])){
	return 0;
      }


      if(!(int)pdf[4]){
	fprintf(stderr, "Not all of your DMEP devices for this pool"
		" are configured!\n");
	fprintf(stderr, "Try running dmep_conf.\n");
	exit(EXIT_FAILURE);
      }

      if(!tmp_buffer_size){
	tmp_buffer_size = bit24_to_uint32(pdf+16);
      }

      if(!tmp_buffer_size || tmp_buffer_size != bit24_to_uint32(pdf+16)){
	fprintf(stderr, "Your buffers are not the same size on all the"
		" DMEP devices in your pool.\n");
	fprintf(stderr, "Try running dmep_conf.\n");
	exit(EXIT_FAILURE);
      }
    }
  } else {
    if(0 > build_dmep_command(0x85, 0x2, seg_num, 0,
			      0, 0, 0, NULL, 0, &cdb,
			      &pdf, &pdf_len)){
      fprintf(stderr, "Could not build DMEP command.\n");
      exit(EXIT_FAILURE);
    }
    /* Send the command for Sense */
    if(0 > send_dmep_command(filename, 0, cdb, &pdf,
			     pdf_len, NULL, 0)){
      return 0;
    }
    if(!(int)pdf[4]){
      tmp_buffer_size = DEFAULT_BUFFER_SIZE;
    } else {
      tmp_buffer_size = bit24_to_uint32(pdf+16);
      if(!tmp_buffer_size){
	tmp_buffer_size = DEFAULT_BUFFER_SIZE;
      }
    }
  }
  buffer_size = tmp_buffer_size;
  return 1;
}
