/******************************************************************************
*******************************************************************************
**
**    Copyright 2000-2001 Sistina Software Inc.
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**	Jonathan Brassow <brassow@sistina.com>
** 
**    Module name: send_pool.c
**
*******************************************************************************
******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/sysmacros.h>

#include "global.h"
#include "pool_lock.h"
#include "pool_ioctl.h"

/**
 * send_to_pool - Sends a DMEP command through pool using IOCTL
 * @fd: File descriptor for a pool device
 * @bid: Buffer ID
 * @cdb: Command Descriptor Block
 * @pdf: The PDF, a double ptr so that data can be returned as well
 * @pdf_len: The length of the pdf
 * @phys_dev: Zero if the command goes to pool else the underlying dev_t
 *
 * Two methods can be used for accessing a DMEP device.  You can
 * go through pool or sg.  This is the function that take the 
 * former approach.
 *
 * Returns: 0 on success, -1 on failure
 */
int send_to_pool(int fd,
		 uint64 bid,
		 u_char *cdb,
		 u_char **pdf,
		 uint32 pdf_len,
		 error_code_t *ec,
		 dev_t phys_dev){

  /* Someday I may put this struct into a .h file, but **
  ** right now I don't know where it fits best.        */
  int error = 0;
  struct {
    uint64 pk_bid;  /* pk_ stands for package, for lack of better name */
    u_char pk_cdb[16];
    uint32 pk_pdf_len;
    u_char *pk_pdf;
    error_code_t pk_ec;
    uint32 pk_major;
    uint32 pk_minor;
  } package;

  package.pk_bid = bid;
  memcpy(package.pk_cdb, cdb, 16);
  package.pk_pdf_len = pdf_len;
  package.pk_pdf = *pdf;
  memset(&(package.pk_ec), 0, sizeof(error_code_t));
  package.pk_minor = minor(phys_dev);
  package.pk_major = major(phys_dev);

  if(ioctl(fd, POOL_MEMEXP, &package) < 0){
    /*fprintf(stderr, "IOCTL failed.\n");*/
    error =  -1;
  }

  if(package.pk_ec.sense_key){
    error = -1;
  }

  if(ec)
    memcpy(ec, &(package.pk_ec), sizeof(error_code_t));

  return error;
}
