/******************************************************************************
*******************************************************************************
**
**    Copyright 1995-2000 Regents of the University of Minnesota
**    Portion Copyright 1999-2001 Sistina Software, Inc.
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      David C. Teigland  <teigland@sistina.com>
**
**    Module name: gfsck.h
**
**    Description:
**      Filesystem checker.
**
*******************************************************************************
******************************************************************************/

#ifndef __GFSCK_H__
#define __GFSCK_H__


#include <string.h>
#include <sys/ioctl.h>
#include <sys/mount.h>


#ifndef GFSCK_EXTERN
#define GFSCK_EXTERN extern
#define GFSCK_INIT(X)
#else
#undef GFSCK_EXTERN
#define GFSCK_EXTERN
#define GFSCK_INIT(X) =X
#endif


/*  Global file descriptor for accessing the file system device  */

GFSCK_EXTERN int fsfd;

/*  Global limits on block numbers  */

GFSCK_EXTERN uint64 last_fs_block GFSCK_INIT((uint64)(-1));
GFSCK_EXTERN uint64 last_data_block GFSCK_INIT((uint64)(-1));
GFSCK_EXTERN uint64 first_data1 GFSCK_INIT(0);

GFSCK_EXTERN uint32 rg_count GFSCK_INIT(0);

/*  Global debugging values and program options  */

GFSCK_EXTERN char *prog_name;
GFSCK_EXTERN int print_structs GFSCK_INIT(0);
GFSCK_EXTERN int print_debug GFSCK_INIT(0);
GFSCK_EXTERN int dry_run GFSCK_INIT(0);

/*  Lists to match link counts with dir entries */

GFSCK_EXTERN osi_list_head_t links_dirents;
GFSCK_EXTERN osi_list_head_t links_nlink;

/*  Incore bitmaps compared to those on disk  */

GFSCK_EXTERN osi_list_head_t bitmaps;



/*  Divide and round up  */

#define DIV_RU(x, y) (((x) + (y) - 1) / (y)) 



/*  Talk to user  */

#define warn(fmt, args...) \
{ \
  printf("%s: ", prog_name); \
  printf(fmt, ## args); \
}

#define die(fmt, args...) \
{ \
  fprintf(stderr, "%s: ", prog_name); \
  fprintf(stderr, fmt, ## args); \
  exit(EXIT_FAILURE); \
}



/*  Function Macros  */

#define type_zalloc(ptr, type, count) \
{ \
  if (((ptr) = (type *)malloc(sizeof(type) * (count))) != NULL) \
    memset((char *)(ptr), 0, sizeof(type) * (count)); \
  else \
  { \
    fprintf(stderr, "%s: unable to allocate memory on line %d.\n", prog_name, __LINE__); \
    exit(EXIT_FAILURE); \
  } \
}


#define do_lseek(fd, off) \
  ((lseek((fd), (off), SEEK_SET) == (off)) ? 0 : -1)

#define do_read(fd, buff, len) \
  ((read((fd), (buff), (len)) == (len)) ? 0 : -1)

#define do_write(fd, buff, len) \
  ((write((fd), (buff), (len)) == (len)) ? 0 : -1)




/*  Use this to keep track of special dinodes  */

struct di_info
{
  osi_list_head_t  din_list;
  uint64           din_addr;
  uint32           din_nlink;
  uint32           din_dirents;
};
typedef struct di_info di_info_t;


/*  Use to manage in core bitmaps  */

struct bitmap_list
{
  osi_list_head_t list;
  char *bm;
  gfs_rgrpd_t *rgd;
};
typedef struct bitmap_list bitmap_list_t;


/*  Generally used to keep list of blocks  */

struct blk_list
{
  osi_list_head_t list;
  uint64 blk;
};
typedef struct blk_list blk_list_t;




/*  Function prototypes for common utility functions  */

#include "util.h"


#endif /* __GFSCK_H__ */





