/******************************************************************************
*******************************************************************************
**
**    Copyright 1995-2000 Regents of the University of Minnesota
**    Portion Copyright 1999-2001 Sistina Software, Inc.
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      David C. Teigland  <teigland@sistina.com>
**
**    Module name: pass5.c  (gfsck)
**
**    Description:
**      Filesystem checker
**
*******************************************************************************
******************************************************************************/

#include "gfs.h"
#include "gfsck.h"

#include "blklist.h"
#include "dir.h"



/**
 * pass_5
 * @sdp:
 *
 */

int pass_5(gfs_sbd_t *sdp)
{
  gfs_rgrpd_t *rgd;
  osi_list_head_t *tmp;
  gfs_inode_t *ip, *pip;
  gfs_inum_t inum;
  di_info_t *dinfo;
  uint64 block; 
  uint32 start_blk, start_buf, bsize = sdp->sd_sb.sb_bsize;
  int error, cnt = 0, count = 0, first, type = 0;
  char buf[bsize];
  osi_qstr_t filename;
  osi_list_head_t visited;


  type_zalloc(ip, gfs_inode_t, 1);
  ip->i_sbd = sdp;

  type_zalloc(pip, gfs_inode_t, 1);  
  pip->i_sbd = sdp;

  memset(&filename, 0, sizeof(osi_qstr_t));
  filename.name = "..";
  filename.len = 2;
  

  for (tmp = sdp->sd_rglist.next; tmp != &sdp->sd_rglist; tmp = tmp->next)
  {
    printf("Pass 5:  %.0f %% \r", 100.0 * ((float)cnt / (float)rg_count));
    fflush(0);
    cnt++;

    rgd = osi_list_entry(tmp, gfs_rgrpd_t, rd_list);

    error = gfs_rgrp_read(rgd);
    if (error)
      return -1;
    
    start_blk = 0;
    start_buf = 0;
    first = 1;

    while (1)
    {
      error = next_rg_metatype(sdp, rgd, &start_blk, &start_buf, &block, buf,
                               GFS_METATYPE_DI, &first);
      if (error)
        break;

      first = 0;

      gfs_dinode_in(&ip->i_di, buf);

      if (ip->i_di.di_flags & GFS_DIF_UNUSED)
        continue;

      if (ip->i_di.di_type != FILE_DIR)
	continue;
      
      ip->i_num = ip->i_di.di_num;
      count++;

      
      osi_list_head_init(&visited);
      type_zalloc(dinfo, di_info_t, 1);
      osi_list_add(&dinfo->din_list, &visited);
      dinfo->din_addr = ip->i_num.no_addr;

      error = gfs_dir_search(ip, &filename, &inum, &type);
      if (error < 0)
      {
	warn("no .. entry found\n");
	continue;
      }

      while (1)
      {
	if (inum.no_addr == sdp->sd_sb.sb_root_di.no_addr)
	  break;

	dinfo = search_list(&visited, inum.no_addr);

	if (dinfo)
	{
	  warn("Found directory cycle\n");
	  break;
	}

	type_zalloc(dinfo, di_info_t, 1);
	osi_list_add(&dinfo->din_list, &visited);
	dinfo->din_addr = inum.no_addr;

	error = gfsck_read_dinode(sdp, &pip->i_di, inum.no_addr);
	if (error)
	{
	  warn("error reading dinode\n");
	  break;
	}
	pip->i_num = pip->i_di.di_num;

	memset(&inum, 0, sizeof(gfs_inum_t));
	error = gfs_dir_search(pip, &filename, &inum, &type);
	if (error < 0)
	{
	  warn("no .. entry found\n");
	  break;
	}
      }

      if (dinfo)
      {
	/* fix cycle */
      }

      while (!osi_list_empty(&visited))
      {
	dinfo = osi_list_entry(visited.next, di_info_t, din_list);
	osi_list_del(&dinfo->din_list);
	free(dinfo);
      }
    }

    gfs_rgrp_relse(rgd);
  }

  free(ip);

  printf("Pass 5:  %d directories checked\n", count);

  return 0;
}
