/******************************************************************************
*******************************************************************************
**
**    Copyright 1995-2000 Regents of the University of Minnesota
**    Portion Copyright 1999-2001 Sistina Software, Inc.
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      David C. Teigland  <teigland@sistina.com>
**
**    Module name: pass6.c  (gfsck)
**
**    Description:
**      Filesystem checker
**
*******************************************************************************
******************************************************************************/

#include "gfs.h"
#include "gfsck.h"



/**
 * pass_6
 * @sdp:
 *
 */

int pass_6(gfs_sbd_t *sdp)
{
  osi_list_head_t *tmp1, *tmp2;
  di_info_t *dinfo1, *dinfo2;
  int error;


  for (tmp1 = links_nlink.next; tmp1 != &links_nlink; tmp1 = tmp1->next)
  {
    dinfo1 = osi_list_entry(tmp1, di_info_t, din_list);

    for (tmp2 = links_dirents.next; tmp2 != &links_dirents; tmp2 = tmp2->next)
    {
      dinfo2 = osi_list_entry(tmp2, di_info_t, din_list);

      if (dinfo1->din_addr == dinfo2->din_addr)
      {
	if (dinfo1->din_nlink == dinfo2->din_dirents)
	{
	  dinfo1->din_addr = 0;
	  dinfo2->din_addr = 0;
	  break;
	}
	else
	{
	  warn("Bad link count on regular file #%"FMT_64"u\n", dinfo1->din_addr);
	  warn("  nlink: %u, directory entries: %u\n", dinfo1->din_nlink, 
	       dinfo2->din_dirents);
	  /* change di_nlink to dinfo2->din_dirents */
	}
      }

    }
  }


  /* files with link count not equal to one */

  
  for (tmp1 = links_nlink.next; tmp1 != &links_nlink; tmp1 = tmp1->next)
  {
    dinfo1 = osi_list_entry(tmp1, di_info_t, din_list);
    
    if (dinfo1->din_addr == 0)
      continue;

    error = check_bitmap_set(sdp, dinfo1->din_addr);

    if (!error)
    {
      warn("It is impossible for this to be printed :)\n");
      /* this should not happen.  if the file is not in a directory,
	 it should have been moved to l+f in pass 4 */
    }
    else
    {
      warn("Bad link count on regular file #%"FMT_64"u\n", dinfo1->din_addr);
      /* set di_nlink to 1 */
    }    
  }

  
  for (tmp2 = links_dirents.next; tmp2 != &links_dirents; tmp2 = tmp2->next)
  {
    dinfo2 = osi_list_entry(tmp2, di_info_t, din_list);
  
    if (dinfo2->din_addr == 0)
      continue;

    warn("Bad link count on regular file #%"FMT_64"u\n", dinfo2->din_addr);

    /* verify di_nlink is 1 and set di_nlink to dinfo2->din_dirents */
  }


  while(!osi_list_empty(&links_nlink))
  {
    dinfo1 = osi_list_entry(links_nlink.next, di_info_t, din_list);
    osi_list_del(&dinfo1->din_list);
    free(dinfo1);
  }

  while(!osi_list_empty(&links_dirents))
  {
    dinfo2 = osi_list_entry(links_dirents.next, di_info_t, din_list);
    osi_list_del(&dinfo2->din_list);
    free(dinfo2);
  }


  return 0;
}




