/******************************************************************************
*******************************************************************************
**
**    Copyright 1999-2001 Sistina Software, Inc.
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      David C. Teigland  <teigland@sistina.com>
**
**    Module name: gfsconf.h
**
**    Description:
**
*******************************************************************************
******************************************************************************/

#ifndef GFSCONF_DOT_H
#define GFSCONF_DOT_H


#include "gfs_ondisk.h"  /* GFS_LOCKNAME_LEN */
#include "stomith.h"
#include "util.h"


#ifndef EXTERN
#define EXTERN extern
#else
#undef EXTERN
#define EXTERN
#endif

EXTERN FILE *fp;
EXTERN char *prog_name;


#define LINE_SIZE (1024)


#define atoll(x) strtoq(x,(char **)NULL,10)


#define die(fmt, args...) \
{ \
  fprintf(stderr, "%s: ", prog_name); \
  fprintf(stderr, fmt, ##args); \
  exit(EXIT_FAILURE); \
}


#define type_zalloc(ptr, type, count) \
{ \
  (ptr) = (type *)malloc(sizeof(type) * (count)); \
  if ((ptr)) \
    memset((char *)(ptr), 0, sizeof(type) * (count)); \
  else \
    die("unable to allocate memory on line %d of file %s\n", \
        __LINE__, __FILE__); \
}


#define do_lseek(fd, off) \
{ \
  if (lseek((fd), (off), SEEK_SET) != (off)) \
    die("bad seek: %s on line %d of file %s\n", \
        strerror(errno),__LINE__, __FILE__); \
}


#define do_read(fd, buff, len) \
{ \
  if (read((fd), (buff), (len)) != (len)) \
    die("bad read: %s on line %d of file %s\n", \
        strerror(errno), __LINE__, __FILE__); \
}


#define do_write(fd, buff, len) \
{ \
  if (write((fd), (buff), (len)) != (len)) \
    die("bad write: %s on line %d of file %s\n", \
        strerror(errno), __LINE__, __FILE__); \
}



/* List of all STOMITH methods defined in config file */

struct stomith_list
{
  uint32                magic;
  char                  name[STM_STRING_LEN];
  struct stomith_list   *next;
  union 
  {
    stm_brcd_zone_t     brcd_zone_s;
    stm_brcd_port_t     brcd_port_s;
    stm_wti_nps_t      	wti_nps_s;
    stm_vacm_emp_t      vacm_emp_s;
    stm_gnbd_t		gnbd_s;
    stm_meatware_t      meatware_s;
    stm_vx_hub_t	vx_hub_s;
    stm_apc_ms_t	apc_ms_s;
  } st;
};
typedef struct stomith_list stomith_list_t;


/* List of all nodes defined in config file */

struct node_list
{
  uint32                ipaddr;
  uint32                cid;
  int                   nstomith;  /* count stomith methods for this node */
  int                   length;    /* keep track of data length used up to 512 */
  stomith_list_t        *stomith;  /* list of stomith methods for this node */
  struct node_list      *next;
};
typedef struct node_list node_list_t;


/* Handle for all info gathered from config file */

struct param
{
  char            cidev[GFS_LOCKNAME_LEN];
  char            datadev[GFS_LOCKNAME_LEN];
  char            lockdev[GFS_LOCKNAME_LEN];
  uint16          cbport;
  uint32          timeout;
  node_list_t     *node_list;
  stomith_list_t  *stomith_list;
  stomith_list_t  *none_names;
};
typedef struct param param_t;


#endif  /*  GFSCONF_DOT_H  */






