%{
/******************************************************************************
*******************************************************************************
**
**    Copyright 1999-2001 Sistina Software, Inc.
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      Michael Tilstra <conrad@sistina.com>
** 
**    Module name: 
**
**    Description:
**
*******************************************************************************
******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include "arglists.h"
#include "util.h"

int yyerror(char*);
int yylex();
void view_config(rootNode *root);
stmNode *new_stomith(stmNode *atch, char *name, keyList *prms);
nodeNode *new_nodenode(nodeNode *atch, char *ip, char *cid, keyList *st, int isip);
keyList *new_key_pair(keyList *atch, char *key, char *value);
extern char *yytext;
extern FILE *yyin;
extern int linenumber;
int errors=0;
rootNode *root;

%}

%union
{
   int num;
   char *lexme;
   keyList *keys;
   stmNode *stm;
   nodeNode *nde;
}

%token DATADEV
%token CIDEV
%token LOCKDEV
%token CBPORT
%token TIMEOUT
%token CCNM
%token STOMITH
%token SMspr
%token NODE
%token COLON

%token <lexme> IPPORT
%token <lexme> JUSTIP
%token <lexme> STRING
%token <lexme> NUMBER

%token ERROR

%type <lexme> allstr
%type <lexme> str_list
%type <keys> st_keys
%type <keys> node_sts
%type <stm> stomith_parts
%type <nde> node_parts

%start conffile

%%

conffile  : generic_parts stomith_parts node_parts
            { root->stomiths = $2;
              root->nodes = $3;
            }
          | generic_parts node_parts
            { yyerror("No STOMITH section.  Better add one."); }
          | stomith_parts
            { yyerror("You forgot the generic stuff."); }
          | node_parts
            { yyerror("You forgot both the generic and STOMITH parts."); }
          ;

generic_parts : generic_parts generic_part
              | generic_part
              ;

generic_part : DATADEV COLON STRING
               { root->datadev = $3; }
             | CIDEV COLON STRING
               { root->cidev = $3; }
             | LOCKDEV COLON IPPORT
               { root->lockdev = $3; }
             | LOCKDEV COLON STRING
               { root->lockdev = $3; }
             | CBPORT COLON NUMBER
               { root->port = atoi($3); }
             | TIMEOUT COLON NUMBER
               { root->timeout = atoi($3); }
             | CCNM COLON STRING
               { root->ccnm = $3; }
             ;

st_keys : st_keys STRING COLON allstr
          { $$ = new_key_pair($1, $2, $4); }
        | STRING COLON allstr
          { $$ = new_key_pair(NULL, $1, $3); }
        ;

stomith_parts : stomith_parts STOMITH COLON STRING st_keys
                { $$ = new_stomith($1, $4, $5); }
              | STOMITH COLON STRING st_keys
                { $$ = new_stomith(NULL, $3, $4); }
              ;

allstr : STRING
       | NUMBER
       | JUSTIP
       ;

str_list : str_list allstr
           {  $1 = (char*)realloc($1, strlen($1) + strlen($2) +2 );
              strcat($1, " ");
              strcat($1, $2);
              $$ = $1;
           }
         | allstr
         ;

/* order of stomith actions on a node is important. so must use right
 * recursion */
node_sts : SMspr COLON STRING str_list node_sts
           { $$ = new_key_pair( $5, $3, $4); }
         | SMspr COLON STRING str_list
           { $$ = new_key_pair(NULL, $3, $4); }
         | SMspr COLON STRING
           { $$ = new_key_pair(NULL, $3, NULL); }
         ;

/* order of node is important. so must use right recurseion. */
node_parts : NODE COLON STRING NUMBER node_sts node_parts
             { $$ = new_nodenode($6, $3, $4, $5, 0); }
           | NODE COLON JUSTIP NUMBER node_sts node_parts
             { $$ = new_nodenode($6, $3, $4, $5, 1); }
           | NODE COLON STRING NUMBER node_sts
             { $$ = new_nodenode(NULL, $3, $4, $5, 0); }
           | NODE COLON JUSTIP NUMBER node_sts
             { $$ = new_nodenode(NULL, $3, $4, $5, 1); }
          ;

%%

/* This main is temporary until we finally hook this into the gfsconf tool
 * For now we just are doing a syntax checker typoe thing.
 */
#if 0
int main (int argc, char **argv)
{
   argc--; argv++;
   if (argc > 0)
      yyin = fopen(argv[0],"r");
   else
      yyin = stdin;

   yyparse();

   fclose(yyin);

   return 0;
}
#else
rootNode *parser_begin(char *filename)
{
   root = (rootNode*)malloc(sizeof(rootNode));
   yyin = fopen(filename, "r");
   yyparse();
   fclose(yyin);

   if(errors != 0) {
      fprintf(stderr," %d Errors were found.\n", errors);
      free(root);
      return NULL;
   }else
      return root;
}
#endif


int yyerror (char *mesg)
{
   errors++;
   fprintf(stderr,"Error found in line %d\n",linenumber);
   fprintf(stderr,"  %s\n",mesg);
   return 0;
}

keyList *new_key_pair(keyList *atch, char *key, char *value)
{
   keyList *tmp;

   tmp = (keyList*)malloc(sizeof(keyList));
   if( !tmp ) yyerror("out of memeory.");
   tmp->key = key;
   tmp->value = value;
   tmp->next = atch;

   return tmp;
}

stmNode *new_stomith(stmNode *atch, char *name, keyList *prms)
{
   stmNode *tmp;

   tmp = (stmNode*)malloc(sizeof(stmNode));
   if(!tmp) yyerror("out of memory");
   tmp->name = name;
   tmp->params = prms;
   tmp->next = atch;
   return tmp;
}

nodeNode *new_nodenode(nodeNode *atch, char *ip, char *cid, keyList *st, int isip)
{
   nodeNode *tmp;
   struct hostent *hnt;

   tmp = (nodeNode*)malloc(sizeof(nodeNode));
   if(!tmp) yyerror("out of memory");
   if(isip) {
      inet_aton(ip, &tmp->IP);
   }else{
      hnt = gethostbyname( ip );
      if( ! hnt ) {
         errors++;
         fprintf(stderr, "Error looking up hostname '%s'\n", ip);
      }
      else tmp->IP = *((unsigned int*)(hnt->h_addr_list[0]));
   }
   tmp->cid = atoi(cid);
   tmp->stms = st;
   tmp->next = atch;
   return tmp;
}

/* for debugging */
void view_config(rootNode *root)
{
   stmNode *st;
   nodeNode *nd;
   keyList *key;

   printf(" Datadev: %s\n", root->datadev);
   printf(" Cidev:   %s\n", root->cidev);
   printf(" lockdev: %s\n", root->lockdev);
   printf(" Callback port:  %u\n", root->port);
   printf(" Timeout: %u\n", root->timeout);


   for(st = root->stomiths; st != NULL; st = st->next) {
      printf("  STOMITH: %s\n", st->name);
      for(key = st->params; key != NULL; key = key->next) {
         printf("         : %s => %s\n", key->key, key->value);
      }
   }

   for(nd = root->nodes; nd != NULL; nd = nd->next) {
      printf("  Node IP 0x%x CID:%u\n", nd->IP, nd->cid);
      for(key = nd->stms; key != NULL; key = key->next) {
         printf("      SM: %s => %s\n", key->key, key->value);
      }
   }

}
