/******************************************************************************
*******************************************************************************
**
**    Copyright 2001 Sistina Software, Inc.
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      Kenneth W. Preslan <kpreslan@sistina.com>
** 
*******************************************************************************
******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>

#include "global.h"



#define die(fmt, args...) \
{ \
  fprintf(stderr, "%s: ", prog_name); \
  fprintf(stderr, fmt, ##args); \
  exit(EXIT_FAILURE); \
}

#define do_lseek(fd, off) \
{ \
  if (lseek((fd), (off), SEEK_SET) != (off)) \
    die("bad seek: %s on line %d of file %s\n", \
        strerror(errno),__LINE__, __FILE__); \
}

#define do_read(fd, buff, len) \
{ \
  if (read((fd), (buff), (len)) != (len)) \
    die("bad read: %s on line %d of file %s\n", \
        strerror(errno), __LINE__, __FILE__); \
}

#define do_write(fd, buff, len) \
{ \
  if (write((fd), (buff), (len)) != (len)) \
    die("bad write: %s on line %d of file %s\n", \
        strerror(errno), __LINE__, __FILE__); \
}



#define BLOCK_SIZE (512)



char *prog_name;





int main(int argc, char *argv[])
{
  int fd;
  char buf[BLOCK_SIZE];
  uint64 offset;
  uint64 num, last_num = 0;
  unsigned int id, clients;
  uint64 skip = 0;

  prog_name = argv[0];


  if (argc < 5)
    die("filename offset id clients\n");

  offset = atoll(argv[2]);
  id = atoi(argv[3]);
  clients = atoi(argv[4]);


  fd = open(argv[1], O_RDWR | O_CREAT, 0644);
  if (fd < 0)
    die("can't open file %s:  %s\n", argv[1], strerror(errno));


  for (;;)
  {
    do_lseek(fd, offset);
    do_read(fd, buf, BLOCK_SIZE);

    num = atoll(buf);
    if (num % clients == id)
    {
      num++;
      sprintf(buf, "%"FMT_64"u\n", num);
      printf("%"FMT_64"u %"FMT_64"u\n", num, skip);

      do_lseek(fd, offset);
      do_write(fd, buf, BLOCK_SIZE);

      if (last_num && last_num + clients != num)
	die("bad\n");

      last_num = num;
      skip = 0;
    }
    else
      skip++;
  }


  close(fd);

  exit(EXIT_SUCCESS);
}

