#!/usr/bin/perl -w

#############################################################
#############################################################
##
##    Copyright 2000-2001 Sistina Software Inc.
##
##    This is free software released under the GNU General
##    Public License.  There is no warranty for this
##    software.  See the file COPYING for details.
##
##    See the file CONTRIBUTORS for a list of contributors.
##
##    This file is maintained by:
##	Jonathan Brassow <brassow@sistina.com>
## 
##    File name : dmep_test.pl
##
#############################################################
#############################################################

####################
# global variables
####################
$do_dmep='';
$dmep_conf='';
$max_line=60;
$force=0;
$verbose=0;
$device='';
%sense_info=();

# turn off line buffering
select((select(STDOUT),$|=1)[0]); 


#############################################################
## Phase 0 :: Aquire pre-test info.
##  1. tcsh is default shell
##  2. do_dmep is found
##  3. device given is a DMEP device AND we can aquire
##     sense information.
#############################################################

# Attention -- Should test to be certain that this is tcsh
#print '$SHELL = ', $ENV{'SHELL'}, "\n";

die "Usage :: dmep_test.pl [-f -v] <device>\n" unless (@ARGV >= 1);

foreach $x (@ARGV){
    if($x =~ /\-f/){ $force = 1; }
    elsif($x =~ /\-v/){ $verbose = 1; }
    elsif($x =~ m%/dev/sg%){ $device = $x; }
}

$device || die "Sorry, this test will only work with SG devices.\n";

`which dmep_conf >& /dev/null`;
if($? == 0){
    $dmep_conf = system('which dmep_conf');
    $verbose && print "Configuration program located, using :: $dmep_conf\n";
} elsif (system("ls do_dmep >& /dev/null") == 0) {
    $dmep_conf = "./dmep_conf";
    $verbose && print "Configuration program located, using :: ./dmep_conf\n";
} else {
    die "Could not locate configuration program :: dmep_conf\n";
}

`which do_dmep >& /dev/null`;
if($? == 0){
    $do_dmep = system('which do_dmep');
    $verbose && print "Test program located, using :: $do_dmep\n";
} elsif (system("ls do_dmep >& /dev/null") == 0) {
    $do_dmep = "./do_dmep";
    $verbose && print "Test program located, using :: ./do_dmep\n";
} else {
    die "Could not locate test program :: do_dmep\n";
}

print "\n";

unless(&get_sense_info){
    print "Sense command failed!\n";
    die "You may need to be root; or this may not be a DMEP device.\n";
}

if($sense_info{'Number of segments'} > 0){
    unless($force){
	print "You already have $sense_info{'Number of segments'}",
	" segments configured.\n";
	print "Proceeding with this test will destroy those segments.\n";
	print "Would you like to proceed (y/n)? ";
	$in = <STDIN>;
	chomp $in;
	die "\nTest aborted.\n" unless ($in eq 'y');
	print "\n";
    }
    &clean_or_die;
}


#############################################################
## Phase 1 :: Creating, enabling, and destroying segments.
## 
#############################################################
print "Phase 1\n";

$statement=" Creating Maximum number of segments";
$statement.=" ($sense_info{'Maximum # of segments'})";
print $statement, ' ' x ($max_line - length $statement), ":: ";

for($i=0; $i < $sense_info{'Maximum # of segments'}; $i++)
{
    $cmd = "$dmep_conf -S $i -n 100 $device";
    `$cmd >& /dev/null`;
    if($? != 0){
	if(&destroy_all_segments){
	    die "failure\n";
	} else {
	    print "failure\n",
	    "Could not exit cleanly!  Buffers may still exist.";
	    unless($verbose){
		print "\nTry using the -v option for more information.";
	    }
	    die "\n";
	}
    }
}

print "success\n";
$statement=" Loading a buffer from each segment";
print $statement, ' ' x ($max_line - length $statement), ":: ";

srand;

#Hash arrays to keep track of info according to BIDs
%in_use=();
%fullness=();
%seq_numbers=();
%seg_numbers=();
%phys_numbers=();

# Load buffers before initial use
for($i=0; $i < $sense_info{'Maximum # of segments'}; $i++)
{
    while(){
	$bid = int(rand 1000000);
	# Attention -- are we sure this produces no duplicates ?
	unless(undef($seq_numbers{$bid})){
	    last;
	}
    }

    $cmd = "$do_dmep $device 85 0 $i $bid";
    @results = `$cmd`;
    map { chomp } @results;

    if($results[0] =~ m/complete/){
	($a, $b) = split(m/ *:: /, $results[1]);
	$in_use{$bid} = $b;
	($a, $b) = split(m/ *:: /, $results[2]);
	$fullness{$bid} = $b;
	($a, $b) = split(m/ *:: /, $results[3]);
	$seq_numbers{$bid} = $b;
	($a, $b) = split(m/ *:: /, $results[4]);
	$phys_numbers{$bid} = $b;
    } else {
	print "failure\n";
	print "\tCould not complete LOAD action.\n";
	if(&destroy_all_segments){
	    die "\n";
	} else {
	    print "\tCould not exit cleanly!  Buffers may still exist.";
	    unless($verbose){
		print "\nTry using the -v option for more information.";
	    }
	    die "\n";
	}
    }
    $seg_numbers{$bid} = $i;
}

print "success\n";
$statement=" Storing data to a buffer from each segment";
print $statement, ' ' x ($max_line - length $statement), ":: ";

foreach $x (keys %in_use){
    $cmd = "$do_dmep $device 89 0 $seg_numbers{$x} $x ";
    $cmd .= "$seq_numbers{$x} $phys_numbers{$x} 1 ";
    $cmd .= "x" x $sense_info{'Data size per buffer'};

    @results = `$cmd`;

    unless($results[0] =~ m/complete/){
	print "failure\n";
	print "\tCould not complete STORE action.\n";
	print "\titeration $x\n";
	print "@results\n";
	if(&destroy_all_segments){
	    die "\n";
	} else {
	    print "\tCould not exit cleanly!  Buffers may still exist.";
	    unless($verbose){
		print "\nTry using the -v option for more information.";
	    }
	    die "\n";
	}
    }
}


print "success\n";
$statement=" Loading stored buffers for comparison";
print $statement, ' ' x ($max_line - length $statement), ":: ";

$count=0;
foreach $x (keys %in_use){
    $cmd = "$do_dmep $device 85 0 $seg_numbers{$x} $x";
    @results = `$cmd`;
    map { chomp } @results;

    if($results[0] =~ m/complete/){
	($a, $b) = split(m/ *:: /, $results[1]);
	$in_use{$bid} = $b;
	($a, $b) = split(m/ *:: /, $results[2]);
	$fullness{$bid} = $b;
	($a, $b) = split(m/ *:: /, $results[3]);
	$seq_numbers{$bid} = $b;
	($a, $b) = split(m/ *:: /, $results[4]);
	$phys_numbers{$bid} = $b;
	($a, $b) = split(m/ *:: /, $results[5]);
	$b =~ m/(\w+)\s*/;
	$data_in = $1;
    } else {
	print "failure\n";
	die "Could not complete LOAD action.\n";
    }
    if("x" x $sense_info{'Data size per buffer'} ne $data_in){
	print "failure\n";
	print "\tLoaded data :: $data_in\n";
	print "\tShould be   :: ", "x" x $sense_info{'Data size per buffer'},
	"\n";
	print "\tSequence number = $seq_numbers{$x}\n";
	print "\tSegment number  = $seg_numbers{$x}\n";
	print "\tBuffer ID       = $x\n";
	print "\tPBN             = $phys_numbers{$x}\n";
	print "\tFailure after $count successfull matches.\n";
	die "\tLOAD did not produce correct data.\n";
    }
    $count++;
}

print "success\n";
$statement=" Destroying all segments";
print $statement, ' ' x ($max_line - length $statement), ":: ";

&clean_or_die;
print "success\n";

#############################################################
## Phase 2 :: Basic testing.
##  1. create large segment.
##  2. fill all buffers.
##  3. verify fullness byte.
##  4. verify sequence numbers are increasing.
#############################################################
print "Phase 2\n";

$statement=" Creating one large segment";
print $statement, ' ' x ($max_line - length $statement), ":: ";

$created_buffers=0;

for($i = 16_000_000; $i > 15_625; $i /= 2){
    $cmd = "$dmep_conf -n $i $device";
    $result = system("$cmd >& /dev/null");
    if($result == 0){
	$created_buffers = $i;
	last;
    }
}

unless(&get_sense_info){
    print "failure\n";
    die "\tSense command failed!\n";
}
$created_buffers = $sense_info{'Number of buffers'};

if($created_buffers > 15_625){
    print "success\n";
    if($created_buffers >= 1_000_000){ 
	$a = $created_buffers / 1_000_000;
	$suffix = "million";
    } else {
	$a = $created_buffers / 1_000;
	$suffix = "thousand";
    }
    $verbose && print "\t$a $suffix buffers created.\n";
} else {
    print "failure\n";
    die "\tCould not even create 15,625 buffers.\n";
}

  
$start_time = time;
$in_use='';
$fullness='';
$seq_numbers='';
$seg_numbers='';
$phys_numbers='';

for($bid = 0; $bid < $created_buffers; $bid++){
    if($bid == 1000){
	$tmp = int ((($created_buffers - $bid)/60)*(time - $start_time)/1000);
	$statement=" Filling all buffers (ETA : ".$tmp." minutes)";
	print $statement, ' ' x ($max_line - length $statement), ":: ";
    }

    $cmd = "$do_dmep $device 85 0 0 $bid";
    @results = `$cmd`;
    
    map { chomp } @results;
    if($results[0] =~ m/complete/){
	($a, $b) = split(m/ *:: /, $results[1]);
	$in_use = $b;
	($a, $b) = split(m/ *:: /, $results[2]);
	$fullness = $b;
	($a, $b) = split(m/ *:: /, $results[3]);
	$seq_numbers = $b;
	($a, $b) = split(m/ *:: /, $results[4]);
	$phys_numbers = $b;
    } else {
	print "failure\n";
	print "\tCould not complete LOAD action.\n";
	&clean_or_die;
	die "\n";
    }

    # Attention -- Be more creative with data.  Entries should
    # be different to catch index errors.

    $cmd = "$do_dmep $device 89 0 0 $bid";
    $cmd .= " $seq_numbers $phys_numbers 1 ";
    $cmd .="x" x $sense_info{'Data size per buffer'};

    @results = `$cmd`;

    if($? != 0 || $results[0] !~ m/complete/){
	print "failure\n";
	print "\tCould not complete STORE action.\n";
	print "\tInteration #",$bid,"\n";
	&clean_or_die;
	die "\n";
    }
}

print "success\n";

$statement=" Checking overflow handling";
print $statement, ' ' x ($max_line - length $statement), ":: ";
# Attention -- this test should cover more...

$cmd = "$do_dmep $device 85 0 0 $created_buffers";
@results = `$cmd`;
$cmd = "$do_dmep $device 89 0 0 $bid";
$cmd .= " $seq_numbers $phys_numbers 0 ";
$cmd .= "x" x $sense_info{'Data size per buffer'};
@results = `$cmd`;

unless($results[0] =~ m/failure/){
    print "failure\n";
    die "\tEverything indicates alotted space is full, yet store succeeds.\n";
}

print "success\n";

$data_in = '';
$start_time = time;

for($bid = 0; $bid < $created_buffers; $bid++){
    if($bid == 1000){
	$tmp = int ((($created_buffers - $bid)/60)*(time - $start_time)/1000);
	$statement=" Checking data integrity & fullness byte";
	$statement.=" (ETA : ".$tmp." minutes)";
	print $statement, ' ' x ($max_line - length $statement), ":: ";
    }

    $cmd = "$do_dmep $device 85 0 0 $bid";
    @results = `$cmd`;
    
    map { chomp } @results;
    if($results[0] =~ m/complete/){
	($a, $b) = split(m/ *:: /, $results[1]);
	$in_use = $b;
	($a, $b) = split(m/ *:: /, $results[2]);
	$fullness = $b;
	($a, $b) = split(m/ *:: /, $results[3]);
	$seq_numbers = $b;
	($a, $b) = split(m/ *:: /, $results[4]);
	$phys_numbers = $b;
	($a, $b) = split(m/ *:: /, $results[5]);
	$b =~ m/(.+\w)\s*$/;
	$data_in = $1;
	unless($data_in){
	    print "failure\n";
	    die "Could not collect data from loaded buffer.\n";
	}
    } else {
	print "failure\n";
	print "\tCould not complete LOAD action.\n";
	&clean_or_die;
	die "\n";
    }

    if($bid == 0){
	unless($fullness eq '0xff'){
	    print "failure\n";
	    die "\tFullness byte is not correctly set on full\n";
	}
    } elsif($bid == int ($created_buffers / 2)){
	unless($fullness eq '0x7f' || $fullness eq '0x80'){
	    print "failure\n";
	    die "\tFullness byte is not correctly set on half-full\n";
	}
    } elsif($bid == ($created_buffers - 1)){
	unless($fullness eq '0x00' || $fullness eq '0x01'){
	    print "failure\n";
	    die "\tFullness byte is not correctly set on empty\n";
	}
    }

    if($data_in ne "x" x $sense_info{'Data size per buffer'}){
	print "failure\n";
	print "\tData integrity was not preserved.\n";
	print "\tData read :: $data_in\n";
	die "\tData expected :: "."x" x $sense_info{'Data size per buffer'}."\n";
    }
    
    $cmd = "$do_dmep $device 89 0 0 $bid";
    $cmd .= " $seq_numbers $phys_numbers 0";

    @results = `$cmd`;

    if($? != 0 || $results[0] !~ m/complete/){
	print "failure\n";
	# Attention -- better error reporting.
	print "\tIteration #",$bid,"\n";
	print "\tCould not complete STORE action.\n";
	&clean_or_die;
	die "\n";
    }
}

print "success\n";

$statement=" Checking buffer reuse and sequence number update";
print $statement, ' ' x ($max_line - length $statement), ":: ";

$cmd = "$do_dmep $device 85 0 0 $bid";
@results = `$cmd`;
    
map { chomp } @results;
if($results[0] =~ m/complete/){
    ($a, $b) = split(m/ *:: /, $results[1]);
    $in_use = $b;
    ($a, $b) = split(m/ *:: /, $results[2]);
    $fullness = $b;
    ($a, $b) = split(m/ *:: /, $results[3]);
    $seq_numbers = $b;
    ($a, $b) = split(m/ *:: /, $results[4]);
    $phys_numbers = $b;
    ($a, $b) = split(m/ *:: /, $results[5]);
    $b =~ m/(.+\w)\s*$/;
} else {
    print "failure\n";
    print "\tCould not complete LOAD action.\n";
    &clean_or_die;
    die "\n";
}
    
$cmd = "$do_dmep $device 89 0 0 $bid";
$cmd .= " $seq_numbers $phys_numbers 1 ";
$cmd .= "x" x $sense_info{'Data size per buffer'};

@results = `$cmd`;

if($? != 0 || $results[0] !~ m/complete/){
    print "failure\n";
    # Attention -- better error reporting.
    print "\tCould not complete STORE action.\n";
    print "\tPossible problem reusing buffers.\n";
    &clean_or_die;
    die "\n";
}

$cmd = "$do_dmep $device 85 0 0 $bid";
@results = `$cmd`;
    
map { chomp } @results;
if($results[0] =~ m/complete/){
    ($a, $b) = split(m/ *:: /, $results[3]);
    if($seq_numbers >= $b && $b != 0){
	print "failure\n";
	die "\tSequence number is not updating properly after a STORE\n";
    }
} else {
    print "failure\n";
    print "\tCould not complete LOAD action.\n";
    &clean_or_die;
    die "\n";
}

print "success\n";

&clean_or_die;

#############################################################
## Phase 3 :: More thorough testing.
##  1. create large buffer segment.
##  2. stress testing on buffers - should include more than
##     one machine.
##  3. verfiy correctness of dump action.
#############################################################
print "Phase 3\n";
print "\nPhase 3 will consist of stress testing which is - at the",
    " moment - incomplete.\n",
    "You have succeeded so far young Skywalker.\n\n";



#############################################################
## get_sense_info()
##
## This routine fills the global varible %sense_info with
## the following hash keys and their respective values
##  Number of segments
##  Maximum # of segments
##  Number of buffers
##  Data size per buffer
##
## The function takes one possible argument, the segment
## number.  If no arguments are given, then the default
## segment number will be 0.
##
## Returns :: 1 on success, 0 otherwise.
#############################################################
sub get_sense_info {
    my $segment_number = 0;
    if(@_ && scalar @_ == 1){
	$segment_number = $_[0];
    }
    my $cmd = "$do_dmep $device 85 2 $segment_number";
    my @results = `$cmd`;

    if(defined($results[0]) && $results[0] =~ m/complete/){
	for($x = 1; $x < @results; $x++){
	    chomp $results[$x];
	    unless(defined $results[$x]){ next; }
	    ($a, $b) = split(m/ *:: /, $results[$x]);
	    unless(defined $b){ next; }
	    $a =~ s/^\s*//;
	    $sense_info{$a} = $b;
	}
    } else {
	return 0;
    }
    $sense_info{'Maximum # of segments'}++;

    return 1;
}


#############################################################
## destory_all_segments()
##
## This subroutine will go through all possible segments,
## check if they are configured, and if so, destroy them.
##
## Returns :: 1 on success, 0 otherwise
#############################################################
sub destroy_all_segments {
    my $results = '';
    &get_sense_info;
    my $max = $sense_info{'Maximum # of segments'};
    my $count = $sense_info{'Number of segments'};

    for($i = 0; $count != 0;){
	if($sense_info{'Number of buffers'}){
	    $results = `$dmep_conf -S $i --destroy $device`;
	    unless($results =~ m/complete/){
		$verbose && print "\n\tCommand failed :: ",
		"$dmep_conf -S $i --destory $device\n";
		return 0;
	    }
	}
	if(++$i < $max){
	    &get_sense_info($i);
	    $count = $sense_info{'Number of segments'};
	} else {
	    $verbose && print "\n\tAll segments reported to be ",
	    "successfully destoryed.\n\tHowever, Sense does not ",
	    "confirm this.\n";
	    return 0;
	}
    }
    return 1;
}

#############################################################
## clean_or_die()
##
## This subroutine simply calls destroy_all_segments() and
## dies if there was a problem
##
## Returns :: simply dies if it doesn't succeed
#############################################################
sub clean_or_die {
    unless(&destroy_all_segments){
	print "\tDestruction of all segments failed.";
	unless($verbose){
	    print "\n\tTry using the -v option for more information.";
	}
	die "\n";
    }
}
