/******************************************************************************
*******************************************************************************
**
**    Copyright 2001 Sistina Software, Inc.
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      Kenneth W. Preslan <kpreslan@sistina.com>
** 
*******************************************************************************
******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>
#include <time.h>



#define T (1)
#define F (0)

#define BLOCKSIZE (512)





int main(int argc, char *argv[])
{
  unsigned char data[BLOCKSIZE];
  int fd;
  unsigned long long int length, x = 0;
  int y;


  srandom(time(NULL));


  if (argc < 3)
  {
    fputs("do_pattern devicename length\n", stderr);
    exit(EXIT_FAILURE);
  }


  if ((fd = open(argv[1], O_WRONLY|O_CREAT, 0644)) < 0)
  {
    fprintf(stderr,"do_pattern:  can't open device %s:  %s\n", argv[1], strerror(errno));
    exit(EXIT_FAILURE);
  }


  length = atoll(argv[2]);
  
  if (length % BLOCKSIZE != 0)
  {
    fprintf(stderr, "do_pattern:  %llu is not divisible bye %u\n", length, BLOCKSIZE);
    exit(EXIT_FAILURE);
  }


  length /= BLOCKSIZE;


  for (x = 0; x < length; x++)
  {
    for (y = 0; y < BLOCKSIZE; y += sizeof(unsigned long long int))
      *((unsigned long long int *)(data + y)) = x + y;

    if (write(fd, data, BLOCKSIZE) != BLOCKSIZE)
    {
      fprintf(stderr, "do_pattern:  bad write():  %s\n", strerror(errno));
      exit(EXIT_FAILURE);
    }
  }


  close(fd);
}
