/******************************************************************************
*******************************************************************************
**
**    Copyright 2001 Sistina Software, Inc.
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      Kenneth W. Preslan <kpreslan@sistina.com>
** 
*******************************************************************************
******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>
#include <time.h>



#define T (1)
#define F (0)

#define BLOCKSIZE (512)





int main(int argc, char *argv[])
{
  unsigned char data[BLOCKSIZE];
  int fd;
  unsigned long long int length, x = 0;
  int y;


  srandom(time(NULL));


  if (argc < 3)
  {
    fputs("do_test devicename length\n", stderr);
    exit(EXIT_FAILURE);
  }


  if ((fd = open(argv[1], O_RDONLY)) < 0)
  {
    fprintf(stderr,"do_test:  can't open device %s:  %s\n", argv[1], strerror(errno));
    exit(EXIT_FAILURE);
  }


  length = atoll(argv[2]);
  
  if (length % BLOCKSIZE != 0)
  {
    fprintf(stderr, "do_test:  %llu is not divisible bye %u\n", length, BLOCKSIZE);
    exit(EXIT_FAILURE);
  }


  length /= BLOCKSIZE;


  while (1)
  {
    x = (((double)random()) / RAND_MAX) * length;
    if (x % 100 == 0)
    printf("%llu\n", x);

    if (lseek(fd, x * BLOCKSIZE, SEEK_SET) != x * BLOCKSIZE)
    {
      fprintf(stderr, "do_test:  bad lseek():  %s\n", strerror(errno));
      exit(EXIT_FAILURE);
    }

    if (read(fd, data, BLOCKSIZE) != BLOCKSIZE)
    {
      fprintf(stderr, "do_test:  bad read():  %s\n", strerror(errno));
      exit(EXIT_FAILURE);
    }

    for (y = 0; y < BLOCKSIZE; y += sizeof(unsigned long long int))
      if (*((unsigned long long int *)(data + y)) != x + y)
      {
	int z;

	for (z = 0; z < BLOCKSIZE; z++)
	{
	  printf("%.2X ", data[z]);
	  if (z % 16 == 15)
	    printf("\n");
	}
	printf("\n");
	fprintf(stderr, "do_test:  bad compare (%llu != %llu) in block %llu\n",
		*((unsigned long long int *)(data + y)), x + y, x);
	exit(EXIT_FAILURE);
      }
  }


  close(fd);
}
