/******************************************************************************
*******************************************************************************
**
**    Copyright 1995-2000 Regents of the University of Minnesota
**    Portions Copyright 2000-2001 Sistina Software, Inc.
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      Kenneth W. Preslan <kpreslan@sistina.com>
** 
**    Module name: 
**      writetest.c
**
**    Description:
**
*******************************************************************************
******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <time.h>
#include <errno.h>



#define WOOHOO (1)
#define DOH (0)

#define BLKSIZE (1048576)



#define do_lseek(fd, off) \
{ \
  if (lseek((fd), (off), SEEK_SET) != (off)) \
    die("bad seek: %s on line %d of file %s\n", \
	strerror(errno),__LINE__, __FILE__); \
}

#define do_read(fd, buff, len) \
{ \
  if (read((fd), (buff), (len)) != (len)) \
    die("bad read: %s on line %d of file %s\n", \
	strerror(errno), __LINE__, __FILE__); \
}

#define do_write(fd, buff, len) \
{ \
  if (write((fd), (buff), (len)) != (len)) \
    die("bad write: %s on line %d of file %s\n", \
	strerror(errno), __LINE__, __FILE__); \
}

#define die(fmt, args...) \
{ \
  fprintf(stderr, "%s: ", prog_name); \
  fprintf(stderr, fmt, ##args); \
  exit(EXIT_FAILURE); \
}



char *prog_name;





int main(int argc,char *argv[])
{
  struct stat st;
  int writetest = WOOHOO;
  int fd1,fd2;
  int it, its, x;
  unsigned long long int filesize, off;
  unsigned int len;
  char *data1, *data2;
  int flags;

  srandom(time(NULL) ^ getpid());
  prog_name = argv[0];


  for (x = strlen(prog_name); x--; )
    if (prog_name[x] == '/')
    {
      prog_name = prog_name + x + 1;
      break;
    }

  if (strcmp(prog_name, "writetest") == 0)
    writetest = WOOHOO;
  else if (strcmp(prog_name, "readtest") == 0)
    writetest = DOH;
  else
    die("strange program name: %s\n", prog_name);
  
  if (writetest)
  {
    if (argc < 5)
      die("arguments: file1 file2 iterations filesize\n");
  }
  else
  {
    if (argc < 4)
      die("arguments: file1 file2 iterations\n");
  }


  data1 = malloc(BLKSIZE);
  if (!data1)
    die("out of memory\n");

  if (!writetest)
  {
    data2 = malloc(BLKSIZE);
    if (!data2)
      die("out of memory\n");
  }


  flags = (writetest) ? (O_CREAT | O_WRONLY | O_TRUNC) : O_RDONLY;

  fd1 = open(argv[1], flags, 0644);
  if (fd1 < 0)
    die("error opening %s\n", argv[1]);

  fd2 = open(argv[2], flags, 0644);
  if (fd2 < 0)
    die("error opening %s\n", argv[2]);

  its = atoi(argv[3]);

  if (writetest)
    filesize = atoll(argv[4]);
  else
  {
    if (fstat(fd1, &st) < 0)
      die("bad stat\n");
    filesize = st.st_size;

    if (fstat(fd2, &st) < 0)
      die("bad stat\n");
    if (st.st_size != filesize)
      die("the files aren't the same size\n");
  }


  for (it = 0; it < its; it++)
  {
    do
    {
      off = filesize * (random() / (RAND_MAX + 1.0));
      len = BLKSIZE * (random() / (RAND_MAX + 1.0));
    }
    while (len + off > filesize);

    do_lseek(fd1, off);
    do_lseek(fd2, off);


    if (writetest)
    {
      for (x = 0; x < len; x++)
	data1[x] = (unsigned char)(256.0 * random() / (RAND_MAX + 1.0));

      do_write(fd1, data1, len);
      do_write(fd2, data1, len);
    }
    else
    {
      do_read(fd1, data1, len);
      do_read(fd2, data2, len);

      if (memcmp(data1, data2, len) != 0)
	die("Read miscompare\n");
    }


    printf("%d:  %llu, %u\n", it, off, len);
  }



  close(fd1);
  close(fd2);
}



