/******************************************************************************
*******************************************************************************
**
**    Copyright 1995-1999 Regents of the University of Minnesota
**    Portions Copyright 1999-2001 Sistina Software, Inc.
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      Kenneth W. Preslan <kpreslan@sistina.com>
** 
**    Module name: mkfs_gfs.h
**
**    Description:
**	Called by mkfs(1M) or alone
**
*******************************************************************************
******************************************************************************/

#ifndef __MKFS_GFS_DOT_H__
#define __MKFS_GFS_DOT_H__


/*  Extern Macro  */

#ifndef EXTERN
#define EXTERN extern
#define INIT(X)
#else
#undef EXTERN
#define EXTERN
#define INIT(X) =X 
#endif



#define die(fmt, args...) \
{ \
  fprintf(stderr, "%s: ", prog_name); \
  fprintf(stderr, fmt, ##args); \
  exit(EXIT_FAILURE); \
}

#define type_zalloc(ptr, type, count) \
{ \
  (ptr) = (type *)malloc(sizeof(type) * (count)); \
  if ((ptr)) \
    memset((char *)(ptr), 0, sizeof(type) * (count)); \
  else \
    die("unable to allocate memory on line %d of file %s\n", \
	__LINE__, __FILE__); \
}

#define type_alloc(ptr, type, count) \
{ \
  (ptr) = (type *)malloc(sizeof(type) * (count)); \
  if (!(ptr)) \
    die("unable to allocate memory on line %d of file %s\n", \
	__LINE__, __FILE__); \
}





struct commandline
{
  char lockproto[GFS_LOCKNAME_LEN];
  char locktable[GFS_LOCKNAME_LEN];

  uint32 bsize;             /*  The block size of the FS  */
  uint32 seg_size;          /*  The journal segment size  */

  int ignore_pool;          /*  Ignore Pool data  */
  uint32 journals;          /*  Number of journals  */
  uint32 jsize;             /*  Size of journals  */

  int debug;                /*  Print out debugging information?  */
  int quiet;                /*  No messages  */

  char *device;             /*  device  */


  /*  Not specified on the command line, but...  */

  uint64 rgrps;             /*  Number of rgrps  */
  int fd;                   /*  fd of the device  */
  gfs_sb_t *sb;             /*  A copy of the superblock  */
  uint64 fssize;            /*  size of the filesystem  */

  uint64 sb_addr;

  uint64 rgrp0_next;        /*  The address of the next available block in rgrp 0  */
};
typedef struct commandline commandline_t;


struct rgrp_list
{
  uint32 subpool;           /*  The subpool who holds this resource group  */

  uint64 rg_offset;         /*  The offset of the beginning of this resource group  */
  uint64 rg_length;         /*  The length of this resource group  */

  gfs_rindex_t *ri;

  struct rgrp_list *next;
};
typedef struct rgrp_list rgrp_list_t;


struct journal_list
{
  uint64 start;
  uint32 segments;

  struct journal_list *next;
};
typedef struct journal_list journal_list_t;


EXTERN char *prog_name;


#define MKFS_DEFAULT_BSIZE	    (4096)
#define MKFS_DEFAULT_SEG_SIZE       (16)
#define MKFS_DEFAULT_JSIZE          (128)


#endif  /*  __MKFS_GFS_DOT_H__  */

