/******************************************************************************
*******************************************************************************
**
**    Copyright 2001 Sistina Software Inc.
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      Jonathan Brassow <brassow@sistina.com>
** 
**    Module name: common.h
**
**    Description:
**	This file, along with common.c, contain all of the functions and
**	structures that can be shared between the Pool related tools.  This
**	is done in an effort to reduce the amount of code and make it easier
**	to read.
**
*******************************************************************************
******************************************************************************/

#ifndef __COMMON_H__
#define __COMMON_H__

#include "global.h"		/* for uint32, etc    */
#include "pool_user.h"		/* user space == for osi.h */
#include "NSP.cf"		/* version numbers    */

#include "pool_ondisk.h"	/* for POOL_NAME_SIZE */
#include "pool_incore.h"	/* for pool structures */

/*********************************************************************
** The following are defined in pool.h as well.  I don't want to be **
** including all of the pool src's .h files though.  So I redeclare **
** them here.  Should be fine if I never include pool.h ........... */
#define POOL_MAJOR		(121)
#define POOL_MAGIC		0x011670
#define POOL_VERSION            (NSPMajorVersion << 16 | \
                                 NSPMinorVersion <<  8 | \
                                 NSPUpdateLevel)
/********************************************************************/

#define BBSIZE          (1<<BBSHIFT)
#define BTOBB(bytes)    (((unsigned long)(bytes) + BBSIZE - 1) >> BBSHIFT)
#define LINE_MAX	256
#define PTOOLS_FORCE	1
#define PTOOLS_VERBOSE	2
#define PTOOLS_GROW	4

/* attention -- this is Linux specific */
#include<linux/kdev_t.h>
#define	makedevice	MKDEV

dev_t devpath_to_devt(char *path);
char *devt_to_devpath(dev_t);
char *get_sptype_name(uint32 type);
void print_pool_t(pool_t *);
pool_list_t *get_pool_info(int);
int remove_pool(char *);
int assemble_pool(char *, int flags);
void compute_and_print_lost_blks(pool_t *);
int scan_and_print_labels(void);
int erase_pool_labels(char *, int);
int get_pool_params_from_file(pool_t **pool, const char *file);
int write_pool_labels(pool_t *pool, int flags);
int grow_pool(pool_t *pool, int flags);

extern void pool_label_out(pool_label_t *pl, char *buf);
extern void pool_label_in(pool_label_t *pl, char *buf);

#endif /* __COMMON_H__ */
