/******************************************************************************
*******************************************************************************
**
**    Copyright 2001 Sistina Software Inc.
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      Jonathan Brassow <brassow@sistina.com>
** 
**    Module name: passemble.c
**
**    Description:
**	passemble is a command that is used to read Pool labels from disk,
**	group those disk according to their labels into a pool, and then
**	make the kernel aware of those pools.
**
*******************************************************************************
******************************************************************************/
#include <stdio.h>
#include <stdlib.h>

#include <unistd.h>

#include "common.h"

void print_usage(void);

int main(int argc, char *argv[]){
  int		c;
  int		flags = 0;
  int		remove = 0;
  char		*poolname = NULL;

  while ((c = getopt(argc, argv, "hr:ve")) != -1) {
    switch(c) {
    case 'h':
      print_usage();
      exit(EXIT_SUCCESS);
      break;
    case 'v':
      flags |= PTOOLS_VERBOSE;
      break;
    case 'r':
      remove = 1;
      poolname = argv[optind-1];
      break;
    }
  }

  if(remove){
    if(remove_pool(poolname) < 0){
      fprintf(stderr, "\nRemoval of pool(s) failed.\n");
      exit(EXIT_FAILURE);
    }
  } else {
    poolname = argv[optind];
    if (!poolname){
      poolname = "all";
    }
    if(assemble_pool(poolname, flags) < 0){
      fprintf(stderr, "\nFailed to assemble pool(s).\n");
      exit(EXIT_FAILURE);
    }
  }

  exit(EXIT_SUCCESS);
}


void print_usage(void){
  printf("Usage: passemble [-hv] [-r all|<poolname>] [<poolname]\n");  
  printf("  -h         Print this help, then exit.\n");
  printf("  -v         Verbose operation.\n");
  printf("  -r         Remove named or all pools from use.\n");
}
