/******************************************************************************
*******************************************************************************
**
**    Copyright 2001 Sistina Software Inc.
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      Jonathan Brassow <brassow@sistina.com>
** 
**    Module name: pgrow.c
**
**    Description:
**     This file houses the details necessary to do growing.
**
*******************************************************************************
******************************************************************************/
#include <stdlib.h>

#include "common.h"

static void print_usage(void);

int main(int argc, char *argv[]){
  int		c;
  int		flags = 0;
  pool_t	*pool = NULL;
  char		*config_file;

  while ((c = getopt(argc, argv, "hfv")) != -1) {
    switch (c) {
    case 'h':
      print_usage();
      exit(EXIT_SUCCESS);
      break;
    case 'f':
      flags |= PTOOLS_FORCE;
      break;
    case 'v':
      flags |= PTOOLS_VERBOSE;
      break;
    default:
      print_usage();
      exit(EXIT_FAILURE);
    }
  }

  config_file = argv[optind];

  if(!config_file){
    print_usage();
    exit(EXIT_FAILURE);
  }

  if (get_pool_params_from_file(&pool, config_file) < 0) {
    fprintf(stderr, "Error occurred while reading parameter file.\n");
    exit(EXIT_FAILURE);
  }
  
  if (grow_pool(pool, flags) < 0){
    fprintf(stderr, "Grow was unsuccessful.\n");
    exit(EXIT_FAILURE);
  }

  exit(EXIT_SUCCESS);
}


static void print_usage(void){
  printf("Usage: pgrow [-hfv] <config file>\n");
  printf("  -h         Print this help, then exit.\n");
  printf("  -f         Force.\n");
  printf("  -v         Verbose.\n");
}
