/******************************************************************************
*******************************************************************************
**
**    Copyright 2000-2001 Sistina Software, Inc.  <nygaard@sistina.com>
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      Erling Nygaard <nygaard@sistina.com>
** 
**    Module name: memexp.c
**
*******************************************************************************
******************************************************************************/



#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>
#include <errno.h>

#include "memexp.h"
#include "scsi_lib.h"

unsigned char cdb[MEMEXP_CMDLEN];

/*******************************
 ***
 ***	DO_MEMEXP_IN_SENSE_CONFIG
 ***	UP_CODE 0x85, ACTION 0x02
 ***
 ******************************/
int do_memexp_in_sense_config (int fd,
			       unsigned char segment_number,			       
			       unsigned char *buffer_number,
			       unsigned char *reply,
			       unsigned int sense_size, 
			       unsigned char *sense_buff,
			       int error
			       ){
  int j = 0;
  int result;
   
  memset(cdb,0,MEMEXP_CMDLEN);
  memset(sense_buff,0,MAX_SENSE);

  cdb[0] = MEMEXP_IN_CMD;    			/* command */
  cdb[1] = ACTION_SENSE_CONFIG; 		/* action */
  cdb[2] = segment_number;			/* segment number */
  cdb[3] = buffer_number[0];			/* buffer number[0], MSB */
  cdb[4] = buffer_number[1]; 			/* buffer number[1] */
  cdb[5] = buffer_number[2];			/* buffer number[2] */
  cdb[6] = buffer_number[3]; 			/* buffer number[3] */  
  cdb[7] = buffer_number[4]; 			/* buffer number[4] */
  cdb[8] = buffer_number[5]; 			/* buffer number[5] */
  cdb[9] = buffer_number[6]; 			/* buffer number[6] */
  cdb[10] = buffer_number[7];       		/* buffer number[7] */
  cdb[11] = buffer_number[8];      		/* buffer number[8], LSB */
  cdb[12] = 0x00;				/* allocation length */
  cdb[13] = 0x00;				/* allocation lenght */
  cdb[14] = 0x12;			 	/* allocation lenght */
  cdb[15] = 0;		       			/* controll */
     
  if ((result = scsi_cmd(fd, 16, cdb, 0, NULL, 20, reply, MAX_SENSE, sense_buff, 0, 0)) < 0){
    if(!error){
      printf("\tFailed CDB: ");
      for(j=0;j<16;j++)
	printf("0x%.2X ",cdb[j]);
      printf("\n");
      printf("Sense retuned: ");
      for(j=0;j<MAX_SENSE;j++){
	printf("0x%.2X ",sense_buff[j]);
      }
      printf("\n");
      return(-1);       
    }
    else
      return(2); /* Operation sucessfully failed */
  }	
  return(1);
}



/*******************************
 ***
 ***	DO_MEMEXP_OUT_SELECT_CONFIG
 ***	UP_CODE 0x89, ACTION 0x02
 ***
 ******************************/
int do_memexp_out_select_config(int fd,
				unsigned char segment_number,			       
				unsigned char *buffer_number,
				unsigned int parameter_length,
				unsigned char *buffers_for_segment,
				unsigned int data_size_for_segment,
				unsigned int sense_size, 
				unsigned char *sense_buff,
				int error
				){
  int j = 0;
  int result;
  unsigned char pdf[20];

  memset(cdb,0,16 );
  memset(pdf,0,20);
  memset(sense_buff,0,MAX_SENSE);

  cdb[0] = MEMEXP_OUT_CMD;          		/* command */
  cdb[1] = ACTION_SELECT_CONFIG; 		/* action */
  cdb[2] = segment_number;			/* segment number */
  cdb[3] = buffer_number[0];			/* buffer number[0], MSB */
  cdb[4] = buffer_number[1]; 			/* buffer number[1] */
  cdb[5] = buffer_number[2];			/* buffer number[2] */
  cdb[6] = buffer_number[3]; 			/* buffer number[3] */  
  cdb[7] = buffer_number[4]; 			/* buffer number[4] */
  cdb[8] = buffer_number[5]; 			/* buffer number[5] */
  cdb[9] = buffer_number[6]; 			/* buffer number[6] */
  cdb[10] = buffer_number[7];       		/* buffer number[7] */
  cdb[11] = buffer_number[8];      		/* buffer number[8], LSB */
  cdb[12] = (char)(parameter_length >> 16);	/* parameter length */
  cdb[13] = (char)(parameter_length >> 8);	/* parameter lenght */
  cdb[14] = (char)(parameter_length >> 00); 	/* parameter lenght */
  cdb[15] = 0;		       			/* controll */
  
  pdf[0] = 0;					/* 00 */     
  pdf[1] = 0;					/* 00 */ 
  pdf[2] = 0x14; 				/* 18  The length is fixed to 0x000012 for this command */
  pdf[3] = 0x02;				/* action */
  pdf[4] = 0x00;				/* reserved*/
  pdf[5] = 0x00;				/* reserved*/
  pdf[6] = 0x00;				/* reserved */
  pdf[7] = 0x00;				/* reserved */
  pdf[8] = buffers_for_segment[0];		/* Number of buffers for segment, MSB */
  pdf[9] = buffers_for_segment[1];		/* Number of buffers for segment */
  pdf[10] = buffers_for_segment[2];		/* Number of buffers for segment */
  pdf[11] = buffers_for_segment[3];		/* Number of buffers for segment */
  pdf[12] = buffers_for_segment[4];		/* Number of buffers for segment */
  pdf[13] = buffers_for_segment[5];		/* Number of buffers for segment */
  pdf[14] = buffers_for_segment[6];		/* Number of buffers for segment */
  pdf[15] = buffers_for_segment[7];		/* Number of buffers for segment, LSB */
  pdf[16] = (char)(data_size_for_segment >> 16);/* Data size for buffers in segment, MSB */
  pdf[17] = (char)(data_size_for_segment >> 8);	/* Data size for buffers in segment */
  pdf[18] = (char)(data_size_for_segment >> 00);/* Data size for buffers in segment, LSB */
  pdf[19] = 0x00;				/* controll */

 if ((result = scsi_cmd(fd, 16, cdb, 20, pdf, 0, NULL, MAX_SENSE, sense_buff, 0, 0)) < 0){
   if(!error){
     printf("\tFailed CDB: ");
     for(j=0;j<16;j++)
       printf("0x%.2X ",cdb[j]);
     printf("\n");
     printf("\tFailed PDF: ");
     for(j=0;j<=19;j++)
       printf("0x%.2X ",pdf[j]);
     printf("\n");
     printf("Sense retuned: ");
     for(j=0;j<MAX_SENSE;j++)
       printf("0x%.2X ",sense_buff[j]);
     printf("\n");
     return(-1);
   }
   else	
     return(2); /* Operation sucessfully failed */
 }
 return(1);
}


/*******************************
 ***
 ***	DO_MEMEXP_OUT_ENABLE_SEGMENT
 ***	UP_CODE 0x89, ACTION 0x03
 ***
 ******************************/
int do_memexp_out_enable_segment(int fd,
				 unsigned char segment_number,			       
				 unsigned char *buffer_number,
				 unsigned int parameter_length,
				 unsigned int sense_size, 
				 unsigned char *sense_buff,
				 int error
				 ){

  int j = 0;
  int result = 0;

  memset(cdb, 0, MEMEXP_CMDLEN );
  memset(sense_buff,0,MAX_SENSE);

  cdb[0] = MEMEXP_OUT_CMD;          		/* command */
  cdb[1] = ACTION_ENABLE_SEGMENT; 		/* action */
  cdb[2] = segment_number;			/* segment number */
  cdb[3] = buffer_number[0];			/* buffer number[0], MSB */
  cdb[4] = buffer_number[1]; 			/* buffer number[1] */
  cdb[5] = buffer_number[2];			/* buffer number[2] */
  cdb[6] = buffer_number[3]; 			/* buffer number[3] */  
  cdb[7] = buffer_number[4]; 			/* buffer number[4] */
  cdb[8] = buffer_number[5]; 			/* buffer number[5] */
  cdb[9] = buffer_number[6]; 			/* buffer number[6] */
  cdb[10] = buffer_number[7];       		/* buffer number[7] */
  cdb[11] = buffer_number[8];      		/* buffer number[8], LSB */
  cdb[12] = (char)(parameter_length >> 16);	/* parameter length */
  cdb[13] = (char)(parameter_length >> 8);	/* parameter lenght */
  cdb[14] = (char)(parameter_length >> 00); 	/* parameter lenght */
  cdb[15] = 0;	 	      			/* controll */
  
  if ((result = scsi_cmd(fd, 16, cdb, 0, NULL, 0, NULL, MAX_SENSE, sense_buff, 0, 0)) < 0){
    if(!error){
      printf("\tFailed CDB: ");
      for(j=0;j<16;j++)
	printf("0x%.2X ",cdb[j]);
      printf("\n");
      printf("Sense retuned: ");
      for(j=0;j<MAX_SENSE;j++)
	printf("0x%.2X ",sense_buff[j]);
      printf("\n");
      return(-1);
    }
    else
      return(2); /* Operation sucessfully failed */ 
  }
  return(1);
}



/*******************************
 ***
 ***	DO_MEMEXP_LOAD_BUFFER
 ***	UP_CODE 0x85, ACTION 0x00
 ***
 ******************************/
int do_memexp_load_buffer(int fd,
			  unsigned char segment_number,			       
			  unsigned char *buffer_number,
			  unsigned int allocation_length,
			  unsigned int reply_size,
			  unsigned char *reply,
			  unsigned int sense_size, 
			  unsigned char *sense_buff,
			  int error
			  ){

  int j = 0;
  int result;

  memset(reply,0,reply_size);
  memset(cdb,0,16);
  memset(sense_buff,0,MAX_SENSE);

  cdb[0] = MEMEXP_IN_CMD;          		/* command */
  cdb[1] = ACTION_LOAD_BUFFER; 			/* action */
  cdb[2] = segment_number;			/* segment number */
  cdb[3] = buffer_number[0];			/* buffer number[0], MSB */
  cdb[4] = buffer_number[1]; 			/* buffer number[1] */
  cdb[5] = buffer_number[2];			/* buffer number[2] */
  cdb[6] = buffer_number[3]; 			/* buffer number[3] */  
  cdb[7] = buffer_number[4]; 			/* buffer number[4] */
  cdb[8] = buffer_number[5]; 			/* buffer number[5] */
  cdb[9] = buffer_number[6]; 			/* buffer number[6] */
  cdb[10] = buffer_number[7];       		/* buffer number[7] */
  cdb[11] = buffer_number[8];      		/* buffer number[8], LSB */
  cdb[12] = (char)(allocation_length >> 16);	/* allocation length */
  cdb[13] = (char)(allocation_length >> 8);	/* allocation lenght */
  cdb[14] = (char)(allocation_length >> 00); 	/* allocation lenght */
  cdb[15] = 0;	   	    			/* controll */

  
  if(result = scsi_cmd(fd, 16, cdb, 0, NULL, reply_size, reply, MAX_SENSE, sense_buff, 0, 0)<0){
    if(!error){
      printf("\tFailed CDB: ");
      for(j=0;j<16;j++)
	printf("0x%.2X ",cdb[j]);
      printf("\n");
      printf("Sense retuned: ");
      for(j=0;j<MAX_SENSE;j++)
	printf("0x%.2X ",sense_buff[j]);
      printf("\n");
      return(-1);
    }	
    else
      return(2); /* Operation sucessfully failed */
  }
  return(1);
}



/*******************************
 ***
 ***	DO_MEMEXP_STORE_BUFFER
 ***	UP_CODE 0x89, ACTION 0x00
 ***
 ******************************/
int do_memexp_store_buffer(int fd,
			   unsigned char segment_number,			       
			   unsigned char *buffer_number,
			   unsigned char in_use,
			   unsigned char *sequence_number,
			   unsigned char *physical_buffer_number,
			   unsigned int data_size,
			   unsigned char *data,
			   unsigned int sense_size, 
			   unsigned char *sense_buff,
			   int error
			   ){

  int j = 0;
  int result;
  unsigned int parameter_length = data_size + STORE_OFFSET;
  unsigned char *data_buffer;
  unsigned char pdf[STORE_OFFSET];

  if (!(data_buffer = malloc(STORE_OFFSET + data_size)))
  {
    printf("do_memexp_store_buffer():  can't get memory\n");
    return(-ENOMEM);
  }
  memset(cdb,0,16);
  memset(sense_buff,0,MAX_SENSE);

  cdb[0] = MEMEXP_OUT_CMD;          		/* command */
  cdb[1] = ACTION_STORE_BUFFER; 		/* action */
  cdb[2] = segment_number;			/* segment number */
  cdb[3] = buffer_number[0];			/* buffer number[0], MSB */
  cdb[4] = buffer_number[1]; 			/* buffer number[1] */
  cdb[5] = buffer_number[2];			/* buffer number[2] */
  cdb[6] = buffer_number[3]; 			/* buffer number[3] */  
  cdb[7] = buffer_number[4]; 			/* buffer number[4] */
  cdb[8] = buffer_number[5]; 			/* buffer number[5] */
  cdb[9] = buffer_number[6]; 			/* buffer number[6] */
  cdb[10] = buffer_number[7];       		/* buffer number[7] */
  cdb[11] = buffer_number[8];      		/* buffer number[8], LSB */
  cdb[12] = (char)(parameter_length >> 16);	/* parameter length */
  cdb[13] = (char)(parameter_length >> 8);	/* parameter lenght */
  cdb[14] = (char)(parameter_length >> 00);  	/* parameter lenght */
  cdb[15] = 0;	       				/* controll */

  pdf[0] = (char)(parameter_length >> 16);	/* length of store parameter */	
  pdf[1] = (char)(parameter_length >> 8);     	/* length of store parameter */	
  pdf[2] = (char)(parameter_length >> 00);   	/* length of store parameter */	
  pdf[3] = 0x00;				/* action for this operation */
  pdf[4] = in_use;				/* First bit to set in_use [1|0]*/
  pdf[5] = 0x00;				/* reserved */
  pdf[6] = 0x00;				/* reserved */
  pdf[7] = 0x00;				/* reserved */
  pdf[8] = sequence_number[0];			/* Sequence number, MSB */
  pdf[9] = sequence_number[1];			/* Sequence number */
  pdf[10] = sequence_number[2];			/* Sequence number */
  pdf[11] = sequence_number[3];			/* Sequence number */
  pdf[12] = sequence_number[4];			/* Sequence number */
  pdf[13] = sequence_number[5];			/* Sequence number */
  pdf[14] = sequence_number[6];			/* Sequence number */
  pdf[15] = sequence_number[7];			/* Sequence number, LSB */
  pdf[16] = physical_buffer_number[0];		/* Physical buffer, MSB */
  pdf[17] = physical_buffer_number[1];		/* Physical buffer */
  pdf[18] = physical_buffer_number[2];		/* Physical buffer */
  pdf[19] = physical_buffer_number[3];		/* Physical buffer */
  pdf[20] = physical_buffer_number[4];		/* Physical buffer */
  pdf[21] = physical_buffer_number[5];		/* Physical buffer */
  pdf[22] = physical_buffer_number[6];		/* Physical buffer */
  pdf[23] = physical_buffer_number[7];		/* Physical buffer */

  memcpy(data_buffer,pdf,sizeof(pdf)); 
  memcpy(data_buffer+STORE_OFFSET,data, data_size);

  if ((result = scsi_cmd(fd, 16, cdb, parameter_length, data_buffer, 0, NULL, MAX_SENSE, sense_buff, 0, 0)) < 0){
    if(!error){
      printf("\tFailed CDB: ");
      for(j=0;j<16;j++)
	printf("0x%.2X ",cdb[j]);
      printf("\n");
      printf("\tFailed PDF: ");
      for(j=0;j<parameter_length;j++)
	printf("0x%.2X ",data_buffer[j]);
      printf("Sense retuned: ");
      for(j=0;j<MAX_SENSE;j++){
	printf("0x%.2X ",sense_buff[j]);
      }
      printf("\n");
      free(data_buffer);
      return(-1);
    }
    else{
      free(data_buffer);
      return(2); /* Operation sucessfully failed */
    }
  }
  free(data_buffer);
  return(1);
}


/*******************************
 ***
 ***	DO_MEMEXP_DUMP_BUFFER
 ***	UP_CODE 0x85, ACTION 0x01
 ***
 ******************************/
int do_memexp_dump_buffer(int fd,
			  unsigned char segment_number,			       
			  unsigned char *buffer_number,
			  unsigned int data_size,
			  unsigned int nr_buffer,
			  unsigned char *reply,
			  unsigned int sense_size, 
			  unsigned char *sense_buff,
			  int error
			  ){

  unsigned int allocation_length;
  unsigned int reply_size;
  int result;
  int j;

  memset(reply,0,reply_size);
  memset(cdb,0,16);
  memset(sense_buff,0,MAX_SENSE);

  allocation_length = ((data_size+28) * nr_buffer) + 8;
  reply_size = allocation_length;

  cdb[0] = MEMEXP_IN_CMD;           		/* command */
  cdb[1] = ACTION_DUMP_BUFFERS;			/* action */
  cdb[2] = segment_number;			/* segment number */
  cdb[3] = buffer_number[0];			/* buffer number[0], MSB */
  cdb[4] = buffer_number[1]; 			/* buffer number[1] */
  cdb[5] = buffer_number[2];			/* buffer number[2] */
  cdb[6] = buffer_number[3]; 			/* buffer number[3] */  
  cdb[7] = buffer_number[4]; 			/* buffer number[4] */
  cdb[8] = buffer_number[5]; 			/* buffer number[5] */
  cdb[9] = buffer_number[6]; 			/* buffer number[6] */
  cdb[10] = buffer_number[7];       		/* buffer number[7] */
  cdb[11] = buffer_number[8];      		/* buffer number[8], LSB */
  cdb[12] = (char)(allocation_length >> 16);	/* allocation length */
  cdb[13] = (char)(allocation_length >> 8);	/* allocation lenght */
  cdb[14] = (char)(allocation_length >> 00); 	/* allocation lenght */
  cdb[15] = 0;	   	    			/* controll */

  
  if(result = scsi_cmd(fd, 16, cdb, 0, NULL, reply_size, reply, MAX_SENSE, sense_buff, 0, 0)<0){
    if(!error){
      printf("\tFailed CDB: ");
      for(j=0;j<16;j++)
	printf("0x%.2X ",cdb[j]);
      printf("\n");
      printf("Sense retuned: ");
      for(j=0;j<MAX_SENSE;j++){
	printf("0x%.2X ",sense_buff[j]);
      }
      printf("\n");
      return(-1);
    }
    else
      return(2); /* Operation sucessfully failed */
  }
  return(1);
}

  /********************************************************************************
   ****
   ****		Do a inquiry to the device before we start
   ****
   ********************************************************************************/
int do_inquiry(int fd){
  
  int j =0;
  int result;

  char input;
  unsigned char sense[20];
  unsigned char reply[95];
  unsigned char cmdblk [6] = 
  { 0x12,  /* command */
    0,  /* lun/reserved */
    0,  /* page code */
    0,  /* reserved */
    96,  /* allocation length */
    0 };/* reserved/flag/link */
  
  memcpy(cdb, cmdblk, sizeof(cmdblk) );
  memset(sense,0,20);

  if ((result = scsi_cmd(fd, sizeof(cmdblk), cdb, 0, NULL, 95, reply, 20, sense, 0, 0)) < 0)
  {
    printf("Inquiry command failed\n");
    printf("Sense retuned: ");
      for(j=0;j<20;j++){
	printf("0x%.2X ",sense[j]);
      }
    printf("\n");
    /*exit(EXIT_FAILURE);*/
  }
  else{
    printf("Info about the device:\n");
    printf("\tManufacturer: ");
    for(j=8;j<15;j++){
      printf("%c",reply[j]);
    }
    printf("\n\tProduct: ");
     for(j=16;j<31;j++){
      printf("%c",reply[j]);
    }
     printf("\n\tRevision: ");
     for(j=32;j<35;j++){
       printf("%c",reply[j]);
     }	
     printf("\n\tVendor unique: ");
     for(j=36;j<55;j++){
       printf("%c",reply[j]);
    } 
     printf("\n");
  }
  return(1);
}
