/******************************************************************************
*******************************************************************************
**
**    Copyright 2000-2001 Sistina Software, Inc.  <nygaard@sistina.com>
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      Erling Nygaard <nygaard@sistina.com>
** 
**    Module name: memexp.h
**
*******************************************************************************
******************************************************************************/


#define MAX_SENSE 20
#define INQUIRY_CMD     0x12
#define INQUIRY_CMDLEN  6
#define INQUIRY_REPLY_LEN 96

#define MEMEXP_IN_CMD	0x85
#define MEMEXP_OUT_CMD	0x89
#define MEMEXP_CMDLEN	16

#define MEMEXP_SENSE_RETURN_LEN	18
#define	MEMEXP_SENSE_CONFIG_LEN 18

#define ACTION_LOAD_BUFFER	0x0
#define	ACTION_STORE_BUFFER	0x0
#define ACTION_DUMP_BUFFERS	0x1
#define	ACTION_SENSE_CONFIG	0x2
#define	ACTION_SELECT_CONFIG	0x2
#define	ACTION_ENABLE_SEGMENT	0x3

#define DATA_OUT	16384
#define REPLY_DATA	16384
#define STORE_OFFSET 24


#define MAX_SEGMENTS 256
#define MAX_BUFFER_SIZE 1024


int do_memexp_in_sense_config(int fd,				/* File pointer to device */
			      unsigned char segment_number,	/* Segment number we are going to operate on */			       
			      unsigned char *buffer_number,	/* Buffer number we are going to operate on */
			      unsigned char *reply,		/* Buffer were we want reply data */
			      unsigned int sense_size, 		/* How much sense data do we want back */
			      unsigned char *sense_buff,  	/* Buffer were we want sense data */
			      int error
			      );

int do_memexp_out_select_config(int fd,				/* File pointer to device*/
				unsigned char segment_number,	/* Segment number we are going to operate on */			       
				unsigned char *buffer_number,	/* Buffer number we are going to operate on */
				unsigned int parameter_length,	/* How much data are we sending? */
				unsigned char *buffers_for_segment,/* How many buffers do we want in this segment */
				unsigned int data_size_for_segment,/* How large should the buffers be */
				unsigned int sense_size, 	/* How much sense do we want back */
				unsigned char *sense_buff,	/* Buffer where we want sense data */
				int error
				);

int do_memexp_out_enable_segment(int fd,		  	/* File pointer to device */
				 unsigned char segment_number,	/* Segment number we are going to operate on */		       
				 unsigned char *buffer_number,	/* Buffer number we are going to operate on */
				 unsigned int parameter_length,	/* how much data we are sending */
				 unsigned int sense_size, 	/* How much sense do we want back */
				 unsigned char *sense_buff,     /* Buffer where we want the sense data */
				 int error
				 );


int do_memexp_store_buffer(int fd,				/* File pointer to device */
			   unsigned char segment_number,	/* Segment number we are going to operate on */			       
			   unsigned char *buffer_number,	/* Buffer number we are going to operate on */
			   unsigned char in_use,		/* Flag for in use / not in use */
			   unsigned char *sequence_number,	/* Sequence number for this buffer */
			   unsigned char *physical_buffer_number,/* Physical buffer number for buffer */
			   unsigned int data_size,		/* data size of buffer */
			   unsigned char *data,			/* Buffer where the data is */
			   unsigned int sense_size, 		/* How much sense data do we want back */
			   unsigned char *sense_buff,           /* Buffer where we want sense data */
			   int error
			   );

int do_memexp_load_buffer(int fd,				/* File pointer to device */
			  unsigned char segment_number,		/* Segment number we are going to operate on */		       
			  unsigned char *buffer_number,		/* Buffer number we are going to operate on */
			  unsigned int allocation_length,	/* How large is the PDF  */
			  unsigned int reply_size,		/* How much data do we want back */
			  unsigned char *reply,			/* Buffer where we want the data */
			  unsigned int sense_size, 		/* How much sense do we want */
			  unsigned char *sense_buff,           	/* Buffer where we want sense data */
			  int error
			  );

int do_memexp_dump_buffer(int fd,				/* Guess what ?? */
			  unsigned char segment_number,		/* Hmmm, wonder what this might be for */			       	
			  unsigned char *buffer_number,		/* Buffer we are going to operate on */
			  unsigned int data_size,		/* Data size of buffers in this segment */
			  unsigned int nr_buffer,		/* How many buffers we want dumped */
			  unsigned char *reply,			/* Buffer where we want the return data */
			  unsigned int sense_size, 		/* How much sense data we want */
			  unsigned char *sense_buff,           	/* Buffer where we want the sense data */
			  int error
			  );
