/******************************************************************************
*******************************************************************************
**
**    Copyright 2000-2001 Sistina Software, Inc.  <nygaard@sistina.com>
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      Erling Nygaard <nygaard@sistina.com>
** 
**    Module name: tester.c
**
*******************************************************************************
******************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>
#include <errno.h>

#define TEST00 1
#define TEST01 1
#define TEST02 1
#define TEST03 1
#define TEST04 1


#include "memexp.h"
#include "scsi_lib.h"


/* Stuff for IN CDB */
unsigned int allocation_length;

/* Stuff for OUT CFD */
unsigned int parameter_length;

/* Stuff for Select Config PDF */
unsigned char buffers_for_segment[8];


int main(int argc, char *argv[] ){

  int fd; /* Pointer to scsi-device */
  
  /* Counters that can be used in the various tests: */
  int i;
  int j;
  int k;
  int l;
  unsigned char c;
  unsigned char d;

  /* Other variables to be used in the various tests: */
  int result;
  int error;
  unsigned int data_size_for_segment;
  unsigned char segment_number;

  /* memory pointers that can beused in the various tests: */
  unsigned char *loaded_data; /* data loaded using load */
  unsigned char *data_buffer; /* data to be stored */
  unsigned char *reply_data;  /* buffer for storing reply data*/	
  unsigned char *dump_data;

  /* Variables for load_buffer */

  /* Variables for store_buffer */

  /* varaibles for dump_buffers */
  unsigned int nr_buffer;
  int dump_offset;
  int dump_data_offset;

  unsigned char buffer_number[9], sense[16];
  unsigned int fullness; 
  unsigned char loaded_sequence_number[8];
  unsigned char loaded_physical_number[8];
  unsigned char loaded_physical_number_10[8];
  unsigned char loaded_physical_number_20[8];
  unsigned char temp_sequence_number[8];
  unsigned char temp_physical_number[8];
  unsigned char loaded_in_use = 0;
  unsigned char in_use;


  if ((fd = scsi_open(argv[1])) < 0){
    printf("DMAP-tester:  can't open device %s:  %s\n", argv[1], strerror(errno));
    exit(-1);
  }
  /**********
   ** Do a Inquiry command to the device 
   *********/
  do_inquiry(fd);


  /************************************************************************************************
   ****
   ****		TEST 00 Test basic commands to see that they are implemented
   ****		
   ****		1)	Do a sense config and see how many segments this baby supports
   ****		2)	Enable ALL segments with one small buffer
   ****		2)      Disable ALL the segments so we start with a clean unit
   ****		3)	Do a select config to configure one segment.
   ****		4)	Enable the segment
   ****		5)	Sense config and verify that that this indeed is what we configured
   ****		6)	Do a Load-Store to write to one buffer
   ****		7)	Do a Load and confirm that the data actually is in the buffer
   ****		8)	Disable all buffers in this segment
   ****		9)	Sense config on the segment and verify that all buffers are disabled
   ****		10)	Select config to clear the segment
   ****		11)	Sense config and verify that the segment is not enabled
   ****
   ***********************************************************************************************/
  if(TEST00){
    printf("Test 00: Doing some basic tests with Select_Config, Enable, Sense_Config, Load and Store commands\n");


    /*******************
     **** Do a MemExp Sense to figure out how many segments we have
     ******************/
    memset(sense,0,MAX_SENSE);
    if (!(reply_data = malloc(20))){
      printf("main():  In test 00 we can't get memory\n");	
      exit(-ENOMEM);
    }

    buffer_number[0] = 0x00;	buffer_number[1] = 0x00;	buffer_number[2] = 0x00;
    buffer_number[3] = 0x00;	buffer_number[4] = 0x00;	buffer_number[5] = 0x00;
    buffer_number[6] = 0x00;	buffer_number[7] = 0x00;	buffer_number[8] = 0x00;

    result = do_memexp_in_sense_config(fd, 
				       segment_number, 
				       buffer_number,  
				       reply_data, 
				       MAX_SENSE, 
				       sense,
				       0);
    if(result > 0){
      l = reply_data[5];
      printf("This device can be configured with up to %d segments\n",l+1);
    }
    else{
      printf("In test 00 the MemExp In Sense Config comamnd failed.\n");
      exit(-1);
    }
    /*****************
     ****
     ****	Configure ALL the segments on the device with each segment having one buffer of 4 bytes
     ****
     ****************/
    printf("Configuring %d segments, each with one buffer of 0x04 bytes\n",l+1);
    for(i=0;i<=l;i++){
      segment_number = i;
      data_size_for_segment = 0x04;

      memset(sense,0,MAX_SENSE);
      buffer_number[0] = 0x00;	buffer_number[1] = 0x00;	buffer_number[2] = 0x00;
      buffer_number[3] = 0x00;	buffer_number[4] = 0x00;	buffer_number[5] = 0x00;
      buffer_number[6] = 0x00;	buffer_number[7] = 0x00;	buffer_number[8] = 0x00;
      parameter_length = 0x14;

      buffers_for_segment[0]=0x00;	buffers_for_segment[1]=0x00;	buffers_for_segment[2]=0x00;
      buffers_for_segment[3]=0x00;	buffers_for_segment[4]=0x00;	buffers_for_segment[5]=0x00;
      buffers_for_segment[6]=0x00;	buffers_for_segment[7]=0x01;

      result = do_memexp_out_select_config(fd,
					   segment_number,			       
					   buffer_number,
					   parameter_length,
					   buffers_for_segment,
					   data_size_for_segment,
					   MAX_SENSE, 
					   sense,
					   0);
      if(result > 0){
      }
      else{
	printf("In test 00 the MemExp_Out_Select_Config command failed when trying to set segment number %d to have 1 buffer of 4 bytes.\n",i+1);
	exit(-1);
      }
    }
    /***************************
     ****
     ****	Unconfigure ALL the segments
     **** 	This is done by configuring all segments to have 0 buffers, each of 0 bytes
     ****	
     **************************/
    printf("Configuring %d segments, each with 0 buffers of 0 bytes. This shall disable all the segments\n",l+1); 
    for(i=0;i<=l;i++){
      segment_number = i;
      data_size_for_segment = 0x00;

      memset(sense,0,MAX_SENSE);
      buffer_number[0] = 0x00;	buffer_number[1] = 0x00;	buffer_number[2] = 0x00;
      buffer_number[3] = 0x00;	buffer_number[4] = 0x00;	buffer_number[5] = 0x00;
      buffer_number[6] = 0x00;	buffer_number[7] = 0x00;	buffer_number[8] = 0x00;
      parameter_length = 0x14;

      buffers_for_segment[0]=0x00;	buffers_for_segment[1]=0x00;	buffers_for_segment[2]=0x00;
      buffers_for_segment[3]=0x00;	buffers_for_segment[4]=0x00;	buffers_for_segment[5]=0x00;
      buffers_for_segment[6]=0x00;	buffers_for_segment[7]=0x00;

      result = do_memexp_out_select_config(fd,
					   segment_number,			       
					   buffer_number,
					   parameter_length,
					   buffers_for_segment,
					   data_size_for_segment,
					   MAX_SENSE, 
					   sense,
					   0);
      if(result > 0){
      }
      else{
	printf("In test 00 the MemExp_Out_Select_Config comamnd failed when trying to clear segment number %d.\n",i+1);
	exit(-1);
      }
    }
    printf("\tAll of the segments can be enabled and disabled\n");
    
    /**************
     ****
     **** Set up a configuration with one segment, 0x0100 buffers and data-size = 0x60 bytes
     ****
     *************/
    printf("Configuring segment 1 to have 0x0100 buffers, each of 0x60 bytes\n");
    data_size_for_segment = 0x60; /* 96 bytes */
    segment_number = 1;
    memset(sense,0,MAX_SENSE);
    buffer_number[0] = 0x00;	buffer_number[1] = 0x00;	buffer_number[2] = 0x00;
    buffer_number[3] = 0x00;	buffer_number[4] = 0x00;	buffer_number[5] = 0x00;
    buffer_number[6] = 0x00;	buffer_number[7] = 0x00;	buffer_number[8] = 0x00;
    parameter_length = 0x14;

    buffers_for_segment[0]=0x00;	buffers_for_segment[1]=0x00;	buffers_for_segment[2]=0x00;
    buffers_for_segment[3]=0x00;	buffers_for_segment[4]=0x00;	buffers_for_segment[5]=0x00;
    buffers_for_segment[6]=0x01;	buffers_for_segment[7]=0x00;

    result = do_memexp_out_select_config(fd,
					 segment_number,			       
					 buffer_number,
					 parameter_length,
					 buffers_for_segment,
					 data_size_for_segment,
					 MAX_SENSE, 
					 sense,
					 0);
    if(result > 0){
    }
    else{
      printf("In test 00 the MemExp_Out_Select_Config command failed.\n");
      exit(-1);
    }
    /**********
     ****Enable segment
     **********/
    printf("Enabling segment 1\n");
    buffer_number[0] = 0x00;	buffer_number[1] = 0x00;	buffer_number[2] = 0x00;
    buffer_number[3] = 0x00;	buffer_number[4] = 0x00;	buffer_number[5] = 0x00;
    buffer_number[6] = 0x00;	buffer_number[7] = 0x00;	buffer_number[8] = 0x00;
    allocation_length = 0x00;

    result = do_memexp_out_enable_segment(fd,
					  segment_number,			       
					  buffer_number,
					  parameter_length,
					  MAX_SENSE, 
					  sense,
					  0);
    if(result > 0){
    }
    else{
      printf("In test 00 the MemExp_Out_Enable commad failed \n");
      exit(-1);
    }
    /********
     ** Sense config
     ********/
    printf("Checking if the segment is correctly configured\n");
    memset(sense,0,MAX_SENSE);
    if (!(reply_data = malloc(20))){
      printf("main():  In test 00 we can't get memory\n");	
      exit(-ENOMEM);
    }

    buffer_number[0] = 0x00;	buffer_number[1] = 0x00;	buffer_number[2] = 0x00;
    buffer_number[3] = 0x00;	buffer_number[4] = 0x00;	buffer_number[5] = 0x00;
    buffer_number[6] = 0x00;	buffer_number[7] = 0x00;	buffer_number[8] = 0x00;

    result = do_memexp_in_sense_config(fd, 
				       segment_number, 
				       buffer_number,  
				       reply_data, 
				       MAX_SENSE, 
				       sense,
				       0);
    if(result > 0){
      if(reply_data[2] != 0x14 || reply_data[0] != 0x00 || reply_data[1] != 0x00){
	/* Right amount of bytes is Return data? */
	printf("The device does returns 0x%.2X%.2X%.2X bytes instead of 0x14 in a Sense Config. This is an error\n",reply_data[0],reply_data[1],reply_data[2]);
      }
      if(reply_data[4] != 0x01){
	/* There should only be one configured segment now */
	printf("The device currently has 0x%.2X segemnt, but we configured it with 0x01 segments. This is an error\n",reply_data[4]);
      }
      if(reply_data[14] != 0x01 || reply_data[8]!=0||reply_data[9]!=0||reply_data[10]!=0||reply_data[11]!=0||reply_data[12]!=0||reply_data[13]!=0||reply_data[15]!=0){
	/* We configured the segment with 0x0100 buffers */
	printf("The device currently has 0x%.2X%.2X%.2X%.2X%.2X%.2X%.2X%.2X buffers, but we configured it with 0x0100. This is an error\n",reply_data[6], reply_data[7],reply_data[8],reply_data[9],reply_data[10],reply_data[11],reply_data[12],reply_data[13]);
      }
      if(reply_data[16] != 0x00 || reply_data[17]!=0x00  || reply_data[18]!=0x60 ){
	/* We configured this segment with 0x000060 as the data size */
	printf("The device currently has data size equal to 0x%.2X%.2X%.2X, but we configured it to be 0x000060. This is an error\n",reply_data[16],reply_data[17],reply_data[18]);
      }
    }
    else{
      printf("In test 00, the MemExp_In_Sense_Config command failed.\n");
      free(reply_data);
      exit(-1);
    }	
    free(reply_data);
    /*********************
     *** Load buffer
     ********************/
    printf("Loading the buffer and storing data back\n");
    if (!(loaded_data = malloc(data_size_for_segment))){
      printf("main(): In test 00 we can't get memory\n");	
      exit(-ENOMEM);
    }
    if (!(data_buffer = malloc(data_size_for_segment))){
      printf("main(): In test 00 we can't get memory\n");	
      free(loaded_data);
      exit(-ENOMEM);
    }
    if (!(reply_data = malloc(STORE_OFFSET + data_size_for_segment))){
      printf("main(): In test 00 we can't get memory\n");	
      free(loaded_data);
      free(data_buffer);
      exit(-ENOMEM);
    }
    memset(sense,0,MAX_SENSE);
    memset(loaded_data,0,data_size_for_segment);
  
    buffer_number[0] = 0x00;	buffer_number[1] = 0x00;	buffer_number[2] = 0x00;
    buffer_number[3] = 0x00;	buffer_number[4] = 0x00;	buffer_number[5] = 0x00;
    buffer_number[6] = 0x00;	buffer_number[7] = 0x00;	buffer_number[8] = 0x20;
  
    allocation_length = STORE_OFFSET + data_size_for_segment;

    result = do_memexp_load_buffer(fd,
				   segment_number,			       
				   buffer_number,
				   allocation_length,
				   STORE_OFFSET + data_size_for_segment,
				   reply_data,
				   MAX_SENSE, 
				   sense,
				   0);
    if(result > 0){
    }
    else{
      printf("In test 00 the Load Buffer command Failed.\n");
      exit(-1);
    }
    memcpy(temp_physical_number,reply_data+16,8);
    memcpy(temp_sequence_number,reply_data+8,8);
    /**************
     **** Store data to buffer 
     **************/
    memset(sense,0,MAX_SENSE);
    memset(data_buffer,0,data_size_for_segment);
  
    buffer_number[0] = 0x00;	buffer_number[1] = 0x00;	buffer_number[2] = 0x00;
    buffer_number[3] = 0x00;	buffer_number[4] = 0x00;	buffer_number[5] = 0x00;
    buffer_number[6] = 0x00;	buffer_number[7] = 0x00;	buffer_number[8] = 0x20;
  
    in_use=0x80;
  
    strcpy(data_buffer,"One Ring to find them all and in darkness bind them");

    result = do_memexp_store_buffer(fd,
				    segment_number,			       
				    buffer_number,
				    in_use,
				    temp_sequence_number,
				    temp_physical_number,
				    data_size_for_segment,
				    data_buffer,
				    MAX_SENSE, 
				    sense,                                        
				    0);
    if(result > 0){
      printf("Data was stored to the buffer. Now loading the buffer again to check the data\n");
    }
    else{
      printf("In test 00 the Store Buffer Failed.\n");
      exit(-1);
    }
    /***************
     *** Load the buffer and compare the data
     **************/
    buffer_number[0] = 0x00;	buffer_number[1] = 0x00;	buffer_number[2] = 0x00;
    buffer_number[3] = 0x00;	buffer_number[4] = 0x00;	buffer_number[5] = 0x00;
    buffer_number[6] = 0x00;	buffer_number[7] = 0x00;	buffer_number[8] = 0x20;
  
    allocation_length = STORE_OFFSET + data_size_for_segment;

    result = do_memexp_load_buffer(fd,
				   segment_number,			       
				   buffer_number,
				   allocation_length,
				   STORE_OFFSET + data_size_for_segment,
				   reply_data,
				   MAX_SENSE, 
				   sense,                                        
				   0);
    if(result > 0){
    }
    else{
      printf("In test 00 the Load Buffer failed the second time we used it.\n");
      exit(-1);
    }
    memcpy(loaded_physical_number,reply_data+16,8);
    memcpy(loaded_sequence_number,reply_data+8,8);
    memcpy(loaded_data,reply_data+STORE_OFFSET,data_size_for_segment);
    result = memcmp(temp_physical_number,loaded_physical_number,8);
    if(result != 0){
      printf("In test 1 the physical number for buffer 0x000000000000000020 changed!!!\nThis is a severe error.");
      exit(-1);
    }
    result = memcmp(loaded_sequence_number,temp_sequence_number,8);
    if(result == 0 || result < 0){
      printf("The sequence number for buffer 0x000000000000000020 did not increase after a sucessfull store\nThis is a severe error");
      exit(-1);
    }
    result = memcmp(loaded_data,data_buffer,data_size_for_segment);
    if(result != 0){
      printf("Loaded data not identical to stored data!!!\n");
      exit(-1);
    }
    else{
      printf("The data written to the buffer is identical to the data read back.\n");
    }
    /*************
     *** Deleting segment with a select config with nr_buffers = 0 and data_size = 0 
     *************/
    printf("Now deleting the segment.\n");
    data_size_for_segment = 0x00;
    segment_number = 1;

    memset(sense,0,MAX_SENSE);
    buffer_number[0] = 0x00;	buffer_number[1] = 0x00;	buffer_number[2] = 0x00;
    buffer_number[3] = 0x00;	buffer_number[4] = 0x00;	buffer_number[5] = 0x00;
    buffer_number[6] = 0x00;	buffer_number[7] = 0x00;	buffer_number[8] = 0x00;
    parameter_length = 0x14;

    buffers_for_segment[0]=0x00;	buffers_for_segment[1]=0x00;	buffers_for_segment[2]=0x00;
    buffers_for_segment[3]=0x00;	buffers_for_segment[4]=0x00;	buffers_for_segment[5]=0x00;
    buffers_for_segment[6]=0x00;	buffers_for_segment[7]=0x00;

    result = do_memexp_out_select_config(fd,
					 segment_number,			       
					 buffer_number,
					 parameter_length,
					 buffers_for_segment,
					 data_size_for_segment,
					 MAX_SENSE, 
					 sense,
					 0);
    if(result > 0){
    }
    else{
      printf("In test 00 the MemExp_Out_Select_Config comamnd failed when trying to clear segment.\n");
      exit(-1);
    }
    /*****************
     **** Sense config to verify that the segment is gone
     ****************/
    memset(sense,0,MAX_SENSE);
    if (!(reply_data = malloc(18))){
      printf("main():  In test 00 we can't get memory\n");	
      exit(-ENOMEM);
    }

    buffer_number[0] = 0x00;	buffer_number[1] = 0x00;	buffer_number[2] = 0x00;
    buffer_number[3] = 0x00;	buffer_number[4] = 0x00;	buffer_number[5] = 0x00;
    buffer_number[6] = 0x00;	buffer_number[7] = 0x00;	buffer_number[8] = 0x00;

    result = do_memexp_in_sense_config(fd, 
				       segment_number, 
				       buffer_number,  
				       reply_data, 
				       MAX_SENSE, 
				       sense,
				       0);
    if(result > 0){
      if(reply_data[2] != 0x14 || reply_data[0] != 0x00 || reply_data[1] != 0x00){
	/* Right amount of bytes is Return data? */
	printf("The device does returns 0x%.2X%.2X%.2X bytes instead of 0x14 in a Sense Config. This is an error\n",reply_data[0],reply_data[1],reply_data[2]);
      }
      if(reply_data[4] != 0x00){
	/* There should be none segments now */
	printf("The device currently has 0x%.2X segemnt, but we disabled the segment. This is an error\n",reply_data[4]);
      }
      if(reply_data[12]!=0||reply_data[6]!=0||reply_data[7]!=0||reply_data[8]!=0||reply_data[9]!=0||reply_data[10]!=0||reply_data[11]!=0||reply_data[13]!=0){
	printf("The device currently has 0x%.2X%.2X%.2X%.2X%.2X%.2X%.2X%.2X buffers, but it should be 0. This is an error\n",reply_data[6], reply_data[7],reply_data[8],reply_data[9],reply_data[10],reply_data[11],reply_data[12],reply_data[13]);
      }
      if(reply_data[14] != 0x00 || reply_data[15]!=0x00  || reply_data[16]!=0x00 ){
	printf("The device currently has data size equal to 0x%.2X%.2X%.2X, but it should be disabled (0). This is an error\n",reply_data[14],reply_data[15],reply_data[16]);
      }
    }
    else{
      printf("In test 00, the MemExp_In_Sense_Config command failed.\n");
      free(reply_data);
      exit(-1);
    }	
    free(reply_data);

  }



  /************************************************************************************************
   ****
   ****		TEST	01 Test fullness byte 
   ****		
   ****		1)	Do a select config to configure one segment.
   ****		2)	Enable the segment
   ****		3)	Sense config and verify that that this indeed is what we configured
   ****		4)	Do Load-Store to all the buffers in this segment
   ****		5)	While doing this, verify that the fullness byte increases
   ****		6)	When all buffers are used, start freeing buffers
   ****		7)	As buffers are being freed, verify that fullness byte decreases
   ****		8)	Disable all buffers in segment
   ****		9)	Sense config on the segment and verify that all buffers are disabled	
   ****
   ***********************************************************************************************/
  if(TEST01){	
    /* Constants for this test */
    printf("\nTest 01: Testing fullness byte. Configuring one segment with 0x10000 buffers\n");
    data_size_for_segment = 0x40;
    segment_number = 1;

    /* Set up configuration, select config  */
    memset(sense,0,MAX_SENSE);
    buffer_number[0] = 0x00;	buffer_number[1] = 0x00;	buffer_number[2] = 0x00;
    buffer_number[3] = 0x00;	buffer_number[4] = 0x00;	buffer_number[5] = 0x00;
    buffer_number[6] = 0x00;	buffer_number[7] = 0x00;	buffer_number[8] = 0x00;
    parameter_length = 0x14;

    buffers_for_segment[0]=0x00;	buffers_for_segment[1]=0x00;	buffers_for_segment[2]=0x00;
    buffers_for_segment[3]=0x00;	buffers_for_segment[4]=0x00;	buffers_for_segment[5]=0x01;
    buffers_for_segment[6]=0x00;	buffers_for_segment[7]=0x00;

    result = do_memexp_out_select_config(fd,
					 segment_number,			       
					 buffer_number,
					 parameter_length,
					 buffers_for_segment,
					 data_size_for_segment,
					 MAX_SENSE, 
					 sense,                                        
					 0);
    if(result > 0){
    }
    else{
      printf("In test 1 MemExp_In_Sense_Config Failed.\n");
      exit(-1);
    }

    /* Enable segment */
    buffer_number[0] = 0x00;	buffer_number[1] = 0x00;	buffer_number[2] = 0x00;
    buffer_number[3] = 0x00;	buffer_number[4] = 0x00;	buffer_number[5] = 0x00;
    buffer_number[6] = 0x00;	buffer_number[7] = 0x00;	buffer_number[8] = 0x00;
    allocation_length = 0x00;

    result = do_memexp_out_enable_segment(fd,
					  segment_number,			       
					  buffer_number,
					  parameter_length,
					  MAX_SENSE, 
					  sense,                                        
					  0);
    if(result > 0){
    }
    else{
      printf("In test 1 MemExp Enable Segment Failed.\n");
      exit(-1);
    }
    /* Sense config */
    memset(sense,0,MAX_SENSE);
    if (!(reply_data = malloc(18))){
      printf("main():  In test 01 we can't get memory\n");	
      exit(-ENOMEM);
    }

    buffer_number[0] = 0x00;	buffer_number[1] = 0x00;	buffer_number[2] = 0x00;
    buffer_number[3] = 0x00;	buffer_number[4] = 0x00;	buffer_number[5] = 0x00;
    buffer_number[6] = 0x00;	buffer_number[7] = 0x00;	buffer_number[8] = 0x00;

    result = do_memexp_in_sense_config(fd, 
				       segment_number, 
				       buffer_number,  
				       reply_data, 
				       MAX_SENSE, 
				       sense,
				       0);
    if(result > 0){
      if(reply_data[2] != 0x14 || reply_data[0] != 0x00 || reply_data[1] != 0x00){
	/* Right amount of bytes is Return data? */
	printf("The device does returns 0x%.2X%.2X%.2X bytes instead of 0x000014 in a Sense Config. This is an error\n",reply_data[0],reply_data[1],reply_data[2]);
      }
      if(reply_data[4] != 0x01){
	/* There should only be one configured segment now */
	printf("The device currently has 0x%.2X segemnt, but we configured it with 0x01 segments. This is an error\n",reply_data[4]);
      }
      if(reply_data[8]!=0||reply_data[9]!=0||reply_data[10]!=0||reply_data[11]!=0||reply_data[12]!=0||reply_data[13]!=0x01||reply_data[14]!=0||reply_data[15]!=0){
	/* We configured the segment with 0x010000 buffers */
	printf("The device currently has 0x%.2X%.2X%.2X%.2X%.2X%.2X%.2X%.2X buffers, but we configured it with 0x010000. This is an error\n",reply_data[8], reply_data[9],reply_data[10],reply_data[11],reply_data[12],reply_data[13],reply_data[14],reply_data[15]);
      }
      if(reply_data[16] != 0x00 || reply_data[17]!=0x00  || reply_data[18]!=0x40 ){
	/* We configured this segment with 0x000040 as the data size */
	printf("The device currently has data size equal to 0x%.2X%.2X%.2X, but we configured it to be 0x000040. This is an error\n",reply_data[16],reply_data[17],reply_data[18]);
      }
    }
    else{
      printf("In test 01, MemExp_In_Sense_Config Failed\n");
      free(reply_data);
      exit(-1);
    }	
    free(reply_data);

    /************************************************
     ****
     ****	Loading and storing all the buffers in the segment
     ****	By doing this the fullness byte should increase from 0x00 (Empty) 
     ****	to 0xFF (full)
     ****
     ***********************************************/
    printf("Configuration OK. Now performing load-store to all buffers\n");
    l=0;
    c = d =0x00;
    for(j = 1; j<=256; j++){
      for(k = 1; k<=256; k++){
	if (!(loaded_data = malloc(data_size_for_segment))){
	  printf("main():  in test 01 we can't get memory\n");	
	  exit(-ENOMEM);
	}
	if (!(data_buffer = malloc(data_size_for_segment))){
	  printf("main():  in test 01 we can't get memory\n");	
	  free(loaded_data);
	  exit(-ENOMEM);
	}
	if (!(reply_data = malloc(STORE_OFFSET + data_size_for_segment))){
	  printf("main():  in test 01 can't get memory\n");	
	  free(loaded_data);
	  free(data_buffer);
	  exit(-ENOMEM);
	}
	memset(sense,0,MAX_SENSE);
	memset(loaded_data,0,data_size_for_segment);
	/*********
	 * Loading a buffer
	 ********/
	buffer_number[0] = 0x00;	buffer_number[1] = 0x00;	buffer_number[2] = 0x00;
	buffer_number[3] = 0x00;	buffer_number[4] = 0x00;	buffer_number[5] = 0x00;
	buffer_number[6] = 0x00;	buffer_number[7] = c;		buffer_number[8] = d;
  
	allocation_length = STORE_OFFSET + data_size_for_segment;

	result = do_memexp_load_buffer(fd,
				       segment_number,			       
				       buffer_number,
				       allocation_length,
				       STORE_OFFSET + data_size_for_segment,
				       reply_data,
				       MAX_SENSE, 
				       sense,                                        
				       0);
	if(result > 0){
	  /*printf("Buffer: 0x%.2X%.2X: reply_data[5] = 0x%.2X\n",c,d,reply_data[5]);*/
	  if(((reply_data[5]) > ((unsigned char)((l/256)+1))) || ((reply_data[5]) < ((unsigned char)((l/256)-1)))){
	    if((l<256) || (l>65279)){
	      if((l==255) && (reply_data[5] == 0x00)){
		printf("\tThere are buffers marked in use, but the fullness byte is 0x%.2X. This is an error\n",reply_data[5]);
		error++;
	      }
	      if((l==65530) && (reply_data[5] == 0xFF)){
		printf("\tThere are still buffers not in use, but the fullness byte is 0x%.2X. This is an error\n",reply_data[5]);
		error++;
	      }
	    }
	    else{
	      printf("After using %d of 65536 buffers the fullness byte is 0x%.2X. It should be 0x%.2X\n",l,reply_data[5], l, (unsigned char)((l)/256));
	      error++;
	    }
	  }
	}
	else{
	  printf("In test 01 Load Buffer Failed.\n");
	  exit(-1);
	}
	/*********
	 * Writing down the sequence and physical numbers
	 ********/
	memcpy(temp_physical_number,reply_data+16,8);
	memcpy(temp_sequence_number,reply_data+8,8);

	/********
	 * Storing data to the buffer
	 *******/
	memset(sense,0,MAX_SENSE);
	memset(data_buffer,0,data_size_for_segment);
  
	buffer_number[0] = 0x00;	buffer_number[1] = 0x00;	buffer_number[2] = 0x00;
	buffer_number[3] = 0x00;	buffer_number[4] = 0x00;	buffer_number[5] = 0x00;
	buffer_number[6] = 0x00;	buffer_number[7] = c;		buffer_number[8] = d;
	in_use=0x80;
	strcpy(data_buffer,"One Ring to rule them all, One Ring to find them");
	result = do_memexp_store_buffer(fd,
					segment_number,			       
					buffer_number,
					in_use,
					temp_sequence_number,
					temp_physical_number,
					data_size_for_segment,
					data_buffer,
					MAX_SENSE, 
					sense,                                        
					0);
	if(result > 0){
	}
	else{
	  printf("In test 01 Store Buffer Failed.\n");
	  exit(-1);
	}
	l++;
	d++;
      }/*d */
      c++;
    }/* c*/
    /********* 
     * ALL buffers in segment should now be filled. Verify that fullness == 0xFF 
     ********/ 
    memset(sense,0,MAX_SENSE);
    memset(loaded_data,0,data_size_for_segment);
    buffer_number[0] = 0x00;	buffer_number[1] = 0x00;	buffer_number[2] = 0x00;
    buffer_number[3] = 0x00;	buffer_number[4] = 0x00;	buffer_number[5] = 0x00;
    buffer_number[6] = 0x00;	buffer_number[7] = 0x00;       	buffer_number[8] = 0x00;
    allocation_length = STORE_OFFSET + data_size_for_segment;	
    result = do_memexp_load_buffer(fd,
				   segment_number,			       
				   buffer_number,
				   allocation_length,
				   STORE_OFFSET + data_size_for_segment,
				   reply_data,
				   MAX_SENSE, 
				   sense,                                        
				   0);
    if(result > 0){ 
      if(reply_data[5] != 0xFF){
	printf("The buffer should be 100\%, but fullness is only 0x%.2X\n",reply_data[5]);
	printf("We have filled %d buffers\n",l);
	error++;
      }
    }
    else{
      printf("In test 1 Load Buffer Failed.\n");
      exit(-1);
    }
	
    /**************************************************
     ***
     ***	Start marking buffers unused
     ***		This is done by load-store the buffers, but marking the buffers not in-use when storing
     ***
     ***************************************************/
    l=0;
    c = d =0x00;
    for(j = 1; j<=256; j++){
      for(k = 1; k<=256; k++){
	
	if (!(loaded_data = malloc(data_size_for_segment))){
	  printf("main():  can't get memory\n");	
	  exit(-ENOMEM);
	}
	if (!(data_buffer = malloc(data_size_for_segment))){
	  printf("main():  can't get memory\n");	
	  free(loaded_data);
	  exit(-ENOMEM);
	}
	if (!(reply_data = malloc(STORE_OFFSET + data_size_for_segment))){
	  printf("main():  can't get memory\n");	
	  free(loaded_data);
	  free(data_buffer);
	  exit(-ENOMEM);
	}
	memset(sense,0,MAX_SENSE);
	memset(loaded_data,0,data_size_for_segment);
	buffer_number[0] = 0x00;	buffer_number[1] = 0x00;	buffer_number[2] = 0x00;
	buffer_number[3] = 0x00;	buffer_number[4] = 0x00;	buffer_number[5] = 0x00;
	buffer_number[6] = 0x00;	buffer_number[7] = c;		buffer_number[8] = d;
	allocation_length = STORE_OFFSET + data_size_for_segment;
	result = do_memexp_load_buffer(fd,
				       segment_number,			       
				       buffer_number,
				       allocation_length,
				       STORE_OFFSET + data_size_for_segment,
				       reply_data,
				       MAX_SENSE, 
				       sense,                                        
				       0);
	if(result > 0){
	}
	else{
	  printf("In test 1 Load Buffer Failed.\n");
	  exit(-1);
	}

	memcpy(temp_physical_number,reply_data+16,8);
	memcpy(temp_sequence_number,reply_data+8,8);
	/***********
	* Store the buffer
	***********/
	memset(sense,0,MAX_SENSE);
	buffer_number[0] = 0x00;	buffer_number[1] = 0x00;	buffer_number[2] = 0x00;
	buffer_number[3] = 0x00;	buffer_number[4] = 0x00;	buffer_number[5] = 0x00;
	buffer_number[6] = 0x00;	buffer_number[7] = c;		buffer_number[8] = d;
	in_use=0;
	result = do_memexp_store_buffer(fd,
					segment_number,			       
					buffer_number,
					in_use,
					temp_sequence_number,
					temp_physical_number,
					0, /* We are not storing any data*/
					NULL,
					MAX_SENSE, 
					sense,                                        
					0);
	if(result > 0){
	}
	else{
	  printf("In test 1 freeing buffer by using Store Buffer with in_use = 0 failed.\n");
	  exit(-1);
	}
	l++;
	d++;
      }/*d */
      c++;
    }/* c*/
    /*********************************
    * ALL buffers in segment should now be marked not in-use. Verify that fullness = 0x00 
    **********************************/
    memset(sense,0,MAX_SENSE);
    memset(loaded_data,0,data_size_for_segment);
    
    buffer_number[0] = 0x00;	buffer_number[1] = 0x00;	buffer_number[2] = 0x00;
    buffer_number[3] = 0x00;	buffer_number[4] = 0x00;	buffer_number[5] = 0x00;
    buffer_number[6] = 0x00;	buffer_number[7] = 0x00;       	buffer_number[8] = 0x00;    
    allocation_length = STORE_OFFSET + data_size_for_segment; 
    result = do_memexp_load_buffer(fd,
				   segment_number,			       
				   buffer_number,
				   allocation_length,
				   STORE_OFFSET + data_size_for_segment,
				   reply_data,
				   MAX_SENSE, 
				   sense,                                        
				   0);
    if(result > 0){      
    }
    else{
      printf("In test 01 Load Buffer Failed.\n");
      exit(-1);
    }
    if(reply_data[5] != 0x00){
      printf("The buffer should be 0\%, but fullness byte is still 0x%.2X\n",reply_data[5]);
      error++;
    }
    if(error==0)
      printf("The fullness byte seems to be working correctly\n");
    else
      printf("There seems to be some errors with the fullness byte\n");
  } /*********************************************************************************************
   ****
   ****		End of test 01
   ****
   ***********************************************************************************************/


  /*******************************************************************************************
   ****
   **** 	Test 02 Dump test
   ****	1) Select a configuration
   ****		
   ****
   ******************************************************************************************/
  if(TEST02){
    printf("\nTest 02: Testing the dump-command\n");
    /*First configure segment as we want */

    /********
     *** MEMEXP_SELECT
     ********/
    memset(sense,0,MAX_SENSE);
    buffer_number[0] = 0x00;	buffer_number[1] = 0x00;	buffer_number[2] = 0x00;
    buffer_number[3] = 0x00;	buffer_number[4] = 0x00;	buffer_number[5] = 0x00;
    buffer_number[6] = 0x00;	buffer_number[7] = 0x00;	buffer_number[8] = 0x00;
    parameter_length = 0x14;

    buffers_for_segment[0]=0x00;	buffers_for_segment[1]=0x00;	buffers_for_segment[2]=0x00;
    buffers_for_segment[3]=0x00;	buffers_for_segment[4]=0x00;	buffers_for_segment[5]=0x00;
    buffers_for_segment[6]=0x01;	buffers_for_segment[7]=0x00;

    data_size_for_segment = 128;

    result = do_memexp_out_select_config(fd,
					 segment_number,			       
					 buffer_number,
					 parameter_length,
					 buffers_for_segment,
					 data_size_for_segment,
					 MAX_SENSE, 
					 sense,                                        
					 0);

    if(result > 0){
    }
    else{
      printf("In test 02 MemExp_In_Select_Config Failed.\n");
    }
    /**********
     **** memexp enable segment
     **********/

    buffer_number[0] = 0x00;	buffer_number[1] = 0x00;	buffer_number[2] = 0x00;
    buffer_number[3] = 0x00;	buffer_number[4] = 0x00;	buffer_number[5] = 0x00;
    buffer_number[6] = 0x00;	buffer_number[7] = 0x00;	buffer_number[8] = 0x00;
    allocation_length = 0x00;

    result = do_memexp_out_enable_segment(fd,
					  segment_number,			       
					  buffer_number,
					  parameter_length,
					  MAX_SENSE, 
					  sense,                                        
					  0);
    if(result > 0){
    }
    else{
      printf("In test 02 MemExp Enable Segment Failed. \n");
    }		
    /************************************************
     ***
     ***	Test the dump command on a segment where we still have not marked
     ***	any buffers as in-use
     ***
     ************************************************/
    printf("Trying dump-command wiht no buffers in use\n");
    error =0;
    nr_buffer = 10;
    memset(buffer_number+1,0,8); /* What number do we want to start with */
    if (!(reply_data = malloc(((data_size_for_segment + 28)* nr_buffer) + 8))){
      printf("main():  can't get memory\n");	
      exit(-ENOMEM);
    }
    memset(reply_data,0,((data_size_for_segment+28) * nr_buffer) + 8);
    if (!(dump_data = malloc(((data_size_for_segment+28) * nr_buffer) + 8))){ /*Buffer for how the dump should be*/
      printf("main():  can't get memory\n");
      free(reply_data);
      exit(-ENOMEM);
    }  
     result = do_memexp_dump_buffer(fd,
				   segment_number,			       
				   buffer_number,
				   data_size_for_segment,
				   nr_buffer,
				   reply_data,
				   MAX_SENSE, 
				   sense,                                        
				   0);
    if(result > 0){
      if((reply_data[0] != 0) || (reply_data[1] != 0) || (reply_data[2] != 0x08)){
	/* Checking reply-length */
	printf("\tError in dump command: When asking for 10 buffers and none are valid, the return length is 0x%.2X%.2X%.2X. It should be 0x000008\n",reply_data[0],reply_data[1],reply_data[2]);
	error++;
      }
      if(reply_data[3] != 0x01){
	/* The action should be 01 */
	printf("\tError in dump command: The Service Action byte in reply should be 0x01, It is 0x%.2X\n",reply_data[3]);
	error++;
      }
      if(reply_data[4] != 0x00){
	printf("\tError in dump command: When there are no valid buffers, the more bit should not be set. It is....\n");
	error++;
      }
    }
    else{
      printf("In test 02 MemExp Dump failed when trying to dump before any buffers in use\n");
    }     
    free(reply_data);
    if(error == 0)
      printf("\tOK\n");
    error = 0;
    /************************************* 
     ***
     ***	Fill up 2 buffers in the segent with data
     ***	Since we are going to dump these, we want to use two buffers that 
     ***	are not next to eachother. We use buffer number 0x10 and 0x20
     ***
     *************************************/

    /*********
     **** Memexp load buffer
     ********/   
    if (!(reply_data = malloc(STORE_OFFSET + data_size_for_segment))){/* Storing reply data */
      printf("main(): Test 02 can't get memory\n");	
      exit(-ENOMEM);
    }
    if(!(data_buffer = malloc(data_size_for_segment))){/* Buffer for storing outgoing data */
      printf("main():  Test 02 can't get memory\n");	
      free(reply_data);
      free(loaded_data);
      exit(-ENOMEM);
    }
    memset(sense,0,MAX_SENSE);
    buffer_number[0] = 0x00;	buffer_number[1] = 0x00;	buffer_number[2] = 0x00;
    buffer_number[3] = 0x00;	buffer_number[4] = 0x00;	buffer_number[5] = 0x00;
    buffer_number[6] = 0x00;	buffer_number[7] = 0x00;	buffer_number[8] = 0x10;
    allocation_length = STORE_OFFSET + data_size_for_segment;
    result = do_memexp_load_buffer(fd,
				   segment_number,			       
				   buffer_number,
				   allocation_length,
				   STORE_OFFSET + data_size_for_segment,
				   reply_data,
				   MAX_SENSE, 
				   sense,                                        
				   0);
    if(result >0){
    }
    else{
      printf("In test 02 MemExp Load Buffer Failed\n");
    }
    memcpy(loaded_physical_number,reply_data+16,8);
    memcpy(loaded_physical_number_10,reply_data+16,8);
    memcpy(loaded_sequence_number,reply_data+8,8);
    
    memset(sense,0,MAX_SENSE);
    memset(data_buffer,0,data_size_for_segment);    
    buffer_number[0] = 0x00;	buffer_number[1] = 0x00;	buffer_number[2] = 0x00;
    buffer_number[3] = 0x00;	buffer_number[4] = 0x00;	buffer_number[5] = 0x00;
    buffer_number[6] = 0x00;	buffer_number[7] = 0x00;	buffer_number[8] = 0x10;
    in_use=0x80;
    strcpy(data_buffer,"One Ring to rule them all, One Ring to find them");
    result = do_memexp_store_buffer(fd,
				    segment_number,			       
				    buffer_number,
				    in_use,
				    loaded_sequence_number,
				    loaded_physical_number,
				    data_size_for_segment,
				    data_buffer,
				    MAX_SENSE, 
				    sense,                                        
				    0);
    if(result >0){
    }
    else{                                                                                                                                                                                                                                                                                                                                                       
      printf("In test 02, MemExp Store failed\n");
    }       
    memset(sense,0,MAX_SENSE);
    buffer_number[0] = 0x00;	buffer_number[1] = 0x00;	buffer_number[2] = 0x00;
    buffer_number[3] = 0x00;	buffer_number[4] = 0x00;	buffer_number[5] = 0x00;
    buffer_number[6] = 0x00;	buffer_number[7] = 0x00;	buffer_number[8] = 0x20;
    allocation_length = STORE_OFFSET + data_size_for_segment;
    result = do_memexp_load_buffer(fd,
				   segment_number,			       
				   buffer_number,
				   allocation_length,
				   STORE_OFFSET + data_size_for_segment,
				   reply_data,
				   MAX_SENSE, 
				   sense,                                        
				   0);
    if(result >0){
    }
    else{
      printf("In test 02 MemExp Load Buffer Failed\n");
    }
    memcpy(loaded_physical_number,reply_data+16,8);
    memcpy(loaded_physical_number_20,reply_data+16,8);
    memcpy(loaded_sequence_number,reply_data+8,8);
    memset(sense,0,MAX_SENSE);
    memset(data_buffer,0,data_size_for_segment);    
    buffer_number[0] = 0x00;	buffer_number[1] = 0x00;	buffer_number[2] = 0x00;
    buffer_number[3] = 0x00;	buffer_number[4] = 0x00;	buffer_number[5] = 0x00;
    buffer_number[6] = 0x00;	buffer_number[7] = 0x00;	buffer_number[8] = 0x20;
    in_use=0x80;
    strcpy(data_buffer,"One Ring to bring them all and in in darkenss bind them");
    result = do_memexp_store_buffer(fd,
				    segment_number,			       
				    buffer_number,
				    in_use,
				    loaded_sequence_number,
				    loaded_physical_number,
				    data_size_for_segment,
				    data_buffer,
				    MAX_SENSE, 
				    sense,                                        
				    0);
    if(result >0){
    }
    else{
      printf("In test 02, MemExp Store failed\n");
    }  
    free(reply_data);
    /********
     **** 			TESTING THE DUMP COMMAND
     ****
     **** 1) Dump 0 buffers
     ****    	This should return a byte number of 0x000006 and more-bit of 1, indicating that none buffers 
     ****	were dumped and more buffers can be dummped	
     ****
     **** 2) Dump only one buffer.
     ****	This should return a byte number of 128 + 28 + 8 = 164 = 0xA4 bytes.
     ****	In addition the more bit should be set. However, since we have 2 in-use buffers in this segment
     ****	and we don't know which one has the "higher" physical number, we have to do this twice. (The dump will
     ****	dump the buffers with equal or smaller (mat) physical buffer id. 
     ****	
     **** 3) Dump both buffers. 
     ****	This should return 128 + 128 + 28 + 28 + 8 = 320 = 0x140 bytes.
     ****	In addition the more bit should not be set.
     ****
     **** 4) Dump 3 buffers.
     ****	Again, 0x13A bytes should be retunred and the more bit should not be set
     *******/
    printf("Trying to dump 0 buffers when 2 buffers are in use\n");
    error =0;
    nr_buffer = 0;
    memcpy(buffer_number+1,loaded_physical_number,8); /* What number do we want to start with */
    if (!(reply_data = malloc(((data_size_for_segment + 28)* nr_buffer) + 8))){
      printf("main():  can't get memory\n");	
      exit(-ENOMEM);
    }
    memset(reply_data,0,((data_size_for_segment+28) * nr_buffer) + 8);
    if (!(dump_data = malloc(((data_size_for_segment+28) * nr_buffer) + 8))){ /*Buffer for how the dump should be*/
      printf("main():  can't get memory\n");
      free(reply_data);
      exit(-ENOMEM);
    }  
     result = do_memexp_dump_buffer(fd,
				   segment_number,			       
				   buffer_number,
				   data_size_for_segment,
				   nr_buffer,
				   reply_data,
				   MAX_SENSE, 
				   sense,                                        
				   0);
    if(result > 0){
      if((reply_data[0] != 0) || (reply_data[1] != 0) || (reply_data[2] != 0x08)){
	/* Checking reply-length */
	printf("\tError in dump command: When asking for 0 buffers, the return length is 0x%.2X%.2X%.2X. It should be 0x000006\n",reply_data[0],reply_data[1],reply_data[2]);
	error++;
      }
      if(reply_data[3] != 0x01){
	/* The action should be 01 */
	printf("\tError in dump command: The Service Action byte in reply should be 0x01, It is 0x%.2X\n",reply_data[3]);
	error++;
      }
      if(reply_data[4] != 0x80){
	printf("\tError in dump command: When dumping 0 buffers, the more bit should be set. It is not....\n");
	error++;
      }
    }
    else{
      printf("In test 02 MemExp Dump failed when dumping 0 buffers\n");
    }     
    free(reply_data);
    if(error == 0)
      printf("\tOK\n");
    error = 0;
    /*****
     *** Dumping 1 buffer
     ****/
    printf("Trying to dump 1 buffer when 2 buffers are in use\n");
    nr_buffer = 1;
    memcpy(buffer_number+1,loaded_physical_number_20,8); /* What number do we want to start with */
    if (!(reply_data = malloc(((data_size_for_segment + 28)* nr_buffer) + 8))){
      printf("main():  can't get memory\n");	
      exit(-ENOMEM);
    }
    memset(reply_data,0,((data_size_for_segment+28) * nr_buffer) + 8);
    if (!(dump_data = malloc(((data_size_for_segment+28) * nr_buffer) + 8))){ /*Buffer for how the dump should be*/
      printf("main():  can't get memory\n");
      free(reply_data);
      exit(-ENOMEM);
    }  
     result = do_memexp_dump_buffer(fd,
				   segment_number,			       
				   buffer_number,
				   data_size_for_segment,
				   nr_buffer,
				   reply_data,
				   MAX_SENSE, 
				   sense,                                        
				   0);
    if(result > 0){
      if((reply_data[0] != 0) || (reply_data[1] != 0) || (reply_data[2] != 0xA4)){
	/* Checking reply-length */
	printf("\tError in dump command: When asking for 1 buffer, the return length is 0x%.2X%.2X%.2X. It should be 0x0000A4\n",reply_data[0],reply_data[1],reply_data[2]);
	error++;
      }
      if(reply_data[3] != 0x01){
	/* The action should be 01 */
	printf("\tError in dump command: The Service Action byte in reply should be 0x01, It is 0x%.2X\n",reply_data[3]);
	error++;
      }
      if(reply_data[4] != 0x80){
	/******************************
	 ***** We have 2 buffers in here, and we asked for one of them. If the other buffer
	 ***** have a "higher" physical ID, the more bit is not set. So we will issue a dump starting with the other buffer,
	 ***** and this time the more-bit should be set
	 *****************************/
	    nr_buffer = 1;
	    memcpy(buffer_number+1,loaded_physical_number_10,8); /* What number do we want to start with */
	    memset(reply_data,0,((data_size_for_segment+28) * nr_buffer) + 8);
	    result = do_memexp_dump_buffer(fd,
					   segment_number,			       
					   buffer_number,
					   data_size_for_segment,
					   nr_buffer,
					   reply_data,
					   MAX_SENSE, 
					   sense,                                        
					   0);
	    if(result > 0){
	      if((reply_data[0] != 0) || (reply_data[1] != 0) || (reply_data[2] != 0xA0)){
		/* Checking reply-length */
		printf("\tError in dump command: When asking for 1 buffer, the return length is 0x%.2X%.2X%.2X. It should be 0x0000A0\n",reply_data[0],reply_data[1],reply_data[2]);
		error++;
	      }
	      if(reply_data[3] != 0x01){
		/* The action should be 01 */
		printf("\tError in dump command: The byte 4 in reply should be 0x01, It is 0x%.2X\n",reply_data[3]);
		error++;
	      }
	      if(reply_data[4] != 0x80){
		printf("\tError in dump command: There are 2 buffers, we are dumping one and the more-bit is NOT set.\n");
		error++;
	      }
	    }
      }
    }
    else{
      printf("In test 02 MemExp Dump failed when bumping 1 buffer\n");
    }      
    free(reply_data);
    if(error == 0)
      printf("\tOK\n");
    error =0;
    /*******
     *** Dump both buffers
     *******/
    printf("Trying to dump 2 buffers when 2 buffers are in use\n");
    nr_buffer = 2;
    memcpy(buffer_number+1,loaded_physical_number,8); /* What number do we want to start with */
    if (!(reply_data = malloc(((data_size_for_segment+28) * nr_buffer) + 8))){
      printf("main():  can't get memory\n");	
      exit(-ENOMEM);
    }
    memset(reply_data,0,((data_size_for_segment+28) * nr_buffer) + 8);
    if (!(dump_data = malloc(((data_size_for_segment+28) * nr_buffer) + 8))){ /*Buffer for how the dump should be*/
      printf("main():  can't get memory\n");
      free(reply_data);
      exit(-ENOMEM);
    }  
     result = do_memexp_dump_buffer(fd,
				   segment_number,			       
				   buffer_number,
				   data_size_for_segment,
				   nr_buffer,
				   reply_data,
				   MAX_SENSE, 
				   sense,                                        
				   0);
    if(result > 0){
      if((reply_data[0] != 0) || (reply_data[1] != 0x01) || (reply_data[2] != 0x40)){
	/* Checking reply-length */
	printf("\tError in dump command: When asking for 2 buffers, the return length is 0x%.2X%.2X%.2X. It should be 0x000140\n",reply_data[0],reply_data[1],reply_data[2]);
	error++;
      }
      if(reply_data[3] != 0x01){
	/* The action should be 01 */
	printf("\tError in dump command: The Service Action byte in reply should be 0x01, It is 0x%.2X\n",reply_data[3]);
	error++;
      }
      if(reply_data[4] != 0x00){
	printf("\tError in dump command: When dumping 2 of 2 buffers, the more bit should NOT be set. It is....\n");
      }
    }
    else{
      printf("In test 02 MemExp Dump failed when trying to dump 2 buffers\n");
    }
    free(reply_data);
    if(error == 0)
      printf("\tOK\n");
    error =0;
    /*********
     *** Try to dump more than 2 buffers
     ********/
    printf("Trying to dump 5 buffers when 2 are in use\n");
    nr_buffer = 5;
    memcpy(buffer_number+1,loaded_physical_number,8); /* What number do we want to start with */
    if (!(reply_data = malloc(((data_size_for_segment+28) * nr_buffer) + 8))){
      printf("main():  can't get memory\n");	
      exit(-ENOMEM);
    }
    memset(reply_data,0,((data_size_for_segment+28) * nr_buffer) + 8);
    if (!(dump_data = malloc(((data_size_for_segment+28) * nr_buffer) + 8))){ /*Buffer for how the dump should be*/
      printf("main():  can't get memory\n");
      free(reply_data);
      exit(-ENOMEM);
    }  
     result = do_memexp_dump_buffer(fd,
				   segment_number,			       
				   buffer_number,
				   data_size_for_segment,
				   nr_buffer,
				   reply_data,
				   MAX_SENSE, 
				   sense,                                        
				   0);
    if(result > 0){
      if((reply_data[0] != 0) || (reply_data[1] != 0x01) || (reply_data[2] != 0x40)){
	/* Checking reply-length */
	printf("\tError in dump command: Trying to dump more than 2 buffers. The return length is 0x%.2X%.2X%.2X. It should be 0x000140\n",reply_data[0],reply_data[1],reply_data[2]);
	error++;
      }
      if(reply_data[3] != 0x01){
	/* The action should be 01 */
	printf("\tError in dump command: The Service Action byte in reply should be 0x01, It is 0x%.2X\n",reply_data[3]);
	error++;
      }
      if(reply_data[4] != 0x00){
	printf("\tError in dump command: When dumping more than 2 of 2 buffers, the more bit should NOT be set. It is....\n");
	error++;
      }
    }
    else{
      printf("In test 02 MemExp Dump failed when trying to dump more than 2 buffers\n");
    }
    if(error == 0)
      printf("\tOK, We dumped 2 buffers\n");  
    free(reply_data);

    /************
     **** Clearing all buffers in the segment by sending a select config with the same configuration
     ****	All buffers should now be marked not in-use 
     ***********/
    memset(sense,0,MAX_SENSE);
    buffer_number[0] = 0x00;	buffer_number[1] = 0x00;	buffer_number[2] = 0x00;
    buffer_number[3] = 0x00;	buffer_number[4] = 0x00;	buffer_number[5] = 0x00;
    buffer_number[6] = 0x00;	buffer_number[7] = 0x00;	buffer_number[8] = 0x00;
    parameter_length = 0x14;
    buffers_for_segment[0]=0x00;	buffers_for_segment[1]=0x00;	buffers_for_segment[2]=0x00;
    buffers_for_segment[3]=0x00;	buffers_for_segment[4]=0x00;	buffers_for_segment[5]=0x00;
    buffers_for_segment[6]=0x01;	buffers_for_segment[7]=0x00;
    data_size_for_segment = 128;
    result = do_memexp_out_select_config(fd,
					 segment_number,			       
					 buffer_number,
					 parameter_length,
					 buffers_for_segment,
					 data_size_for_segment,
					 MAX_SENSE, 
					 sense,                                        
					 0);
    if(result > 0){
    }
    else{
      printf("In test 02 final MemExp_In_Select_Config failed.\n");
    }	     
  }/******************
   ***
   *** End of test 02
   ***
   *****************/

  /*****************************************************************************************
   *****
   ***** TEST IF SEQUENCE NUMBER INCREASES AND PHYSICAL BUFFER STAYS CONSTANT
   *****
   *****  1) Select and configure a segment
   *****  2) In a loop that runs for a long time:
   *****	-Load one buffer
   *****	-Store the buffer
   *****	-Load the buffer and verify that the sequence number has increased since the first
   *****	 load and that the physical number is the same
   *****	-Compare the loaded data with the data we stored
   ***** 3) Clear the segment
   *****
   ********************************************************************************************/
  if(TEST03){
    /********
     *** 
     *** Configuring a segment with 0x100 buffer each 64 bytes
     ***
     ********/
    printf("\nNow verifying that sequence number increases with stores\n");
    memset(sense,0,MAX_SENSE);
    buffer_number[0] = 0x00;	buffer_number[1] = 0x00;	buffer_number[2] = 0x00;
    buffer_number[3] = 0x00;	buffer_number[4] = 0x00;	buffer_number[5] = 0x00;
    buffer_number[6] = 0x00;	buffer_number[7] = 0x00;	buffer_number[8] = 0x00;
    parameter_length = 0x14;
    data_size_for_segment = 64;

    buffers_for_segment[0]=0x00;	buffers_for_segment[1]=0x00;	buffers_for_segment[2]=0x00;
    buffers_for_segment[3]=0x00;	buffers_for_segment[4]=0x00;	buffers_for_segment[5]=0x00;
    buffers_for_segment[6]=0x01;	buffers_for_segment[7]=0x00;

    result = do_memexp_out_select_config(fd,
					 segment_number,			       
					 buffer_number,
					 parameter_length,
					 buffers_for_segment,
					 data_size_for_segment,
					 MAX_SENSE, 
					 sense,                                        
					 0);
    if(result > 0){
    }
    else{
      printf("In test 03 MemExp_In_Select_Config Failed.\n");
    }
    /**********
     **** Enabling the segemnt
     **********/
    buffer_number[0] = 0x00;	buffer_number[1] = 0x00;	buffer_number[2] = 0x00;
    buffer_number[3] = 0x00;	buffer_number[4] = 0x00;	buffer_number[5] = 0x00;
    buffer_number[6] = 0x00;	buffer_number[7] = 0x00;	buffer_number[8] = 0x00;
    allocation_length = 0x00;
    result = do_memexp_out_enable_segment(fd,
					  segment_number,			       
					  buffer_number,
					  parameter_length,
					  MAX_SENSE, 
					  sense,                                        
					  0);
    if(result > 0){
    }
    else{
      printf("In test 03 MemExp Enable Segment Failed.\n");
    }
    /*****************************
     *****
     ***** Starting loop to load, store and laod buffers
     *****	Each buffer is loaded to get sequence and physical number, then stored with data and loaded to comapre the data
     *****
     *******************************/
    for(j=0;j<20000;j++){
    /*********
     **** Memexp load buffer
     ********/
      if (!(reply_data = malloc(STORE_OFFSET + data_size_for_segment))){
	printf("main():  can't get memory\n");	
	exit(-ENOMEM);
      }
      if(!(data_buffer = malloc(data_size_for_segment))){/* Buffer for storing outgoing data */
	printf("main():  Test 03 can't get memory\n");	
	free(reply_data);
	exit(-ENOMEM);
      }
      if (!(loaded_data = malloc(data_size_for_segment))){
	  printf("main():  can't get memory\n");	
	  exit(-ENOMEM);
	}
    memset(sense,0,MAX_SENSE);
    memset(loaded_data,0,data_size_for_segment);
    memset(data_buffer,0,data_size_for_segment);
    memset(reply_data,0,STORE_OFFSET + data_size_for_segment);
 
    buffer_number[0] = 0x00;	buffer_number[1] = 0x00;	buffer_number[2] = 0x00;
    buffer_number[3] = 0x00;	buffer_number[4] = 0x00;	buffer_number[5] = 0x00;
    buffer_number[6] = 0x00;	buffer_number[7] = 0x00;	buffer_number[8] = 0x01;
    allocation_length = STORE_OFFSET + data_size_for_segment;

    result = do_memexp_load_buffer(fd,
				   segment_number,			       
				   buffer_number,
				   allocation_length,
				   STORE_OFFSET + data_size_for_segment,
				   reply_data,
				   MAX_SENSE, 
				   sense,                                        
				   0);
    if(result > 0){
    }
    else{
      printf("In test 03 first Load buffer in loop  failed\n");
    }
    memcpy(loaded_physical_number,reply_data+16,8);
    memcpy(loaded_sequence_number,reply_data+8,8);
    memcpy(loaded_data,reply_data+STORE_OFFSET,data_size_for_segment);
    if(0){
      printf("Load Buffer OK:\n");
      printf("\tLength of returend data: 0x%.2X%.2X%.2X\n",reply_data[0],reply_data[1],reply_data[2]);
      printf("\tLoaded sequence number: 0x");
      for(i=0;i<8;i++)
	printf("%.2X",loaded_sequence_number[i]);
      printf("\n");
      printf("\tLoaded physical number: 0x");
      for(i=0;i<8;i++)
	printf("%.2X",loaded_physical_number[i]);
      printf("\n");
      printf("\tLoaded buffer in use: 0x%.2X\n",reply_data[4]);
      printf("\tFullness: %d\%\n",reply_data[5]);
      printf("\tThe buffer contains: ");
      for(i=0;i<data_size_for_segment;i++)
	printf("0x%.2X ",loaded_data[i]);
      printf("\n");
    }
    free(reply_data);
    /**************
     **** Store Buffer
     *************/
    memset(sense,0,MAX_SENSE);
    memset(data_buffer,0,data_size_for_segment);
    buffer_number[0] = 0x00;	buffer_number[1] = 0x00;	buffer_number[2] = 0x00;
    buffer_number[3] = 0x00;	buffer_number[4] = 0x00;	buffer_number[5] = 0x00;
    buffer_number[6] = 0x00;	buffer_number[7] = 0x00;	buffer_number[8] = 0x01;

    in_use=0x80;

    /*** Fill the buffer with a quote and a number ***/
    strcpy(data_buffer,"One Ring to rule them all, One Ring to find them");
    memset(data_buffer+data_size_for_segment-4,j,1);

    result = do_memexp_store_buffer(fd,
				    segment_number,			       
				    buffer_number,
				    in_use,
				    loaded_sequence_number,
				    loaded_physical_number,
				    data_size_for_segment,
				    data_buffer,
				    MAX_SENSE, 
				    sense,                                        
				    0);


    /*********
     **** Memexp load buffer
     ********/
    memset(sense,0,MAX_SENSE);
    memset(loaded_data,0,data_size_for_segment);

    if (!(reply_data = malloc(STORE_OFFSET + data_size_for_segment))){
      printf("main():  can't get memory\n");	
      exit(-ENOMEM);
    }
    buffer_number[0] = 0x00;	buffer_number[1] = 0x00;	buffer_number[2] = 0x00;
    buffer_number[3] = 0x00;	buffer_number[4] = 0x00;	buffer_number[5] = 0x00;
    buffer_number[6] = 0x00;	buffer_number[7] = 0x00;	buffer_number[8] = 0x01;

    allocation_length = STORE_OFFSET + data_size_for_segment;

    result = do_memexp_load_buffer(fd,
				   segment_number,			       
				   buffer_number,
				   allocation_length,
				   STORE_OFFSET + data_size_for_segment,
				   reply_data,
				   MAX_SENSE, 
				   sense,                                        
				   0);

    memcpy(temp_physical_number,reply_data+16,8);
    memcpy(temp_sequence_number,reply_data+8,8);
    memcpy(loaded_data,reply_data+STORE_OFFSET,data_size_for_segment);
 
    result = memcmp(temp_physical_number,loaded_physical_number,8);
    if(result != 0)
      printf("In test 03 the physical number for a buffer got changed. This is a sever error!!!\n");
     result = memcmp(temp_sequence_number,loaded_sequence_number,8);
     if(result == 0 || result < 0)
       printf("In test 03 the sequence number for a biffer did not increase after a store. This is a sever error!!!\n");
    result = memcmp(loaded_data,data_buffer,data_size_for_segment);
    if(result != 0)
      printf("Loaded data not identical to stored data!!!\n");    
    }
  }
  /*************
   *** END OF TEST 03
   *************/



  scsi_close(fd);
  exit(0);

}









