#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <getopt.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <sys/stat.h>

char *where = NULL;
int type = 1;
int do_writes = 0;

void map_file(char *name)
{
	struct stat st;
	int fd = open(name, O_RDWR);
	if (fd < 0) {
		perror(name);
		exit(-1);
	}
	if (fstat(fd, &st) < 0) {
		perror("fstat");
		exit(-1);
	}
	if (st.st_size < 32) {
		fprintf(stderr, "File has to be at least 32 bytes long\n");
		exit(-1);
	}
	where = mmap(NULL, st.st_size, do_writes ? PROT_READ|PROT_WRITE : PROT_READ, type ? MAP_SHARED : MAP_PRIVATE, fd, 0);
	if (where == MAP_FAILED) {
		perror("mmap");
		exit(-1);
	}
	printf("File mapped successfully. Sleeping\n");
}

void poke_mapping(void)
{
static unsigned char buf[32];

	printf("About to poke mapping...\n");
	if (do_writes)
		memset(where, 0, 32);
	else
		memcpy(buf, where, 32);
	printf("Mapping poked. Sleeping\n");
}

int main(int argc, char *argv[])
{
	int opt;
	int when = 20;

	while((opt = getopt(argc, argv, "pt:?")) != EOF) {
		switch(opt) {
			case 'p':
				type = 0;
				break;
			case 't':
				when = atoi(optarg);
				break;
			case 'w':
				do_writes = 1;
				break;
			case '?':
			default:
usage:
				fprintf(stderr, "Usage: mmt [-t <secs>] file\n");
				exit(-1);
		}
	}

	if ((argc - optind) != 1)
		goto usage;

	map_file(argv[optind]);

	while(1) {
		sleep(when);
		poke_mapping();
	}

	return 0;
}

