/******************************************************************************
 *    Copyright 2000-2001 Sistina Software, Inc.
 *
 *    This is free software released under the GNU General Public License.
 *    There is no warranty for this software.  See the file COPYING for
 *    details.
 *
 *    See the file CONTRIBUTORS for a list of contributors.
 *
 *    This file is maintained by:
 *      Mike C Tilstra <conrad@sistina.com>
 *
 * This header file contains the info that it used by lock_state in the
 * memexp module.  We make a copy here sine the header files with the
 * needed info have many things that don't play nice in user space, and
 * other general incompatibilities.
 */
#ifndef __lsm_info_h__
#define __lsm_info_h__

/* from lm_interface.h */
#define LM_TYPE_RESERVED       (0x00)
#define LM_TYPE_CIDBUF         (0x01)
#define LM_TYPE_MOUNT          (0x02)
#define LM_TYPE_NONDISK        (0x03)
#define LM_TYPE_INODE          (0x04)
#define LM_TYPE_RGRP           (0x05)
#define LM_TYPE_META           (0x06)
#define LM_TYPE_NOPEN          (0x07)
#define LM_TYPE_FLOCK          (0x08)
#define LM_TYPE_PLOCK          (0x09)
#define LM_TYPE_PLOCK_HEAD     (0x0a)
#define LM_TYPE_LVB_MASK       (0x80)



/* from memexp.h  (not the server's memexp.h)*/
#define STATE_UNLOCKED          (0)
#define STATE_EXCLUSIVE         (1)
#define STATE_DEFERRED          (2)
#define STATE_SHARED            (3)

#define NO_CONV                 (0)
#define YES_CONV                (1)

#define CID_NOT_USED            (0)
#define CID_USED                (1)
#define CID_STOMITH             (2)
#define CID_STOMITH_DONE        (3)
#define CID_EXPIRED_MARKING     (4)
#define CID_EXPIRED             (5)
#define CID_RESETING            (6)

#define OWNER_NONE              (65535)

#define MOUNT_STATE_NULL        (0)
#define MOUNT_STATE_INIT        (1)
#define MOUNT_STATE_RESET       (2)
#define MOUNT_STATE_OK          (3)



/* from lock_state.h */
/**** *** Defines Local to state machine *** ****/

#define INUSE                   (1 << 7)
#define NINUSE                  (0)

/**** *** *** *** Data Structures *** *** *** ****/

#define MAX_CLIENTS 32

struct lsm_lock_s{
  u8  state; 
  u8  conv:4, lvb_perm:2, lvb_lock:2; 
  u16 expired_holders; 
  u16 live_holders;
  u16 lvb_holders;
  u8  holders[MAX_CLIENTS / 8]; 
  u8  lvb[MAX_CLIENTS / 8];
  u8  expired[MAX_CLIENTS / 8]; 
  u8  conversion[MAX_CLIENTS / 8]; 
}; typedef struct lsm_lock_s lsm_lock;


struct cidbuf_s
{
  uint64 counter;         /* increment this to prove you're alive */
  uint16 cleaner;         /* put your cid here to claim recovery rights */
  unsigned char inuse;    /* state of buffer */
  unsigned char dead;     /* local use only - TRUE if counter hasn't changed */
};
typedef struct cidbuf_s cidbuf_t;

struct mlbuf_s
{
  uint16 state;
  uint16 owner;
};
typedef struct mlbuf_s mlbuf_t;


/* hopefully lvb.h will stay fairly clean...... */
typedef void lm_lvb_t; /* hack*/
#include "lvb.h"

#endif /*__lsm_info_h__*/
