/******************************************************************************
 *    Copyright 2000-2001 Sistina Software, Inc.
 *
 *    This is free software released under the GNU General Public License.
 *    There is no warranty for this software.  See the file COPYING for
 *    details.
 *
 *    See the file CONTRIBUTORS for a list of contributors.
 *
 *    This file is maintained by:
 *      Mike C Tilstra <conrad@sistina.com>
 */
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/uio.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <readline/readline.h>
#include <readline/history.h>
#include <signal.h>
#include <string.h>
#include <errno.h>
#include "global.h"
#include "memexp.h"

#include "lsm_info.h"

static unsigned int cmdport = DEF_PORT;

#undef die
void die(char *s)
{
   fprintf(stderr, "%s: %m\n", s);
   exit(1);
}

void dump_buffer(void *buf, int len)
{
   int i;
   u8 *c = buf;

   for(i=0; i<len; i++) {
      if( i % 16 == 0 ) printf("\n%p: ", &c[i]);
      printf("%2x ", c[i]);
   }
   printf("\n");
}

/**
 * help - prints help
 * @void: 
 * 
 * should get this to print a list of cmds, then have the user do a
 * `help <cmd>' to get the details.
 * 
 * Returns: void
 */
void help(char *s)
{
   if(!s) {
      printf("Commands: \n");
      printf(" help\n");
      printf(" load {<bid>|<lock number> <lock type>}\n");
      printf(" store <bid> <InUse> <Seq> <pbn> <hex data>\n");
      printf(" dump <pbn> <count>\n");
      printf(" verify <pbn>\n");
      printf(" modepage [<Number buffers> <Memory Buffer Size>]\n");
      printf(" Reset\n");
      printf(" Decode [True|False]\n");
      printf(" Ban <IP>\n");
      printf(" Unban <IP>\n");
      printf(" Stats\n");
      printf(" Exit\n");
      printf(" Quit\n");
      printf(" Q\n");
   } else
   if( strncasecmp("help", s, 4) ==0) {
    printf(" Help\n");
    printf("   This Text\n");
   } else
   if( strncasecmp("load", s, 4) ==0) {
    printf(" Load <bid>\n");
    printf("   Get the memory buffer for <bid>\n");
    printf("   <bid> can be expressed as a plain string, `likethis' or as a hex\n");
    printf("   value, `0x6c696b6574686973'.  hex must start with `0x'.  If you\n");
    printf("   give a string or hex that is too long for a bid, it will be\n");
    printf("   trucated.  If it is too short, it is padded with 0 for hex and\n");
    printf("   spaces for strings.  (so `shortbid' is equal to `shortbid  ' (not\n");
    printf("   that you could ever type that...(no you could, just as \n");
    printf("   0x73686f72746269642020)))\n");
    printf(" Load <lock number> <lock type>\n");
    printf("   Get the memory buffer for the bid that is constructed from the\n");
    printf("   <lock number> and <lock type> in the same maner that the memexp\n");
    printf("   module does.\n");
    printf("   <lock number> is 64bit digit.\n");
    printf("   <lock type> is one of: RESV CID META NOPEN FLOCK NONDISK\n");
   } else
   if( strncasecmp("store", s, 5) ==0) {
    printf(" Store <bid> <InUse> <Seq> <pbn> <hex data>\n");
    printf("   Put the <hex data> into the memory buffer for <bid>.\n");
    printf("   Both <seq> and <pbn> Must match those on server for this to succed.\n");
    printf("   <InUse> is `1' or `0'.  If `0' this memory buffer will be freed.\n");
    printf("   <hex data> is a run of hex bytes.  Only values legal for hex will\n");
    printf("   be parsed. The rest are skipped over, this lets you intermix spaces\n");
    printf("   into the hexdata for readablity.  It also lets you do stupid things\n");
    printf("   like `09gh08', and it will only see `0908'.\n");
    printf("   Support for the lsm structures may happen. Hell might also freeze\n");
   } else
   if( strncasecmp("dump", s, 4) ==0) {
    printf(" Dump <pbn> <cnt>\n");
    printf("   Dump out memory buffers starting at <pbn>.  \n");
    printf("   <cnt> is the number of buffers you want to be returned.\n");
   } else
   if( strncasecmp("verify", s, 6) ==0) {
    printf(" Verify <pbn>\n");
    printf("   Determins if the requested PBN has be succesfully received.\n");
    printf("   This command is really worthless unless you fully understand the\n");
    printf("   details of how the protocol works.\n");
   } else
   if( strncasecmp("modepage", s, 8) ==0) {
    printf(" ModePage [<Number buffers> <Memory Buffer Size>]\n");
    printf("   If no args, gets the current modepage. (which will update internal\n");
    printf("   marks.)\n");
    printf("   If both args, sets the new values.\n");
   } else
   if( strncasecmp("reset", s, 5) ==0) {
    printf(" Reset\n");
    printf("   shortcut for doing a modepage write with the current buffer size and\n");
    printf("   as many buffers as we can get.  The effect it has is to reset all of\n");
    printf("   the inuse buffers on the server to not in use.\n");
    printf("   Much quicker than killing ang restarting the server.\n");
   } else
   if( strncasecmp("decode", s, 6) ==0) {
    printf(" Decode [True|False]\n");
    printf("   Without params, reports what the current value is.  Otherise sets it.\n");
   } else
   if( strncasecmp("ban", s, 3) ==0) {
    printf(" Ban <IP>\n");
    printf("   Tell the server to ignore requests from IP\n");
   } else
   if( strncasecmp("unban", s, 5) ==0) {
    printf(" Unban <IP>\n");
    printf("   Tell the server to accept requests from IP\n");
   } else
   if( strncasecmp("stats", s, 5) ==0) {
    printf(" Stats [hsh|cnt|");
#ifdef LPS_CODE
    printf("lps|");
#endif
#ifdef LDUR_CODE
    printf("ldr|");
#endif
    printf("cli]\n");
    printf("   Ask the server to quick gather some statistics about its self and\n");
    printf("   return them here to be displayed.\n");
    printf("\n");
    printf("   hsh returns a list that tells how many hash chains are of each"
           " length\n");
    printf("   cnt returns a list of counters that are held in the sevrer.\n");
#ifdef LPS_CODE
    printf("   lps returns stats on Locks Per second.  Must have compiled with"
           " this on.\n");
#endif
#ifdef LDUR_CODE
    printf("   ldr returns stats on Lock op durrations.  Must have compiled "
           "with this on.\n");
#endif
    printf("   cli returns the Client Tables.\n");
   } else
   if( strncasecmp("Exit", s, 4) ==0 ||
       strncasecmp("Quit", s, 4) ==0 ||
       strncasecmp("Q", s, 1) ==0) {
    printf(" Exit\n");
    printf(" Quit\n");
    printf(" Q\n");
    printf("   duh.\n");
   } else {
      printf(" Unrecognised \"%s\". Try again.\n",s);
   }
}

/******************************************************************************
 * strtobid
 * takes a string pointer and translates that string to a valid bid.
 * Assumes that bid points to a correctly sized buffer.
 * Trick is that these following forms are all correct:
 *  "somestr" 12345677 0x01234567890abcef0123
 *  I think I may say anthing starting with 0x is hex all rest is string
 *  data.  (kill middle type of raw numbers. too hard.)
 */
int strtobid( char *s, u8 *bid)
{
   int l,i,j=0,c;
   u8 *p;
   if( strncmp("0x", s, 2) == 0) {
      /* its in hex. */
      s+=2; /* skip past the `0x' */
      l = MIN( strlen(s), 20); /* 20 chars makes 10 bytes. */
      for(i=0; i < l; i+=2 ) {
         p = &s[i];
         sscanf(p,"%2x", &c);
#if 0
         printf("read byte %u\n", c);
#endif
         bid[j++] = c;
         if(j>=10) break;
      }
   } else {
      /* its raw string. */
      memset(bid, ' ', 10); /* `blank' values. */
      l = MIN( strlen(s), 10);
      memcpy(bid, s, l);
   }
   return 0;
}

/**
 * pack_bid - takes a lock number and type and makes a bid
 * @ln: > the number
 * @lt: > the type
 * @bid: < the new bid
 * 
 * Returns: void
 */
void pack_bid(u64 ln, u8 lt, u8 bid[BIDSIZE])
{
   bid[0] = lt & 0xFF;

   bid[1] = (ln >> 56) & 0xFF;
   bid[2] = (ln >> 48) & 0xFF;
   bid[3] = (ln >> 40) & 0xFF;
   bid[4] = (ln >> 32) & 0xFF;
   bid[5] = (ln >> 24) & 0xFF;
   bid[6] = (ln >> 16) & 0xFF;
   bid[7] = (ln >>  8) & 0xFF;
   bid[8] = (ln >>  0) & 0xFF;
   bid[9] = 0;
}

void decode_bid(u8 bid[BIDSIZE])
{
   u64 ln=0;
   ln  = (u64)bid[1] << 56;
   ln |= (u64)bid[2] << 48;
   ln |= (u64)bid[3] << 40;
   ln |= (u64)bid[4] << 32;
   ln |= (u64)bid[5] << 24;
   ln |= (u64)bid[6] << 16;
   ln |= (u64)bid[7] <<  8;
   ln |= (u64)bid[8] <<  0;
   
   printf("%"FMT_64"u ",ln);

   if( (bid[0] & LM_TYPE_LVB_MASK) == LM_TYPE_LVB_MASK) printf("LVB:");
   switch(bid[0] & ~LM_TYPE_LVB_MASK) {
#define CasePrint(x) case (x): printf("%s\n",#x); break
      CasePrint(LM_TYPE_RESERVED);
      CasePrint(LM_TYPE_CIDBUF);
      CasePrint(LM_TYPE_MOUNT);
      CasePrint(LM_TYPE_NONDISK);
      CasePrint(LM_TYPE_INODE);
      CasePrint(LM_TYPE_RGRP);
      CasePrint(LM_TYPE_META);
      CasePrint(LM_TYPE_NOPEN);
      CasePrint(LM_TYPE_FLOCK);
      CasePrint(LM_TYPE_PLOCK);
      CasePrint(LM_TYPE_PLOCK_HEAD);
#undef CasePrint
      default: printf("%d\n", bid[0]); break;
   }
}

/**
 * build_bid - build up a valid bid best we can
 * @a: a string
 * @b: a string
 * @bid: the bid if no err
 * 
 * If b in NULL, assumes that a has either a string or hex value of the
 * bid.  Otherwise, a is the lock number and b is the lock type as memexp
 * understands them.
 * 
 * Returns: 0 if ok, <0 if error.
 */
int build_bid(char *a, char *b, u8 bid[BIDSIZE])

{
  if( b == NULL ) 
      return strtobid( a, bid);
   else {
      u64 ln;
      u8 lt;
      sscanf(a, "%"FMT_64"u", &ln);

      if( strlen(b) ==1 ) /* assume its the type in digit form */
	lt = atoi(b);

      else if(strncasecmp("resv", b, 4)==0) lt = LM_TYPE_RESERVED;
      else if(strncasecmp("cid", b, 3)==0) lt = LM_TYPE_CIDBUF;
      else if(strncasecmp("meta", b, 4)==0) lt = LM_TYPE_META;
      else if(strncasecmp("nopen", b, 5)==0) lt = LM_TYPE_NOPEN;
      else if(strncasecmp("flock", b, 5)==0) lt = LM_TYPE_FLOCK;
      else if(strncasecmp("nondisk", b, 7)==0) lt = LM_TYPE_NONDISK;
      else if(strncasecmp("mount", b, 5)==0) lt = LM_TYPE_MOUNT;
      else return -1;

      pack_bid(ln, lt, bid);
   }
   return 0;
}

/**
 * printbid - prints out a bid in hex.
 * @bid: bid buffer to print.
 */
void printbid(u8 bid[BIDSIZE])
{
   int i;
   printf("0x");
   for(i=0;i<BIDSIZE;i++) printf("%02x",bid[i]&0xFF);
}

/**
 * print_cidbuf - print out a lsm_cid buffer
 * @data: 
 * 
 * Returns: void
 */
void print_cidbuf(u8 *data)

{
   cidbuf_t *cid = (cidbuf_t*)data;
   printf(" cid:\n");
   printf("  counter  %"FMT_64"u\n", be64_to_cpu(cid->counter));
   printf("  cleaner  %u\n", be16_to_cpu(cid->cleaner));
   printf("  inuse    ");
   switch(cid->inuse) {
#define CasePrint(x) case (x): printf("%s\n",#x); break
      CasePrint(CID_NOT_USED);
      CasePrint(CID_USED);
      CasePrint(CID_STOMITH);
      CasePrint(CID_STOMITH_DONE);
      CasePrint(CID_EXPIRED_MARKING);
      CasePrint(CID_EXPIRED);
      CasePrint(CID_RESETING);
#undef CasePrint
      default: printf("%d\n", cid->inuse); break;
   }
   printf("  dead     %s\n",(cid->dead)?"True":"False");
}



void print_mlbuf(u8 *data)
{
  mlbuf_t *ml = (mlbuf_t *)data;
  printf("Mount Lock:\n");
  printf("  state  ");
  switch(be16_to_cpu(ml->state)) {
#define CasePrint(x) case (x): printf("%s\n",#x); break
    CasePrint(MOUNT_STATE_NULL);
    CasePrint(MOUNT_STATE_INIT);
    CasePrint(MOUNT_STATE_RESET);
    CasePrint(MOUNT_STATE_OK);
#undef CasePrint
    default: printf("%d\n", be16_to_cpu(ml->state)); break;
  }
  printf("  owner  %u\n", be16_to_cpu(ml->owner));
}
  


/**
 * print_lockbuf - printout a lsm_lock buffer.
 * @data: 
 * 
 * 
 * Returns: void
 */
void print_lockbuf(u8 * data)

{
   lsm_lock *lck = (lsm_lock*)data;
   int i;
   printf(" lock:\n");
   printf("  state     ");
   switch(lck->state){
#define CasePrint(x) case (x): printf("%s\n",#x); break
      CasePrint(STATE_UNLOCKED);
      CasePrint(STATE_SHARED);
      CasePrint(STATE_EXCLUSIVE);
      CasePrint(STATE_DEFERRED);
#undef CasePrint
      default: printf("%d\n", lck->state); break;
   }
   printf("  conv      ");
   switch(lck->conv){
#define CasePrint(x) case (x): printf("%s\n",#x); break
      CasePrint(NO_CONV);
      CasePrint(YES_CONV);
#undef CasePrint
      default: printf("%d\n", lck->conv); break;
   }
   printf("  lvb perm  %u\n", lck->lvb_perm);
   printf("  lvb lock  %u\n", lck->lvb_lock);
   printf("  exp hold  %u\n", be16_to_cpu(lck->expired_holders));
   printf("  live hold %u\n", be16_to_cpu(lck->live_holders));
   printf("  holders:  ");
   for(i=0;i<MAX_CLIENTS;i++)
      if( (lck->holders[i>>3]) & (1<<(i&0x7)) ) printf("%u ",i);
   printf("\n");
   printf("  lvb holders:  ");
   for(i=0;i<MAX_CLIENTS;i++)
      if( (lck->lvb[i>>3]) & (1<<(i&0x7)) ) printf("%u ",i);
   printf("\n");
   printf("  expired:  ");
   for(i=0;i<MAX_CLIENTS;i++)
      if( (lck->expired[i>>3]) & (1<<(i&0x7)) ) printf("%u ",i);
   printf("\n");
   printf("  convs:    ");
   for(i=0;i<MAX_CLIENTS;i++)
      if( (lck->conversion[i>>3]) & (1<<(i&0x7)) ) printf("%u ",i);
   printf("\n");
}

/**
 * print_plock_elem - print a plock lvb.
 * @data: < yep, thats data.
 */
void print_plock_elem(u8 *data)
{
   plock_element_t *pl = (plock_element_t*)data;
   if( be32_to_cpu(pl->magic) == PLOCK_ELEM_MAGIC ) {
      printf(" plock element:\n");
      printf("  range begin: %"FMT_64"u\n", be64_to_cpu(pl->begin));
      printf("  range end  : %"FMT_64"u\n", be64_to_cpu(pl->end));
      printf("  magic      : %x\n", be32_to_cpu(pl->magic));
      printf("  pid        : %u\n", be32_to_cpu(pl->pid));
      printf("  Next       : %u\n", be16_to_cpu(pl->Next));
      printf("  ltype      : %u\n", pl->ltype);
   }else{
      plock_jid_head_t *pljid = (plock_jid_head_t*)data;
      printf(" plock jid head:\n");
      printf("  magic      : %x\n", be32_to_cpu(pljid->magic));
      printf("  lock_group : %x\n", be16_to_cpu(pljid->lock_group));
      printf("  lock_count : %x\n", be16_to_cpu(pljid->lock_count));
      printf("  first      : %x\n", be16_to_cpu(pljid->first));
   }
}

/**
 * print_plock_head - print a plock head lvb
 * @data: < yep, thats data.
 */
void print_plock_head(u8 *data)
{
   plock_head_t *plhd = (plock_head_t*)data;
   printf(" plock list head:\n");
   printf("  magic     : %x\n", be32_to_cpu(plhd->magic));
   printf("  Jid Group : %u\n", be32_to_cpu(plhd->Jid_group));
}

/**
 * print_data_dump - decode the data portion based on the decode.
 * @data: < buffer that holds the data
 * @dlen: < how much of it there is
 * @decode: < how to printout the data
 */
void print_data_dump(u8 *data, size_t dlen, unsigned int decode)
{
   switch( decode ) {
      case LM_TYPE_CIDBUF:
         print_cidbuf(data);
         break;
      case LM_TYPE_MOUNT:
         print_mlbuf(data);
         break;
      case LM_TYPE_META:
      case LM_TYPE_NOPEN:
      case LM_TYPE_FLOCK:
      case LM_TYPE_NONDISK:
      case LM_TYPE_INODE:
      case LM_TYPE_PLOCK:
      case LM_TYPE_PLOCK_HEAD:
         print_lockbuf(data);
         break;
      case (LM_TYPE_PLOCK|LM_TYPE_LVB_MASK):
         print_plock_elem(data);
         break;
      case (LM_TYPE_PLOCK_HEAD|LM_TYPE_LVB_MASK):
         print_plock_head(data);
         break;
      default:
         printf("data field:");
         dump_buffer(data, dlen);
         printf("\n");
         break;
   }
}
/**
 * printdumpelem - Print out a dumped element.
 * @rbuf: 
 * @datalen: 
 * @decode: try to decode buffer contents?
 * 
 * Returns: void
 */
void printdumpelem(u8 *rbuf, size_t datalen, char decode)

{
   dump_element_t *rdf = (dump_element_t *)rbuf;
   u8 *data = rbuf + sizeof(dump_element_t);

   printf("dump Used %u\n", rdf->InUse);
   printf("dump bid  ");printbid(rdf->bid);
   if(decode) {printf(" : "); decode_bid(rdf->bid);} else printf("\n");
   printf("dump pbn  %"FMT_64"u\n", be64_to_cpu(rdf->pbn));
   printf("dump seq  %"FMT_64"u\n", be64_to_cpu(rdf->seq));

   print_data_dump(data, datalen, decode?rdf->bid[0]:(~0));
}

/**
 * printrdf - Printout a formated response from a load
 * @rbuf: 
 * @datalen: 
 * @decode: decode style, if not recognised, defaults to hexdump
 * 
 * Returns: void
 */
void printrdf(u8 *rbuf, size_t datalen, unsigned int decode)
{
   me_rdf_t* rdf = (me_rdf_t*)rbuf;
   u8 *data = rbuf+ sizeof(me_rdf_t);

   printf("rdf => InUse    %u\n", rdf->InUse);
   printf("rdf => fullness %u\n", rdf->fullness);
   printf("rdf => seq      %"FMT_64"u\n", be64_to_cpu(rdf->seq));
   printf("rdf => pbn      %"FMT_64"u\n", be64_to_cpu(rdf->pbn));

   print_data_dump(data, datalen, decode);
}

/**
 * show_error - printout the errorcode
 * @rbuf: 
 * 
 * 
 * Returns: void
 */
void show_error(u8 *rbuf)

{
   printf(" Error received: ");
#define CaseError(x) case (x): printf("%s\n",#x); break
   switch ( *rbuf ) {
      CaseError(Mel_Err_BadSize);
      CaseError(Mel_Err_BidZero);
      CaseError(Mel_Err_NoBid);
      CaseError(Mel_Err_BadSeq);
      CaseError(Mel_Err_BadPbn);
      CaseError(Mel_Pbn_Space_Full);
      CaseError(Mel_You_are_Banned);
      CaseError(Mel_Verify_Failed);
      CaseError(Mel_Err_NotImpl);
      default: printf("Unknown errorcode %d\n", *rbuf);
   }
#undef CaseError
}

typedef enum {
   apcalc,
   gnuplot
} stat_ouptu_style;

#ifdef LPS_CODE
/**
 * lps_print_vals - print out info based on the lps data
 * @lps: 
 * 
 * 
 * Returns: void
 */
void lps_print_vals(lps_t *lps, stat_ouptu_style sop)
{
   u64 devi = 0;
   u64 part1;/* These parts may have to be 64bit to avoid overflow.*/
   u64 part2;
   u64 part3;
   u64 mean=0;

   part1 = (u64)lps->count * (u64)lps->std;
   part2 = (u64)lps->total * (u64)lps->total;
   part3 = (u64)lps->count * ((u64)lps->count -(u64)1);
   if(part3 != 0)
      devi  = (part1 - part2) / part3;
   if(lps->count !=0)
         mean = lps->total / lps->count;

   switch(sop){
   case apcalc:
      printf("usec per lock op:\n");
      printf(" Min: %"FMT_64"u  Max: %"FMT_64"u  Mean: %"FMT_64"u\n",
            lps->min, lps->max, mean);
      printf(" Called %"FMT_64"u times, with total rate %"FMT_64"u\n",
            lps->count, lps->total);
      printf(" Squared Std Deviation: %"FMT_64"u\n", devi);
      printf("  (%"FMT_64"u - %"FMT_64"u) / %"FMT_64"u\n",
            part1, part2, part3);
      printf(" 99%%conf = (2.576 * sqrt(%"FMT_64"u))/sqrt(%"FMT_64"u)\n",
            devi, lps->count);
      break;
   case gnuplot:
      printf("set data style lines\n");
      printf("ssd = %"FMT_64"u\n", devi);
      printf("mu = %"FMT_64"u\n", mean);
      printf("nd(x) = (1/sqrt(2*pi*ssd))*exp((-(x-mu)**2)/(2*ssd))\n");
      printf("plot nd(x) \n");
      break;
   default:
      printf("bad code\n");
   }

}
#endif /*LPS_CODE*/

#ifdef LDUR_CODE
/**
 * ldur_print_vals - Prints lock op durration statitics.
 * @ldur: < stuct we got from server.
 */
void ldur_print_vals(ldur_t *ldur, stat_ouptu_style sop)
{
   u64 devi = 0;
   u64 part1;/* These parts may have to be 64bit to avoid overflow.*/
   u64 part2;
   u64 part3;
   u64 mean=0;

   part1 = (u64)ldur->count * (u64)ldur->std;
   part2 = (u64)ldur->total * (u64)ldur->total;
   part3 = (u64)ldur->count * ((u64)ldur->count -(u64)1);
   if(part3 != 0)
      devi  = (part1 - part2) / part3;
   if(ldur->count !=0)
         mean = ldur->total / ldur->count;

   switch(sop) {
   case apcalc:
      printf("Lock op durration values:\n");
      printf(" Min: %"FMT_64"u  Max: %"FMT_64"u  Mean: %"FMT_64"u\n",
            ldur->min, ldur->max, mean);
      printf(" Called %"FMT_64"u times, with total rate %"FMT_64"u\n",
            ldur->count, ldur->total);
      printf(" Squared Std Deviation: %"FMT_64"u\n", devi);
      printf("  (%"FMT_64"u - %"FMT_64"u) / %"FMT_64"u\n",
            part1, part2, part3);
      printf(" 99%%conf = (2.576 * sqrt(%"FMT_64"u))/sqrt(%"FMT_64"u)\n",
            devi, ldur->count);
      break;
   case gnuplot:
      printf("set data style lines\n");
      printf("ssd = %"FMT_64"u\n", devi);
      printf("mu = %"FMT_64"u\n", mean);
      printf("nd(x) = (1/sqrt(2*pi*ssd))*exp((-(x-mu)**2)/(2*ssd))\n");
      printf("plot nd(x) \n");
      break;
   default:
      printf("bad code\n");
   }

}
#endif /*LDUR_CODE*/

/******************************************************************************
 * stuffhexdata
 * This takes a string of hex numbers, ignoring whitspace, converts the
 * hexinto actual values putting those into data.
 *
 * If there is more hexdata than space in data, the extra is ignored.
 * If there is less hexdata, the remaining is padded with 0s.
 *
 * the incomming hex string MUST be null terminated.
 *
 * I'm not completely sure I did this one right.....
 * seems to work though.
 */
void stuffhexdata(char *t, u8 *data, size_t dlen)
{
   char nextb[2];
   int nb=0, da=0, c;

   memset(data, 0, dlen);

   while( *t != '\0' ) {
      if( (*t >= 0x30 && *t <= 0x39) ||
          (*t >= 0x61 && *t <= 0x66) ||
          (*t >= 0x41 && *t <= 0x46) ) {
         /* hex value. */
         nextb[nb++] = *t;
         nb %= 2;
         if( nb == 0 ) {
            sscanf(nextb, "%2x", &c);
#if 0
            printf("Found byte %x\n", c);
#endif
            data[da++] = c;
            if( da > dlen ) return;
         }
      }
      t++;
   }
}

/******************************************************************************
 * main
 */
int main(int argc, char **argv)
{
   int sock, n, datasize=0;
   struct sockaddr_in adr;
   char *line=NULL;
   char cbuf[30], rbuf[256];
   char *data;
   me_hdr_t hdr;
   me_cdb_t *cdb = (me_cdb_t*)cbuf;
   me_rdf_t *rdf = (me_rdf_t*)rbuf;
   char decodebufs= TRUE;

   signal(SIGPIPE, SIG_IGN);

   rl_readline_name = "uc"; /* let conditional be in .inputrc */

   data = rbuf + sizeof(me_rdf_t);

   if( argc < 2 || argc > 3 ) die("usage: uc <ip> [<port>]");
   if( argc == 3) cmdport = atoi( argv[2] );

   if( (sock = socket(AF_INET, SOCK_STREAM, 0)) <0)
      die("Failed to create socket");

   memset(&adr, 0, sizeof(struct sockaddr_in));
   adr.sin_family = AF_INET;
   adr.sin_addr.s_addr = inet_addr( argv[1] );
   adr.sin_port = htons( cmdport );

   if( connect(sock, (struct sockaddr*)&adr, sizeof(struct sockaddr_in))<0)
      die("Couldn't connect");

   { /* do modepage so we have some needed info. */
      modepage_t *mp = (modepage_t*)cbuf;
      memset(cbuf, 0, 30);

      hdr.Opcode = Mel_ModePgR;
      hdr.Key = 0;
      hdr.length = cpu_to_be16(sizeof(modepage_t));

      n = write(sock, &hdr, sizeof(me_hdr_t));
      n = write(sock, cbuf, sizeof(modepage_t));
#if 0
      printf(">>Sent cdb (%d)\n",n);
#endif
      n = read(sock, &hdr, sizeof(me_hdr_t));
         if(n<0) printf("read got n=%d\n",n);
      n = read(sock, cbuf, sizeof(modepage_t));
         if(n<0) printf("read got n=%d\n",n);

      datasize = be32_to_cpu( mp->datasize );
      printf(" ModePage returned: Datasize=%d Numbufs=%"FMT_64"u \n",
            datasize, be64_to_cpu( mp->numbufs ) );

   }

   for(;;) {
      char *token;
     /* changed my mind.
      * We're gonna do this like a shell.
      * you'll get a prompt with readline, and you'll issue commands.  Then
      * I'll have code to parse those apart and do the work.
      */
      line = readline("==> ");
      if( ! (line && *line)) continue;
      add_history(line);

      memset(cbuf, 0, 30);
      memset(rbuf, 0, 256);

      token = strtok(line, " \t");
      if( strncasecmp("help", token, 4) == 0) {
         token = strtok(NULL, " \t");
         help(token);
      } else
      if( strncasecmp("ban", token, 3) == 0) {
         u32 IP;
         token = strtok(NULL, " \t");
         if( ! token ) {printf("Bad params.\n"); continue;}
         IP = inet_addr( token );

         hdr.Opcode = Mel_BanReq;
         hdr.Key = 0;
         hdr.length = cpu_to_be16(sizeof(u32));

         n = write(sock, &hdr, sizeof(me_hdr_t));
         n = write(sock, &IP, sizeof(u32));

         n = read(sock, &hdr, sizeof(me_hdr_t));
         if(n<0) printf("read got n=%d\n",n);
         if( hdr.Opcode == Mel_Error) {
            n = read(sock, rbuf, sizeof(errorcode_t));
            show_error(rbuf);
         }else {
            printf("IP banned\n");
         }
      } else
      if( strncasecmp("unban", token, 5) == 0) {
         u32 IP;
         token = strtok(NULL, " \t");
         if( ! token ) {printf("Bad params.\n"); continue;}
         IP = inet_addr( token );

         hdr.Opcode = Mel_UnBanMe;
         hdr.Key = 0;
         hdr.length = cpu_to_be16(sizeof(u32));

         n = write(sock, &hdr, sizeof(me_hdr_t));
         n = write(sock, &IP, sizeof(u32));

         n = read(sock, &hdr, sizeof(me_hdr_t));
         if(n<0) printf("read got n=%d\n",n);
         if( hdr.Opcode == Mel_Error) {
            n = read(sock, rbuf, sizeof(errorcode_t));
            show_error(rbuf);
         }else {
            printf("IP Unbanned\n");
         }
      } else
      if( strncasecmp("load", token, 4) == 0) {
         char *btok;
         token = strtok(NULL, " \t");
         if( ! token ) {printf("Bad params.\n"); continue;}
         btok = strtok(NULL, " \t");

         if(build_bid(token, btok, cdb->bid)<0)
            {printf("Bad params.\n"); continue;}

         cdb->allocLen = cpu_to_be16(sizeof(me_rdf_t) + datasize);
         hdr.Opcode = Mel_Load;
         hdr.Key = 0;
         hdr.length = cpu_to_be16(sizeof(me_cdb_t));

         n = write(sock, &hdr, sizeof(me_hdr_t));
         n = write(sock, cbuf, sizeof(me_cdb_t));

         n = read(sock, &hdr, sizeof(me_hdr_t));
         if(n<0) printf("read got n=%d\n",n);
         printf("Getting %u bytes back\n", be16_to_cpu(hdr.length));
         if( hdr.Opcode == Mel_Error) {
            n = read(sock, rbuf, sizeof(errorcode_t));
            show_error(rbuf);
         }else{
            n = read(sock, rbuf, (sizeof(me_rdf_t) + datasize));
            printrdf(rbuf, datasize, (decodebufs)?cdb->bid[0]:(~0));
         }

      } else
      if( strncasecmp("verify", token, 4) == 0) {
         u64 pbn, nbp;
         token = strtok(NULL, "\t");
         if( ! token) {printf("Bad params.\n"); continue;}
         sscanf(token, "%"FMT_64"u", &pbn);
         nbp = cpu_to_be64( pbn );

         hdr.Opcode = Mel_Verify;
         hdr.Key = 0;
         hdr.length = cpu_to_be16(sizeof(u64));

         n = write(sock, &hdr, sizeof(me_hdr_t));
         n = write(sock, &nbp, sizeof(u64));

         n = read(sock, &hdr, sizeof(me_hdr_t));
         if(n<0) printf("read got n=%d\n",n);
         if( hdr.Opcode == Mel_Error) {
            n = read(sock, rbuf, sizeof(errorcode_t));
            show_error(rbuf);
         }else{
            printf("Verify: Ok\n");
         }
      } else
      if( strncasecmp("store", token, 4) == 0) {
         u64 sq;
         token = strtok(NULL, " \t");
         if( ! token ) {printf("Bad params.\n"); continue;}
         strtobid(token, cdb->bid);
         cdb->allocLen = cpu_to_be16(sizeof(me_rdf_t) + datasize);
         hdr.Opcode = Mel_Store;
         hdr.Key = 0;
         hdr.length = cpu_to_be16(sizeof(me_cdb_t)+sizeof(me_rdf_t)+datasize);

         token = strtok(NULL, " \t");
         if( ! token ) {printf("Bad params.\n"); continue;}
         if( strncmp(token, "1", 1) == 0) rdf->InUse = 1;
         else rdf->InUse = 0;

         token = strtok(NULL, " \t");
         if( ! token ) {printf("Bad params.\n"); continue;}
         sscanf(token, "%"FMT_64"u", &sq);
         rdf->seq = cpu_to_be64( sq );

         token = strtok(NULL, " \t");
         if( ! token ) {printf("Bad params.\n"); continue;}
         sscanf(token, "%"FMT_64"u", &sq);
         rdf->pbn = cpu_to_be64( sq );

         token = strtok(NULL, "\n"); /* eat up to end of line*/
         if( ! token ) {printf("Bad params.\n"); continue;}
         stuffhexdata(token, data, datasize);

         n = write(sock, &hdr, sizeof(me_hdr_t));
         n = write(sock, cbuf, sizeof(me_cdb_t));
         n = write(sock, rbuf, sizeof(me_rdf_t) + datasize);

         n = read(sock, &hdr, sizeof(me_hdr_t));
         if(n<0) printf("read got n=%d\n",n);
         printf("Getting %u bytes back\n", be16_to_cpu(hdr.length));
         if( hdr.Opcode == Mel_Error) {
            n = read(sock, rbuf, sizeof(errorcode_t));
            show_error(rbuf);
         }else {
            printf("Data stored\n");
         }

      } else
      if( strncasecmp("dump", token, 4) == 0) {
         u64 pbn; u32 cnt;
         dumpb_cdb_t *dcdb = (dumpb_cdb_t*)cbuf;

         token = strtok(NULL, " \t");
         if( ! token ) {printf("Bad params.\n"); continue;}
         sscanf(token, "%"FMT_64"u", &pbn);
         dcdb->pbn = cpu_to_be64( pbn );

         token = strtok(NULL, " \t");
         if( ! token ) {printf("Bad params.\n"); continue;}
         cnt = atol( token );
         dcdb->want = cpu_to_be32( cnt );

         hdr.Opcode = Mel_Dump;
         hdr.Key = 0;
         hdr.length = cpu_to_be16(sizeof(dumpb_cdb_t));

         n = write(sock, &hdr, sizeof(me_hdr_t));
         n = write(sock, cbuf, sizeof(dumpb_cdb_t));

         n = read(sock, &hdr, sizeof(me_hdr_t));
         if(n<0) printf("read got n=%d\n",n);
         printf("Getting %u bytes back\n", be16_to_cpu(hdr.length));
         if( hdr.Opcode == Mel_Error ) {
            n = read(sock, rbuf, sizeof(errorcode_t));
            show_error(rbuf);
         } else {
            n = read(sock, rbuf, 8); rbuf[8] = '\0';
         if(n<0) printf("read got n=%d\n",n);
            if( strncmp(rbuf, "DUMPSTRT", 8) != 0)
               printf("DUMPSTRT not seen!!!!!\n");
            n = read(sock, rbuf, sizeof(u32));
         if(n<0) printf("read got n=%d\n",n);
            cnt = be32_to_cpu( *(u32*)rbuf );
            printf("Dump is returning %d elements\n", cnt);

            while( cnt > 0) {
               n = read(sock, rbuf, sizeof(dump_element_t) + datasize);
               printdumpelem(rbuf, datasize, decodebufs);

               cnt--;
            }
         }

      } else
      if( strncasecmp("modepage", token, 4) == 0) {
         u64 nb; u32 ds;
         modepage_t *mp = (modepage_t*)cbuf;
         /* Need to add support for setting values. */
         
         token = strtok(NULL, " \t");
         if( token ) {
            sscanf(token, "%"FMT_64"u", &nb);
            token = strtok(NULL, " \t");
            ds = atoi( token );

            mp->datasize = cpu_to_be32( ds );
            mp->numbufs = cpu_to_be64( nb );

            hdr.Opcode = Mel_ModePgW;
         } else
            hdr.Opcode = Mel_ModePgR;
         hdr.Key = 0;
         hdr.length = cpu_to_be16(sizeof(modepage_t));

         n = write(sock, &hdr, sizeof(me_hdr_t));
         n = write(sock, cbuf, sizeof(modepage_t));
         n = read(sock, &hdr, sizeof(me_hdr_t));
         if(n<0) printf("read got n=%d\n",n);
         n = read(sock, cbuf, sizeof(modepage_t));
         if(n<0) printf("read got n=%d\n",n);

         datasize = be32_to_cpu( mp->datasize );
         printf(" ModePage returned: Datasize=%d Numbufs=%"FMT_64"u \n",
               datasize, be64_to_cpu( mp->numbufs ) );
      } else
      if( strncasecmp("reset", token, 4) == 0) {
         /* just do the modepage with what we already have.  This will
          * reset the contents of server to empties.
          */
         modepage_t *mp = (modepage_t*)cbuf;
         
         hdr.Opcode = Mel_ModePgW;
         hdr.Key = 0;
         hdr.length = cpu_to_be16(sizeof(modepage_t));

         mp->datasize = cpu_to_be32( datasize );
         mp->numbufs = cpu_to_be64( ~0UL ); /* as many as we can get */

         n = write(sock, &hdr, sizeof(me_hdr_t));
         n = write(sock, cbuf, sizeof(modepage_t));
         n = read(sock, &hdr, sizeof(me_hdr_t));
         if(n<0) printf("read got n=%d\n",n);
         n = read(sock, cbuf, sizeof(modepage_t));
         if(n<0) printf("read got n=%d\n",n);

         datasize = be32_to_cpu( mp->datasize );
         printf(" ModePage returned: Datasize=%d Numbufs=%"FMT_64"u \n",
               datasize, be64_to_cpu( mp->numbufs ) );
      } else
      if( strncasecmp("stats", token, 4) == 0) {
         u8 Sel;

         token = strtok(NULL, " \t");
         if( !token ) {printf("Bad params.\n"); continue;}

         if(strncasecmp("hsh", token, 3) == 0) Sel = STAT_HSHCNT;
         else if(strncasecmp("cnt", token, 3) == 0) Sel = STAT_counts;
#ifdef LPS_CODE
         else if(strncasecmp("lps", token, 3) == 0) Sel = STAT_LPS;
#endif
#ifdef LDUR_CODE
         else if(strncasecmp("ldr", token, 3) == 0) Sel = STAT_LDUR;
#endif
         else if(strncasecmp("cli", token, 3) == 0) Sel = STAT_CLILST;
         else {printf("Unknown stat type \"%s\"\n",token);continue;}

         hdr.Opcode = Mel_StatReq;
         hdr.Key = 0;
         hdr.length = cpu_to_be16(sizeof(u8));
         n = write(sock, &hdr, sizeof(me_hdr_t));
         n = write(sock, &Sel, sizeof(u8));


         n = read(sock, &hdr, sizeof(me_hdr_t));
         if(n<0) printf("read got n=%d\n",n);
         if( hdr.Opcode == Mel_Error ) {
            n = read(sock, rbuf, sizeof(errorcode_t));
            show_error(rbuf);
         } else {
         switch(Sel) {
            case STAT_HSHCNT:
               {
                  hshcnt_t hshcnt;
                  int i;

                  n = read(sock, &hshcnt, sizeof(hshcnt_t));
                  if(n<0) printf("read got n=%d\n",n);
                  printf("hash size : number of buckets at this size\n");
                  for(i=0;i<MAXdens;i++) {
                     printf("  %.2u: %7u   ", i, be16_to_cpu(hshcnt.bcnt[i]));
                     if( i % 4 ==3 ) printf("\n");
                  }
                  printf(" There are %u memory buffers in use\n",
                        be32_to_cpu(hshcnt.inuse));
                  printf(" There are %u keys in the hashtable\n",
                        be32_to_cpu(hshcnt.numkeys));
                  printf(" There is a maximum of %u elements.\n",
                        be32_to_cpu(hshcnt.maxelems));
               }
               break;
            case STAT_counts:
               {
                  counters_t cnts;
                  n = read(sock, &cnts, sizeof(counters_t));
                  if(n<0) printf("read got n=%d\n",n);
                  printf("There have been:\n");
                  printf("   loads : %u\n", be32_to_cpu(cnts.loads));
                  printf("   stores: %u\n", be32_to_cpu(cnts.stores));
                  printf("   sucstr: %u\n", be32_to_cpu(cnts.sucstr));
                  printf("   verify: %u\n", be32_to_cpu(cnts.verify));
                  printf("   dumps : %u\n", be32_to_cpu(cnts.dumps));
                  printf("   bans  : %u\n", be32_to_cpu(cnts.bans));
                  printf("   unbans: %u\n", be32_to_cpu(cnts.unbans));
                  printf("   errors: %u\n", be32_to_cpu(cnts.errors));
               }
               break;
            case STAT_LPS:
               {
#ifdef LPS_CODE
                  lps_t lps;
                  n = read(sock, &lps, sizeof(lps_t));
                  if(n<0) printf("read got n=%d\n",n);
                  lps.min = be64_to_cpu(lps.min);
                  lps.max = be64_to_cpu(lps.max);
                  lps.std = be64_to_cpu(lps.std);
                  lps.total = be64_to_cpu(lps.total);
                  lps.count = be64_to_cpu(lps.count);

                  lps_print_vals(&lps, apcalc);
#endif /*LPS_CODE*/
               }
               break;
            case STAT_LDUR:
               {
#ifdef LDUR_CODE
                  ldur_t ld;
                  n = read(sock, &ld, sizeof(ldur_t));
                  if(n<0) printf("read got n=%d\n",n);
                  ld.min = be64_to_cpu(ld.min);
                  ld.max = be64_to_cpu(ld.max);
                  ld.std = be64_to_cpu(ld.std);
                  ld.total = be64_to_cpu(ld.total);
                  ld.count = be64_to_cpu(ld.count);

                  ldur_print_vals(&ld, apcalc);
                  ldur_print_vals(&ld, gnuplot);
#endif /*LDUR_CODE*/
               }
               break;
            case STAT_CLILST:
               {
                  u32 *ips;
                  u8 *bans;
                  int i;
                  u16 s = be16_to_cpu(hdr.length);

                  ips = malloc(s * sizeof(u32));
                  if( !ips) {printf("Failed to get memeory\n");continue;}
                  memset(ips,0,s * sizeof(u32));
                  bans = malloc(s);
                  memset(bans,0,s);
                  if( !bans) {printf("Failed to get memeory\n");continue;}

                  n = read(sock, ips, s * sizeof(u32));
                  if(n<0) printf("read got n=%d\n",n);
                  n = read(sock, bans, s );
                  if(n<0) printf("read got n=%d\n",n);

                  for(i=0;i<DEF_CLIENTS;i++) {
                     printf("IP:0x%08x  Flags:0x%02x\t%s",
                           ips[i], bans[i], (i%2)==1?"\n":"");
                  }
                  printf("\n");
                  free(ips);
                  free(bans);
               }
               break;
         }
         }
      } else
      if( strncasecmp("exit", token, 4) == 0) {
         break;
      } else
      if( strncasecmp("quit", token, 4) == 0) {
         break;
      } else
      if( strncasecmp("q", token, 1) == 0) {
         break;
      } else
      if( strncasecmp("!", token, 1) == 0) {
         token = strtok(NULL, "\n"); /* eat up to end of line*/
         if( ! token ) {printf("Bad params.\n"); continue;}
         /* i should probably inspect this string first..... */
         system( token );
      } else
      if( strncasecmp("decode", token, 6) == 0) {
         token = strtok(NULL, " \t");
         if( ! token ) {
            printf("Automagic buffer decoding is %s\n",
                  (decodebufs)?"True":"False");
            continue;
         }
         if( strncasecmp("true", token, 4) == 0)
            decodebufs = TRUE;
         else if( strncasecmp("false", token, 5) == 0)
            decodebufs = FALSE;
         else printf("Bad params.\n");
      } else
      {
         printf("Unknown command: \"%s\". try again.\n", token);
      }

      free(line); /* made it here, its got data we want to kill. */
   }

   return 0;
}

/* vim:ai cin et sw=3 ts=3
 */
